@@ -20,13 +20,21 @@ Boston, MA 02111-1307, USA.  */
 
 #ifndef __CMD_ATTACH_H__
 #define __CMD_ATTACH_H__
+#include <list>
+using namespace std;
+
+class symbol;
+class Pin_t;
+typedef list<Pin_t*> PinList_t;
 
 class cmd_attach : public command
 {
 public:
 
   cmd_attach(void);
-  void attach(char_list *);
+  void attach(list <string> *);
+  void attach(list <Value*> *);
+  void attach(Value*, PinList_t *);
 
 };
 
