@@ -23,21 +23,23 @@ Boston, MA 02111-1307, USA.  */
 
 #include <unistd.h>
 #include <glib.h>
+#include "command.h"
 
+class Value;
+#define CMDBREAK_BAD_BREAK_NUMBER 0xffff
 class cmd_break : public command
 {
 public:
 
-  cmd_break(void);
-  void list(void);
-
-  void set_break(int bit_flag);
-  void set_break(int bit_flag,guint64 value);
-  void set_break(int bit_flag,int reg, int value,int mask);
-  void set_break(int bit_flag,char *sym);
-  void set_break(int bit_flag,char *sym, int value,int mask);
+  cmd_break();
+  void list(guint64 value=CMDBREAK_BAD_BREAK_NUMBER);
 
+  unsigned int set_break(cmd_options *co, bool bLog=false);
+  unsigned int set_break(cmd_options *co, ExprList_t *pEL, bool bLog=false);
 
+private:
+  unsigned int set_break(cmd_options *co, Expression *pExpr1, Expression *pExpr2=0, bool bLog=false);
+  unsigned int set_break(int bit_flag, bool bLog);
 };
 
 extern cmd_break c_break;
