@@ -19,12 +19,11 @@ the Free Software Foundation, 59 Temple Place - Suite 330,
 Boston, MA 02111-1307, USA.  */
 
 
-#include <iostream.h>
-#include <iomanip.h>
+#include <iostream>
+#include <iomanip>
 #include <string>
 
 #include "command.h"
-#include "parse.h"
 #include "cmd_echo.h"
 
 // See cmd_echo.h for why this file is basically a skeleton
@@ -33,7 +32,7 @@ cmd_echo echo;
 
 static cmd_options cmd_echo_options[] =
 {
-  NULL,0,0
+  {0,0,0}
 };
 
 
