@@ -29,6 +29,7 @@ Boston, MA 02111-1307, USA.  */
 
 #include "../src/sim_context.h"
 #include "../src/processor.h"
+#include "../src/fopen-path.h"
 
 extern int parser_warnings;
 extern void redisplay_prompt(void);  // in input.cc
@@ -88,7 +89,6 @@ cmd_load::cmd_load(void)
 //
 //
 #define MAX_LINE_LENGTH 256  
-int parse_string(char *cmd_string);
 extern void process_command_file(const char * file_name);
 
 /**
@@ -131,25 +131,41 @@ int cmd_load::load(int bit_flag,const char *filename)
   // is being loaded.
 
   redisplay_prompt();
+
   return iReturn;
 }
 
-int cmd_load::load(Value *file, Value *pProcessorType) {
-  cout << endl;
-  string sFile;
-  string sProcType;
-  const char * psProcessorType = 0;
-  sFile = file->toString();
-  if (pProcessorType) {
-    sProcType = pProcessorType->toString();
-    psProcessorType = sProcType.c_str();
+//------------------------------------------------------------------------
+//
+// load 
+
+static int load1(const char *s1, const char *s2)
+{
+  FILE *fp = fopen_path(s1,"rb");
+  if (fp) {
+    fclose (fp);
+    return gpsim_open(get_active_cpu(), s1, s2, 0);
   }
 
-  return gpsim_open(get_active_cpu(), sFile.c_str(),
-    psProcessorType);
+  return s2 ? gpsim_open(get_active_cpu(), s2, 0, s1) : 0;
+
+}
+int cmd_load::load(Value *file, Value *pProcessorType)
+{
+  cout << endl;
+
+  char s1[256];
+  file->toString(s1, sizeof(s1));
+
+  if (pProcessorType) {
+    char s2[256];
+    pProcessorType->toString(s2, sizeof(s2));
+    return load1(s1, s2);
+  }
+  return load1(s1,0);
 }
 
-int cmd_load::load(const char *file, const char *pProcessorType) {
-  return gpsim_open(get_active_cpu(), file,
-    pProcessorType);
+int cmd_load::load(const char *file, const char *pProcessorType)
+{
+  return load1 (file, pProcessorType);
 }
