@@ -26,9 +26,9 @@ class cmd_load : public command
 public:
 
   cmd_load(void);
-  void load(void) { cout << "load command\n";};
-
-  void load(int bit_flag,char *filename);
+  int load(int bit_flag,const char *filename);
+  int load(Value *file, Value * pProcessorType = NULL);
+  int load(const char *file, const char *pProcessorType);
 };
 
 extern cmd_load c_load;
