@@ -19,8 +19,8 @@ the Free Software Foundation, 59 Temple Place - Suite 330,
 Boston, MA 02111-1307, USA.  */
 
 
-#include <iostream.h>
-#include <iomanip.h>
+#include <iostream>
+#include <iomanip>
 #include <string>
 
 #include "command.h"
@@ -28,15 +28,11 @@ Boston, MA 02111-1307, USA.  */
 
 #include "../src/pic-processor.h"
 
-#ifdef HAVE_GUI
-void gui_simulation_has_stopped(void);
-#endif
-
 cmd_reset reset;
 
 static cmd_options cmd_reset_options[] =
 {
-  NULL,0,0
+  {0,0,0}
 };
 
 
@@ -53,7 +49,7 @@ cmd_reset::cmd_reset(void)
 
 void cmd_reset::reset(void)
 {
-  if(cpu)
-    cpu->reset(POR_RESET);
+  if(GetActiveCPU())
+    GetActiveCPU()->reset(POR_RESET);
 
 }
