@@ -19,8 +19,8 @@ the Free Software Foundation, 59 Temple Place - Suite 330,
 Boston, MA 02111-1307, USA.  */
 
 
-#include <iostream.h>
-#include <iomanip.h>
+#include <iostream>
+#include <iomanip>
 #include <string>
 
 #include "command.h"
@@ -28,11 +28,13 @@ Boston, MA 02111-1307, USA.  */
 
 #include "../src/pic-processor.h"
 
+extern void redisplay_prompt(void);  // in input.cc
+
 cmd_run c_run;
 
 static cmd_options cmd_run_options[] =
 {
-  NULL,0,0
+  {0,0,0}
 };
 
 
@@ -40,27 +42,24 @@ cmd_run::cmd_run(void)
 { 
   name = "run";
 
-  brief_doc = string("Execute the pic program");
+  brief_doc = string("Initiate the simulation");
 
   long_doc = string ("run\n\
 \tStart simulating and don't stop until a break is encountered.\n\
+\tUse CTRL->C to halt the simulation execution.\n\
 \n");
 
   op = cmd_run_options; 
 }
 
-void cmd_run::run(void)
+void cmd_run::run()
 {
 
-  if(NULL == cpu)
-    {
-      cout << "no cpu has been selected\n";
-      return;
-    }
+  Integer &verbosity = *get_symbol_table().findInteger("sim.verbosity");
 
-  cout << "running...\n";
-
-  cpu->run();
+  get_interface().start_simulation();
 
+  if((int)verbosity)
+    redisplay_prompt();
 
 }
