@@ -21,36 +21,21 @@ Boston, MA 02111-1307, USA.  */
 #ifndef __CMD_STIMULUS_H__
 #define __CMD_STIMULUS_H__
 #include "../src/stimulus_orb.h"
+#include "command.h"
 
 class cmd_stimulus : public command
 {
 public:
   int valid_options,options_entered;
   int have_data;
-  /*
-  unsigned int period,
-    phase,
-    high_time,
-    start_cycle,
-    bit_pos,
-    states,
-    initial_state,
-    time_flag,
-    digital;
 
-  char *stim_name;
-
-  vector<StimulusDataType>  temp_array;
-  */
   cmd_stimulus(void);
   void stimulus(void);
 
   void stimulus(int bit_flag);
-  void stimulus(cmd_options_num *con);
+  void stimulus(cmd_options_expr *coe);
   void stimulus(cmd_options_str *cos);
-  void stimulus(cmd_options_float *cof);
-  void data_point(guint64 new_int_data_point);
-  void data_point(float new_float_data_point);
+  void stimulus(ExprList_t *);
   void end(void);
 
   bool can_span_lines(void) {return 1;};
