@@ -21,6 +21,9 @@ Boston, MA 02111-1307, USA.  */
 #ifndef __CMD_SYMBOL_H__
 #define __CMD_SYMBOL_H__
 
+class Expression;
+class Value;
+
 class cmd_symbol : public command
 {
 public:
@@ -28,10 +31,14 @@ public:
   cmd_symbol(void);
 
   void dump_all(void);
-  void dump_one(char *sym_name);
-  void add_one(char *sym_name, char *sym_type, int value);
+  void dump_one(const char *sym_name);
+  void dump_one(Value *);
+  void dump(Value *, ExprList_t*);
+  void EvaluateAndDisplay(Expression *pExpr);
 
+  void Set(Value *s, ExprList_t*e, Expression *pExpr);
 
+  void add_one(const char *sym_name, Expression *);
 };
 
 extern cmd_symbol c_symbol;
