@@ -19,8 +19,8 @@ the Free Software Foundation, 59 Temple Place - Suite 330,
 Boston, MA 02111-1307, USA.  */
 
 
-#include <iostream.h>
-#include <iomanip.h>
+#include <iostream>
+#include <iomanip>
 #include <string>
 
 #include "command.h"
@@ -32,14 +32,14 @@ extern void gpsim_version(void);
 
 static cmd_options cmd_version_options[] =
 {
-  NULL,0,0
+  {0,0,0}
 };
 
 
 cmd_version::cmd_version(void)
 { 
   name = "version";
-
+  abbreviation = "ver";
 
   brief_doc = string("Display the gpsim's version");
 
