@@ -21,7 +21,7 @@ Boston, MA 02111-1307, USA.  */
 #ifndef __MISC_H__
 #define __MISC_H__
 
-extern int verbose; 
+#include "../src/cmd_gpsim.h"
 
 // miscellaneous definitions that are used
 
@@ -38,26 +38,15 @@ struct cmd_options_num {
   int n;
 };
 
-   /* Command option with a float numeric parameter */
-
-struct cmd_options_float {
-  cmd_options *co;
-  float f;
-};
-
    /* Command option with a string parameter */
 
-struct cmd_options_str {
+class cmd_options_str
+{
+ public:
   cmd_options *co;
   char *str;
-};
-
-// Char list.
-// Here's a singly linked-list of char *'s.
-
-struct char_list {
-  char *name;
-  char_list *next;
+  cmd_options_str(const char *);
+  ~cmd_options_str();
 };
 
 #endif
