@@ -1,7 +1,7 @@
-/* A Bison parser, made by GNU Bison 1.875c.  */
+/* A Bison parser, made by GNU Bison 2.0.  */
 
 /* Skeleton parser for Yacc-like parsing with Bison,
-   Copyright (C) 1984, 1989, 1990, 2000, 2001, 2002, 2003 Free Software Foundation, Inc.
+   Copyright (C) 1984, 1989, 1990, 2000, 2001, 2002, 2003, 2004 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -73,46 +73,54 @@
      LITERAL_BOOL_T = 299,
      LITERAL_FLOAT_T = 300,
      LITERAL_STRING_T = 301,
-     SYMBOL_T = 302,
-     PIN_T = 303,
-     PORT_T = 304,
-     EQU_T = 305,
-     AND_T = 306,
-     COLON_T = 307,
-     COMMENT_T = 308,
-     DIV_T = 309,
-     EOLN_T = 310,
-     MINUS_T = 311,
-     MPY_T = 312,
-     OR_T = 313,
-     PLUS_T = 314,
-     SHL_T = 315,
-     SHR_T = 316,
-     XOR_T = 317,
-     LOR_T = 318,
-     LAND_T = 319,
-     NE_T = 320,
-     EQ_T = 321,
-     ABS_T = 322,
-     MAX_T = 323,
-     MIN_T = 324,
-     GE_T = 325,
-     GT_T = 326,
-     LE_T = 327,
-     LT_T = 328,
-     IND_T = 329,
-     BITS_T = 330,
-     BIT_T = 331,
-     WORD_T = 332,
-     LADDR_T = 333,
-     HIGH_T = 334,
-     LOW_T = 335,
-     INDEXED_T = 336,
-     UNARYOP_PREC = 337,
-     ONESCOMP_T = 338,
-     LNOT_T = 339,
-     POW_T = 340,
-     REG_T = 341
+     LITERAL_ARRAY_T = 302,
+     SYMBOL_T = 303,
+     PIN_T = 304,
+     PORT_T = 305,
+     EQU_T = 306,
+     AND_T = 307,
+     COLON_T = 308,
+     COMMENT_T = 309,
+     DIV_T = 310,
+     EOLN_T = 311,
+     MINUS_T = 312,
+     MPY_T = 313,
+     OR_T = 314,
+     PLUS_T = 315,
+     SHL_T = 316,
+     SHR_T = 317,
+     XOR_T = 318,
+     INDEXERLEFT_T = 319,
+     INDEXERRIGHT_T = 320,
+     DECLARE_TYPE = 321,
+     DECLARE_INT_T = 322,
+     DECLARE_FLOAT_T = 323,
+     DECLARE_BOOL_T = 324,
+     DECLARE_CHAR_T = 325,
+     LOR_T = 326,
+     LAND_T = 327,
+     NE_T = 328,
+     EQ_T = 329,
+     ABS_T = 330,
+     MAX_T = 331,
+     MIN_T = 332,
+     GE_T = 333,
+     GT_T = 334,
+     LE_T = 335,
+     LT_T = 336,
+     IND_T = 337,
+     BITS_T = 338,
+     BIT_T = 339,
+     WORD_T = 340,
+     LADDR_T = 341,
+     HIGH_T = 342,
+     LOW_T = 343,
+     INDEXED_T = 344,
+     UNARYOP_PREC = 345,
+     ONESCOMP_T = 346,
+     LNOT_T = 347,
+     POW_T = 348,
+     REG_T = 349
    };
 #endif
 #define ABORT 258
@@ -159,52 +167,60 @@
 #define LITERAL_BOOL_T 299
 #define LITERAL_FLOAT_T 300
 #define LITERAL_STRING_T 301
-#define SYMBOL_T 302
-#define PIN_T 303
-#define PORT_T 304
-#define EQU_T 305
-#define AND_T 306
-#define COLON_T 307
-#define COMMENT_T 308
-#define DIV_T 309
-#define EOLN_T 310
-#define MINUS_T 311
-#define MPY_T 312
-#define OR_T 313
-#define PLUS_T 314
-#define SHL_T 315
-#define SHR_T 316
-#define XOR_T 317
-#define LOR_T 318
-#define LAND_T 319
-#define NE_T 320
-#define EQ_T 321
-#define ABS_T 322
-#define MAX_T 323
-#define MIN_T 324
-#define GE_T 325
-#define GT_T 326
-#define LE_T 327
-#define LT_T 328
-#define IND_T 329
-#define BITS_T 330
-#define BIT_T 331
-#define WORD_T 332
-#define LADDR_T 333
-#define HIGH_T 334
-#define LOW_T 335
-#define INDEXED_T 336
-#define UNARYOP_PREC 337
-#define ONESCOMP_T 338
-#define LNOT_T 339
-#define POW_T 340
-#define REG_T 341
+#define LITERAL_ARRAY_T 302
+#define SYMBOL_T 303
+#define PIN_T 304
+#define PORT_T 305
+#define EQU_T 306
+#define AND_T 307
+#define COLON_T 308
+#define COMMENT_T 309
+#define DIV_T 310
+#define EOLN_T 311
+#define MINUS_T 312
+#define MPY_T 313
+#define OR_T 314
+#define PLUS_T 315
+#define SHL_T 316
+#define SHR_T 317
+#define XOR_T 318
+#define INDEXERLEFT_T 319
+#define INDEXERRIGHT_T 320
+#define DECLARE_TYPE 321
+#define DECLARE_INT_T 322
+#define DECLARE_FLOAT_T 323
+#define DECLARE_BOOL_T 324
+#define DECLARE_CHAR_T 325
+#define LOR_T 326
+#define LAND_T 327
+#define NE_T 328
+#define EQ_T 329
+#define ABS_T 330
+#define MAX_T 331
+#define MIN_T 332
+#define GE_T 333
+#define GT_T 334
+#define LE_T 335
+#define LT_T 336
+#define IND_T 337
+#define BITS_T 338
+#define BIT_T 339
+#define WORD_T 340
+#define LADDR_T 341
+#define HIGH_T 342
+#define LOW_T 343
+#define INDEXED_T 344
+#define UNARYOP_PREC 345
+#define ONESCOMP_T 346
+#define LNOT_T 347
+#define POW_T 348
+#define REG_T 349
 
 
 
 
 #if ! defined (YYSTYPE) && ! defined (YYSTYPE_IS_DECLARED)
-#line 104 "parse.yy"
+#line 138 "parse.yy"
 typedef union YYSTYPE {
   guint32              i;
   guint64             li;
@@ -226,13 +242,12 @@ typedef union YYSTYPE {
 
   StringList_t             *StringList_P;
   ExprList_t               *ExprList_P;
-//  SymbolList_t             *SymbolList_P;
   PinList_t                *PinList_P;
 
   Macro                    *Macro_P;
 } YYSTYPE;
-/* Line 1275 of yacc.c.  */
-#line 236 "parse.h"
+/* Line 1318 of yacc.c.  */
+#line 251 "parse.h"
 # define yystype YYSTYPE /* obsolescent; will be withdrawn */
 # define YYSTYPE_IS_DECLARED 1
 # define YYSTYPE_IS_TRIVIAL 1
