@@ -8,11 +8,11 @@
 /*****
 * eXdbm.c : eXdbm main source file
 *
-* This file Version	$Revision: 1.2 $
+* This file Version	$Revision: 313 $
 *
-* Last modification: 	$Date: 2002/01/04 13:12:25 $
-* By:					$Author: rfg $
-* Current State:		$State: Exp $
+* Last modification: 	$Date: 2003-10-26 13:54:54 -0800 (Sun, 26 Oct 2003) $
+* By:					$Author: sdattalo $
+* Current State:		$State$
 *
 * Copyright (C) 1997 Fred Pesch 
 * All Rights Reserved
@@ -35,10 +35,10 @@
 *
 *****/
 
-/* 	$Id: eXdbm.c,v 1.2 2002/01/04 13:12:25 rfg Exp $	 */
+/* 	$Id: eXdbm.c 313 2003-10-26 21:54:54Z sdattalo $	 */
 
 #ifndef lint
-static char vcid[] = "$Id: eXdbm.c,v 1.2 2002/01/04 13:12:25 rfg Exp $";
+static char vcid[] = "$Id: eXdbm.c 313 2003-10-26 21:54:54Z sdattalo $";
 #endif /* lint */
 
 /* includes */
@@ -301,7 +301,7 @@ int eXdbmOpenDatabase(char *filename,  DB_ID *dbid)
 
 int eXdbmNewDatabase(char *filename,  DB_ID *dbid)
 {
-  FILE *f;
+  /* FILE *f; */
   DB_ID temp_id;
   int ret;
   int i;
@@ -338,7 +338,7 @@ int eXdbmNewDatabase(char *filename,  DB_ID *dbid)
     DbmDbList->dblist = (TDbmDatabase *) realloc( DbmDbList->dblist, sizeof(TDbmDatabase) * DbmDbList->array_size);
     if(DbmDbList->dblist==NULL) {
       RaiseError(DBM_ALLOC);
-      fclose(f);
+      /* fclose(f); */
       return -1;
     }
     
@@ -362,7 +362,7 @@ int eXdbmNewDatabase(char *filename,  DB_ID *dbid)
   DbmDbList->dblist[temp_id].root = (TDbmListEntry *) malloc (sizeof(TDbmListEntry));
   if(DbmDbList->dblist[temp_id].root == NULL) {
     RaiseError(DBM_ALLOC);
-    fclose(f);
+    /* fclose(f); */
     return -1;
   }
 
@@ -377,7 +377,7 @@ int eXdbmNewDatabase(char *filename,  DB_ID *dbid)
   DbmDbList->dblist[temp_id].root->order = (TDbmListEntry **) malloc(sizeof(TDbmListEntry *) * MIN_ORDER_SIZE);
   if(DbmDbList->dblist[temp_id].root->order == NULL) {
     RaiseError(DBM_ALLOC);
-    fclose(f);
+    /* fclose(f); */
     return(-1);
   }
 
@@ -388,7 +388,7 @@ int eXdbmNewDatabase(char *filename,  DB_ID *dbid)
   
   if(DbmDbList->dblist[temp_id].root->child == NULL) {
     RaiseError(DBM_ALLOC);
-    fclose(f);
+    /* fclose(f); */
     return(-1);
   }
 
@@ -1496,7 +1496,7 @@ int eXdbmCreateVarReal(DB_ID dbid, DB_LIST list, char *entryname, char *comment,
 
   if (node==NULL) return(-1);
 
-  node->value.int_val = ceil(value);
+  node->value.int_val = (int)ceil(value);
   node->value.real_val = value;
 
   return (1);
