@@ -1,14 +1,14 @@
 
-/* 	$Id: test3.c,v 1.1 2001/12/27 16:31:10 sdattalo Exp $	 */
+/* 	$Id: test3.c 230 2002-04-15 22:03:02Z linas $	 */
 
 /*****
 * test3.c : eXdbm database tool
 *
-* This file Version	$Revision: 1.1 $
+* This file Version	$Revision: 230 $
 *
-* Last modification: 	$Date: 2001/12/27 16:31:10 $
-* By:					$Author: sdattalo $
-* Current State:		$State: Exp $
+* Last modification: 	$Date: 2002-04-15 15:03:02 -0700 (Mon, 15 Apr 2002) $
+* By:					$Author: linas $
+* Current State:		$State$
 *
 * Copyright (C) 1997 Fred Pesch 
 * All Rights Reserved
@@ -32,7 +32,7 @@
 *****/
 
 #ifndef lint
-static char vcid[] = "$Id: test3.c,v 1.1 2001/12/27 16:31:10 sdattalo Exp $";
+static char vcid[] = "$Id: test3.c 230 2002-04-15 22:03:02Z linas $";
 #endif /* lint */
 
 
