@@ -73,7 +73,8 @@ enum window_types {
   WT_scope_window,
   WT_status_bar,
   WT_sfr_window,
-  WT_list_source_window
+  WT_list_source_window,
+  WT_SourceWindow
 };
 
 //
@@ -192,13 +193,23 @@ public:
 
 };
 
+//------------------------------------------------------------------------
+// The KeyEvent class is used for key mapping (e.g. mapping the 's' 
+// key to step)
+
+class KeyEvent
+{
+public:
+  virtual void press(gpointer data = 0) = 0;
+  virtual void release(gpointer data = 0) = 0;
+};
+
 //
 // External references and function prototypes
 //
 
 extern GtkItemFactory *item_factory;
-// gui_processor.c
-extern GUI_Processor *gp;
+extern GUI_Processor *gpGuiProcessor;
 
 void exit_gpsim(void);
 
@@ -215,6 +226,7 @@ int config_get_variable(const char *module, const char *entry, int *value);
 int config_set_variable(const char *module, const char *entry, int value);
 int config_get_string(const char *module, const char *entry, char **string);
 int config_set_string(const char *module, const char *entry, const char *string);
+int config_remove(const char *module, const char *entry);
 
 gint gui_object_configure_event(GtkWidget *widget, GdkEventConfigure *e, GUI_Object *go);
 
@@ -224,6 +236,18 @@ void ProfileWindow_notify_stop_callback(Profile_Window *pw);
 int gui_get_value(char *prompt);
 
 
+//------------------------------------------------------------------------
+// Functions to abstract differences between gtk 1.x and 2.x
+#if GTK_MAJOR_VERSION >= 2
+int gStringWidth(PangoFontDescription *font, const char *str);
+int gStringHeight(PangoFontDescription *font, const char *str);
+GdkFont *gFontFromDescription(PangoFontDescription *);
+#else
+int gStringWidth(GdkFont *font, const char *str);
+int gStringHeight(GdkFont *font, const char *str);
+GdkFont *gFontFromDescription(GdkFont *pF) { return pF;}
+#endif
+
 #endif // __GUI_H__
 
 #endif // HAVE_GUI
