@@ -74,7 +74,7 @@ void link_src_to_gpsim(GUI_Processor *gp);
  * --- Global variables
  */
 
-GUI_Processor *gp=0;
+GUI_Processor *gpGuiProcessor=0;
 GSList *gui_processors=0;
 unsigned int interface_id=0;
 
@@ -161,7 +161,7 @@ GUI_Interface::~GUI_Interface()
     gp->trace_window->set_config();
     gp->profile_window->set_config();
     gp->stopwatch_window->set_config();
-    //gp->scope_window->set_config();
+    gp->scope_window->set_config();
   }
 }
 
@@ -234,6 +234,7 @@ static void *SimulationHasStopped( void *ptr )
       lgp->trace_window->Update();
       lgp->profile_window->Update();
       lgp->stopwatch_window->Update();
+      lgp->scope_window->Update();
 
 
       
@@ -371,6 +372,32 @@ void GUI_Interface::Update(gpointer object)
 
 
 
+/*------------------------------------------------------------------
+ * 
+ */
+#if GTK_MAJOR_VERSION >= 2
+int gStringWidth(PangoFontDescription *font, const char *str)
+{
+  return (font && str) ? gdk_string_width (gdk_font_from_description(font),str) : 0;
+}
+int gStringHeight(PangoFontDescription *font, const char *str)
+{
+  return (font && str) ? gdk_string_height (gdk_font_from_description(font),str) : 0;
+}
+GdkFont *gFontFromDescription(PangoFontDescription *font)
+{
+  return (font ? gdk_font_from_description(font) : 0);
+}
+#else
+int gStringWidth(GdkFont *font, const char *str)
+{
+  return (font && str) ? gdk_string_width (font,str) : 0;
+}
+int gStringHeight(GdkFont *font, const char *str)
+{
+  return (font && str) ? gdk_string_height (font,str) : 0;
+}
+#endif
 
 
 /*------------------------------------------------------------------
@@ -440,8 +467,8 @@ int gui_init (int argc, char **argv)
 
 
   gte();
-  gp = new GUI_Processor();
-  interface_id = get_interface().add_interface(new GUI_Interface(gp));
+  gpGuiProcessor = new GUI_Processor();
+  interface_id = get_interface().add_interface(new GUI_Interface(gpGuiProcessor));
   gtl();
 
   return(0);
