@@ -65,7 +65,7 @@ typedef struct _note_book_item
 
 GtkItemFactory *item_factory=0;
 
-extern GUI_Processor *gp;
+extern GUI_Processor *gpGuiProcessor;
 
 static void 
 do_quit_app(GtkWidget *widget) 
@@ -124,37 +124,17 @@ about_cb (gpointer             callback_data,
 
 }
 
+//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+
+#if defined(NEW_SOURCE_BROWSER)
+
+//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+
 
 //========================================================================
-
-// class ColorSelection 
 //
-// A class to keep track of the state of configurable colors.
-//
-class ColorSelection
-{
-public:
-  ColorSelection(GdkColor *pC)
-    : m_current(pC)
-  {
-    m_preferred=0;
-    setPreferred(pC);
-  }
-  bool hasChanged()    { return gdk_color_equal(m_current,m_preferred) != TRUE;}
-  GdkColor *getPreferred() { return m_preferred;}
-  const char *getPreferred(char *);
-  void setPreferred(GdkColor *cP_preferred) 
-  { 
-    if (m_preferred)
-      gdk_color_free(m_preferred);
-    m_preferred = gdk_color_copy(cP_preferred);
-  }
-protected:
-  GdkColor *m_current;
-  GdkColor *m_preferred;
-};
-
-
 // class ColorButton
 //
 // Creates a GtkColorButton and places it into a parent widget.
@@ -162,54 +142,136 @@ protected:
 // call back into this class and keep track of the selected 
 // color state.
 class SourceBrowserPreferences;
-class ColorButton : public ColorSelection
+class ColorButton
 {
 public:
   ColorButton (GtkWidget *pParent, 
-	       GdkColor *pC, 
+	       TextStyle *pStyle,
 	       const char *label,
 	       SourceBrowserPreferences *
 	       );
   static void setColor_cb(GtkColorButton *widget,
 			  ColorButton    *This);
   void apply();
-  void addTagRange(int start_index, int end_index);
-  void setBuffer(GtkTextBuffer *pBuffer)
-  {
-    m_buffer = pBuffer; 
-    char cParray[20];
-    m_tag = gtk_text_buffer_create_tag (pBuffer, m_label,
-				      "foreground",
-				      getPreferred(cParray),
-				      NULL);
-
-
-  }
+  void cancel();
+  TextStyle *m_pStyle;
 private:
   SourceBrowserPreferences *m_prefs;
-  GtkTextTag *m_tag;
   const char *m_label;
-  GtkTextBuffer *m_buffer;
 };
 
-class SourceBrowserPreferences
+//========================================================================
+//
+// class MarginButton
+//
+// Creates a GtkCheckButton that is used to select whether line numbers,
+// addresses or opcodes will be displayed in the source browser margin.
+
+class MarginButton
+{
+public:
+  enum eMarginType {
+    eLineNumbers,
+    eAddresses,
+    eOpcodes
+  };
+  MarginButton (GtkWidget *pParent,
+		const char *pName,
+		eMarginType id,
+		SourceBrowserPreferences *
+	       );
+  static void toggle_cb(GtkToggleButton *widget,
+			MarginButton    *This);
+  void set_active();
+private:
+  GtkWidget *m_button;
+  SourceBrowserPreferences *m_prefs;
+  eMarginType m_id;
+};
+
+//========================================================================
+//
+// class TabButton
+//
+// Creates a GtkCheckButton that is used to select whether line numbers,
+// addresses or opcodes will be displayed in the source browser margin.
+
+class TabButton
+{
+public:
+  TabButton (GtkWidget *pParent, GtkWidget *pButton,
+	     int id,
+	     SourceBrowserPreferences *
+	     );
+  static void toggle_cb(GtkToggleButton *widget,
+			TabButton    *This);
+  void set_active();
+private:
+  GtkWidget *m_button;
+  SourceBrowserPreferences *m_prefs;
+  int m_id;
+};
+
+//========================================================================
+//
+// class FontSelection
+//
+
+class FontSelection
+{
+public:
+  FontSelection (GtkWidget *pParent, 
+		 SourceBrowserPreferences *
+		 );
+  static void setFont_cb(GtkFontButton *widget,
+			FontSelection  *This);
+  void setFont();
+private:
+  SourceBrowserPreferences *m_prefs;
+  GtkWidget *m_fontButton;
+};
+
+//========================================================================
+//
+class SourceBrowserPreferences : public SourceWindow
 {
 public:
   SourceBrowserPreferences(GtkWidget *pParent);
 
   void apply();
+  void cancel();
   void update();
-  void parseLine(const char*);
+  void toggleBreak(int line);
+  void movePC(int line);
+
+  virtual int getPCLine(int page);
+  virtual int getAddress(NSourcePage *pPage, int line);
+  virtual bool bAddressHasBreak(int address);
+  virtual int getOpcode(int address);
+  void setTabPosition(int);
+  void setFont(const char *);
+  const char *getFont();
 private:
-  GtkTextView *m_view;
+
   ColorButton *m_LabelColor;
   ColorButton *m_MnemonicColor;
   ColorButton *m_SymbolColor;
   ColorButton *m_CommentColor;
   ColorButton *m_ConstantColor;
 
-  GtkWidget   *m_SampleNotebook;
-  GtkPositionType m_TabPosition;
+  MarginButton *m_LineNumbers;
+  MarginButton *m_Addresses;
+  MarginButton *m_Opcodes;
+
+  int m_currentTabPosition;
+  int m_originalTabPosition;
+  TabButton    *m_Up;
+  TabButton    *m_Left;
+  TabButton    *m_Down;
+  TabButton    *m_Right;
+  TabButton    *m_None;
+
+  FontSelection *m_FontSelector;
 };
 
 //------------------------------------------------------------------------
@@ -219,8 +281,6 @@ public:
   gpsimGuiPreferences();
   ~gpsimGuiPreferences();
 
-  void apply();
-
   static void setup (gpointer             callback_data,
 		     guint                callback_action,
 		     GtkWidget           *widget);
@@ -230,9 +290,9 @@ private:
   SourceBrowserPreferences *m_SourceBrowser;
 
   static gint cancel_cb (gpsimGuiPreferences *Self);
-
   static gint apply_cb  (gpsimGuiPreferences *Self);
-
+  void apply() { m_SourceBrowser->apply();}
+  void cancel() { m_SourceBrowser->cancel();}
   GtkWidget *window;
 };
 static GtkWidget *LocalWindow=0;
@@ -242,53 +302,35 @@ void gpsimGuiPreferences::setup (gpointer             callback_data,
 				 guint                callback_action,
 				 GtkWidget           *widget)
 {
-  if (0)
-    new gpsimGuiPreferences();
+  new gpsimGuiPreferences();
 }
 
 
 gint gpsimGuiPreferences::cancel_cb (gpsimGuiPreferences  *Self)
 {
-  printf (" cancel preferences %p \n",Self);
+
+  Self->cancel();
   delete Self;
   return TRUE;
 }
 gint gpsimGuiPreferences::apply_cb  (gpsimGuiPreferences *Self)
 {
-  printf (" apply preferences\n");
+
   Self->apply();
   delete Self;
   return TRUE;
 }
 
-//========================================================================
-struct sTest
-{
-  const char *cName;
-  int id;
-};
-
-static void setFont_cb (GtkWidget     *pFontButton,
-			gpointer      callback_data)
-{
-
-  sTest *s = (sTest *) callback_data;
-  if (s)
-    printf("setFont_cb %s %d\n", s->cName, s->id);
-
-  printf("Font %s\n", gtk_font_button_get_font_name (GTK_FONT_BUTTON(pFontButton)));
-}
-
 //------------------------------------------------------------------------
 // ColorButton Constructor
-ColorButton::ColorButton(GtkWidget *pParent, GdkColor *pColor, 
+ColorButton::ColorButton(GtkWidget *pParent, TextStyle *pStyle,
 			 const char *colorName,SourceBrowserPreferences *prefs)
-  : ColorSelection(pColor), m_prefs(prefs), m_label(colorName)
+  : m_pStyle(pStyle),m_prefs(prefs), m_label(colorName)
 {
   GtkWidget *hbox        = gtk_hbox_new(0,0);
   gtk_box_pack_start (GTK_BOX (pParent), hbox, FALSE, TRUE, 0);
 
-  GtkWidget *colorButton = gtk_color_button_new_with_color (pColor);
+  GtkWidget *colorButton = gtk_color_button_new_with_color (pStyle->mFG.CurrentColor());
   gtk_color_button_set_title (GTK_COLOR_BUTTON(colorButton), colorName);
   gtk_box_pack_start (GTK_BOX(hbox),colorButton,FALSE, FALSE, 0);
   gtk_widget_show(colorButton);
@@ -297,34 +339,154 @@ ColorButton::ColorButton(GtkWidget *pParent, GdkColor *pColor,
 		      "color-set", 
 		      GTK_SIGNAL_FUNC(setColor_cb),
 		      this);
-  printf("create color button %s %p %p\n",colorName,colorButton,this);
   const int cBORDER = 10; // pixels
   GtkWidget *label       = gtk_label_new(colorName);
-  gtk_box_pack_start (GTK_BOX(hbox),label,TRUE, TRUE, cBORDER);
+  gtk_box_pack_start (GTK_BOX(hbox),label,FALSE,FALSE, cBORDER);
   gtk_widget_show (label);
 
   gtk_widget_show (hbox);
-
-
 }
+
 //------------------------------------------------------------------------
 void ColorButton::setColor_cb(GtkColorButton *widget,
 			      ColorButton    *This)
 {
-  printf("setColor_cb  %p %p\n",widget,This);
   GdkColor newColor;
   gtk_color_button_get_color (widget, &newColor);
-  This->setPreferred(&newColor);
-  This->apply();
-}
-const char *ColorSelection::getPreferred(char *cParray)
-{
-  sprintf(cParray,"#%04X%04X%04X",
-	  m_preferred->red, m_preferred->green, m_preferred->blue);
-  return cParray;
+  This->m_pStyle->setFG(&newColor);
 }
 
-bool isButtonEvent (GdkEventType type)
+void ColorButton::apply()
+{
+  m_pStyle->apply();
+}
+void ColorButton::cancel()
+{
+  m_pStyle->revert();
+}
+
+//------------------------------------------------------------------------
+MarginButton::MarginButton(GtkWidget *pParent, const char *pName,
+			   eMarginType id,
+			   SourceBrowserPreferences *prefs)
+  : m_prefs(prefs), m_id(id)
+{
+  m_button = gtk_check_button_new_with_label (pName);
+  bool bState = false;
+  switch (m_id) {
+  case eLineNumbers:
+    bState = m_prefs->margin().bLineNumbers();
+    break;
+  case eAddresses:
+    bState = m_prefs->margin().bAddresses();
+    break;
+  case eOpcodes:
+    bState = m_prefs->margin().bOpcodes();
+    break;
+  }
+  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(m_button),
+				bState);
+  gtk_box_pack_start (GTK_BOX (pParent), m_button, FALSE, TRUE, 10);
+
+  gtk_signal_connect (GTK_OBJECT(m_button),
+		      "toggled", 
+		      GTK_SIGNAL_FUNC(toggle_cb),
+		      this);
+}
+//------------------------------------------------------------------------
+void MarginButton::toggle_cb(GtkToggleButton *widget,
+			     MarginButton    *This)
+{
+  //  This->m_pStyle->setFG(&newColor);
+  This->set_active();
+}
+void MarginButton::set_active()
+{
+  bool bNewState = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(m_button)) ? true : false;
+  switch (m_id) {
+  case eLineNumbers:
+    m_prefs->margin().enableLineNumbers(bNewState);
+    break;
+  case eAddresses:
+    m_prefs->margin().enableAddresses(bNewState);
+    break;
+  case eOpcodes:
+    m_prefs->margin().enableOpcodes(bNewState);
+    break;
+  }
+
+}
+
+//------------------------------------------------------------------------
+TabButton::TabButton(GtkWidget *pParent, GtkWidget *pButton,
+		     int id,
+		     SourceBrowserPreferences *prefs)
+  : m_button(pButton), m_prefs(prefs), m_id(id)
+{
+  gtk_box_pack_start (GTK_BOX (pParent), m_button, FALSE, TRUE, 5);
+
+  gtk_signal_connect (GTK_OBJECT(m_button),
+		      "toggled", 
+		      GTK_SIGNAL_FUNC(toggle_cb),
+		      this);
+}
+//------------------------------------------------------------------------
+void TabButton::toggle_cb(GtkToggleButton *widget,
+			  TabButton    *This)
+{
+  //  This->m_pStyle->setFG(&newColor);
+  This->set_active();
+}
+void TabButton::set_active()
+{
+  if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(m_button)))
+    m_prefs->setTabPosition(m_id);
+}
+
+//------------------------------------------------------------------------
+FontSelection::FontSelection (GtkWidget *pParent,
+			      SourceBrowserPreferences *pPrefs)
+  : m_prefs(pPrefs)
+{
+  GtkWidget *frame = gtk_frame_new ("Font");
+
+  gtk_box_pack_start (GTK_BOX (pParent), frame, FALSE, TRUE, 0);
+  GtkWidget *hbox        = gtk_hbox_new(0,0);
+  gtk_container_add (GTK_CONTAINER (frame), hbox);
+
+  m_fontButton = gtk_font_button_new_with_font (m_prefs->getFont());
+  const char *fontDescription = "Font Selector";
+  gtk_font_button_set_title (GTK_FONT_BUTTON(m_fontButton), fontDescription);
+  gtk_box_pack_start (GTK_BOX(hbox),m_fontButton,FALSE, FALSE, 0);
+  gtk_widget_show(m_fontButton);
+
+  gtk_signal_connect (GTK_OBJECT(m_fontButton), 
+		      "font-set", 
+		      GTK_SIGNAL_FUNC(setFont_cb),
+		      this);
+
+  const char *fontName = "font";
+  const int cBORDER = 10; // pixels
+  GtkWidget *label       = gtk_label_new(fontName);
+  gtk_box_pack_start (GTK_BOX(hbox),label,TRUE, TRUE, cBORDER);
+  gtk_widget_show (label);
+
+  gtk_widget_show (hbox);
+}
+//------------------------------------------------------------------------
+void FontSelection::setFont_cb (GtkFontButton *pFontButton,
+				FontSelection *This)
+{
+  This->setFont();
+}
+void FontSelection::setFont()
+{
+  m_prefs->setFont(gtk_font_button_get_font_name (GTK_FONT_BUTTON(m_fontButton)));
+}
+
+//------------------------------------------------------------------------
+#if 0 // defined but not used
+static bool isButtonEvent (GdkEventType type)
 {
   return 
     type == GDK_BUTTON_PRESS ||
@@ -333,7 +495,8 @@ bool isButtonEvent (GdkEventType type)
     type == GDK_BUTTON_RELEASE;
 
 }
-gboolean    TagEvent  (GtkTextTag *texttag,
+
+static gboolean    TagEvent  (GtkTextTag *texttag,
 		       GObject *arg1,
 		       GdkEvent *event,
 		       GtkTextIter *arg2,
@@ -353,372 +516,224 @@ gboolean    TagEvent  (GtkTextTag *texttag,
   }
   return FALSE;
 }
-
-//------------------------------------------------------------------------
-// addTagRange(int start_index, int end_index)
-//
-// Each color button has an associated buffer and tag. 
-// Addtag range applies the tag state to a range of text in the buffer
-void ColorButton::addTagRange(int start_index, int end_index)
-{
-  GtkTextIter    start;
-  GtkTextIter    end;
-  gtk_text_buffer_get_iter_at_offset (m_buffer, &start, start_index);
-  gtk_text_buffer_get_iter_at_offset (m_buffer, &end, end_index);
-
-  gtk_text_buffer_apply_tag (m_buffer, m_tag, &start, &end);
-
-  g_signal_connect (G_OBJECT (m_tag), "event",
-		    GTK_SIGNAL_FUNC(TagEvent),
-		    this);
-  printf("Added TagRange event for %p\n",this);
-
-}
-
-void ColorButton::apply()
-{
-  char array[20];
-  g_object_set(m_tag, "foreground" , getPreferred(array), NULL);
-}
 //------------------------------------------------------------------------
 
 static void preferences_AddFontSelect(GtkWidget *pParent, const char *fontDescription,
 				      const char *fontName  )
 {
-  GtkWidget *frame = gtk_frame_new ("Font");
-
-  gtk_box_pack_start (GTK_BOX (pParent), frame, FALSE, TRUE, 0);
-  GtkWidget *hbox        = gtk_hbox_new(0,0);
-  //gtk_box_pack_start (GTK_BOX (pParent), hbox, FALSE, TRUE, 0);
-  gtk_container_add (GTK_CONTAINER (frame), hbox);
-
-  //GtkWidget *fontButton = gtk_font_button_new_with_font (fontName);
-  GtkWidget *fontButton = gtk_font_button_new ();
-  gtk_font_button_set_title (GTK_FONT_BUTTON(fontButton), fontDescription);
-  gtk_box_pack_start (GTK_BOX(hbox),fontButton,FALSE, FALSE, 0);
-  gtk_widget_show(fontButton);
-
-  sTest *s = new sTest();
-  s->cName = fontDescription;
-  s->id = 42;
-
-  gtk_signal_connect (GTK_OBJECT(fontButton), 
-		      "font-set", 
-		      GTK_SIGNAL_FUNC(setFont_cb),
-		      s);
-
-  const int cBORDER = 10; // pixels
-  GtkWidget *label       = gtk_label_new(fontName);
-  gtk_box_pack_start (GTK_BOX(hbox),label,TRUE, TRUE, cBORDER);
-  gtk_widget_show (label);
-
-  gtk_widget_show (hbox);
 }
+#endif
 
 
-//========================================================================
-static int isString(const char *cP)
+void SourceBrowserPreferences::toggleBreak(int line)
 {
-  int i=0;
 
-  if (isalpha(*cP) || *cP=='_')
-    while (isalnum(cP[i]) || cP[i]=='_')
-      i++;
-  return i;
 }
 
-static int isWhiteSpace(const char *cP)
+
+void SourceBrowserPreferences::movePC(int line)
 {
-  int i=0;
-
-  while (cP[i]==' ' || cP[i]=='\t')
-    i++;
-  return i;
 }
 
-static int isHexNumber(const char *cP)
-{
-  int i=0;
-  if ((*cP == '0' && toupper(cP[1])=='X') ||
-      (*cP == '$')) {
-    i = (*cP=='0') ? 2 : 1;
 
-    while (isxdigit(cP[i]))
-      i++;
-  }
-  return i;
-}
-static int isNumber(const char *cP)
-{
-  int i=isHexNumber(cP);
-  if (!i)
-    while (isdigit(cP[i]))
-      i++;
-  return i;
-}
-
-static bool isEnd(const char c)
-{
-  return c=='\n' || c==0;
-}
-
-static int isComment(const char *cP)
-{
-  int i = (*cP==';') ? 1 : 0;
-  if (i)
-    while (!isEnd(cP[i]))
-      i++;
-  return i;
-}
-
-//#include "../xpms/break.xpm"
-static char * break_xpm[] = {
-"12 12 3 1",
-" 	c None",
-".	c gray",
-"X	c white",
-"   .......    ",
-"  .........   ",
-" ...........  ",
-" ...XXXXX.... ",
-" ...XXXXX.... ",
-" ...XXXXX.... ",
-" ...XXXXX.... ",
-" ...XXXXX.... ",
-" ...........  ",
-"  .........   ",
-"   .......    ",
-"              "};
-
-void SourceBrowserPreferences::parseLine(const char *cP)
-{
-  GtkTextBuffer *buffer = gtk_text_view_get_buffer (m_view);
-  GtkTextIter iEnd;
-
-  gtk_text_buffer_get_end_iter (buffer, &iEnd);
-  if(1){
-
-    //GtkWidget *widget = gtk_image_new_from_stock (GTK_STOCK_GO_FORWARD,
-    //						  GTK_ICON_SIZE_BUTTON);
-    //GtkWidget *widget = gtk_check_button_new();
-
-    static GdkPixmap *pixmap_break = 0;
-    static GdkBitmap *bp_mask = 0;
-
-    if (pixmap_break == 0) {
-      GtkStyle  *style;
-
-      style=gtk_style_new();
-
-      pixmap_break = gdk_pixmap_create_from_xpm_d(LocalWindow->window,
-						  &bp_mask,
-						  &style->bg[GTK_STATE_NORMAL],
-						  (gchar**)break_xpm);
-    GtkWidget *widget = gtk_image_new_from_pixmap (pixmap_break,
-						   bp_mask);
-
-    GtkTextChildAnchor *anchor =
-      gtk_text_buffer_create_child_anchor (buffer, &iEnd);
-
-    gtk_text_view_add_child_at_anchor (m_view,
-				       widget,
-				       anchor);
-
-    gtk_widget_show_all (widget);
-    }
-
-  }
-
-
-  //gtk_text_buffer_get_end_iter (buffer, &iEnd);
-
-  char buf[64];
-  int line_number = gtk_text_buffer_get_line_count(buffer);
-  static int opcode=0x1234;
-
-  snprintf(buf, sizeof(buf), "\t%5d %04X ",line_number,opcode);
-  //gtk_text_buffer_insert_with_tags_by_name (buffer, &iEnd, buf, -1, "margin", NULL);
-  gtk_text_buffer_insert (buffer, &iEnd, buf, -1);
-
-  //gtk_text_buffer_get_end_iter (buffer, &iEnd);
-  int offset = gtk_text_iter_get_offset (&iEnd);
-
-  //gtk_text_buffer_insert_with_tags_by_name (buffer, &iEnd, cP, -1, "margin", NULL);
-  gtk_text_buffer_insert (buffer, &iEnd, cP, -1);
-
-  int i=0;
-  int j=0;
-  bool bHaveMnemonic = false;
-
-  if (i != (j = isString(cP))) {
-    //printf ("label %d:%d %d\n",i,j,offset);
-    m_LabelColor->addTagRange(i+offset,j+offset);
-    i=j;
-  }
-  while (!isEnd(cP[i])) {
-
-    if ( (j=isWhiteSpace(&cP[i])) != 0) {
-      //printf ("White space %d:%d\n",i,i+j);
-      i += j;
-    } else if ( (j=isString(&cP[i])) != 0) {
-      if (bHaveMnemonic)
-	m_SymbolColor->addTagRange(i+offset,i+j+offset);
-      else
-	m_MnemonicColor->addTagRange(i+offset,i+j+offset);
-      bHaveMnemonic = true;
-      i += j;
-    } else if ( (j=isNumber(&cP[i])) != 0) {
-      m_ConstantColor->addTagRange(i+offset,i+j+offset);
-      //printf ("number %d:%d\n",i,i+j);
-      i += j;
-    } else if ( (j=isComment(&cP[i])) != 0) {
-      m_CommentColor->addTagRange(i+offset,i+j+offset);
-      //printf ("comment %d:%d\n",i,i+j);
-      i += j;
-      return;
-    } else 
-      i++;
-  }
-}
 //========================================================================
 SourceBrowserPreferences::SourceBrowserPreferences(GtkWidget *pParent)
+  : SourceWindow(0,0,false,0)
 {
 
-  if (!gp && !gp->source_browser)
-    return;
-  SourceBrowserAsm_Window *sbaw = gp->source_browser->getChild(0);
-  if (!sbaw)
+  if (!gpGuiProcessor && !gpGuiProcessor->source_browser)
     return;
 
-  GtkWidget *hbox2 = gtk_hbox_new(0,0);
-  gtk_box_pack_start (GTK_BOX (pParent), hbox2, FALSE, TRUE, 0);
+  GtkWidget *notebook = gtk_notebook_new();
+  gtk_notebook_set_tab_pos((GtkNotebook*)notebook,GTK_POS_TOP);
+  gtk_box_pack_start (GTK_BOX (pParent), notebook, TRUE, TRUE, 0);
+  gtk_widget_show(notebook);
 
-  GtkWidget *vbox3 = gtk_vbox_new(0,0);
-  gtk_box_pack_start (GTK_BOX (hbox2), vbox3, FALSE, TRUE, 0);
+  m_pParent = gpGuiProcessor->source_browser;
+  GtkWidget *label;
 
   {
     // Color Frame for Source Browser configuration
+
+    GtkWidget *vbox = gtk_vbox_new(0,0);
+
     GtkWidget *colorFrame = gtk_frame_new ("Colors");
-    gtk_box_pack_start (GTK_BOX (hbox2), colorFrame, FALSE, TRUE, 0);
+    gtk_box_pack_start (GTK_BOX (vbox), colorFrame, FALSE, TRUE, 0);
 
     GtkWidget *colorVbox = gtk_vbox_new(0,0);
     gtk_container_add (GTK_CONTAINER (colorFrame), colorVbox);
 
-    m_LabelColor    = new ColorButton(GTK_WIDGET(colorVbox), 
-				      &sbaw->label_text_style->fg[GTK_STATE_NORMAL],
+    m_LabelColor    = new ColorButton(colorVbox, 
+				      m_pParent->mLabel,
 				      "Label", this);
-    m_MnemonicColor = new ColorButton(GTK_WIDGET(colorVbox), 
-				      &sbaw->instruction_text_style->fg[GTK_STATE_NORMAL],
+    m_MnemonicColor = new ColorButton(colorVbox, 
+				      m_pParent->mMnemonic,
 				      "Mnemonic", this);
-    m_SymbolColor   = new ColorButton(GTK_WIDGET(colorVbox), 
-				      &sbaw->symbol_text_style->fg[GTK_STATE_NORMAL],
+    m_SymbolColor   = new ColorButton(colorVbox, 
+				      m_pParent->mSymbol,
 				      "Symbols", this);
-    m_ConstantColor = new ColorButton(GTK_WIDGET(colorVbox),
-				      &sbaw->number_text_style->fg[GTK_STATE_NORMAL],
+    m_ConstantColor = new ColorButton(colorVbox,
+				      m_pParent->mConstant,
 				      "Constants", this);
-    m_CommentColor  = new ColorButton(GTK_WIDGET(colorVbox),
-				      &sbaw->comment_text_style->fg[GTK_STATE_NORMAL],
+    m_CommentColor  = new ColorButton(colorVbox,
+				      m_pParent->mComment,
 				      "Comments", this);
+
+    // Font selector
+    //preferences_AddFontSelect(GTK_WIDGET(vbox), "Font Selector", "font");
+    m_FontSelector = new FontSelection(vbox,this);
+
+    label = gtk_label_new("Font");
+    gtk_notebook_append_page(GTK_NOTEBOOK(notebook),vbox,label);
+
+
+
   }
 
   {
     // Tab Frame for the Source browser
-    m_TabPosition = GTK_POS_TOP;
-    GtkWidget *tabFrame = gtk_frame_new ("Tabs");
-    gtk_box_pack_start (GTK_BOX (vbox3), tabFrame, FALSE, TRUE, 0);
+    m_currentTabPosition = m_pParent->getTabPosition();
+    m_originalTabPosition = m_currentTabPosition;
 
-    GtkWidget *radioUp    = gtk_radio_button_new_with_label (NULL,"up");
-    GtkRadioButton *rb    = GTK_RADIO_BUTTON(radioUp);
-    GtkWidget *radioLeft  = gtk_radio_button_new_with_label_from_widget (rb,"left");
-    GtkWidget *radioDown  = gtk_radio_button_new_with_label_from_widget (rb,"down");
-    GtkWidget *radioRight = gtk_radio_button_new_with_label_from_widget (rb,"right");
-    GtkWidget *radioNone  = gtk_radio_button_new_with_label_from_widget (rb,"none");
+    GtkWidget *hbox = gtk_hbox_new(0,0);
+    GtkWidget *tabFrame = gtk_frame_new ("Notebook Tabs");
+    gtk_box_pack_start (GTK_BOX (hbox), tabFrame, FALSE, TRUE, 0);
+
+    GtkWidget *radioUp  = gtk_radio_button_new_with_label (NULL,"up");
+    GtkRadioButton *rb  = GTK_RADIO_BUTTON(radioUp);
 
     GtkWidget *tabVbox = gtk_vbox_new(0,0);
     gtk_container_add (GTK_CONTAINER (tabFrame), tabVbox);
-    
-    gtk_box_pack_start (GTK_BOX (tabVbox), radioUp,   FALSE, TRUE, 0);
-    gtk_box_pack_start (GTK_BOX (tabVbox), radioLeft, FALSE, TRUE, 0);
-    gtk_box_pack_start (GTK_BOX (tabVbox), radioDown, FALSE, TRUE, 0);
-    gtk_box_pack_start (GTK_BOX (tabVbox), radioRight,FALSE, TRUE, 0);
-    gtk_box_pack_start (GTK_BOX (tabVbox), radioNone, FALSE, TRUE, 0);
+
+    m_Up    = new TabButton(tabVbox, radioUp, GTK_POS_TOP, this);
+    m_Left  = new TabButton(tabVbox, gtk_radio_button_new_with_label_from_widget (rb,"left"),
+			   GTK_POS_LEFT, this);
+    m_Down  = new TabButton(tabVbox, gtk_radio_button_new_with_label_from_widget (rb,"down"),
+			   GTK_POS_BOTTOM, this);
+    m_Right = new TabButton(tabVbox, gtk_radio_button_new_with_label_from_widget (rb,"right"),
+			   GTK_POS_RIGHT, this);
+    m_None  = new TabButton(tabVbox, gtk_radio_button_new_with_label_from_widget (rb,"none"),
+			   -1, this);
+
+
+    // Source browser margin 
+    GtkWidget *marginFrame = gtk_frame_new ("Margin");
+    gtk_box_pack_start (GTK_BOX (hbox), marginFrame, FALSE, TRUE, 0);
+    GtkWidget *marginVbox = gtk_vbox_new(0,0);
+    gtk_container_add (GTK_CONTAINER (marginFrame), marginVbox);
+
+    m_LineNumbers = new MarginButton(marginVbox, "Line Numbers", 
+				     MarginButton::eLineNumbers, this);
+    m_Addresses   = new MarginButton(marginVbox, "Addresses", 
+				     MarginButton::eAddresses, this);
+    m_Opcodes     = new MarginButton(marginVbox, "Opcodes", 
+				     MarginButton::eOpcodes, this);
+
+    label = gtk_label_new("Margins");
+    gtk_notebook_append_page(GTK_NOTEBOOK(notebook),hbox,label);
 
   }
 
-  preferences_AddFontSelect(GTK_WIDGET(vbox3), "Mnemonic", "font");
-
   {
-
-    GtkTextBuffer *buffer;
-    GtkTextIter start, end;
-    PangoFontDescription *font_desc;
-    GdkColor color;
-    GtkTextTag *tag;
-    char cParray[20];
-
-    m_view = GTK_TEXT_VIEW (gtk_text_view_new ());
-    gtk_text_view_set_wrap_mode (m_view, GTK_WRAP_NONE);
-    gtk_text_view_set_editable  (m_view, FALSE);
-
-    /* Change default font throughout the widget */
-    font_desc = pango_font_description_from_string ("Courier 12");
-    gtk_widget_modify_font (GTK_WIDGET (m_view), font_desc);
-    pango_font_description_free (font_desc);
-
-    buffer = gtk_text_view_get_buffer (m_view);
-
-    m_LabelColor->setBuffer(buffer);
-    m_MnemonicColor->setBuffer(buffer);
-    m_CommentColor->setBuffer(buffer);
-    m_SymbolColor->setBuffer(buffer);
-    m_ConstantColor->setBuffer(buffer);
-
-    gtk_text_buffer_create_tag (buffer, "margin",
-				"left_margin", 50,
-				NULL);
-    parseLine("Label: MOVF    Temp1,W ;Comment\n");
-    parseLine("       MOVLW   0x42    ;Comment");
-
-
-    m_LabelColor->apply();
-    m_MnemonicColor->apply();
-    m_CommentColor->apply();
-    m_SymbolColor->apply();
-    m_ConstantColor->apply();
-
+    SourceBuffer *pBuffer = new SourceBuffer (m_pParent->getTagTable(),0,m_pParent);
 
 
     GtkWidget *frame = gtk_frame_new ("Sample");
     gtk_box_pack_start (GTK_BOX (pParent), frame, FALSE, TRUE, 0);
     
-    m_SampleNotebook = gtk_notebook_new();
-    gtk_notebook_set_tab_pos((GtkNotebook*)m_SampleNotebook,m_TabPosition);
+    m_Notebook = gtk_notebook_new();
+    //m_currentTabPosition = m_pParent->getTabPosition();
+    //gtk_notebook_set_tab_pos((GtkNotebook*)m_Notebook,m_TabPosition);
+    setTabPosition(m_pParent->getTabPosition());
 
-    gtk_container_add (GTK_CONTAINER (frame), m_SampleNotebook);
+    gtk_container_add (GTK_CONTAINER (frame), m_Notebook);
 
-    GtkWidget *label = gtk_label_new("file1.asm");
-    gtk_notebook_append_page(GTK_NOTEBOOK(m_SampleNotebook),
-			     GTK_WIDGET(m_view),
-			     label);
+    bIsBuilt = true;
 
+    int id = AddPage (pBuffer, "file1.asm");
+
+    pages[id]->m_pBuffer->parseLine( "        MOVLW   0x34       ; Comment",1);
+    pages[id]->m_pBuffer->parseLine( "; Comment only",1);
+    pages[id]->m_pBuffer->parseLine( "Label:  ADDWF  Variable,F  ; Comment",1);
+
+#if 0
     label = gtk_label_new("file2.asm");
     GtkWidget *emptyBox = gtk_hbox_new(0,0);
-    gtk_notebook_append_page(GTK_NOTEBOOK(m_SampleNotebook),emptyBox,label);
+    gtk_notebook_append_page(GTK_NOTEBOOK(m_Notebook),emptyBox,label);
+
+#endif
 
 
 
   }
 
+  gtk_widget_show_all(notebook);
 
-  gtk_widget_show_all(hbox2);
-  gtk_widget_show_all(vbox3);
+}
 
+void SourceBrowserPreferences::setTabPosition(int tabPosition)
+{
+  m_currentTabPosition = tabPosition;
+  m_pParent->setTabPosition(tabPosition);
+  if (tabPosition >= 0) {
+    gtk_notebook_set_show_tabs(GTK_NOTEBOOK(m_Notebook),TRUE);
+    gtk_notebook_set_tab_pos(GTK_NOTEBOOK(m_Notebook), (GtkPositionType) m_currentTabPosition);
+  } else {
+    gtk_notebook_set_show_tabs(GTK_NOTEBOOK(m_Notebook),FALSE);
+  }
+  Update();
+}
+
+void SourceBrowserPreferences::setFont(const char *cpFont)
+{
+  m_pParent->setFont(cpFont);
+}
+
+const char *SourceBrowserPreferences::getFont()
+{
+  return m_pParent->getFont();
 }
 
 void SourceBrowserPreferences::apply()
 {
-  printf ("apply source browser preferences\n");
+  m_LabelColor->apply();
+  m_MnemonicColor->apply();
+  m_SymbolColor->apply();
+  m_ConstantColor->apply();
+  m_CommentColor->apply();
+
+  m_pParent->setTabPosition(m_currentTabPosition);
 }
+
+void SourceBrowserPreferences::cancel()
+{
+
+  m_LabelColor->cancel();
+  m_MnemonicColor->cancel();
+  m_SymbolColor->cancel();
+  m_ConstantColor->cancel();
+  m_CommentColor->cancel();
+
+  m_pParent->setTabPosition(m_originalTabPosition);
+}
+
+int SourceBrowserPreferences::getPCLine(int page)
+{
+  return 1;
+}
+int SourceBrowserPreferences::getAddress(NSourcePage *pPage, int line)
+{
+  return 0x1234;
+}
+bool SourceBrowserPreferences::bAddressHasBreak(int address)
+{
+  return false;
+}
+int SourceBrowserPreferences::getOpcode(int address)
+{
+  return 0xABCD;
+}
+
 //========================================================================
 gpsimGuiPreferences::gpsimGuiPreferences()
 {
@@ -728,36 +743,14 @@ gpsimGuiPreferences::gpsimGuiPreferences()
   window = LocalWindow;
   gtk_widget_show (window);
 
-  gtk_window_set_title (GTK_WINDOW (window), "Preferences");
+  gtk_window_set_title (GTK_WINDOW (window), "Source Browser configuration");
   gtk_container_set_border_width (GTK_CONTAINER (window), 0);
 
   GtkWidget *vbox = GTK_DIALOG (window)->vbox;
 
-  GtkWidget *notebook = gtk_notebook_new();
-  gtk_notebook_set_tab_pos((GtkNotebook*)notebook,GTK_POS_TOP);
-  gtk_box_pack_start (GTK_BOX (vbox), notebook, TRUE, TRUE, 0);
-  gtk_widget_show(notebook);
+  m_SourceBrowser = new SourceBrowserPreferences(vbox);
+  gtk_widget_show_all(vbox);
 
-  GtkWidget *label;
-  GtkWidget *vbox2;
-
-
-  // source browser preferences...
-  
-  vbox2 = gtk_vbox_new(0,0);
-  gtk_container_set_border_width (GTK_CONTAINER (vbox2), 3);
-  label = gtk_label_new("Source Browser");
-  gtk_notebook_append_page(GTK_NOTEBOOK(notebook),vbox2,label);
-
-  m_SourceBrowser = new SourceBrowserPreferences(vbox2);
-  gtk_widget_show_all(vbox2);
-
-
-  vbox2 = gtk_vbox_new(0,0);
-  gtk_container_set_border_width (GTK_CONTAINER (vbox2), 3);
-  label=gtk_label_new("RAM");
-  gtk_notebook_append_page(GTK_NOTEBOOK(notebook),vbox2,label);
-  gtk_widget_show(vbox2);
 
   // Cancel and Apply buttons
 
@@ -788,23 +781,24 @@ gpsimGuiPreferences::gpsimGuiPreferences()
   gtk_widget_show (window);
   gtk_grab_add (window);
 
-  printf("Created preferences %p %p %s\n",this, window, gtk_widget_get_name(window));
 }
 
 gpsimGuiPreferences::~gpsimGuiPreferences()
 {
 
-  printf("Destroying preferences %p %p\n",this, window);
 
   gtk_widget_destroy (window);
 
   //delete m_SourceBrowser;
 }
 
-void gpsimGuiPreferences::apply()
-{
-  printf ("apply gui preferences\n");
-}
+//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+
+#endif //if defined(NEW_SOURCE_BROWSER)
+
+//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 
 //========================================================================
 void
@@ -824,10 +818,10 @@ file_selection_ok (GtkWidget        *w,
   const char *file;
   char msg[200];
 
-  if(gp)
+  if(gpGuiProcessor)
   {
     file=gtk_file_selection_get_filename (fs);
-    if(!gpsim_open(gp->cpu, file, NULL))
+    if(!gpsim_open(gpGuiProcessor->cpu, file, 0, 0))
     {
       sprintf(msg, "Open failedCould not open \"%s\"", (char *)file);
       gui_message(msg);
@@ -895,47 +889,47 @@ toggle_window (gpointer             callback_data,
 
   menu_item = gtk_item_factory_get_item (item_factory,
 					 gtk_item_factory_path_from_widget (widget));
-  if(gp && menu_item) {
+  if(gpGuiProcessor && menu_item) {
     
     int view_state =  GTK_CHECK_MENU_ITEM(menu_item)->active ? VIEW_SHOW : VIEW_HIDE;
 			
     switch(callback_action) {
     case WT_opcode_source_window:
-      gp->program_memory->ChangeView(view_state);
+      gpGuiProcessor->program_memory->ChangeView(view_state);
       break;
     case WT_asm_source_window:
-      gp->source_browser->ChangeView(view_state);
+      gpGuiProcessor->source_browser->ChangeView(view_state);
       break;
     case WT_register_window:
-      gp->regwin_ram->ChangeView(view_state);
+      gpGuiProcessor->regwin_ram->ChangeView(view_state);
       break;
     case WT_eeprom_window:
-      gp->regwin_eeprom->ChangeView(view_state);
+      gpGuiProcessor->regwin_eeprom->ChangeView(view_state);
       break;
     case WT_watch_window:
-      gp->watch_window->ChangeView(view_state);
+      gpGuiProcessor->watch_window->ChangeView(view_state);
       break;
     case WT_symbol_window:
-      gp->symbol_window->ChangeView(view_state);
+      gpGuiProcessor->symbol_window->ChangeView(view_state);
       break;
     case WT_breadboard_window:
-      gp->breadboard_window->ChangeView(view_state);
+      gpGuiProcessor->breadboard_window->ChangeView(view_state);
       break;
     case WT_stack_window:
-      gp->stack_window->ChangeView(view_state);
+      gpGuiProcessor->stack_window->ChangeView(view_state);
       break;
     case WT_trace_window:
-      gp->trace_window->ChangeView(view_state);
+      gpGuiProcessor->trace_window->ChangeView(view_state);
       break;
     case WT_profile_window:
-      gp->profile_window->ChangeView(view_state);
+      gpGuiProcessor->profile_window->ChangeView(view_state);
       break;
     case WT_stopwatch_window:
-      gp->stopwatch_window->ChangeView(view_state);
+      gpGuiProcessor->stopwatch_window->ChangeView(view_state);
       break;
     case WT_scope_window:
-      //gp->scope_window->ChangeView(view_state);
-      cout << " The Scope is disabled right now\n";
+      gpGuiProcessor->scope_window->ChangeView(view_state);
+      //cout << " The Scope is disabled right now\n";
       break;
     default:
       puts("unknown menu action");
@@ -950,50 +944,48 @@ toggle_window (gpointer             callback_data,
 static void 
 runbutton_cb(GtkWidget *widget)
 {
-  if(gp && gp->cpu)
-    gp->cpu->pma->run();
+  if(gpGuiProcessor && gpGuiProcessor->cpu)
+    gpGuiProcessor->cpu->pma->run();
 }
 
 static void 
 stopbutton_cb(GtkWidget *widget)
 {
-  if(gp && gp->cpu)
-    gp->cpu->pma->stop();
+  if(gpGuiProcessor && gpGuiProcessor->cpu)
+    gpGuiProcessor->cpu->pma->stop();
 }
     
 static void 
 stepbutton_cb(GtkWidget *widget)
 {
-  if(gp && gp->cpu) 
-    gp->cpu->pma->step(1);
+  if(gpGuiProcessor && gpGuiProcessor->cpu) 
+    gpGuiProcessor->cpu->pma->step(1);
 }
     
 static void 
 overbutton_cb(GtkWidget *widget)
 {
-  if(gp && gp->cpu) 
-    gp->cpu->pma->step_over();
+  if(gpGuiProcessor && gpGuiProcessor->cpu) 
+    gpGuiProcessor->cpu->pma->step_over();
 
 }
     
 static void 
 finishbutton_cb(GtkWidget *widget)
 {
-  if(gp && gp->cpu) 
-    gp->cpu->pma->finish();
+  if(gpGuiProcessor && gpGuiProcessor->cpu) 
+    gpGuiProcessor->cpu->pma->finish();
 }
 
 static void 
 resetbutton_cb(GtkWidget *widget)
 {
-  if(gp && gp->cpu)
-    gp->cpu->reset(POR_RESET);
+  if(gpGuiProcessor && gpGuiProcessor->cpu)
+    gpGuiProcessor->cpu->reset(POR_RESET);
 }
 
 
 int gui_animate_delay; // in milliseconds
-extern int realtime_mode;
-extern int realtime_mode_with_gui;
 
 
 //========================================================================
@@ -1075,19 +1067,19 @@ UpdateRateMenuItem::UpdateRateMenuItem(GtkWidget *parent,
 
 void UpdateRateMenuItem::Select()
 {
-  realtime_mode = bRealTime ? 1 : 0;
-  realtime_mode_with_gui = bWithGui ? 1 : 0;
+  EnableRealTimeMode(bRealTime);
+  EnableRealTimeModeWithGui(bWithGui);
 
   if(bAnimate) {
     gui_animate_delay = update_rate;
-    gi.set_update_rate(1);
+    get_interface().set_update_rate(1);
   } else {
     gui_animate_delay = 0;
-    gi.set_update_rate(update_rate);
+    get_interface().set_update_rate(update_rate);
   }
 
-  if(gp && gp->cpu)
-    gp->cpu->pma->stop();
+  if(gpGuiProcessor && gpGuiProcessor->cpu)
+    gpGuiProcessor->cpu->pma->stop();
 
   config_set_variable("dispatcher", "SimulationMode", id);
 
@@ -1145,7 +1137,7 @@ public:
     : TimeFormatter(tw,menu,"MicroSeconds") {}
   void Format(char *buf, int size)
   {
-    double time_db = gp->cpu->get_InstPeriod() * cycles.value * 1e6;
+    double time_db = gpGuiProcessor->cpu->get_InstPeriod() * get_cycles().value * 1e6;
     snprintf(buf,size, "%19.2f us",time_db);
   }
 };
@@ -1157,7 +1149,7 @@ public:
     : TimeFormatter(tw,menu,"MilliSeconds") {}
   void Format(char *buf, int size)
   {
-    double time_db = gp->cpu->get_InstPeriod() * cycles.value * 1e3;
+    double time_db = gpGuiProcessor->cpu->get_InstPeriod() * get_cycles().value * 1e3;
     snprintf(buf,size, "%19.3f ms",time_db);
   }
 };
@@ -1169,7 +1161,7 @@ public:
     : TimeFormatter(tw,menu,"Seconds") {}
   void Format(char *buf, int size)
   {
-    double time_db = gp->cpu->get_InstPeriod() * cycles.value;
+    double time_db = gpGuiProcessor->cpu->get_InstPeriod() * get_cycles().value;
     snprintf(buf,size, "%19.3f Sec",time_db);
   }
 };
@@ -1181,7 +1173,7 @@ public:
     : TimeFormatter(tw,menu,"HH:MM:SS.mmm") {}
   void Format(char *buf, int size)
   {
-    double time_db = gp->cpu->get_InstPeriod() * cycles.value;
+    double time_db = gpGuiProcessor->cpu->get_InstPeriod() * get_cycles().value;
     double v=time_db;
     int hh=(int)(v/3600),mm,ss,cc;
     v-=hh*3600.0;
@@ -1200,7 +1192,7 @@ public:
     : TimeFormatter(tw,menu,"Cycles (Hex)") {}
   void Format(char *buf, int size)
   {
-    snprintf(buf,size,"0x%016" PRINTF_INT64_MODIFIER "x",cycles.value);
+    snprintf(buf,size,"0x%016" PRINTF_INT64_MODIFIER "x",get_cycles().value);
   }
 };
 
@@ -1211,7 +1203,7 @@ public:
     : TimeFormatter(tw,menu,"Cycles (Dec)") {}
   void Format(char *buf, int size)
   {
-    snprintf(buf,size,"%016" PRINTF_INT64_MODIFIER "d",cycles.value);
+    snprintf(buf,size,"%016" PRINTF_INT64_MODIFIER "d",get_cycles().value);
   }
 };
 
@@ -1379,8 +1371,10 @@ static GtkItemFactoryEntry menu_items[] =
   { "/Windows/St_opwatch",      0, TOGGLE_WINDOW,WT_stopwatch_window,"<ToggleItem>" },
   { "/Windows/Sco_pe",          0, TOGGLE_WINDOW,WT_scope_window,"<ToggleItem>" },
 
+#if defined(NEW_SOURCE_BROWSER)
   { "/_Edit",     0, 0,       0, "<Branch>" },
   { "/Edit/Preferences",        0, (GtkItemFactoryCallback)gpsimGuiPreferences::setup, 0 },
+#endif
 
   { "/_Help",            0,         0,                     0, "<LastBranch>" },
   { "/Help/_About",      0,         (GtkItemFactoryCallback)about_cb,       0 },
@@ -1428,7 +1422,7 @@ void dispatch_Update()
   if(!dispatcher_window)
     return;
 
-  if(gp && gp->cpu) {
+  if(gpGuiProcessor && gpGuiProcessor->cpu) {
 
     TheWindow.Update();
 
