@@ -88,7 +88,7 @@ private:
 
 public:
 
-  virtual void execute(void)=0;
+  virtual void execute()=0;
 
 };
 
@@ -101,7 +101,7 @@ private:
 
 class Menu_BreakClear : public RegWindowMenuItem {
 public:
-  virtual void execute(void) {
+  virtual void execute() {
     printf("BreakClear");
   }
 
@@ -153,7 +153,7 @@ Register_Window *popup_rw;
 // get_register
 // get the "real" register. If 'bTopLevelOnly' is true
 // 
-Register *GUIRegister::get_register(void)
+Register *GUIRegister::get_register()
 {
   if(!rma)
     return 0;
@@ -181,7 +181,7 @@ void GUIRegister::put_shadow(RegisterValue new_value)
   shadow = new_value;
 }
 
-unsigned int GUIRegister::get_value(void)
+unsigned int GUIRegister::get_value()
 {
   Register *reg = get_register();
 
@@ -192,7 +192,7 @@ unsigned int GUIRegister::get_value(void)
 }
 
 
-RegisterValue GUIRegister::getRV(void)
+RegisterValue GUIRegister::getRV()
 {
   Register *reg = get_register();
 
@@ -239,7 +239,7 @@ bool GUIRegister::hasChanged(RegisterValue &current_value) const
   return (shadow != current_value);
 }
 
-void GUIRegister::Clear_xref(void)
+void GUIRegister::Clear_xref()
 {
   Register *reg = get_register();
 
@@ -258,7 +258,7 @@ void GUIRegister::Assign_xref(CrossReferenceToGUI *new_xref)
   xref = new_xref;
 }
 
-bool GUIRegister::hasBreak(void)
+bool GUIRegister::hasBreak()
 {
 
   if(rma)
@@ -267,17 +267,23 @@ bool GUIRegister::hasBreak(void)
   return false;
 }
 
-char *GUIRegister::name(void)
+char *GUIRegister::name()
 {
 
+  static char buffer[128];
+
   Register *reg = get_register();
+  if (!reg) {
+    sprintf(buffer,"NULL");
+    return buffer;
+  }
+
   register_symbol * pRegSym = get_symbol_table().findRegisterSymbol(
     reg->address);
 
   if(!reg || reg->isa()==Register::INVALID_REGISTER)
     return 0;
 
-  static char buffer[128];
   if(bIsAliased) {
     sprintf(buffer,"alias (%s)", reg->name().c_str());
   }
@@ -292,14 +298,14 @@ char *GUIRegister::name(void)
   return buffer;
 }
 
-bool GUIRegister::bIsValid(void)
+bool GUIRegister::bIsValid()
 {
   if(rma && (*rma)[address].getReg())
     return true;
   return false;
 }
 
-bool GUIRegister::bIsSFR(void)
+bool GUIRegister::bIsSFR()
 {
   if(rma && (*rma)[address].isa() == Register::SFR_REGISTER)
     return true;
@@ -366,9 +372,9 @@ public:
   { 
     printf("(gui_regwin)Warning: writing to invalid register\n");
   };
-  unsigned int get_value(void) { return 0;};
+  unsigned int get_value() { return 0;};
 
-  InvalidGuiRegister(void) {
+  InvalidGuiRegister() {
     rma=0;
   }
 private:
@@ -378,7 +384,37 @@ private:
 static InvalidGuiRegister THE_invalid_register;
 
 //========================================================================
+// GtkSheet extensions
+//
+// The gtk_sheet api does not provide access to row and column labels.
+// This means we can manipulate the font the way the cells can be manipulated
 
+#if 0 // defined but not used
+static GtkSheetButton *
+gtk_sheet_row_button_get(GtkSheet *sheet, gint row)
+{
+  g_return_val_if_fail (sheet != NULL, NULL);
+  g_return_val_if_fail (GTK_IS_SHEET (sheet), NULL);
+
+  if(row < 0 || row > sheet->maxrow) return NULL;
+
+  return (&sheet->row[row].button);
+}
+#endif
+
+static void
+gtk_sheet_REALLY_set_row_height(GtkSheet *sheet, gint row, gint height)
+{
+  g_return_if_fail (sheet != NULL);
+  g_return_if_fail (GTK_IS_SHEET (sheet));
+
+  if (row < 0 || row > sheet->maxrow)
+    return;
+
+  sheet->row[row].height = height;
+}
+
+//========================================================================
 // get_value
 static void a_cb(GtkWidget *w, gpointer user_data)
 {
@@ -765,7 +801,7 @@ popup_activated(GtkWidget *widget, gpointer data)
 	  {
 	    address=popup_rw->row_to_address[j]+i;
 	    printf("break on read \n");
-	    bp.set_read_break(popup_rw->gp->cpu, address);
+	    get_bp().set_read_break(popup_rw->gp->cpu, address);
 	  }
       break;
     case MENU_BREAK_WRITE:
@@ -773,7 +809,7 @@ popup_activated(GtkWidget *widget, gpointer data)
 	for(i=range.col0;i<=range.coli;i++)
 	  {
 	    address=popup_rw->row_to_address[j]+i;
-	    bp.set_write_break(popup_rw->gp->cpu, address);
+	    get_bp().set_write_break(popup_rw->gp->cpu, address);
 	  }
       break;
     case MENU_BREAK_READ_VALUE:
@@ -784,7 +820,7 @@ popup_activated(GtkWidget *widget, gpointer data)
 	for(i=range.col0;i<=range.coli;i++)
 	  {
 	    address=popup_rw->row_to_address[j]+i;
-	    bp.set_read_value_break(popup_rw->gp->cpu,address,value);
+	    get_bp().set_read_value_break(popup_rw->gp->cpu,address,value);
 	  }
       break;
     case MENU_BREAK_WRITE_VALUE:
@@ -795,7 +831,7 @@ popup_activated(GtkWidget *widget, gpointer data)
 	for(i=range.col0;i<=range.coli;i++)
 	  {
 	    address=popup_rw->row_to_address[j]+i;
-	    bp.set_write_value_break(popup_rw->gp->cpu,address,value);
+	    get_bp().set_write_value_break(popup_rw->gp->cpu,address,value);
 	  }
       break;
     case MENU_BREAK_CLEAR:
@@ -803,7 +839,7 @@ popup_activated(GtkWidget *widget, gpointer data)
 	for(i=range.col0;i<=range.coli;i++)
 	  {
 	    address=popup_rw->row_to_address[j]+i;
-	    bp.clear_all_register(popup_rw->gp->cpu,address);
+	    get_bp().clear_all_register(popup_rw->gp->cpu,address);
 	  }
       break;
     case MENU_ADD_WATCH:
@@ -820,26 +856,27 @@ popup_activated(GtkWidget *widget, gpointer data)
     case MENU_LOG_SETTINGS:
       gui_get_log_settings(&filename, &mode);
       if(filename!=0)
-	trace_log.enable_logging(filename,mode);
+        GetTraceLog().enable_logging(filename,mode);
       break;
     case MENU_LOG_READ:
-      for(j=range.row0;j<=range.rowi;j++)
-	for(i=range.col0;i<=range.coli;i++)
-	  {
-	    address=popup_rw->row_to_address[j]+i;
-	    trace_log.enable_logging();
-	    // FIXME the register type is ignored here (and in all other cases
-	    // where we're logging -- it's assumed that the register address is
-	    // for ram, even if in fact the user requests eeprom.
-	    bp.set_notify_read(popup_rw->gp->cpu,address);
-	  }
+      for(j=range.row0;j<=range.rowi;j++) {
+        for(i=range.col0;i<=range.coli;i++)
+        {
+          address=popup_rw->row_to_address[j]+i;
+          GetTraceLog().enable_logging();
+          // FIXME the register type is ignored here (and in all other cases
+          // where we're logging -- it's assumed that the register address is
+          // for ram, even if in fact the user requests eeprom.
+          get_bp().set_notify_read(popup_rw->gp->cpu,address);
+        }
+      }
       break;
     case MENU_LOG_WRITE:
       for(j=range.row0;j<=range.rowi;j++)
 	for(i=range.col0;i<=range.coli;i++)
 	  {
 	    address=popup_rw->row_to_address[j]+i;
-	    bp.set_notify_write(popup_rw->gp->cpu,address);
+	    get_bp().set_notify_write(popup_rw->gp->cpu,address);
 	  }
       break;
     case MENU_LOG_READ_VALUE:
@@ -851,7 +888,7 @@ popup_activated(GtkWidget *widget, gpointer data)
 	for(i=range.col0;i<=range.coli;i++)
 	  {
 	    address=popup_rw->row_to_address[j]+i;
-	    bp.set_notify_read_value(popup_rw->gp->cpu,address, value, mask);
+	    get_bp().set_notify_read_value(popup_rw->gp->cpu,address, value, mask);
 
 	  }
       break;
@@ -864,7 +901,7 @@ popup_activated(GtkWidget *widget, gpointer data)
         for(i=range.col0;i<=range.coli;i++)
           {
           address=popup_rw->row_to_address[j]+i;
-          bp.set_notify_write_value(popup_rw->gp->cpu,address, value, mask);
+          get_bp().set_notify_write_value(popup_rw->gp->cpu,address, value, mask);
           }
       break;
     case MENU_REGWIN_REFRESH:
@@ -927,29 +964,29 @@ static gint
 do_popup(GtkWidget *widget, GdkEventButton *event, Register_Window *rw)
 {
 
-    GtkWidget *popup;
-//	GdkModifierType mods;
-    GtkSheet *sheet;
+  GtkWidget *popup;
+  //	GdkModifierType mods;
+  GtkSheet *sheet;
 
-    popup=rw->popup_menu;
+  popup=rw->popup_menu;
     
   if(widget==0 || event==0 || rw==0)
-  {
+    {
       printf("Warning do_popup(%p,%p,%p)\n",widget,event,rw);
       return 0;
-  }
+    }
   
-    sheet=GTK_SHEET(widget);
+  sheet=GTK_SHEET(widget);
 
-    if( (event->type == GDK_BUTTON_PRESS) &&  (event->button == 3) )
+  if( (event->type == GDK_BUTTON_PRESS) &&  (event->button == 3) )
     {
 
-	popup_rw = rw;
+      popup_rw = rw;
   
-	gtk_menu_popup(GTK_MENU(popup), 0, 0, 0, 0,
-			   3, event->time);
+      gtk_menu_popup(GTK_MENU(popup), 0, 0, 0, 0,
+		     3, event->time);
     }
-    return FALSE;
+  return FALSE;
 }
 
 // The following routine will convert the first number it finds in
@@ -1033,39 +1070,47 @@ set_cell(GtkWidget *widget, int row, int col, Register_Window *rw)
 
   if(errno != EINVAL && n != (int) reg->get_shadow().data)
     {
-      reg->put_value(n & gp->cpu->register_mask());
+      reg->put_value(n & gpGuiProcessor->cpu->register_mask());
       rw->UpdateASCII(row);
     }
 
 }
 
 //------------------------------------------------------------------------
+// int column_width(int col)
+//
+// Return the width of one of the register sheet columns. 
+//
+// Column = -1 is the row label
+// Columns 0-REGISTERS_PER_ROW are ram/registers
+// Column REGISTERS_PER_ROW is an ASCII string of the row data.
+//
+// The width of the column is based on width of a single character,
+// char_width. This width is depends on the font and is computed in
+// LoadStyles()
+//
+// FIXME: column_width assumes there are fewer than 0x1000 registers.
 int Register_Window::column_width(int col)
 {
-
-  int esthetic_padding = 6;   // pixels 
-
   if(!char_width)
     return 0;
 
+  // Row Labels
   if(col < 0)
-    return char_width * 3 + esthetic_padding;
+    return char_width * 3;
 
+  // Register data
   if(col < REGISTERS_PER_ROW)
-    return char_width * chars_per_column + esthetic_padding;
+    return char_width * chars_per_column;
 
-  return char_width * REGISTERS_PER_ROW + esthetic_padding;
+  // ASCII column
+  return char_width * (REGISTERS_PER_ROW + 1) + char_width/2;
 }
 
 //------------------------------------------------------------------------
-int Register_Window::row_height(int col)
+int Register_Window::row_height(int row)
 {
-  int esthetic_padding = 6;  // pixels 
-
-  if(!char_width)
-    return 0;
-
-  return 3 * char_width + esthetic_padding;
+  return char_height ? char_height : 20;
 }
 
 //------------------------------------------------------------------------
@@ -1073,26 +1118,26 @@ int Register_Window::row_height(int col)
 //
 //
 
-void Register_Window::UpdateLabel(void)
+void Register_Window::UpdateLabel()
 {
-  int row, col;
+  int row = -1, col = -1;
 
-  //row=register_sheet->active_cell.row;
-  //col=register_sheet->active_cell.col;
-  gtk_sheet_get_active_cell(register_sheet, &row, &col);
+  if(register_sheet != 0) {
+    gtk_sheet_get_active_cell(register_sheet, &row, &col);
 
-  if(col >= REGISTERS_PER_ROW)
-    gtk_label_set(GTK_LABEL(location), "  ascii  ");
-  else {
+    if(col > -1 && row > -1) {
+      if(col >= REGISTERS_PER_ROW)
+        gtk_label_set(GTK_LABEL(location), "  ascii  ");
+      else {
 
-    GUIRegister *reg = getRegister(row,col);
+        GUIRegister *reg = getRegister(row,col);
 
-    const char *n = reg ? reg->name() : "INVALID_REGISTER";
+        const char *n = reg ? reg->name() : "INVALID_REGISTER";
 
-    gtk_label_set(GTK_LABEL(location), n);
+        gtk_label_set(GTK_LABEL(location), n);
+      }
+    }
   }
-
-
 }
 
 //------------------------------------------------------------------------
@@ -1100,26 +1145,28 @@ void Register_Window::UpdateLabel(void)
 //
 //
 
-void Register_Window::UpdateEntry(void)
+void Register_Window::UpdateEntry()
 {
   gint row, col;
 
   const char *text; 
   GtkWidget * sheet_entry;
 
-  sheet_entry = gtk_sheet_get_entry(register_sheet);
-  gtk_sheet_get_active_cell(register_sheet, &row, &col);
+  if(register_sheet != 0) {
+    sheet_entry = gtk_sheet_get_entry(register_sheet);
+    gtk_sheet_get_active_cell(register_sheet, &row, &col);
 
-  if(row_to_address[row] < 0)
-    return;
+    if(row_to_address[row] < 0)
+      return;
 
-  GUIRegister *reg = getRegister(row,col);
+    GUIRegister *reg = getRegister(row,col);
 
-  if(reg && reg->bIsValid() )
+    if(reg && reg->bIsValid() )
     {
       if((text=gtk_entry_get_text (GTK_ENTRY(sheet_entry))))
           gtk_entry_set_text(GTK_ENTRY(entry), text);
     }
+  }
 }
 
 //------------------------------------------------------------------------
@@ -1127,7 +1174,7 @@ void Register_Window::UpdateEntry(void)
 //
 //
 
-void Register_Window::UpdateLabelEntry(void)
+void Register_Window::UpdateLabelEntry()
 {
   UpdateLabel();
   UpdateEntry();
@@ -1136,8 +1183,6 @@ void Register_Window::UpdateLabelEntry(void)
 static gint configure_event(GtkWidget *widget, GdkEventConfigure *e, gpointer data)
 {
 
-  Dprintf((" configure_event\n"));
-
   if(widget->window==0)
     return 0;
     
@@ -1145,7 +1190,43 @@ static gint configure_event(GtkWidget *widget, GdkEventConfigure *e, gpointer da
   return 0; // what should be returned?, FIXME
 }
 
-int Register_Window::LoadStyles(void)
+//------------------------------------------------------------------------
+void Register_Window::UpdateStyle()
+{
+  if (!register_sheet || !normalfont)
+    return;
+
+  GtkSheetRange range;
+
+  //gtk_sheet_freeze(register_sheet);
+
+  // Update the font for the cells
+  range.row0=0;
+  range.rowi=register_sheet->maxrow;
+  range.col0=0;
+  range.coli=register_sheet->maxcol;
+  gtk_sheet_range_set_font(register_sheet, &range, normalfont);
+
+  // Update the font for the row and column labels
+  gtk_widget_modify_font(GTK_WIDGET(register_sheet),normalfont);
+
+  // Adjust the cell sizes based on the font size
+  int i;
+  for(i=0; i<=register_sheet->maxcol; i++)
+    gtk_sheet_set_column_width (register_sheet, i, column_width(i));
+  for(i=0; i<=register_sheet->maxrow; i++)
+    gtk_sheet_REALLY_set_row_height (register_sheet, i, row_height(i));
+
+
+  gtk_sheet_set_row_titles_width(register_sheet, column_width(-1));
+  gtk_sheet_set_column_titles_height(register_sheet, row_height(0));
+  //gtk_sheet_thaw(register_sheet);
+
+
+}
+
+//------------------------------------------------------------------------
+int Register_Window::LoadStyles()
 {
 #if GTK_MAJOR_VERSION >= 2
   normalfont = pango_font_description_from_string(normalfont_string);
@@ -1156,19 +1237,23 @@ int Register_Window::LoadStyles(void)
   if(!normalfont)
   {
     char_width = 0;
+    char_height = 0;
     return 0;
   }
 
 #if GTK_MAJOR_VERSION >= 2
-  //char_width = gdk_string_width(gdk_font_from_description(normalfont), "9");
-  //printf("%s\npango size %d, char_width %d\n",normalfont_string,
-  //	 pango_font_description_get_size(normalfont),char_width);
-  // FIXME - this clearly is not correct.
-  // what is wanted is a way to determine the width of a 'typical' character.
-  // We have the 'Font Description', but not the 'Font'. There is a way 
-  // to get the width of a character given the font by calling:
-  // pango_font_metrics_get_approximate_digit_width(PangoFontMetrics *)
-  char_width =pango_font_description_get_size(normalfont)*2/(3*1024);
+  {
+    PangoRectangle rect;
+    PangoLayout *layout;
+
+    layout = gtk_widget_create_pango_layout (GTK_WIDGET(register_sheet), "A");
+    pango_layout_set_font_description (layout, normalfont);
+
+    pango_layout_get_extents (layout, NULL, &rect);
+    char_width =  PANGO_PIXELS(rect.width);
+    char_height = PANGO_PIXELS(rect.height + (rect.height<<1))>>1;
+    g_object_unref(G_OBJECT(layout));
+  }
 
 #else
   char_width = gdk_string_width (normalfont,"9");
@@ -1185,7 +1270,7 @@ static void settingsok_cb(GtkWidget *w, gpointer user_data)
     settings_active=0;
 }
 
-int Register_Window::SettingsDialog(void)
+int Register_Window::SettingsDialog()
 {
   static GtkWidget *dialog=0;    // fixme
   GtkWidget *button;
@@ -1193,9 +1278,7 @@ int Register_Window::SettingsDialog(void)
   static GtkWidget *normalfontstringentry;   //fixme
   GtkWidget *label;
   int fonts_ok=0;
-  GtkSheet *sheet=0;
-  GtkSheetRange range;
-  int i;
+
 
   if(!dialog)
   {
@@ -1243,14 +1326,15 @@ int Register_Window::SettingsDialog(void)
 
 
 
+#if GTK_MAJOR_VERSION >= 2
+  PangoFontDescription *font=0;
+#else
+  GdkFont *font=0;
+#endif
+
   while(fonts_ok!=1)
   {
     char fontname[256];
-#if GTK_MAJOR_VERSION >= 2
-    PangoFontDescription *font;
-#else
-    GdkFont *font;
-#endif
 
     settings_active=1;
     while(settings_active)
@@ -1287,25 +1371,8 @@ int Register_Window::SettingsDialog(void)
   }
 
   gtk_sheet_freeze(register_sheet);
-  range.row0=0;
-  range.rowi=sheet->maxrow;
-  range.col0=0;
-  range.coli=sheet->maxcol;
-  //gtk_sheet_range_set_font(sheet, &range, normalfont);
-
-  // FIXME - this code doesn't look right. 'i' iterates over both
-  // the columns and rows. If there are more rows then columns, then
-  // this code breaks.
-
-  for(i=0; i<=register_sheet->maxcol; i++){
-    gtk_sheet_set_column_width (register_sheet, i, column_width(i));
-    //gtk_sheet_set_row_height (register_sheet, i, row_height(i));
-  }
-
-  gtk_sheet_set_row_titles_width(register_sheet, column_width(-1));
-  gtk_sheet_set_column_titles_height(register_sheet, row_height(0));
+  UpdateStyle();
   gtk_sheet_thaw(register_sheet);
-
   gtk_widget_hide(dialog);
 
   return 0;
@@ -1499,12 +1566,12 @@ show_entry(GtkWidget *widget, Register_Window *rw)
 static gint
 activate_sheet_cell(GtkWidget *widget, gint row, gint column, Register_Window *rw) 
 {
-  Dprintf((" activate_sheet_cell\n"));
+  Dprintf((" activate_sheet_cell rma=%p\n",(rw? rw->rma :0)));
 
-  GtkSheet *sheet=0;
-    
-  if(rw)
-    sheet=rw->register_sheet;
+  GtkSheet *sheet = rw ? rw->register_sheet : 0;
+
+  if(!sheet)
+    return 0;
 
   if(widget==0 || row>sheet->maxrow || row<0||
      column>sheet->maxcol || column<0 || rw==0)
@@ -1574,12 +1641,13 @@ void Register_Window::SelectRegister(int regnumber)
   col=registers->Get(regnumber)->col;
   range.row0=range.rowi=row;
   range.col0=range.coli=col;
+
   gtk_sheet_select_range(GTK_SHEET(register_sheet),&range);
   if(register_sheet != NULL &&
      (GTK_SHEET(register_sheet)->view.col0>range.col0 ||
      GTK_SHEET(register_sheet)->view.coli<range.coli ||
      GTK_SHEET(register_sheet)->view.row0>range.row0 ||
-     GTK_SHEET(register_sheet)->view.rowi<range.rowi))
+      GTK_SHEET(register_sheet)->view.rowi<range.rowi))
     gtk_sheet_moveto(GTK_SHEET(register_sheet),row,col,0.5,0.5);
 
   UpdateLabelEntry();
@@ -1646,7 +1714,7 @@ void Register_Window::UpdateASCII(gint row)
 
     name[i] = registers->Get(row_to_address[row] + i)->get_shadow().data;
 
-    if( (name[i] < ' ') || (name[i]>'z'))
+    if( (name[i] < ' ') || (name[i]>'~'))
       name[i] = '.';
 
   }
@@ -1677,6 +1745,9 @@ gboolean Register_Window::UpdateRegisterCell(unsigned int reg_number)
   
   GUIRegister *guiReg = registers->Get(reg_number);
 
+  if (!guiReg || !guiReg->rma)
+    return 0;
+
   if(reg_number >= guiReg->rma->get_size())
     return 0;
 
@@ -1788,7 +1859,7 @@ gboolean Register_Window::UpdateRegisterCell(unsigned int reg_number)
 }
 
 
-void Register_Window::Update(void)
+void Register_Window::Update()
 {
 
   int address;
@@ -1809,7 +1880,7 @@ void Register_Window::Update(void)
     return;
   }
 
-
+  gtk_sheet_freeze(register_sheet);
   for(j = 0; j<=GTK_SHEET(register_sheet)->maxrow; j++) {
 
     if(row_to_address[j]==-1)
@@ -1830,10 +1901,13 @@ void Register_Window::Update(void)
     if(bRowChanged)
       UpdateASCII(j);
   }
+
+  gtk_sheet_thaw(register_sheet);
+
 }
 
 //------------------------------------------------------------------------
-void Register_Window::SetRegisterSize(void)
+void Register_Window::SetRegisterSize()
 {
   if(gp && gp->cpu)
     register_size = gp->cpu->register_size();
@@ -1853,8 +1927,8 @@ void Register_Window::SetRegisterSize(void)
     int i;
     char buffer[10];
 
+    // Column labels
     for(i=0; i<register_sheet->maxcol; i++){
-
       sprintf(buffer,"%02x",i);
       gtk_sheet_column_button_add_label(register_sheet, i, buffer);
       gtk_sheet_set_column_title(register_sheet, i, buffer);
@@ -1862,7 +1936,7 @@ void Register_Window::SetRegisterSize(void)
     }
 
 
-
+    // ASCII column
     i = REGISTERS_PER_ROW;
     sprintf(buffer,"ASCII");
     gtk_sheet_column_button_add_label(register_sheet, i, buffer);
@@ -1910,9 +1984,6 @@ GUIRegisterList::~GUIRegisterList() {
 //------------------------------------------------------------------------
 void Register_Window::NewProcessor(GUI_Processor *_gp)
 {
-
-
-#define NAME_SIZE 32
   gint i,j, border_mask, border_width;
   unsigned int reg_number;
   CrossReferenceToGUI *cross_reference;
@@ -1931,18 +2002,21 @@ void Register_Window::NewProcessor(GUI_Processor *_gp)
   }
       
   row_created=FALSE;
+  unsigned int nRegs;
+  nRegs = (rma->get_size() < MAX_REGISTERS) ? (rma->get_size()) : MAX_REGISTERS;
+
+  if (!nRegs)
+    return;
 
   gtk_sheet_freeze(register_sheet);
     
   j=0;
   i=0;
 
-  gtk_sheet_set_row_height (register_sheet, j, row_height(i));
+  gtk_sheet_REALLY_set_row_height (register_sheet, j, row_height(i));
 
   SetRegisterSize();
 
-  unsigned int nRegs;
-  nRegs = (rma->get_size() < MAX_REGISTERS) ? (rma->get_size()) : MAX_REGISTERS;
 
   for(reg_number=0;reg_number<nRegs;reg_number++) {
     i=reg_number%REGISTERS_PER_ROW;
@@ -1980,7 +2054,7 @@ void Register_Window::NewProcessor(GUI_Processor *_gp)
         if(register_sheet->maxrow<j)
         {
           gtk_sheet_add_row(register_sheet,1);
-          gtk_sheet_set_row_height (register_sheet, j, row_height(0));
+          gtk_sheet_REALLY_set_row_height (register_sheet, j, row_height(0));
         }
 
         sprintf(row_label,"%x0",reg_number/REGISTERS_PER_ROW);
@@ -1997,14 +2071,13 @@ void Register_Window::NewProcessor(GUI_Processor *_gp)
     gtk_sheet_delete_rows(register_sheet,j,register_sheet->maxrow-j);
 
   registers_loaded = 1;
-    
+
   range.row0=0;
   range.rowi=register_sheet->maxrow;
   range.col0=0;
   range.coli=register_sheet->maxcol;
 
-  //gtk_sheet_range_set_font(register_sheet, &range, normalfont);
-
+  UpdateStyle();
   border_mask = GTK_SHEET_RIGHT_BORDER |
     GTK_SHEET_LEFT_BORDER |
     GTK_SHEET_BOTTOM_BORDER |
@@ -2022,17 +2095,15 @@ void Register_Window::NewProcessor(GUI_Processor *_gp)
 
   gtk_sheet_range_set_border(register_sheet, &range, border_mask, border_width, 0);
 
+  gtk_sheet_thaw(register_sheet);
+
   // set values in the sheet
   Update();
-
-  gtk_sheet_thaw(register_sheet);
-    
   SelectRegister(0);
-
 }
 
 static int show_event(GtkWidget *widget,
-                        Register_Window *rw)
+		      Register_Window *rw)
 {
   Dprintf((" show_event\n"));
 
@@ -2054,12 +2125,13 @@ static int delete_event(GtkWidget *widget,
 //
 //
 
-void Register_Window::Build(void)
+void Register_Window::Build()
 {
 
   if(bIsBuilt)
     return;
 
+  Dprintf((" Register_Window::Build()\n"));
   GtkWidget *main_vbox;
   GtkWidget *scrolled_window;
 
@@ -2094,20 +2166,8 @@ void Register_Window::Build(void)
     register_sheet=GTK_SHEET(gtk_sheet_new(1,MAXCOLS,"gpsim Register Viewer [EEPROM]"));
     gtk_window_set_title(GTK_WINDOW(window), "register viewer [EEPROM]");
   }
-    
-  /// DEBUG ///
-  //#if defined(GTK_VERSION)
-#if 0
-  const char *gtk_version = GTK_VERSION;
-  if (gtk_version[0] == '2' && gtk_version[2] >= '6') {
-    gtk_sheet_hide_row_titles(register_sheet);
-    gtk_sheet_hide_column_titles(register_sheet);
-  }
-#endif
-  /// DEBUG ///
-
-  //  GTK_WIDGET_UNSET_FLAGS(register_sheet,GTK_CAN_DEFAULT);
-
+  //gtk_sheet_hide_column_titles(register_sheet);
+  //gtk_sheet_hide_row_titles(register_sheet);
   /* create popupmenu */
   popup_menu=build_menu(this);
 
@@ -2119,7 +2179,7 @@ void Register_Window::Build(void)
 
   /**************************** load fonts *********************************/
 #if GTK_MAJOR_VERSION >= 2
-#define DEFAULT_NORMALFONT "Courier Roman 14"
+#define DEFAULT_NORMALFONT "Monospace 10"
 #else
 #define DEFAULT_NORMALFONT "-adobe-courier-*-r-*-*-*-140-*-*-*-*-*-*"
 #endif
@@ -2139,7 +2199,7 @@ void Register_Window::Build(void)
       SettingsDialog();
       }
   }
-
+  UpdateStyle();
   gtk_signal_connect(GTK_OBJECT (window), "delete_event",
 		     GTK_SIGNAL_FUNC(delete_event), this);
 
@@ -2222,7 +2282,7 @@ void Register_Window::Build(void)
 }
 
 //------------------------------------------------------------------------
-Register_Window::Register_Window(void)
+Register_Window::Register_Window()
 {
   register_sheet = NULL;
   printf("WARNING: calling default constructor: %s\n",__FUNCTION__);
@@ -2276,6 +2336,7 @@ void RAM_RegisterWindow::NewProcessor(GUI_Processor *_gp)
 
   rma = &_gp->cpu->rma;
   registers = _gp->m_pGUIRamRegisters;
+  Dprintf((" RAM_RegisterWindow::NewProcessor rma=%p\n",rma));
 
   Register_Window::NewProcessor(_gp);
   if(sbw)
@@ -2314,6 +2375,7 @@ void EEPROM_RegisterWindow::NewProcessor(GUI_Processor *_gp)
   rma = &_gp->cpu->ema;
   registers = _gp->m_pGUIEEPromRegisters;
 
+  Dprintf((" EEPROM_RegisterWindow::NewProcessor rma=%p\n",rma));
 
   Register_Window::NewProcessor(_gp);
 }
