@@ -207,20 +207,10 @@ static void unselect_row(GtkCList *clist,
 }
 
 
-static char *_strndup(const char *s, int len)
-{
-#if defined(strndup)
-  return strndup(s,len);
-#else
-  return strdup(s);
-#endif
-}
-
 void Symbol_Window::Update(void)
 {
 
 
-  GList *iter;
 
   load_symbols=1;
     
@@ -233,13 +223,17 @@ void Symbol_Window::Update(void)
   gtk_clist_clear(GTK_CLIST(symbol_clist));
 
   // free all old allocations
+  /*
+  GList *iter;
   for(iter=symbols;iter!=0;)
     {
       GList *next=iter->next;
       g_list_remove(iter,iter->data);
       iter=next;
     }
-  symbols=0;
+  */
+  while (symbols)
+    symbols = g_list_remove(symbols,symbols->data);
 
   Symbol_Table &st = CSimulationContext::GetContext()->GetSymbolTable();
   Symbol_Table::iterator symIt;
@@ -260,8 +254,8 @@ void Symbol_Window::Update(void)
     char **entry = (char**)malloc(3*sizeof(char*));
     const int cMaxLength = 32;
 
-    entry[0] = _strndup(sym->name().c_str(), cMaxLength);
-    entry[1] = _strndup(sym->showType().c_str(), cMaxLength);
+    entry[0] = g_strndup(sym->name().c_str(), cMaxLength);
+    entry[1] = g_strndup(sym->showType().c_str(), cMaxLength);
     entry[2] = (char*)malloc(cMaxLength);
     if (typeid(*sym) == typeid(register_symbol)) {
       Register * pReg = ((register_symbol*)sym)->getReg();
