@@ -51,7 +51,7 @@ Boston, MA 02111-1307, USA.  */
 #define BITCOL 5
 #define LASTCOL BITCOL
 
-static char *watch_titles[]={"name","address","dec","hex ","ascii","bits"};
+static char *watch_titles[]={"name","address","dec","hex","ascii","bits"};
 
 #define COLUMNS sizeof(watch_titles)/sizeof(char*)
 
@@ -128,7 +128,7 @@ WatchEntry::WatchEntry()
 }
 //========================================================================
 ColumnData::ColumnData()
-  : ww(0),isVisible(1), bIsValid(true)
+  : isVisible(1), bIsValid(true), ww(0)
 {
 }
 
@@ -190,6 +190,62 @@ void Watch_Window::UpdateMenus(void)
   }
 }
 
+int Watch_Window::set_config(void) {
+  int iRet = GUI_Object::set_config();
+  WriteSymbolList();
+  return iRet;
+}
+
+void Watch_Window::WriteSymbolList()
+{
+  // delete previous list
+  DeleteSymbolList();
+  // write the current list
+  WatchEntry *entry;
+  guint uSize = g_list_length(watches);
+  char cwv[100];
+  char *vname;
+  for (guint i = 0; i<uSize; i++) {
+    snprintf(cwv, sizeof(cwv), "WV%d",i);
+    vname = 0;
+    entry = (WatchEntry*) g_list_nth_data(watches, i);
+    if (entry && entry->pRegSymbol)
+      config_set_string(name(), cwv, entry->pRegSymbol->name().c_str());
+  }
+  
+}
+
+void Watch_Window::DeleteSymbolList() {
+  int i=0;
+  char cwv[100];
+  while (i<1000) {
+    snprintf(cwv, sizeof(cwv), "WV%d",i++);
+    if (config_remove(name(), cwv) == 0 ) {
+      break;
+    }
+  }
+}
+
+void Watch_Window::ReadSymbolList()
+{
+  // now read symbols watched from a prior simulation session
+  int i=0;
+  char cwv[100];
+  char *vname;
+  while (i<1000) {
+    snprintf(cwv, sizeof(cwv), "WV%d",i++);
+    vname = 0;
+    if (config_get_string(name(), cwv, &vname) ) {
+      Value *val = get_symbol_table().find(vname);
+      if(val != 0) {
+        Add(val);
+      }
+    }
+    else
+      break;
+  }
+}
+
 static void unselect_row(GtkCList *clist,
 			 gint row,
 			 gint column,
@@ -238,25 +294,25 @@ popup_activated(GtkWidget *widget, gpointer data)
       entry->put_value(value);
       break;
     case MENU_BREAK_READ:
-      bp.set_read_break(entry->cpu,entry->address);
+      get_bp().set_read_break(entry->cpu,entry->address);
       break;
     case MENU_BREAK_WRITE:
-      bp.set_write_break(entry->cpu,entry->address);
+      get_bp().set_write_break(entry->cpu,entry->address);
       break;
     case MENU_BREAK_READ_VALUE:
       value = gui_get_value("value to read for breakpoint:");
       if(value<0)
 	break; // Cancel
-      bp.set_read_value_break(entry->cpu,entry->address,value);
+      get_bp().set_read_value_break(entry->cpu,entry->address,value);
       break;
     case MENU_BREAK_WRITE_VALUE:
       value = gui_get_value("value to write for breakpoint:");
       if(value<0)
 	break; // Cancel
-      bp.set_write_value_break(entry->cpu,entry->address,value);
+      get_bp().set_write_value_break(entry->cpu,entry->address,value);
       break;
     case MENU_BREAK_CLEAR:
-      bp.clear_all_register(entry->cpu,entry->address);
+      get_bp().clear_all_register(entry->cpu,entry->address);
       break;
     case MENU_COLUMNS:
       select_columns(popup_ww, popup_ww->watch_clist);
@@ -513,7 +569,6 @@ void Watch_Window::UpdateWatch(WatchEntry *entry)
 
   RegisterValue rvMaskedNewValue;
   unsigned int uBitmask;
-  unsigned int uBitmaskForMaskedValue = entry->cpu->register_mask();
 
   entry->put_shadow(rvNewValue);
 
@@ -655,11 +710,6 @@ void Watch_Window::Add( REGISTER_TYPE type, GUIRegister *reg, register_symbol *
 
   UpdateMenus();
 
-  static int wv=0;
-  char cwv[100];
-  snprintf(cwv, sizeof(cwv), "WV%d",wv);
-  wv++;
-  config_set_string(name(), cwv, vname);
 }
 
 //---
@@ -693,23 +743,7 @@ void Watch_Window::NewProcessor(GUI_Processor *_gp)
   if (!gp || !gp->cpu)
     return;
 
-  int i;
-
-  // now read symbols watched from a prior simulation session
-  i=0;
-  char cwv[100];
-  char *vname;
-  while (i<1000) {
-    snprintf(cwv, sizeof(cwv), "WV%d",i++);
-    vname = 0;
-    if (config_get_string(name(), cwv, &vname) ) {
-      Value *val = get_symbol_table().find(vname);
-      Add(val);
-    }
-    else
-      break;
-  }
-  
+  ReadSymbolList();
 }
 //------------------------------------------------------------------------
 // ClearWatches
@@ -845,7 +879,16 @@ Watch_Window::Watch_Window(GUI_Processor *_gp)
 
     if (!config_get_variable(name(),watch_titles[i],&coldata[i].isVisible))
       config_set_variable(name(),watch_titles[i],1);
+  }
 
+  { 
+    // Fix a db error in previous versions of gpsim
+    int j;
+    while (config_get_variable(name(),"hex",&j))
+      config_remove(name(), "hex");
+
+    const int hexIndex=3;
+    config_set_variable(name(),watch_titles[hexIndex],coldata[hexIndex].isVisible);
   }
 
   if(enabled)
