@@ -41,14 +41,32 @@ support:
   Module * getmodule(char *module_type) - creates a new module
  */
 
-#include <iostream.h>
-#include <dlfcn.h>
+/* IN_MODULE should be defined for modules */
+#define IN_MODULE
+
+#include <iostream>
 #include <stdio.h>
+
+#include "../config.h"    // get the definition for HAVE_GUI
+
 #include "../src/modules.h"
-#include "binary_indicator.h"
+
 #include "resistor.h"
 #include "usart.h"
-#include "paraface.h"
+#ifndef _WIN32
+//#include "paraface.h"
+#endif
+#include "switch.h"
+#include "logic.h"
+#ifdef HAVE_GUI
+#include "led.h"
+#include "push_button.h"
+//#include "video.h"
+#include "encoder.h"
+#endif
+#include "stimuli.h"
+#include "ttl.h"
+#include "i2c-eeprom.h"
 
 /*
 class Module_Types
@@ -61,20 +79,59 @@ public:
 */
 Module_Types available_modules[] =
 {
-  { "binary_indicator", "bi",   Binary_Indicator::construct },
 
-  //  { "resistor",         "res",  Resistor::construct }
-  { "pullup",           "pu",   PullupResistor::pu_construct },
-  { "pulldown",         "pd",   PullupResistor::pd_construct },
-
-  // USART
-  { "usart",            "usart", USARTModule::USART_construct},
 
+#ifndef _WIN32
   // Parallel port interface
-  { "parallel_interface",         "paraface", Paraface::construct},
+  /*
+    TSD - removed 16APR06 - The parallel port interface uses the deprecated
+    IOPORT class. 
+  { {"parallel_interface",         "paraface"}, Paraface::construct},
+  */
+#endif
+
+  // Switch
+  { {"switch",         "sw"}, Switches::Switch::construct},
+
+  // Logic
+  { {"and2", "and2"}, AND2Gate::construct},
+  { {"or2",  "or2"},  OR2Gate::construct},
+  { {"xor2", "xor2"}, XOR2Gate::construct},
+  { {"not",  "not"},  NOTGate::construct},
+
+#ifdef HAVE_GUI
+  // Leds
+  { {"led_7segments", "led7s"}, Leds::Led_7Segments::construct},
+  { {"led", "led"}, Leds::Led::construct},
+  { {"push_button",      "pb"},   PushButton::construct },
+#endif
+  { {"PortStimulus",     "ps"},   ExtendedStimuli::PortStimulus::construct },
+  { {"pullup",           "pu"},   PullupResistor::pu_construct },
+  { {"pulldown",         "pd"},   PullupResistor::pd_construct },
+  { {"pulsegen",         "pg"},   ExtendedStimuli::PulseGen::construct },
+
+  /*
+    TSD Removed 17APR06
+  // Video
+  { {"PAL_video", "video"}, Video::construct},
+  */
+#ifdef HAVE_GUI
+  // Encoder
+  { {"Encoder", "encoder"}, Encoder::construct},
+#endif
+  // USART
+  { {"usart",            "usart"}, USARTModule::USART_construct},
+
+  // TTL devices
+  { {"TTL377", "ttl377"}, TTL::TTL377::construct},
+
+  // I2c EEPROM
+  { {"I2C-EEPROM2k", "e24xx024"}, I2C_EEPROM_Modules::I2C_EE_Module::construct_2k},
+  { {"I2C-EEPROM16k", "e24xx16b"}, I2C_EEPROM_Modules::I2C_EE_Module::construct_16k},
+  { {"I2C-EEPROM256k", "e24xx256"}, I2C_EEPROM_Modules::I2C_EE_Module::construct_256k},
 
   // No more modules
-  { NULL,NULL,NULL}
+  { {0,0},0}
 };
 
 #ifdef __cplusplus
@@ -103,12 +160,13 @@ extern "C" {
 void mod_list(void)
 {
 
-  int number_of = sizeof(available_modules) / sizeof(Module_Types);
-  int i,j,k,l,longest;
+  unsigned int number_of = sizeof(available_modules) / sizeof(Module_Types);
+  unsigned int i,j,l;
+  unsigned int k,longest;
 
   for(i=0,longest=0; i<number_of; i++)
     {
-      k = strlen(available_modules[i].names[1]);
+      k = (unsigned int)strlen(available_modules[i].names[1]);
       if(k>longest)
 	longest = k;
     }
@@ -122,7 +180,7 @@ void mod_list(void)
 	  cout << available_modules[k].names[1];
 	  if(i<3)
 	    {
-	      l = longest + 2 - strlen(available_modules[k].names[1]);
+	      l = longest + 2 - (unsigned int)strlen(available_modules[k].names[1]);
 	      for(j=0; j<l; j++)
 		cout << ' ';
 	    }
@@ -165,23 +223,6 @@ void test(void)
 
 }
 
-Module * getmodule(void)
-{
-  Module *bi = Binary_Indicator::construct();
-
-  cout << "gpsim_modules created a binary indicator name:  " 
-       << bi->name() <<'\n';
-  return bi;
-}
-
 #ifdef __cplusplus
 }
 #endif /* __cplusplus */
-
-void test2(void)
-{
-  printf("%s - c++ declared\n", __FUNCTION__);
-}
-
-
-
