@@ -37,6 +37,7 @@ Boston, MA 02111-1307, USA.  */
 
 /* IN_MODULE should be defined for modules */
 #define IN_MODULE
+#define DEFAULT_BAUD    9600
 
 #include <errno.h>
 #include <stdlib.h>
@@ -45,6 +46,7 @@ Boston, MA 02111-1307, USA.  */
 
 #include "../config.h"    // get the definition for HAVE_GUI
 #ifdef HAVE_GUI
+#define GTK_ENABLE_BROKEN
 #include <gtk/gtk.h>
 #endif
 
@@ -62,8 +64,6 @@ Boston, MA 02111-1307, USA.  */
 #include "../src/trace.h"
 #include "../src/uart.h"
 
-#include "../src/bitlog.h"
-
 
 //#define DEBUG
 #if defined(DEBUG)
@@ -72,6 +72,7 @@ Boston, MA 02111-1307, USA.  */
 #define Dprintf(arg) {}
 #endif
 
+#define HAVE_TXFIFO
 
 static bool bIsLow(char state)
 {
@@ -106,101 +107,6 @@ confined to the limited number of discrete baud rates.
 **********************************************************************************/
 
 
-//--------------------------------------------------------------
-//
-//  BRG
-//
-// Serial Port Baud Rate Generator
-//
-//
-//--------------------------------------------------------------
-
-class BRG       //  : public TriggerObject
-{
-public:
-  double baud;
-  guint64 time_per_bit;
-  guint64 last_time;
-  guint64 start_time;
-  guint64 future_time;
-
-
-  BRG(void) {
-    set_baud_rate(125000); // Default baud rate.
-    
-  }
-
-#if 0
-  virtual void callback(void) {
-    cout << "SPBRG::" << __FUNCTION__ << " is not implemented\n";
-
-    //pic_processor *cpu = gpsim_get_active_cpu();
-    last_time = get_cycles().get();
-
-    cout << "SPBRG rollover at cycle " << last_time << '\n';
-
-    //if(rcsta->value & _RCSTA::SPEN) {
-      // If the serial port is enabled, then set another 
-      // break point for the next clock edge.
-      get_next_cycle_break();
-
-      //}
-
-  };
-#endif
-
-  virtual void start(void) {
-
-    last_time = get_cycles().get();
-    start_time = last_time;
-
-    //future_time = last_time + time_per_bit;
-
-    //gpsim_set_break(future_time, this);
-    //get_next_cycle_break();
-
-
-    //if(verbose)
-    //cout << "SPBRG::start   last_cycle = " << 
-    //  hex << last_time << " future_cycle = " << future_time << '\n';
-
-  };
-
-#if 0
-  virtual void get_next_cycle_break(void) {
-    future_time = last_time + time_per_bit;
-
-    get_cycles().set_break(future_time, this);
-
-
-  };
-#endif
-
-  void set_baud_rate(double new_baud) {
-    //cout << "SPBRG::" << __FUNCTION__ << "\n";
-
-    baud = new_baud;
-
-    if(get_active_cpu() && baud>0.0) {
-
-      time_per_bit = (guint64)(get_active_cpu()->get_frequency()/baud);
-
-      Dprintf(("Baud:%f time_per_bit=%lld\n",baud,time_per_bit));
-    }
-  };
-
-#if 0
-  virtual guint64 get_cpu_cycle(unsigned int edges_from_now) {
-    cout << "SPBRG::" << __FUNCTION__ << " is not implemented\n";
-  };
-#endif
-
-};
-
-
-
-
-
 //--------------------------------------------------------------
 //
 //
@@ -212,13 +118,12 @@ public:
   USARTModule *usart;
 
   USART_RXPIN (USARTModule *_usart,
-	       unsigned int b, 
-	       char *opt_name=NULL) : IO_bi_directional_pu(0,b,opt_name) { 
+	       char *opt_name=NULL) : IO_bi_directional_pu(opt_name) { 
 
     usart = _usart;
 
     string n(usart->name());
-    n = n + ".RX";
+    n = n + ".RXPIN";
     new_name(n.c_str());
 
     // Let the pin think it's in the high state. If this is wrong,
@@ -272,14 +177,14 @@ public:
   USARTModule *usart;
 
   USART_TXPIN (USARTModule *_usart,
-	       unsigned int b, 
-	       char *opt_name=NULL) : IO_bi_directional(0,b,0) 
+	       char *opt_name=NULL) 
   { 
 
     usart = _usart;
 
     string n(usart->name());
-    n = n + ".TX";
+    n = n + ".TXPIN";
+    IO_bi_directional(n.c_str());
     new_name(n.c_str());
     bDrivingState = true;
     update_direction(1,true);   // Make the TX pin an output.
@@ -303,7 +208,6 @@ public:
 class TXREG : public TriggerObject
 {
 private:
-  //BRG         *brg;
 
   bool empty_flag;
   double baud;
@@ -337,23 +241,24 @@ private:
   USARTModule *usart;
 
   virtual bool is_empty() { return empty_flag;};
-  virtual void empty() {empty_flag = 0;};
-  virtual void full()  {empty_flag = 1;};
+  virtual void empty() {empty_flag = 1;};
+  virtual void full()  {empty_flag = 0;};
   virtual void assign_pir_set(PIR_SET *new_pir_set){};
 
   TXREG(void) {
     txpin = 0;
     usart = 0;
 
-    baud = 9600;
     bits_per_byte = 8;
     stop_bits = 1;
     use_parity = 0;
+    set_baud_rate(DEFAULT_BAUD);
 
     tx_byte = '0';
 
     update_packet_time();
 
+    empty();
   }
 
 
@@ -368,12 +273,13 @@ private:
 
     if(get_active_cpu()) {
       time_per_packet = 
-	(guint64)( get_active_cpu()->get_frequency() * ( (1.0 +             // start bit
-							  bits_per_byte +   // data bits
-							  stop_bits  +      // stop bit(s)
-							  use_parity)       //
-							 /baud));
-      time_per_bit = (guint64)(get_active_cpu()->get_frequency() / baud);
+	(guint64)( get_cycles().instruction_cps() * ( 
+			  (1.0 +             // start bit
+			  bits_per_byte +   // data bits
+			  stop_bits  +      // stop bit(s)
+			  use_parity)       //
+			 /baud));
+      time_per_bit = (guint64)(get_cycles().instruction_cps() / baud);
     } else
       time_per_packet = time_per_bit = 0;
 
@@ -409,7 +315,7 @@ private:
 
 
   virtual void callback(void) {
-    if(1) {
+    if(0) {
       cout << " usart module TXREG::" << __FUNCTION__ << "\n";
     }
 
@@ -418,7 +324,11 @@ private:
 
     if(txpin) {
       txpin->putState((txr & 1) ? true : false);
-      cout << "usart tx module sent a " << (txr&1) <<  " bit count " << bit_count << '\n';
+      if (0)
+      {
+          cout << "usart tx module sent a " << (txr&1) <<  
+		" bit count " << bit_count << '\n';
+      }
     }
 
     if(bit_count) {
@@ -428,10 +338,12 @@ private:
       get_cycles().set_break(future_time, this);
     } else {
       // We've sent the whole byte. 
-
+      /* output data from buffer if configured */
+#ifdef HAVE_TXFIFO
       if(usart && usart->mGetTxByte(tx_byte))
 	mSendByte(tx_byte);
       else
+#endif
 	empty();
     }
 
@@ -441,16 +353,17 @@ private:
   void mSendByte(unsigned _tx_byte) 
   {
     
-    if(1) {
+    if(0) {
       cout << "\n\n";
       cout << "TXREG::" << __FUNCTION__ << "\n";
       cout << "\n\n";
     }
 
-    mBuildTXpacket(tx_byte);
+    mBuildTXpacket(_tx_byte);
     last_time = get_cycles().get();
     future_time = last_time + time_per_bit;
     get_cycles().set_break(future_time, this);
+    full();
   }
 
 private:
@@ -464,8 +377,11 @@ private:
     // total bits = byte + start and stop bits
     bit_count = bits_per_byte + 1 + 1;
 
-    cout << hex << "TXREG::" << __FUNCTION__ << " byte to send 0x" << tb 
+    if (0)
+    {
+        cout << hex << "TXREG::" << __FUNCTION__ << " byte to send 0x" << tb 
 	 <<" txr 0x" << txr << "  bits " << bit_count << '\n';
+    }
 
   }
 
@@ -485,7 +401,6 @@ class RCREG : public TriggerObject
 
   USART_RXPIN *rxpin;
 
-#define DEFAULT_BAUD    250000
 #define MAX_PW  0xfffffff
 
 #define RX_ERR_OVERRUN        1
@@ -494,24 +409,12 @@ class RCREG : public TriggerObject
 
   enum RX_STATES {
     RS_WAITING_FOR_START,
-    RS_0,
-    RS_1,
-    RS_2,
-    RS_3,
-    RS_4,
     RS_RECEIVING,
     RS_STOPPED,
-    RS_OVERRUN
+    RS_OVERRUN,
+    RS_START_BIT
   } receive_state;
 
-  enum RXErrors {
-    eNoError,
-    eNoStartBit,
-    eNoStopBit,
-    eBadParity
-    
-  };
-
   /**************************/
   // RCREG constructor
   /**************************/
@@ -534,13 +437,13 @@ class RCREG : public TriggerObject
 
     if(get_active_cpu()) {
       time_per_packet = 
-	(guint64)(get_active_cpu()->get_frequency() * (1.0 +     // start bit
-						       bits_per_byte +   // data bits
-						       stop_bits  +      // stop bit(s)
-						       use_parity)       //
-		  /baud);
+	(guint64)(get_cycles().instruction_cps() * (1.0 +     // start bit
+			       bits_per_byte +   // data bits
+			       stop_bits  +      // stop bit(s)
+			       use_parity)       //
+		  	       /baud);
 
-      time_per_bit = (guint64)(get_active_cpu()->get_frequency() / baud);
+      time_per_bit = (guint64)(get_cycles().instruction_cps() / baud);
     } else
       time_per_packet = time_per_bit = 0;
 
@@ -570,12 +473,10 @@ class RCREG : public TriggerObject
   virtual void callback();
 
   void start();
-  RXErrors decode_byte(unsigned sindex, unsigned int &aByte);
   void new_rx_edge(bool bit);
 
 private:
   USARTModule *m_usart;
-  ThreeStateEventLogger *rx_event;
 
   char m_cLastRXState;
   unsigned int start_bit_event;
@@ -594,13 +495,14 @@ private:
   bool    use_parity;
   bool    parity;         // 0 = even, 1 = odd
   double  baud;
+  unsigned int rx_byte;
+  int	  rx_count;
 
   guint64 time_per_packet;
 
   bool autobaud;
 
   IOPIN   *rcpin;
-  //BRG     brg;
   unsigned int *fifo;
 
 
@@ -610,53 +512,72 @@ private:
 //------------------------------------------------------------------------
 
 RCREG::RCREG(USARTModule *pUsart)
-  : m_usart(pUsart), rcpin(0), start_bit_event(0), m_cLastRXState('?')
+  : m_usart(pUsart), m_cLastRXState('?'), start_bit_event(0), rcpin(0)
 {
   assert(m_usart);
 
-  rx_event = new ThreeStateEventLogger(1024);
 
   receive_state = RS_WAITING_FOR_START;
 
   autobaud = false;
   set_bits_per_byte(8);
-  set_stop_bits(1.0);
+  set_stop_bits(0.9);
   set_noparity();
 
-  //set_baud_rate(DEFAULT_BAUD);
-  set_baud_rate(250000);
+  set_baud_rate(DEFAULT_BAUD);
 }
 
 //------------------------------------------------------------------------
 void RCREG::callback() 
 {
 
-  Dprintf((" usart module RCREG\n"));
+  Dprintf((" usart module RCREG time:0x%llx=%lld\n",get_cycles().get(),get_cycles().get()));
 
 
   switch(receive_state) {
   case RS_WAITING_FOR_START:
     Dprintf(("waiting for start\n"));
     break;
+
+  case RS_START_BIT:	// should now be in middle of start bit
+     if (bIsLow(m_cLastRXState))
+     {
+	receive_state = RS_RECEIVING;
+	rx_count = bits_per_byte + use_parity;
+	rx_byte = 0;
+  	future_time = get_cycles().get() + time_per_bit;
+
+  	if(!autobaud) 
+    	  get_cycles().set_break(future_time, this);
+       
+     }
+     else // Not valid start bit
+     {
+       receive_state = RS_WAITING_FOR_START;
+     }
+     break;
+
   case RS_RECEIVING:
 
-    if (bIsHigh(m_cLastRXState)) {
+    if (rx_count--)
+    {
+	rx_byte = (rx_byte >> 1) | (bIsHigh(m_cLastRXState) ?  
+		1<<(bits_per_byte-1) : 0);
+  	future_time = get_cycles().get() + time_per_bit;
 
-      receive_state = RS_WAITING_FOR_START;
-
-      Dprintf((" received a byte\n"));
-      unsigned int rx_byte=0;
-      RXErrors re = decode_byte(start_bit_event,rx_byte);
-      if (re == eNoError) {
-	Dprintf((" Successfully recieved 0x%02x\n",rx_byte));
+  	if(!autobaud) 
+    	  get_cycles().set_break(future_time, this);
+    }
+    else if (bIsHigh(m_cLastRXState)) // on stop bit
+    {
 	m_usart->newRxByte(rx_byte);
-      } else {
-	Dprintf((" Failed to decode byte\n"));
-      }
-
-    } else {
-      receive_state = RS_WAITING_FOR_START;
-      cout << "Looks like we've overrun\n";
+	m_usart->show_tx(rx_byte);
+        receive_state = RS_WAITING_FOR_START;
+    }
+    else
+    {
+	cout << "USART module RX overrun error\n";
+        receive_state = RS_WAITING_FOR_START;
     }
 
     break;
@@ -675,11 +596,10 @@ void RCREG::callback()
 void RCREG::start() 
 {
 
-  receive_state = RS_RECEIVING;
+  receive_state = RS_START_BIT;
 
-  start_bit_event = rx_event->get_index();
 
-  future_time = rx_event->get_time(start_bit_event) + time_per_packet;
+  future_time = get_cycles().get() + time_per_bit/2;
 
   if(!autobaud) {
     get_cycles().set_break(future_time, this);
@@ -688,59 +608,6 @@ void RCREG::start()
   Dprintf((" usart module RCREG current cycle=%lld future_cycle=%lld\n", get_cycles().value,future_time));
 }
 
-//------------------------------------------------------------------------
-// decode_byte
-// 
-// decode_byte will examine the data logged from the receiver's I/O 
-// pin and attempt to decipher it.
-//
-//    +---------------------------------------------  Edge of start bit
-//    | +-------------------------------------------  Middle of start bit
-//  __v v  ___ ___ ___ ___ ___ ___ ___ ___ ____
-//    \___/___X___X___X___X___X___X___X___/    
-//          ^   ^   ^   ^   ^   ^   ^   ^   ^
-//          |   |   |   |   |   |   |   |   +-------  Middle of stop bit
-//          +---+---+---+---+---+---+---+-----------  Middle of data bits
-//
-// The input to decode_byte is the index into the event log holding
-// the captured waveform. The baud rate for the receiver is used to
-// compute the bit time and this in turn is used to compute the times
-// at which the data samples should be taken.
-//
-
-RCREG::RXErrors RCREG::decode_byte(unsigned int sindex, unsigned int &rx_byte)
-{
-
-  Dprintf((" decode_byte start_bit_index=%d \n", sindex));
-
-  rx_byte = 0;
-
-  if (!bIsLow(rx_event->get_state(sindex)))
-    return eNoStartBit;
-
-  guint64 sample_time = rx_event->get_time(sindex) + time_per_bit/2;
-
-  Dprintf((" decode_byte sample_time=%lld start_bit_time=%lld time_per_bit=%d\n", sample_time,
-	   rx_event->get_time(sindex), time_per_bit));
-
-  if (!bIsLow(rx_event->get_state(sample_time)))
-    return eNoStartBit;
-
-  sample_time += time_per_bit;
-
-  unsigned int msb = 1<<(bits_per_byte-1);
-  for (int i=0; i<bits_per_byte; i++, sample_time+=time_per_bit)
-    rx_byte = (rx_byte>>1) | (bIsHigh(rx_event->get_state(sample_time))?msb:0);
-
-  if (!bIsHigh(rx_event->get_state(sample_time)))
-    return eNoStopBit;
-
-  Dprintf((" decoded %d=%x=%c \n", rx_byte,rx_byte,rx_byte));
-
-  return eNoError;
-}
-
-
 //------------------------------------------------------------------------
 //  new_rx_edge(bool bit)
 //
@@ -751,43 +618,9 @@ RCREG::RXErrors RCREG::decode_byte(unsigned int sindex, unsigned int &rx_byte)
 
 void RCREG::new_rx_edge(bool bit) 
 {
-#if defined(DEBUG)
-  cout << "USART MODULE RCREG::" << __FUNCTION__ << "\n";
-  switch(receive_state) {
-  case RS_WAITING_FOR_START:
-    cout << "state = WAITING_FOR_START\n";
-    break;
-  case RS_RECEIVING:
-    cout << "state = RECEIVING\n";
-    break;
-  case RS_STOPPED:
-    cout << "state = STOPPED\n";
-    break;
-  case RS_OVERRUN:
-    cout << "state = OVERRUN\n";
-    break;
-  case RS_0:
-    cout << "state = RS_0\n";
-    break;
-  case RS_1:
-    cout << "state = RS_1\n";
-    break;
-  case RS_2:
-    cout << "state = RS_2\n";
-    break;
-  case RS_3:
-    cout << "state = RS_3\n";
-    break;
-  case RS_4:
-    cout << "state = RS_4\n";
-    break;
-
-  }
-#endif
 
   // Save the event state
   char currentRXState = rxpin->getBitChar();
-  rx_event->event(currentRXState);
 
   if (currentRXState != m_cLastRXState) {
 
@@ -797,7 +630,7 @@ void RCREG::new_rx_edge(bool bit)
     case RS_WAITING_FOR_START:
       if(bIsLow(currentRXState)) {
 	start();
-	Dprintf(("Start bit at t=0x%llx\n",rx_event->get_time(start_bit_event)));
+	Dprintf(("Start bit at t=0x%llx\n",get_cycles().get()));
       }
 
       break;
@@ -819,63 +652,49 @@ void RCREG::new_rx_edge(bool bit)
 
 
 //------------------------------------------------------------------------
-class USART_IO : public IOPIN
+class USART_IO : public IO_bi_directional_pu
 {
 public:
 
+  USARTModule *usart;
 
   USART_IO(void) {
     cout << "USART_IO constructor - do nothing\n";
   }
-  USART_IO (unsigned int b, char *opt_name=NULL) : IOPIN(0,b,opt_name) { };
 
-};
+  USART_IO ( USARTModule *_usart,
+	     unsigned int b,
+             char *opt_name ) : IO_bi_directional_pu(opt_name) {
+    usart = _usart;
 
+    string n(usart->name());
+    n = n + "." + opt_name;
+    new_name(n.c_str());
 
+    bDrivenState = true;
+    update_direction(0,true);   // Make the RX pin an input.
 
-
-//--------------------------------------------------------------
-//  Usart Attributes - derived from gpsim's FloatAttribute
-// 
-//--------------------------------------------------------------
-class UsartAttribute : public Float {
-
-public:
-  enum UA_TYPE {
-    UA_BAUDRATE,
-    UA_STARTBITS,
-    UA_STOPBITS,
-  } ;
-
-  UA_TYPE type;
-
-
-  UsartAttribute(char *_name, UA_TYPE new_type, double def_val) :
-    Float(def_val)
-  {
-
-    cout << "USART Attribute constructor\n";
-
-    type = new_type;
-    new_name(_name);
+    bPullUp = true;
+    Zpullup = 10e3;
   }
 
+  void setDrivenState(bool new_dstate) { 
+    bool diff = new_dstate ^ bDrivenState;
 
-  void set(double new_val) {
+//    Dprintf((" usart module %s new state=%d\n",name(),new_dstate));
 
-    cout << "Setting usart attribute\n";
+    if( usart && diff ) {
 
-    Float::set(new_val);
+      bDrivenState = new_dstate;
+      IOPIN::setDrivenState(new_dstate);
 
-  };
+    }
 
-
-  void set(int v) {
-    set(double(v));
-  };
+  }
 };
 
 
+
 //
 //  USART attributes
 //
@@ -902,7 +721,7 @@ public:
   RCREG *rcreg;
 
   RxBaudRateAttribute(RCREG *prcreg)
-    : Integer("rxbaud",9600,"USART Module Receiver baud rate"), rcreg(prcreg)
+    : Integer("rxbaud",DEFAULT_BAUD,"USART Module Receiver baud rate"), rcreg(prcreg)
   {
     assert(rcreg);
   }
@@ -911,12 +730,12 @@ public:
   void set(Value *v) {
 
 
-    cout << "Setting Rx baud rate attribute!\n";
     Integer::set(v);
 
     gint64 b;
     get(b);
     rcreg->set_baud_rate(b);
+    cout << "Setting Rx baud rate attribute to " << dec << b << "\n";
 
   };
   virtual string toString()
@@ -933,7 +752,7 @@ public:
   TXREG *txreg;
 
   TxBaudRateAttribute(TXREG *ptxreg)
-    : Integer("txbaud",9600,"USART Module Transmitter baud rate"), txreg(ptxreg)
+    : Integer("txbaud",DEFAULT_BAUD,"USART Module Transmitter baud rate"), txreg(ptxreg)
   {
     assert(txreg);
   }
@@ -945,6 +764,7 @@ public:
     gint64 b;
     get(b);
     txreg->set_baud_rate(b);
+    cout << "Setting Tx baud rate attribute to " << dec << b << "\n";
   }
   virtual string toString()
   {
@@ -955,20 +775,20 @@ public:
 
 class TxBuffer : public Integer
 {
-  TXREG *txreg;
+  USARTModule *usart;
 public:
-  TxBuffer(TXREG *_txreg)
-    : Integer("tx",0,"UART Transmit Register"),txreg(_txreg)
+  TxBuffer(USARTModule *_usart)
+    : Integer("tx",0,"UART Transmit Register"),usart(_usart)
   {
   }
   virtual void set(gint64 i)
   {
     i &= 0xff;
 
-    cout << name() << " sending byte 0x" << hex << i << endl;
+    //cout << name() << " sending byte 0x" << hex << i << endl;
 
-    if(txreg)
-      txreg->mSendByte(i);
+    if(usart)
+      usart->SendByte(i);
 
     Integer::set(i);
   }
@@ -1014,9 +834,14 @@ void USARTModule::new_rx_edge(unsigned int bit)
 void USARTModule::newRxByte(unsigned int aByte)
 {
   m_RxBuffer->newByte(aByte);
+  if(m_loop->getVal())
+  {
+	SendByte(aByte);
+  }
 }
 
 //--------------------------------------------------------------
+#ifndef HAVE_TXFIFO
 static unsigned int _tx_index=0;
 static unsigned char Test_Hello[] = {
   0x1b,0xff, 0x87,0x05, 'H', 'E',  'L', 'L', 'O', 0x17, 0x55
@@ -1030,6 +855,20 @@ bool USARTModule::mGetTxByte(unsigned int &aByte)
   return true;
 
 }
+#else
+bool USARTModule::mGetTxByte(unsigned int &aByte)
+{
+    if ( m_FifoHead == m_FifoTail )
+        return false;
+    aByte = m_TxFIFO[m_FifoTail];
+    if ( m_FifoTail < m_FifoLen-1 )
+        m_FifoTail++;
+    else
+        m_FifoTail = 0;
+    return true;
+}
+#endif
+
 //--------------------------------------------------------------
 // create_iopin_map 
 //
@@ -1065,14 +904,14 @@ void USARTModule::create_iopin_map(void)
   //   need to reference these newly created I/O pins (like
   //   below) then we can call the member function 'get_pin'.
 
-  USART_TXPIN *txpin = new USART_TXPIN(this, 0,"TX");
-  USART_RXPIN *rxpin = new USART_RXPIN(this, 1,"RX");
+  USART_TXPIN *txpin = new USART_TXPIN(this, "TXPIN");
+  USART_RXPIN *rxpin = new USART_RXPIN(this, "RXPIN");
 
 
   assign_pin(1, txpin);
   assign_pin(2, rxpin);
-  assign_pin(3, new USART_IO(2,"CTS"));
-  assign_pin(4, new USART_IO(3,"RTS"));
+  assign_pin(3, new USART_IO(this, 2, "CTS"));
+  assign_pin(4, new USART_IO(this, 3, "RTS"));
 
   // Complete the usart initialization
 
@@ -1080,10 +919,6 @@ void USARTModule::create_iopin_map(void)
   m_txreg->usart = this; // Point back to the module
   m_rcreg->rxpin = rxpin;
 
-  //usart->brg->put(0);
-  //usart->spbrg->set_baud_rate(38400);
-  //usart->rcsta->put(_RCSTA::SPEN | _RCSTA::CREN);
-
 }
 //--------------------------------------------------------------
 void USARTModule::get(char *cP, int len)
@@ -1110,6 +945,12 @@ USARTModule::USARTModule(const char *_name)
   assert(_name);
   new_name(_name);
 
+#ifdef HAVE_TXFIFO
+  m_TxFIFO = new unsigned char[64];
+  m_FifoLen = 64;
+  m_FifoHead = m_FifoTail = 0;
+#endif
+
   m_rcreg = new RCREG(this);
   m_txreg = new TXREG;
 
@@ -1122,9 +963,20 @@ USARTModule::USARTModule(const char *_name)
   m_RxBuffer = new RxBuffer(m_rcreg);
   add_attribute(m_RxBuffer);
 
-  m_TxBuffer = new TxBuffer(m_txreg);
+  m_TxBuffer = new TxBuffer(this);
   add_attribute(m_TxBuffer);
 
+  m_CRLF = new Boolean("crlf", true, "if true, carriage return and linefeeds generate new lines in the terminal");
+  add_attribute(m_CRLF);
+
+  m_loop = new Boolean("loop", false, "if true, received characters looped back to transmit");
+  add_attribute(m_loop);
+
+  m_console = new Boolean("console", false, "if true, display received character to the terminal window");
+  add_attribute(m_console);
+
+  CreateGraphics();
+
 
   assert(m_rcreg);
   assert(m_txreg);
@@ -1140,3 +992,170 @@ USARTModule::~USARTModule()
     // FIXME
 }
 
+//--------------------------------------------------------------
+void USARTModule::SendByte(unsigned tx_byte)
+{
+#ifdef HAVE_TXFIFO
+    Dprintf (( "SendByte <%02X> : head=%d, tail=%d, txreg=%p\n",
+               tx_byte, m_FifoHead, m_FifoTail, m_txreg ))
+    if ( m_FifoHead != m_FifoTail || !m_txreg || !m_txreg->is_empty() )
+    {
+        int newHead;
+
+        m_TxFIFO[m_FifoHead] = tx_byte;
+        newHead = m_FifoHead+1;
+        if ( newHead >= m_FifoLen )
+            newHead = 0;
+        if ( newHead == m_FifoTail )
+        {
+            int newLen = m_FifoLen + 32;
+            unsigned char * newFIFO;
+            newFIFO = new unsigned char[newLen];
+            int oldTail = m_FifoTail;
+            int dIdx = 0;
+            int sIdx;
+            for ( sIdx = oldTail; sIdx < m_FifoLen; )
+                newFIFO[dIdx++] = m_TxFIFO[sIdx++];
+            for ( sIdx = 0; sIdx < newHead; )
+                newFIFO[dIdx++] = m_TxFIFO[sIdx++];
+            unsigned char * oldFIFO = m_TxFIFO;
+            m_TxFIFO = newFIFO;
+            m_FifoTail -= oldTail;
+            m_FifoHead = dIdx;
+            m_FifoLen = newLen;
+            delete oldFIFO;
+        }
+        else
+            m_FifoHead = newHead;
+        //cout << "Byte added to queue\n";
+    }
+    else
+#endif
+        if (m_txreg)
+            m_txreg->mSendByte(tx_byte);
+};
+
+#ifdef HAVE_GUI
+static bool ctl = false;	// true when ctrl key is down
+
+static gint
+key_press(GtkWidget *widget,
+          GdkEventKey *key,
+          gpointer data)
+{
+	unsigned int c = key->keyval;
+
+        gtk_signal_emit_stop_by_name (GTK_OBJECT (widget), "key_press_event");
+	if(c == 0xffe3 || c == 0xffe4) // key is left or right ctrl
+	{
+	    ctl = true;
+	    return(1);
+	}
+	if (ctl && c < 0xff00)	// build control character
+	{
+		Dprintf(("CTL 0x%02x\n", c));
+		c &= 0x1f;
+	}
+	if ( c < 0xff20)	// send character to usart
+	{
+	    c &= 0xff;
+	    ((USARTModule *)data)->USARTModule::SendByte(c);
+	    Dprintf(("Send %c 0x%x\n", c,c));
+	}
+	else
+	{
+		Dprintf(( "0x%02x\n", c));
+	}
+	return(1);
+}
+static gint
+key_release(GtkWidget *widget,
+          GdkEventKey *key,
+          gpointer data)
+{
+	unsigned int c = key->keyval;
+	if(c == 0xffe3 || c == 0xffe4)	// Capture release of ctrl key
+	{
+	    ctl = false;
+	}
+	return(1);
+}
+#endif //HAVE_GUI
+
+// Display character from usart on GUI text window
+void USARTModule::show_tx(unsigned int data)
+{
+  data &= 0xff;
+
+  if(m_console->getVal()) {
+    if (isascii(data) && (isprint(data) || '\n' == data || '\r' == data))
+      putchar(data);
+    else
+      printf("<%02X>", data);
+  }
+
+#ifdef HAVE_GUI
+  if(get_interface().bUsingGUI()) {
+      GtkTextBuffer *buff = gtk_text_view_get_buffer(GTK_TEXT_VIEW(text));
+      GtkTextIter iter;
+      gtk_text_buffer_get_end_iter(buff, &iter);
+      if ((isascii(data) && isprint(data)) || 
+	  (m_CRLF->getVal() && ('\n' == data || '\r' == data))) {
+        char ch = data;
+        gtk_text_buffer_insert(buff, &iter, &ch, 1);
+      }
+      else {
+        char hex[5];
+        sprintf (hex, "<%02X>", data);
+        gtk_text_buffer_insert(buff, &iter, hex, 4);  
+      }
+      gtk_text_view_scroll_to_iter(GTK_TEXT_VIEW(text), &iter, 0.0, TRUE, 0.0, 1.0);
+  }
+#endif //HAVE_GUI
+}
+
+// Create a GUI text window 
+void USARTModule::CreateGraphics()
+{
+#ifdef HAVE_GUI
+  if(get_interface().bUsingGUI()) {
+    window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
+    gtk_window_set_title(GTK_WINDOW(window), "USART");
+    gtk_window_set_default_size (GTK_WINDOW (window), 300, 100);
+
+    GtkWidget *pSW = gtk_scrolled_window_new (0,0);
+    gtk_container_add (GTK_CONTAINER (window), pSW);
+    gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (pSW),
+				    GTK_POLICY_AUTOMATIC,
+				    GTK_POLICY_AUTOMATIC);
+    text = gtk_text_view_new ();
+    gtk_text_view_set_editable (GTK_TEXT_VIEW (text), TRUE);
+    gtk_container_add (GTK_CONTAINER (pSW), text);
+
+    /* Change default font throughout the widget */
+    PangoFontDescription *font_desc;
+    font_desc = pango_font_description_from_string ("Courier 10");
+    gtk_widget_modify_font (text, font_desc);
+    pango_font_description_free (font_desc);
+
+    gtk_widget_add_events(window, GDK_KEY_RELEASE_MASK);
+
+    gtk_signal_connect(GTK_OBJECT(text),"key_press_event",
+		       (GtkSignalFunc) key_press,
+		       this);
+    gtk_signal_connect(GTK_OBJECT(text),"key_release_event",
+		       (GtkSignalFunc) key_release,
+		       this);
+
+    gtk_signal_connect (GTK_OBJECT (window), "destroy",
+                        GTK_SIGNAL_FUNC (gtk_main_quit), NULL);
+                                                                                
+
+    gtk_widget_show_all(window);
+
+  } else {
+    window = 0;
+    text = 0;
+  }
+#endif
+}
