@@ -22,75 +22,71 @@ Boston, MA 02111-1307, USA.  */
 #ifndef __USART_MODULE_H__
 #define __USART_MODULE_H__
 
+/* IN_MODULE should be defined for modules */
+#define IN_MODULE
+
 #include "../config.h"
 
 #ifdef HAVE_GUI
+#define GTK_ENABLE_BROKEN
 #include <gtk/gtk.h>
 #endif
 
 
 #include <glib.h>
 
-#include "../src/attribute.h"
 #include "../src/modules.h"
-#include "../src/packages.h"
-#include "../src/stimuli.h"
-#include "../src/symbol.h"
-#include "../src/trace.h"
-#include "../src/uart.h"
 
-class USART_IOPORT;
-class USARTModule;
-class EventLogger;
 
 class TXREG;
 class RCREG;
-class USART_IOPORT;
+class RxBaudRateAttribute;
+class TxBaudRateAttribute;
+class TxBuffer;
+class RxBuffer;
+class Boolean;
 
-class USART_CORE //: public USART_MODULE
+class USARTModule : public Module
 {
  public:
 
-  /* */
-  USART_IOPORT *port;
+#ifdef HAVE_GUI
+  GtkWidget *window, *text;
+#endif // HAVE_GUI
+ 
+  void CreateGraphics(void);
+  virtual void show_tx(unsigned int data);
+  virtual void SendByte(unsigned tx_byte);
 
-  /*  receiver stuff **/
-  RCREG *rcreg;
-
-
-
-  /* Transmitter stuff **/
-  TXREG *txreg;
-
-  EventLogger *tx_event;
-
-  Attribute *baud_rate;
-
-  USARTModule *um;
-
-  virtual void new_rx_edge(unsigned int);
-
-  void initialize(USART_IOPORT *new_iop=NULL);
-
-  USART_CORE(void);
-};
-
-class USARTModule : public ExternalModule
-{
- public:
-
-  USART_IOPORT *port;
-  USART_CORE   *usart;    // Use the PIC usart core 
 
   // Inheritances from the Package class
-  virtual void create_iopin_map(void);
+  virtual void create_iopin_map();
 
-  // Inheritances from the Module class
-  virtual void set_attribute(char *attr, char *val);
+  USARTModule(const char *new_name);
+  ~USARTModule();
 
-  USARTModule(char *new_name = NULL);
+  static Module *USART_construct(const char *new_name=NULL);
 
-  static ExternalModule *USART_construct(char *new_name=NULL);
+  virtual void new_rx_edge(unsigned int);
+  virtual bool mGetTxByte(unsigned int &);
+  virtual void newRxByte(unsigned int);
+  virtual void get(char *, int len);
+private:
+  RxBaudRateAttribute *m_RxBaud;
+  TxBaudRateAttribute *m_TxBaud;
+  Boolean  *m_CRLF;
+  Boolean  *m_loop;
+  Boolean  *m_console;
+  TxBuffer *m_TxBuffer;
+  RxBuffer *m_RxBuffer;
+
+  RCREG *m_rcreg;
+  TXREG *m_txreg;
+
+  unsigned char * m_TxFIFO;
+  int m_FifoLen;
+  int m_FifoHead;
+  int m_FifoTail;
 
 };
 #endif //  __USART_MODULE_H__
