@@ -27,27 +27,106 @@ Boston, MA 02111-1307, USA.  */
 
 #include <string>
 #include "stimuli.h"
+#include "trace.h"
 
 extern unsigned int config_word;
 
 
+//========================================================================
+// 
+
+class OptionTraceObject : public RegisterWriteTraceObject
+{
+public:
+  OptionTraceObject(Processor *_cpu, OPTION_REG *pOptionReg, RegisterValue trv)
+    : RegisterWriteTraceObject(_cpu, pOptionReg, trv)
+  {
+  }
+  void print(FILE *fp)
+  {
+    char sFrom[16];
+    char sTo[16];
+
+    if(reg)
+      fprintf(fp, "  Option: from 0x%s to 0x%s\n",
+	      from.toString(sFrom,sizeof(sFrom)),
+	      to.toString(sTo,sizeof(sTo)));
+
+  }
+};
+
+//========================================================================
+class OptionTraceType : public TraceType
+{
+public:
+  OptionTraceType(Processor *_cpu, OPTION_REG *pOptionReg)
+    : TraceType(1), m_cpu(_cpu),m_pOptionReg(pOptionReg)
+  {
+  }
+
+  TraceObject *decode(unsigned int tbi)
+  {
+
+    unsigned int tv = trace.get(tbi);
+    RegisterValue rv = RegisterValue(tv&0xff,0);
+    OptionTraceObject *oto = new OptionTraceObject(m_cpu, m_pOptionReg, rv);
+
+    return oto;
+  }
+
+  int dump_raw(Trace *pTrace, 
+	       unsigned int tbi,
+	       char *buf, int bufsize)
+  {
+    if (!pTrace)
+      return 0;
+
+    int n = TraceType::dump_raw(pTrace, tbi,buf,bufsize);
+
+    buf += n;
+    bufsize -= n;
+
+    unsigned int tv = pTrace->get(tbi);
+    //unsigned int subtype = (tv >> 8) & 0xfff;
+
+    int  m = snprintf(buf, bufsize,
+		      "  Option Reg: was 0x%0X ", tv & 0xff);
+
+    return m>0 ? (m+n) : n;
+  }
+protected:
+  Processor *m_cpu;
+  OPTION_REG *m_pOptionReg;
+};
+
+
 //-------------------------------------------------------------------
-_12bit_processor::_12bit_processor(void)
+_12bit_processor::_12bit_processor(const char *_name, const char *desc)
+  : pic_processor(_name, desc)
 {
   pc = new Program_Counter();
 
-  pc->set_trace_command(trace.allocateTraceType(new PCTraceType(this,0,1)));
+  pc->set_trace_command(trace.allocateTraceType(new PCTraceType(this,1)));
 
+  mOptionTT = new OptionTraceType(this,&option_reg);
+  trace.allocateTraceType(mOptionTT);
+  RegisterValue rv( (mOptionTT->type() & 0xff000000) | 0, 0);
+  option_reg.set_write_trace(rv);
+  option_reg.set_read_trace(rv);
 }
 
-void _12bit_processor::create_symbols(void)
+_12bit_processor::~_12bit_processor()
+{
+}
+
+void _12bit_processor::create_symbols()
 {
   pic_processor::create_symbols();
   // add a special symbol for W
   symbol_table.add_w(W);
 }
 
-void _12bit_processor::por(void)
+void _12bit_processor::por()
 {
   pic_processor::por();
 }
@@ -75,7 +154,7 @@ bool _12bit_processor::set_config_word(unsigned int address,unsigned int cfg_wor
 
     config_word = cfg_word;
 
-    if(verbose && config_modes)
+    if((bool)verbose && config_modes)
       config_modes->print();
 
     return true;
@@ -84,7 +163,7 @@ bool _12bit_processor::set_config_word(unsigned int address,unsigned int cfg_wor
   return false;
 }
 
-void _12bit_processor::create(void)
+void _12bit_processor::create()
 {
 
   if(verbose)
@@ -109,6 +188,12 @@ void _12bit_processor::create(void)
   //1 tmr0.start(0);
 
 }
+//-------------------------------------------------------------------
+void _12bit_processor::create_config_memory()
+{
+  m_configMemory = new ConfigMemory *[1];
+  *m_configMemory = new ConfigMemory("CONFIG", 0,"Configuration Word",this,0xfff);
+}
 
 //-------------------------------------------------------------------
 void _12bit_processor::option_new_bits_6_7(unsigned int bits)
@@ -121,7 +206,7 @@ void _12bit_processor::option_new_bits_6_7(unsigned int bits)
 }
 
 //-------------------------------------------------------------------
-void _12bit_processor::dump_registers (void)
+void _12bit_processor::dump_registers ()
 {
 
 
