@@ -26,28 +26,62 @@ Boston, MA 02111-1307, USA.  */
 //#include "14bit-registers.h"
 //#include "14bit-instructions.h"
 #include "14bit-processors.h"
+#include "pic-ioports.h"
 
 #include <string>
 #include "stimuli.h"
 
+//========================================================================
+// Generic Configuration word for the midrange family.
 
-//-------------------------------------------------------------------
-//
-pic_processor * _14bit_processor::construct(void)
+class Generic14bitConfigWord : public ConfigMemory 
 {
+public:
+  Generic14bitConfigWord(_14bit_processor *pCpu)
+    : ConfigMemory("CONFIG", 0x3fff, "Configuration Word", pCpu, 0x2007)
+  {
+  }
 
-  cout << " Can't create a generic 14bit processor\n";
+  enum {
+    FOSC0  = 1<<0,
+    FOSC1  = 1<<1,
+    WDTEN  = 1<<2,
+    PWRTEN = 1<<3
+  };
+
+  virtual void set(gint64 v)
+  {
+    gint64 oldV = getVal();
+
+    Integer::set(v);
+    if (m_pCpu) {
+
+      gint64 diff = oldV ^ v;
+
+      if (diff & WDTEN)
+	m_pCpu->wdt.initialize((v & WDTEN) == WDTEN);
+
+    }
+
+  }
+
+};
 
-  return 0;
 
-}
 //-------------------------------------------------------------------
-_14bit_processor::_14bit_processor(void)
+_14bit_processor::_14bit_processor(const char *_name, const char *_desc)
+  : pic_processor(_name,_desc), intcon(0)
 {
   pc = new Program_Counter();
-  pc->set_trace_command(trace.allocateTraceType(new PCTraceType(this,0,1)));
+  pc->set_trace_command(trace.allocateTraceType(new PCTraceType(this,1)));
 
 }
+
+_14bit_processor::~_14bit_processor()
+{
+  //  delete pc;
+}
+
 //-------------------------------------------------------------------
 //
 // 
@@ -95,6 +129,27 @@ void _14bit_processor::option_new_bits_6_7(unsigned int bits)
   cout << "14bit, option bits 6 and/or 7 changed\n";
 }
 
+
+//------------------------------------------------------------------
+// Fetch the rom contents at a particular address.
+unsigned int _14bit_processor::get_program_memory_at_address(unsigned int address)
+{
+  unsigned int uIndex = map_pm_address2index(address);
+
+  
+  if (uIndex < program_memory_size())
+    return  program_memory[uIndex] ? program_memory[uIndex]->get_opcode() : 0xffffffff;
+
+  return get_config_word(address);
+}
+
+//-------------------------------------------------------------------
+void _14bit_processor::create_config_memory()
+{
+  m_configMemory = new ConfigMemory *[1];
+  *m_configMemory = new Generic14bitConfigWord(this);
+}
+
 #if 0
 //-------------------------------------------------------------------
 class PortBSink;
@@ -160,7 +215,8 @@ void PortBSink::setPullups(bool new_pullupState)
 #endif
 
 //-------------------------------------------------------------------
-Pic14Bit::Pic14Bit()
+Pic14Bit::Pic14Bit(const char *_name, const char *_desc)
+  : _14bit_processor(_name,_desc)
 {
   m_porta = new PicPortRegister("porta",8,0x1f);
   m_trisa = new PicTrisRegister("trisa",m_porta);
@@ -170,9 +226,13 @@ Pic14Bit::Pic14Bit()
 
   m_portb = new PicPortBRegister("portb",8,0xff);
   m_trisb = new PicTrisRegister("trisb",m_portb);
-
 }
 
+//-------------------------------------------------------------------
+Pic14Bit::~Pic14Bit()
+{
+
+}
 //-------------------------------------------------------------------
 void Pic14Bit::create_symbols(void)
 {
@@ -181,11 +241,6 @@ void Pic14Bit::create_symbols(void)
   // add a special symbol for W
   symbol_table.add_w(W);
 
-  symbol_table.add_register(m_portb);
-  symbol_table.add_register(m_porta);
-  symbol_table.add_register(m_trisb);
-  symbol_table.add_register(m_trisa);
-
 }
 
 //-------------------------------------------------------------------
