@@ -1,5 +1,6 @@
 /*
    Copyright (C) 1998 T. Scott Dattalo
+   Copyright (C) 2006 Roy R Rankin
 
 This file is part of gpsim.
 
@@ -36,7 +37,7 @@ Boston, MA 02111-1307, USA.  */
 //#define DEBUG
 
 #if defined(DEBUG)
-#define Dprintf(arg) {printf("%s:%d",__FILE__,__LINE__); printf arg; }
+#define Dprintf(arg) {printf("%s:%d ",__FILE__,__LINE__); printf arg; }
 #else
 #define Dprintf(arg) {}
 #endif
@@ -79,8 +80,6 @@ void CCPRL::capture_tmr()
   //trace.register_write(ccprh->address,ccprh->value.get());
   ccprh->value.put(tmrl->tmrh->value.get());
 
-  tmrl->pir_set->set_ccpif();
-
   int c = value.get() + 256*ccprh->value.get();
   if(verbose & 4)
     cout << "CCPRL captured: " << c << '\n';
@@ -114,8 +113,12 @@ void CCPRL::start_pwm_mode()
   //cout << "CCPRL: starting pwm mode\n";
 
   ccprh->pwm_mode = 1;
+}
+void CCPRL::stop_pwm_mode()
+{
+  //cout << "CCPRL: stopping pwm mode\n"; 
 
-
+  ccprh->pwm_mode = 0;
 }
 
 //--------------------------------------------------
@@ -146,6 +149,14 @@ CCPRH::CCPRH()
   pwm_value = 0;
 }
 
+// put_value allows PWM code to put data
+void CCPRH::put_value(unsigned int new_value)
+{
+      trace.raw(write_trace.get() | value.get());
+      //trace.register_write(address,value.get());
+
+      value.put(new_value);
+}
 void CCPRH::put(unsigned int new_value)
 {
 
@@ -153,10 +164,7 @@ void CCPRH::put(unsigned int new_value)
 
   if(pwm_mode == 0)   // In pwm_mode, CCPRH is a read-only register.
     {
-      trace.raw(write_trace.get() | value.get());
-      //trace.register_write(address,value.get());
-
-      value.put(new_value);
+      put_value(new_value);
 
       if(ccprl && ccprl->tmrl && ccprl->tmrl->compare_mode)
 	ccprl->start_compare_mode();   // Actually, re-start with new capture value.
@@ -168,11 +176,9 @@ unsigned int CCPRH::get()
 {
   //cout << "CCPRH get\n";
 
-  unsigned int read_value =  (pwm_mode) ? (pwm_value >>2) : value.get();
-
   trace.raw(read_trace.get() | value.get());
   //trace.register_read(address, read_value);
-  return read_value;
+  return value.get();
 }
 
 //--------------------------------------------------
@@ -272,6 +278,7 @@ void CCPCON::new_edge(unsigned int level)
     case CAP_FALLING_EDGE:
       if (level == 0 && ccprl) {
 	ccprl->capture_tmr();
+	pir_set->set_ccpif();
 	Dprintf(("--CCPCON caught falling edge\n"));
       }
       break;
@@ -279,14 +286,15 @@ void CCPCON::new_edge(unsigned int level)
     case CAP_RISING_EDGE:
       if (level && ccprl) {
 	ccprl->capture_tmr();
+	pir_set->set_ccpif();
 	Dprintf(("--CCPCON caught rising edge\n"));
       }
       break;
 
     case CAP_RISING_EDGE4:
       if (level && --edges <= 0) {
-	if (ccprl)
-	  ccprl->capture_tmr();
+	ccprl->capture_tmr();
+	pir_set->set_ccpif();
 	edges = 4;
 	Dprintf(("--CCPCON caught 4th rising edge\n"));
       }
@@ -296,8 +304,8 @@ void CCPCON::new_edge(unsigned int level)
 
     case CAP_RISING_EDGE16:
       if (level && --edges <= 0) {
-	if (ccprl)
-	  ccprl->capture_tmr();
+	ccprl->capture_tmr();
+	pir_set->set_ccpif();
 	edges = 16;
 	Dprintf(("--CCPCON caught 4th rising edge\n"));
       }
@@ -387,7 +395,6 @@ void CCPCON::pwm_match(int level)
   if( (value.get() & PWM0) == PWM0) {
 
     m_cOutputState = level ? '1' : '0';
-    m_PinModule->updatePinModule();
 
     // if the level is 'high', then tmr2 == pr2 and the pwm cycle
     // is starting over. In which case, we need to update the duty
@@ -397,7 +404,11 @@ void CCPCON::pwm_match(int level)
     if(level) {
       ccprl->ccprh->pwm_value = ((value.get()>>4) & 3) | 4*ccprl->value.get();
       tmr2->pwm_dc(ccprl->ccprh->pwm_value, address);
+      ccprl->ccprh->put_value(ccprl->value.get());
+      if (!ccprl->ccprh->pwm_value)  // if duty cycle == 0 output stays low
+	m_cOutputState = '0';
     }
+    m_PinModule->updatePinModule();
 
     //cout << "iopin should change\n";
   }  else
@@ -424,7 +435,10 @@ void CCPCON::put(unsigned int new_value)
     case ALL_OFF2:
     case ALL_OFF3:
       if (ccprl)
+      {
 	ccprl->stop_compare_mode();
+	ccprl->stop_pwm_mode();
+      }
       if (tmr2)
 	tmr2->stop_pwm(address);
       m_bInputEnabled = false;
@@ -434,6 +448,7 @@ void CCPCON::put(unsigned int new_value)
     case CAP_RISING_EDGE:
       edges = 0;
       ccprl->stop_compare_mode();
+      ccprl->stop_pwm_mode();
       tmr2->stop_pwm(address);
       m_bInputEnabled = true;
       m_bOutputEnabled = false;
@@ -442,6 +457,7 @@ void CCPCON::put(unsigned int new_value)
     case CAP_RISING_EDGE4:
       edges &= 3;
       ccprl->stop_compare_mode();
+      ccprl->stop_pwm_mode();
       tmr2->stop_pwm(address);
       m_bInputEnabled = true;
       m_bOutputEnabled = false;
@@ -449,6 +465,7 @@ void CCPCON::put(unsigned int new_value)
 
     case CAP_RISING_EDGE16:
       ccprl->stop_compare_mode();
+      ccprl->stop_pwm_mode();
       tmr2->stop_pwm(address);
       m_bInputEnabled = true;
       m_bOutputEnabled = false;
@@ -460,6 +477,7 @@ void CCPCON::put(unsigned int new_value)
     case COM_TRIGGER:
       ccprl->tmrl->ccpcon = this;
       ccprl->start_compare_mode();
+      ccprl->stop_pwm_mode();
       tmr2->stop_pwm(address);
 
       if(adcon0)
@@ -475,8 +493,10 @@ void CCPCON::put(unsigned int new_value)
     case PWM2:
     case PWM3:
       ccprl->stop_compare_mode();
+/* do this when TMR2 == PR2
       ccprl->start_pwm_mode();
       tmr2->pwm_dc( ccprl->ccprh->pwm_value, address);
+*/
       m_bInputEnabled = false;
       m_bOutputEnabled = true;
       m_cOutputState = '0';
@@ -602,7 +622,7 @@ unsigned int TMRH::get_value()
 // member functions for the TMRL base class
 //--------------------------------------------------
 TMRL::TMRL()
-  : m_cState('?'), m_bExtClkEnabled(false)
+  : m_cState('?'), m_bExtClkEnabled(false), m_Interrupt(0)
 {
 
   value.put(0);
@@ -615,7 +635,6 @@ TMRL::TMRL()
 
   tmrh    = 0;
   t1con   = 0;
-  pir_set = 0;
   ccpcon  = 0;
   new_name("TMRL");
 
@@ -623,6 +642,8 @@ TMRL::TMRL()
 
 void TMRL::setIOpin(PinModule *extClkSource)
 {
+  Dprintf(("TMRL::setIOpin\n"));
+
   if (extClkSource)
     extClkSource->addSink(this);
 }
@@ -637,10 +658,22 @@ void TMRL::setSinkState(char new3State)
   }
 }
 
+//------------------------------------------------------------
+// setInterruptSource()
+//
+// This Timer can be an interrupt source. When the interrupt
+// needs to be generated, then the InterruptSource object will
+// direct the interrupt to where it needs to go (usually this
+// is the Peripheral Interrupt Register).
+
+void TMRL::setInterruptSource(InterruptSource *_int)
+{
+  m_Interrupt = _int;
+}
+
 void TMRL::increment()
 {
-  if(verbose & 4)
-    cout << "TMRL increment because of external clock ";
+  Dprintf(("TMRL increment because of external clock\n"));
 
   if(--prescale_counter == 0) {
     prescale_counter = prescale;
@@ -656,8 +689,8 @@ void TMRL::increment()
 
     tmrh->value.put((value_16bit >> 8) & 0xff);
     value.put(value_16bit & 0xff);
-    if(value_16bit == 0)
-      pir_set->set_tmr1if();
+    if(value_16bit == 0 && m_Interrupt)
+      m_Interrupt->Trigger();
   }
 
 }
@@ -666,8 +699,9 @@ void TMRL::on_or_off(int new_state)
 {
 
   if(new_state) {
-    if(verbose & 0x4)
-      cout << "TMR1 is being turned on\n";
+
+    Dprintf(("TMR1 is being turned on\n"));
+
     // turn on the timer
 
     // Effective last cycle
@@ -678,8 +712,8 @@ void TMRL::on_or_off(int new_state)
     update();
   }
   else {
-    if(verbose & 0x4)
-      cout << "TMR1 is being turned off\n";
+
+    Dprintf(("TMR1 is being turned off\n"));
 
     // turn off the timer and save the current value
     current_value();
@@ -697,8 +731,7 @@ void TMRL::on_or_off(int new_state)
 void TMRL::update()
 {
 
-  if(verbose & 0x4)
-    cout << "TMR1 update "  << hex << get_cycles().value << '\n';
+  Dprintf(("TMR1 update 0x%llx\n",get_cycles().value));
 
   if(t1con->get_tmr1on()) {
 
@@ -891,7 +924,8 @@ void TMRL::callback()
       // The break was due to a roll-over
 
       //cout<<"TMRL rollover: " << hex << cycles.value.lo << '\n';
-      pir_set->set_tmr1if();
+      if (m_Interrupt)
+	m_Interrupt->Trigger();
 
       // Reset the timer to 0.
 
@@ -933,9 +967,9 @@ void PR2::put(unsigned int new_value)
 
   if(value.get() != new_value)
     {
-      value.put(new_value);
       if (tmr2)
-	tmr2->new_pr2();
+	tmr2->new_pr2(new_value);
+      value.put(new_value);
     }
   else
     value.put(new_value);
@@ -957,11 +991,19 @@ T2CON::T2CON()
 void T2CON::put(unsigned int new_value)
 {
   trace.raw(write_trace.get() | value.get());
-  //trace.register_write(address,value.get());
+
+  unsigned int diff = value.get() ^ new_value;
   value.put(new_value);
-  if (tmr2)
+
+
+  if (tmr2) {
+    
     tmr2->new_pre_post_scale();
 
+    if( diff & TMR2ON)
+      tmr2->on_or_off(value.get() & TMR2ON);
+
+  }
 }
 
 
@@ -975,7 +1017,6 @@ TMR2::TMR2()
   update_state = TMR2_PWM1_UPDATE | TMR2_PWM2_UPDATE | TMR2_PR2_UPDATE;
   pwm_mode = 0;
   value.put(0);
-  synchronized_cycle=0;
   future_cycle = 0;
   prescale=1;
   new_name("TMR2");
@@ -996,6 +1037,33 @@ void TMR2::start()
 
 }
 
+
+void TMR2::on_or_off(int new_state)
+{
+
+  if(new_state) {
+
+    Dprintf(("TMR2 is being turned on\n"));
+
+    // turn on the timer
+
+    // Effective last cycle
+    // Compute the "effective last cycle", i.e. the cycle
+    // at which TMR2 was last 0 had it always been counting.
+
+    last_cycle = get_cycles().value - value.get()*prescale;
+    update();
+  }
+  else {
+
+    Dprintf(("TMR2 is being turned off\n"));
+
+    // turn off the timer and save the current value
+    current_value();
+  }
+
+}
+
 //
 // pwm_dc - 
 //
@@ -1011,11 +1079,11 @@ void TMR2::pwm_dc(unsigned int dc, unsigned int ccp_address)
 
       duty_cycle1 = dc;
 
-      // Update the cycle break iff this is the first time to go into pwm mode
+      // Update the cycle break if this is the first time to go into pwm mode
       if( (pwm_mode & TMR2_PWM1_UPDATE) == 0)
 	{
 	  pwm_mode |= TMR2_PWM1_UPDATE;
-	  update();
+//wait for next TMR2 update	  update();	
 	}
     }
   else if(ccp_address == ccp2con->address)
@@ -1024,12 +1092,12 @@ void TMR2::pwm_dc(unsigned int dc, unsigned int ccp_address)
 
       duty_cycle2 = dc;
 
-      // Update the cycle break iff this is the first time to go into pwm mode
+      // Update the cycle break if this is the first time to go into pwm mode
       if( (pwm_mode & TMR2_PWM2_UPDATE) == 0)
 	{
 	  pwm_mode |= TMR2_PWM2_UPDATE;
 
-	  update();
+//wait for next TMR2 update	  update();	
 	}
     }
   else
@@ -1054,15 +1122,20 @@ void TMR2::stop_pwm(unsigned int ccp_address)
     {
       // cout << "TMR2:  stopping pwm mode with ccp1.\n";
       pwm_mode &= ~TMR2_PWM1_UPDATE;
+      if(last_update & TMR2_PWM1_UPDATE)
+         update_state &= (~TMR2_PWM1_UPDATE);
+
     }
   else if(ccp_address == ccp2con->address)
     {
       // cout << "TMR2:  stopping pwm mode with ccp2.\n";
       pwm_mode &= ~TMR2_PWM2_UPDATE;
+      if(last_update & TMR2_PWM2_UPDATE)
+         update_state &= (~TMR2_PWM2_UPDATE);
     }
 
-  if(pwm_mode ^ old_pwm)
-    update();
+  if((pwm_mode ^ old_pwm) && future_cycle && t2con->get_tmr2on())
+    update(update_state);
 
 }
 
@@ -1078,86 +1151,92 @@ void TMR2::stop_pwm(unsigned int ccp_address)
 void TMR2::update(int ut)
 {
 
-  //cout << "TMR2 update. cpu cycle " << cycles.value <<'\n';
+  //cout << "TMR2 update. cpu cycle " << hex << cycles.value <<'\n';
 
-  if(t2con->get_tmr2on())
-    {
-      if(future_cycle)
+  if(t2con->get_tmr2on()) {
+
+    if(future_cycle) {
+
+      // If TMR2 is enabled (i.e. counting) then 'future_cycle' is non-zero,
+      // which means there's a cycle break point set on TMR2 that needs to
+      // be moved to a new cycle.
+
+      current_value();
+
+      // Assume that we are not in pwm mode (and hence the next break will
+      // be due to tmr2 matching pr2)
+
+      break_value = 1 + pr2->value.get();
+      guint64 fc = get_cycles().value + (break_value - value.get()) * prescale;
+
+      last_update = TMR2_PR2_UPDATE;
+
+      if (pwm_mode)
+      {
+          break_value <<= 2;    // now pwm value
+          fc = last_cycle + break_value * prescale;
+      }
+
+      if(pwm_mode & ut & TMR2_PWM1_UPDATE) {
+
+	// We are in pwm mode... So let's see what happens first: a pr2 compare
+	// or a duty cycle compare. (recall, the duty cycle is really 10-bits)
+
+	if( (duty_cycle1 > (value.get()*4) ) && (duty_cycle1 < break_value))
+	  {
+	    //cout << "TMR2:PWM1 update\n";
+	    last_update = TMR2_PWM1_UPDATE;
+            fc = last_cycle + duty_cycle1 * prescale;
+	  }
+      }
+
+      if(pwm_mode & ut & TMR2_PWM2_UPDATE)
 	{
-	  // If TMR2 is enabled (i.e. counting) then 'future_cycle' is non-zero,
-	  // which means there's a cycle break point set on TMR2 that needs to
-	  // be moved to a new cycle.
-
-	  current_value();
-
-	  // Assume that we are not in pwm mode (and hence the next break will
-	  // be due to tmr2 matching pr2)
-
-	  break_value = (1 + pr2->value.get()) << 2;
-	  unsigned int pwm_break_value = break_value;
-
-	  last_update = TMR2_PR2_UPDATE;
-
-	  if(pwm_mode & ut & TMR2_PWM1_UPDATE)
-	    {
-	      // We are in pwm mode... So let's see what happens first: a pr2 compare
-	      // or a duty cycle compare. (recall, the duty cycle is really 10-bits)
-
-	      if( (duty_cycle1 > (value.get()*4*prescale) ) && (duty_cycle1 < break_value))
-		{
-		  pwm_break_value = duty_cycle1;
-		  last_update = TMR2_PWM1_UPDATE;
-		  //cout << "TMR2:PWM1 update\n";
-		}
-	    }
-
-	  if(pwm_mode & ut & TMR2_PWM2_UPDATE)
-	    {
-	      // We are in pwm mode... So let's see what happens first: a pr2 compare
-	      // or a duty cycle compare. (recall, the duty cycle is really 10-bits)
-
-	      if( (duty_cycle2 > (value.get()*4*prescale) ) && (duty_cycle2 < break_value))
-		{
-		  pwm_break_value = duty_cycle2;
-		  last_update = TMR2_PWM2_UPDATE;
-		  //cout << "TMR2:PWM2 update\n";
-		}
-	    }
-
-
-	  // If TMR2 is configured for pwm'ing,
-	  // make sure that the new break cycle is beyond the current one (this
-	  // is necessary because the 'duty_cycle' may be larger than the 'period'.)
-	  //cout << "TMR2: break_value " <<hex<<break_value << " pwm_break_value " << pwm_break_value <<'\n';
-	  if(pwm_break_value >= break_value)
-	    {
-	      last_update = TMR2_PR2_UPDATE;
-	      update_state = TMR2_PWM1_UPDATE | TMR2_PWM2_UPDATE | TMR2_PR2_UPDATE;
-	      last_cycle = get_cycles().value;
-	    }
-	  else
-	    break_value = pwm_break_value;
-
-	  guint64 fc = last_cycle + ((break_value>>2) - value.get())  * prescale;
-
-	  if(fc <= future_cycle)
-	    {
-	      cout << "TMR2: update BUG! future_cycle is screwed\n";
-	    }
-
-	  //cout << "TMR2: update new break at cycle "<<hex<<fc<<'\n';
-	  get_cycles().reassign_break(future_cycle, fc, this);
-
-	  future_cycle = fc;
-
+	  // We are in pwm mode... So let's see what happens first: a pr2 compare
+	  // or a duty cycle compare. (recall, the duty cycle is really 10-bits)
 
+	  if( (duty_cycle2 > (value.get()*4) ) && (duty_cycle2 < break_value))
+	  {
+	      //cout << "TMR2:PWM2 update\n";
+                                                                                
+            if (last_update == TMR2_PWM1_UPDATE)
+            {
+                // set break for first duty cycle change
+                if (duty_cycle2 < duty_cycle1)
+                {
+                    fc = last_cycle + duty_cycle2 * prescale;
+                    last_update = TMR2_PWM2_UPDATE;
+                }
+                else if (duty_cycle2 == duty_cycle1)
+                    last_update |= TMR2_PWM2_UPDATE;
+            }
+            else
+            {
+                last_update = TMR2_PWM2_UPDATE;
+                fc = last_cycle + duty_cycle2 * prescale;
+            }
+                                                                                
 	}
-      else
-	{
-	  cout << "TMR2 BUG!! tmr2 is on but has no cycle_break set on it\n";
-	}
+      }
+
+      if(fc < future_cycle)
+        cout << "TMR2: update note: new breakpoint=" << hex << fc <<
+           " before old breakpoint " << future_cycle << endl;
+
+      if (fc != future_cycle)
+      {
+          // cout << "TMR2: update new break at cycle "<<hex<<fc<<'\n';
+          get_cycles().reassign_break(future_cycle, fc, this);
+          future_cycle = fc;
+      }
 
     }
+    else
+      {
+	cout << "TMR2 BUG!! tmr2 is on but has no cycle_break set on it\n";
+      }
+
+  }
   else
     {
       //cout << "TMR2 is not running (no update occurred)\n";
@@ -1166,6 +1245,9 @@ void TMR2::update(int ut)
 }
 void TMR2::put(unsigned int new_value)
 {
+  current_value();
+
+  unsigned int old_value = value.get();
 
 
   trace.raw(write_trace.get() | value.get());
@@ -1179,15 +1261,62 @@ void TMR2::put(unsigned int new_value)
       // which means there's a cycle break point set on TMR2 that needs to
       // be moved to a new cycle.
 
-      last_cycle = get_cycles().value;
-      guint64 fc = last_cycle + ((pr2->value.get() - value.get()) & 0xff) * prescale;
+      guint64 current_cycle = get_cycles().value;
+      unsigned int delta = (future_cycle - last_cycle);
+      int shift = (new_value - old_value) * prescale;
 
-      get_cycles().reassign_break(future_cycle, fc, this);
+      if (pwm_mode)
+	shift <<= 2;
+  
+      // set cycle when tmr2 would have been zero
 
-      future_cycle = fc;
+      last_cycle = current_cycle - shift;
+      unsigned int now = (current_cycle - last_cycle);
+      
 
-      // 'clear' the post scale counter. (I've actually implemented the post scale counter
-      // as a count-down counter, so 'clearing it' means resetting it to the starting point.
+      guint64 fc;
+
+      /*
+	Three possible cases
+	   1> TMR2 is still before the next break point.
+		Adjust the breakpoint to ocurr at correct TMR2 value
+	   2> TMR2 is now greater the PR2
+		Assume TMR2 must count up to 0xff, roll-over and then
+		we are back in business. High CCP outputs stay high.
+	   3> TMR2 is now less than PR2 but greater than a CCP duty cycle point.
+		The CCP output stays as the duty cycle comparator does not 
+		match on this cycle.
+      */
+
+      if (now < delta) // easy case, just shift break.
+      {
+          fc = last_cycle + delta;
+          get_cycles().reassign_break(future_cycle, fc, this);
+          future_cycle = fc;
+      }
+      else if (now >= break_value * prescale)  // TMR2 now greater than PR2
+      {
+        // set break to when TMR2 will overflow
+	last_update |= TMR2_WRAP;
+	if (pwm_mode)
+	    fc = last_cycle + (0x100 * prescale << 2);
+	else
+	    fc = last_cycle + 0x100 * prescale;
+        get_cycles().reassign_break(future_cycle, fc, this);
+        future_cycle = fc;
+      }
+      else	// new break < PR2 but > duty cycle break
+      {
+          update(update_state);
+      }
+
+
+
+     /* 
+	'clear' the post scale counter. (I've actually implemented the 
+	post scale counter as a count-down counter, so 'clearing it' 
+	means resetting it to the starting point.
+     */
       if (t2con)
 	post_scale = t2con->get_post_scale();
     }
@@ -1198,10 +1327,6 @@ unsigned int TMR2::get()
 
   if(t2con->get_tmr2on())
     {
-      ///int new_value = (cycles.value - last_cycle)/ prescale;
-
-      ///value = new_value;
-
       current_value();
     }
 
@@ -1216,10 +1341,6 @@ unsigned int TMR2::get_value()
 
   if(t2con->get_tmr2on())
     {
-      ///int new_value = (cycles.value - last_cycle)/ prescale;
-
-      ///value = new_value;
-
       current_value();
     }
 
@@ -1229,7 +1350,7 @@ unsigned int TMR2::get_value()
 void TMR2::new_pre_post_scale()
 {
 
-  //cout << "T2CON was written to, so update TMR2\n";
+  //cout << "T2CON was written to, so update TMR2 " << t2con->get_tmr2on() << "\n";
 
   if(!t2con->get_tmr2on()) {
     // TMR2 is not on. If has just been turned off, clear the callback breakpoint.
@@ -1237,10 +1358,14 @@ void TMR2::new_pre_post_scale()
     if(future_cycle) {
       get_cycles().clear_break(this);
       future_cycle = 0;
-      return;
     }
+    return;
   }
 
+  unsigned int old_prescale = prescale;
+  prescale = t2con->get_pre_scale();
+  post_scale = t2con->get_post_scale();
+
   if(future_cycle)
     {
       // If TMR2 is enabled (i.e. counting) then 'future_cycle' is non-zero,
@@ -1249,93 +1374,117 @@ void TMR2::new_pre_post_scale()
 
       // Get the current value of TMR2
       ///value = (cycles.value - last_cycle)/prescale;
+
       current_value();
 
       //cout << "cycles " << cycles.value.lo  << " old prescale " << prescale;
 
-      prescale = t2con->get_pre_scale();
 
       //cout << " prescale " << prescale;
 
-      // Now compute the 'last_cycle' as though if TMR2 had been running on the 
-      // new prescale all along. Recall, 'last_cycle' records the value of the cpu's
-      // cycle counter when TMR2 last rolled over.
 
-      last_cycle = get_cycles().value - value.get() * prescale;
-      //cout << " effective last_cycle " << last_cycle << '\n';
+      if (prescale != old_prescale)	// prescaler value change
+      {
+	// togo is number of cycles to next callback based on new prescaler.
+	guint64 togo = (future_cycle - get_cycles().value) * prescale / old_prescale;
 
-      //cout << "tmr2's current value " << value << '\n';
+	if (!togo)	// I am not sure this can happen RRR
+	    callback();
+	else
+	{
+	   guint64 fc = togo + get_cycles().value;
 
-      guint64 fc = get_cycles().value;
-
-      if(pr2->value.get() == value.get())
-	fc += 0x100 * prescale;
-      else
-	fc +=  ((pr2->value.get() - value.get()) & 0xff) * prescale;
-
-      //cout << "moving break from " << future_cycle << " to " << fc << '\n';
-
-      get_cycles().reassign_break(future_cycle, fc, this);
-
-      future_cycle = fc;
+          get_cycles().reassign_break(future_cycle, fc, this);
+          future_cycle = fc;
+	}
+      }
     }
   else
     {
       //cout << "TMR2 was off, but now it's on.\n";
 
-      prescale = t2con->get_pre_scale();
-      if(pr2->value.get() == value.get())
-	future_cycle = 0x100 * prescale;
+      if (value.get() == pr2->value.get()) // TMR2 == PR2
+      {
+        future_cycle = get_cycles().value;
+        get_cycles().set_break(future_cycle, this);
+	callback();
+      }
+      else if (value.get() > pr2->value.get()) // TMR2 > PR2
+      {
+	cout << "Warning TMR2 turned on with TMR2 greater than PR2\n";
+	// this will cause TMR2 to wrap
+        future_cycle  = get_cycles().value + 
+		(1 + pr2->value.get() + (0x100 -  value.get())) * prescale;
+        get_cycles().set_break(future_cycle, this);
+      }
       else
-	future_cycle =  ((pr2->value.get() - value.get()) & 0xff) * prescale;
-
-      last_cycle = get_cycles().value;
-      future_cycle += get_cycles().value;
-      get_cycles().set_break(future_cycle, this);
-    }
-
-  post_scale = t2con->get_post_scale();
+      {
+          future_cycle = get_cycles().value + 1;
+          get_cycles().set_break(future_cycle, this);
+	  last_cycle = get_cycles().value - value.get();
+          update(update_state);
+      }
+  }
 
 }
 
-void TMR2::new_pr2()
+void TMR2::new_pr2(unsigned int new_value)
 {
 
   if(t2con->get_tmr2on())
     {
-      //update the tmr2 break point...
-      ///      value = (cycles.value - last_cycle)/ prescale;
-      current_value();
 
-      // Get the current value of the prescale counter (because
-      // writing to pr2 doesn't affect the pre/post scale counters).
+      unsigned int cur_break = (future_cycle - last_cycle)/prescale;
+      unsigned int new_break = (pwm_mode)? (1 + new_value) << 2 : 1 + new_value;
+      unsigned int now_cycle = (get_cycles().value - last_cycle) / prescale;
 
-      guint64 curr_prescale = value.get() * prescale - (get_cycles().value - last_cycle);
+      guint64 fc = last_cycle;
 
-      guint64 fc = get_cycles().value + curr_prescale;
+      /*
+	PR2 change casses
 
-      if(pr2->value.get() == value.get())
-	{  // May wanta ignore the == case and instead allow the cycle break handle it...
-	  fc += 0x100 * prescale;
-	  last_cycle += 0x100 * prescale;
-	}
-      else
-	fc +=  ((pr2->value.get() - value.get()) & 0xff) * prescale;
+	1> TMR2 greater than new PR2
+		TMR2 wraps through 0xff
+	2> New PR2 breakpoint less than current breakpoint or
+	   current break point due to PR2
+		Change breakpoint to new value based on PR2
+	3> Other breakpoint < PR2 breakpoint
+		No need to do anything.
+     */
 
-      get_cycles().reassign_break(future_cycle, fc, this);
 
-      future_cycle = fc;
+      if (now_cycle > new_break)	// TMR2 > PR2 do wrap
+      {
+        // set break to when TMR2 will overflow
+	last_update |= TMR2_WRAP;
+	if (pwm_mode)
+	    fc += (0x100 * prescale << 2);
+	else
+	    fc += 0x100 * prescale;
+        get_cycles().reassign_break(future_cycle, fc, this);
+        future_cycle = fc;
+      }
+      else if (cur_break == break_value ||	// breakpoint due to pr2
+	       new_break < cur_break)		// new break less than current
+      {
+	fc += new_break * prescale;
+        get_cycles().reassign_break(future_cycle, fc, this);
+        future_cycle = fc;
+      }
     }
-
-
 }
 
 void TMR2::current_value()
 {
-  value.put((unsigned int)((future_cycle - get_cycles().value)/ prescale ));
+  unsigned int tmr2_val = (get_cycles().value - last_cycle)/ prescale;
 
-  if(value.get()>0xff)
-    cout << "TMR2 BUG!! value = " << value.get() << " which is greater than 0xff\n";
+  if (pwm_mode)
+       tmr2_val >>= 2;
+
+  value.put(tmr2_val & 0xff);
+
+  if(tmr2_val > 0x100)	// Can get to 0x100 during transition
+   cout << "TMR2 BUG!! value = " << value.get() << " which is greater than 0xff\n";
 }
 
 // TMR2 callback is called when the cycle counter hits the break point that
@@ -1354,21 +1503,30 @@ void TMR2::callback()
 
       // What caused the callback: PR2 match or duty cyle match ?
 
-      //if((pwm_mode) && (break_value == duty_cycle1))
-      if(last_update == TMR2_PWM1_UPDATE)
+      if (last_update & TMR2_WRAP) // TMR2 > PR2 
+      {
+	last_update &= ~TMR2_WRAP;
+	// This (implicitly) resets the timer to zero:
+	last_cycle = get_cycles().value;
+      }
+      else if( last_update & (TMR2_PWM1_UPDATE | TMR2_PWM2_UPDATE))
+      {
+
+        if(last_update & TMR2_PWM1_UPDATE)
 	{
 	  // duty cycle match
 	  //cout << "TMR2: duty cycle match for pwm1 \n";
 	  update_state &= (~TMR2_PWM1_UPDATE);
 	  ccp1con->pwm_match(0);
 	}
-      else if(last_update == TMR2_PWM2_UPDATE)
+        if(last_update & TMR2_PWM2_UPDATE)
 	{
 	  // duty cycle match
 	  //cout << "TMR2: duty cycle match for pwm2 \n";
 	  update_state &= (~TMR2_PWM2_UPDATE);
 	  ccp2con->pwm_match(0);
 	}
+      }
       else
 	{
 	  // matches PR2
@@ -1378,11 +1536,14 @@ void TMR2::callback()
 	  // This (implicitly) resets the timer to zero:
 	  last_cycle = get_cycles().value;
 
-	  if(pwm_mode & TMR2_PWM1_UPDATE)
-	    ccp1con->pwm_match(1);
+         if (ssp_module)
+               ssp_module->tmr2_clock();
 
-	  if(pwm_mode & TMR2_PWM2_UPDATE)
-	    ccp2con->pwm_match(1);
+          if ((ccp1con->value.get() & CCPCON::PWM0) == CCPCON::PWM0)
+               ccp1con->pwm_match(1);
+
+          if ((ccp2con->value.get() & CCPCON::PWM0) == CCPCON::PWM0)
+               ccp2con->pwm_match(1);
 
 	  if(--post_scale < 0)
 	    {
