@@ -20,8 +20,8 @@ Boston, MA 02111-1307, USA.  */
 
 // T. Scott Dattalo 16bit core routines
 
-#include <iostream.h>
-#include <iomanip.h>
+#include <iostream>
+#include <iomanip>
 #include <string>
 #include <list>
 
@@ -38,38 +38,6 @@ Boston, MA 02111-1307, USA.  */
 #include "p16x8x.h"
 #include "p16x6x.h"
 #include "p16x7x.h"
-#include "p12x.h"
-
-
-// This is a whole cleaner/simpler way to decode an instruction
-// set... This technique will eventually be used to decode
-// the other instruction sets (12 and 14-bit) (and the "don't
-// fix what ain't broken" rule will be violated in the 
-// process). In addition this will be used to dynamically
-// instantiate processors.
-//
-// The way it works is the 'instruction_constructor' structure
-// contains three pieces of info for each instruction:
-//   inst_mask - a bit mask indicating which bits uniquely
-//               identify an instruction
-//   opcode    - What those unique bits should be
-//   inst_constructor - A pointer to the static member function
-//                      'construct' in the instruction class.
-//
-// An instruction is decoded by finding a matching entry in
-// the instruction_constructor array. A match is defined to
-// be:
-//    inst_mask & passed_opcode == opcode
-// which means that the opcode that is passed to the decoder
-// is ANDed with the instruction mask bits and compared to
-// the base bits of the opcode. If this test passes, then the
-// 'inst_constructor' will be called.
-
-struct instruction_constructor {
-  unsigned int inst_mask;
-  unsigned int opcode;
-  instruction * (*inst_constructor) (pic_processor *cpu, unsigned int inst);
-};
 
 
 /* PIC 16-bit instruction set */
@@ -77,6 +45,12 @@ struct instruction_constructor {
 
 struct instruction_constructor op_18cxx[] = {
 
+  { 0xfe00,  0xe800,  ADDFSR::construct },
+  { 0xffff,  0x0014,  CALLW::construct },
+  { 0xff00,  0xeb00,  MOVSF::construct },
+  { 0xff00,  0xea00,  PUSHL::construct },
+  { 0xff00,  0xeb00,  MOVSF::construct },
+
   { 0xff00,  0x0f00,  ADDLW16::construct },
   { 0xfc00,  0x2400,  ADDWF16::construct },
   { 0xfc00,  0x2000,  ADDWFC::construct },
@@ -118,7 +92,8 @@ struct instruction_constructor op_18cxx[] = {
   { 0xf000,  0xc000,  MOVFF::construct },
   { 0xff00,  0x0100,  MOVLB::construct },
   { 0xff00,  0x0e00,  MOVLW::construct },
-  { 0xfe00,  0x6e00,  MOVWF16::construct },
+  { 0xff00,  0x6f00,  MOVWF16::construct },
+  { 0xff00,  0x6e00,  MOVWF16a::construct },
   { 0xff00,  0x0d00,  MULLW::construct },
   { 0xfe00,  0x0200,  MULWF::construct },
   { 0xfe00,  0x6c00,  NEGF::construct },
@@ -218,20 +193,16 @@ const int NUM_OP_17CXX	= sizeof(op_17cxx) / sizeof(op_17cxx[0]);
 instruction * disasm16 (pic_processor *cpu, unsigned int address, unsigned int inst)
 {
   instruction *pi;
-  char buf[50];
 
-  cpu16->current_disasm_address = address;
+  cpu16->setCurrentDisasmAddress(address);
 
-  pi = NULL;
+  pi = 0;
   for(int i =0; i<NUM_OP_18CXX; i++)
     if((op_18cxx[i].inst_mask & inst) == op_18cxx[i].opcode)
       pi = op_18cxx[i].inst_constructor(cpu, inst);
 
-  if(pi == NULL)
+  if(pi == 0)
     pi = invalid_instruction::construct(cpu, inst);
 
-  //  if(pi != NULL)
-  //    cout << hex << "0x" << address << '\t' << pi->name(buf) << '\n';
-
   return (pi);
 }
