@@ -46,6 +46,8 @@ public:
   int destination_index;
   unsigned int absolute_destination_index;
 
+  Branching(Processor *new_cpu, unsigned int new_opcode);
+
   virtual void execute(void){ };
   virtual void debug(void){ };
   virtual char *name(char *,int);
@@ -64,8 +66,10 @@ class multi_word_instruction : public instruction
   unsigned int PMindex;
   bool initialized;
 
+  multi_word_instruction(Processor *new_cpu, unsigned int new_opcode);
+
   virtual int instruction_size(void) { return 2;}
-  virtual INSTRUCTION_TYPES isa(void) {return MULTIWORD_INSTRUCTION;};
+  virtual enum INSTRUCTION_TYPES isa(void) {return MULTIWORD_INSTRUCTION;};
   virtual bool isBase() { return true;}
 
   virtual void initialize(bool init_state) { initialized = init_state; }
@@ -77,12 +81,100 @@ class multi_word_branch : public multi_word_instruction
  public:
   unsigned int destination_index;
 
+  multi_word_branch(Processor *new_cpu, unsigned int new_opcode);
+
   void runtime_initialize(void);
   virtual void execute(void){};
   virtual char *name(char *,int);
 
 };
 
+//---------------------------------------------------------
+class ADDULNK : public instruction 
+{
+
+public:
+  ADDULNK(Processor *new_cpu, unsigned int new_opcode,const char *);
+  virtual bool isBase() { return true;}
+  virtual void execute(void);
+  virtual char *name(char *,int);
+protected:
+  unsigned int m_lit;
+};
+
+//---------------------------------------------------------
+class ADDFSR : public instruction 
+{
+
+public:
+  ADDFSR(Processor *new_cpu, unsigned int new_opcode,const char *);
+  virtual bool isBase() { return true;}
+  virtual void execute(void);
+  virtual char *name(char *,int);
+  static instruction *construct(Processor *new_cpu, unsigned int new_opcode)
+  {
+    if (((new_opcode>>6)&3) == 3)
+      if (new_opcode & 0x100)
+	return new ADDULNK(new_cpu,new_opcode,"subulnk");
+      else
+	return new ADDULNK(new_cpu,new_opcode,"addulnk");
+    if (new_opcode & 0x100)
+      return new ADDFSR(new_cpu,new_opcode,"subfsr");
+    return new ADDFSR(new_cpu,new_opcode,"addfsr");
+  }
+protected:
+  unsigned int m_fsr;
+  unsigned int m_lit;
+  Indirect_Addressing *ia;
+};
+
+//---------------------------------------------------------
+class CALLW : public instruction
+{
+public:
+  CALLW(Processor *new_cpu, unsigned int new_opcode);
+  virtual bool isBase() { return true;}
+  virtual void execute(void);
+  virtual char *name(char *,int);
+  static instruction *construct(Processor *new_cpu, unsigned int new_opcode)
+  {
+    return new CALLW(new_cpu,new_opcode);
+  }
+};
+//---------------------------------------------------------
+class MOVSF : public multi_word_instruction
+{
+public:
+
+  MOVSF(Processor *new_cpu, unsigned int new_opcode);
+  virtual void execute(void);
+  virtual char *name(char *,int);
+  void runtime_initialize(void);
+
+  static instruction *construct(Processor *new_cpu, unsigned int new_opcode)
+    {return new MOVSF(new_cpu,new_opcode);}
+protected:
+  unsigned int source,destination;
+
+};
+
+//---------------------------------------------------------
+class PUSHL : public instruction
+{
+public:
+  PUSHL(Processor *new_cpu, unsigned int new_opcode);
+  virtual bool isBase() { return true;}
+  virtual void execute(void);
+  virtual char *name(char *,int);
+  static instruction *construct(Processor *new_cpu, unsigned int new_opcode)
+  {
+    return new PUSHL(new_cpu,new_opcode);
+  }
+protected:
+  unsigned int m_lit;
+};
+
+
 //---------------------------------------------------------
 class ADDLW16 : public ADDLW 
 {
