@@ -31,8 +31,73 @@ Boston, MA 02111-1307, USA.  */
 #include "stimuli.h"
 #include "symbol.h"
 #include "eeprom.h"
+
+
+//------------------------------------------------------------------------
+// Configuration bits
+//
+// The 16bit-core PIC devices contain configuration memory starting at 
+// address 0x300000.
+//
+
+
+//------------------------------------------------------------------------
+// Config1H - default 
+
+class Config1H : public ConfigMemory 
+{
+#define FOSC0   (1<<0)
+#define FOSC1   (1<<1)
+#define FOSC2   (1<<2)
+#define OSCEN   (1<<5)
+
+#define CONFIG1H_default (OSCEN | FOSC2 | FOSC1 | FOSC0)
+public:
+  Config1H(_16bit_processor *pCpu, unsigned int addr)
+    : ConfigMemory("CONFIG1H", CONFIG1H_default, "Oscillator configuration", pCpu, addr)
+  {
+  }
+};
+
+
+//------------------------------------------------------------------------
+// Config2H - default 
+//  The default Config2H register controls the 18F series WDT. 
+class Config2H : public ConfigMemory 
+{
+#define WDTEN   (1<<0)
+#define WDTPS0  (1<<1)
+#define WDTPS1  (1<<2)
+#define WDTPS2  (1<<3)
+
+#define CONFIG2H_default (WDTEN | WDTPS0 | WDTPS1 | WDTPS2)
+public:
+  Config2H(_16bit_processor *pCpu, unsigned int addr)
+    : ConfigMemory("CONFIG2H", CONFIG2H_default, "WatchDog configuration", pCpu, addr)
+  {
+  }
+  virtual void set(gint64 v)
+  {
+    Integer::set(v);
+    if (m_pCpu)
+      m_pCpu->wdt.initialize((v & WDTEN) == WDTEN);
+  }
+};
+
+
+
+#define PWRTEN  1<<0
+#define BOREN   1<<1
+#define BORV0   1<<2
+#define BORV1   1<<2
+
+#define CCP2MX  1<<0
+
+#define STVREN  1<<0
 //-------------------------------------------------------------------
-_16bit_processor::_16bit_processor()
+_16bit_processor::_16bit_processor(const char *_name, const char *desc)
+  : pic_processor(_name,desc),
+    pir1(0,0), pir2(0,0)
 {
 
   package = 0;
@@ -40,7 +105,7 @@ _16bit_processor::_16bit_processor()
   pll_factor = 2;
 
   pc = new Program_Counter16();
-  pc->set_trace_command(trace.allocateTraceType(new PCTraceType(this,0,1)));
+  pc->set_trace_command(trace.allocateTraceType(new PCTraceType(this,1)));
 
   m_porta = new PicPortRegister("porta",8,0x7f);
   m_trisa = new PicTrisRegister("trisa", m_porta);
@@ -56,6 +121,13 @@ _16bit_processor::_16bit_processor()
   m_latc  = new PicLatchRegister("latc", m_portc);
 
 }
+
+//-------------------------------------------------------------------
+_16bit_processor::~_16bit_processor()
+{
+
+}
+
 //-------------------------------------------------------------------
 pic_processor *_16bit_processor::construct()
 {
@@ -68,7 +140,7 @@ pic_processor *_16bit_processor::construct()
 
   p->create();
   p->create_invalid_registers();
-  p->pic_processor::create_symbols();
+  p->create_symbols();
   p->name_str = "generic 16bit processor";
   symbol_table.add_module(p,p->name_str.c_str());
 
@@ -108,14 +180,14 @@ void _16bit_processor :: create_sfr_map()
   add_sfr_register(&ipr2,	  0xfa2,porv,"ipr2");
 
 
-  usart16.initialize(&pir_set_def,&(*m_portc)[6], &(*m_portc)[7],
-		     new TXREG_16(), new RCREG_16());
+  usart.initialize(&pir_set_def,&(*m_portc)[6], &(*m_portc)[7],
+		   new _TXREG(&usart), new _RCREG(&usart));
 
-  add_sfr_register(&usart16.rcsta,    0xfab,porv,"rcsta");
-  add_sfr_register(&usart16.txsta,    0xfac,RegisterValue(0x02,0),"txsta");
-  add_sfr_register(usart16.txreg,     0xfad,porv,"txreg");
-  add_sfr_register(usart16.rcreg,     0xfae,porv,"rcreg");
-  add_sfr_register(&usart16.spbrg,    0xfaf,porv,"spbrg");
+  add_sfr_register(&usart.rcsta,    0xfab,porv,"rcsta");
+  add_sfr_register(&usart.txsta,    0xfac,RegisterValue(0x02,0),"txsta");
+  add_sfr_register(usart.txreg,     0xfad,porv,"txreg");
+  add_sfr_register(usart.rcreg,     0xfae,porv,"rcreg");
+  add_sfr_register(&usart.spbrg,    0xfaf,porv,"spbrg");
 
   add_sfr_register(&t3con,	  0xfb1,porv,"t3con");
   add_sfr_register(&tmr3l,	  0xfb2,porv,"tmr3l");
@@ -128,23 +200,16 @@ void _16bit_processor :: create_sfr_map()
   add_sfr_register(&ccpr1l,	  0xfbe,porv,"ccpr1l");
   add_sfr_register(&ccpr1h,	  0xfbf,porv,"ccpr1h");
 
-//   add_sfr_register(&adcon1,	  0xfc1,porv,"adcon1");
-//   add_sfr_register(&adcon2,	  0xfc2,porv,"adcon2");
-//   add_sfr_register(&adresl,	  0xfc3,porv,"adresl");
-//   add_sfr_register(&adresh,	  0xfc4,porv,"adresh");
+  add_sfr_register(&adcon1,	  0xfc1,porv,"adcon1");
+  add_sfr_register(&adcon0,	  0xfc2,porv,"adcon0");
+  add_sfr_register(&adresl,	  0xfc3,porv,"adresl");
+  add_sfr_register(&adresh,	  0xfc4,porv,"adresh");
 
-  //Fixme - temporary hack
-  add_file_registers(0xfc1, 0xfc4, 0);
-  rma[0xfc1].new_name("adcon1");
-  rma[0xfc2].new_name("adcon2");
-  rma[0xfc3].new_name("adresl");
-  rma[0xfc4].new_name("adresh");
-
-  add_sfr_register(&ssp.sspcon1,  0xfc5,porv,"sspcon1");
-  add_sfr_register(&ssp.sspcon2,  0xfc6,porv,"sspcon2");
+  add_sfr_register(&ssp.sspcon2,  0xfc5,porv,"sspcon2");
+  add_sfr_register(&ssp.sspcon,   0xfc6,porv,"sspcon1");
   add_sfr_register(&ssp.sspstat,  0xfc7,porv,"sspstat");
   add_sfr_register(&ssp.sspadd,   0xfc8,porv,"sspadd");
-  //add_sfr_register(&ssp.sspbuf,   0xfc9,porv,"sspbuf");
+  add_sfr_register(&ssp.sspbuf,   0xfc9,porv,"sspbuf");
 
   add_sfr_register(&t2con,	  0xfca,porv,"t2con");
   add_sfr_register(&pr2,	  0xfcb,RegisterValue(0xff,0),"pr2");
@@ -210,9 +275,10 @@ void _16bit_processor :: create_sfr_map()
     delete pcl;
   pcl = new PCL16();
 
-  add_sfr_register(pcl,    0xff9);
-  add_sfr_register(pclath, 0xffa);
-  //add_sfr_register(&pclatu, 0xffb);
+  add_sfr_register(pcl,     0xff9);
+  add_sfr_register(pclath,  0xffa);
+  add_sfr_register(&pclatu, 0xffb, porv, "pclatu");
+
   stack = &stack16;
   add_sfr_register(&stack16.stkptr,  0xffc,porv,"stkptr");
   add_sfr_register(&stack16.tosl,    0xffd,porv,"tosl");
@@ -230,14 +296,14 @@ void _16bit_processor :: create_sfr_map()
   }
 
   // Initialize all of the register cross linkages
-  //get_pir_set()->set_pir1(get_pir1());
-  //get_pir_set()->set_pir2(get_pir2());
   pir_set_def.set_pir1(&pir1);
   pir_set_def.set_pir2(&pir2);
 
+  tmr2.ssp_module = &ssp;
+
   tmr1l.tmrh   = &tmr1h;
   tmr1l.t1con  = &t1con;
-  tmr1l.pir_set =  &pir_set_def; //get_pir_set();
+  tmr1l.setInterruptSource(new InterruptSource(&pir1, PIR1v1::TMR1IF));
   tmr1l.ccpcon = &ccp1con;
 
   tmr1h.tmrl  = &tmr1l;
@@ -255,7 +321,7 @@ void _16bit_processor :: create_sfr_map()
 
   tmr3l.tmrh  = &tmr3h;
   tmr3l.t1con = &t3con;
-  tmr3l.pir_set = &pir_set_def; //get_pir_set();
+  tmr3l.setInterruptSource(new InterruptSource(&pir2, PIR2v2::TMR3IF));
   tmr3l.ccpcon = &ccp1con;
 
   tmr3h.tmrl  = &tmr3l;
@@ -266,25 +332,78 @@ void _16bit_processor :: create_sfr_map()
   t3con.ccpr2l = &ccpr2l;
 
   ccp1con.setCrosslinks(&ccpr1l, &pir_set_def, &tmr2);
+  ccp1con.setIOpin(&((*m_portc)[2]));
   ccpr1l.ccprh  = &ccpr1h;
   ccpr1l.tmrl   = &tmr1l;
   ccpr1h.ccprl  = &ccpr1l;
 
-  pir1.intcon = &intcon; //get_pir1()->intcon = &intcon;
-  pir1.pie = &pie1; //get_pir1()->pie    = &pie1;
-  pie1.pir    = &pir1; //get_pir1();
+  pir1.set_intcon(&intcon);
+  pir1.set_pie(&pie1);
+  pie1.pir = &pir1;
   pie1.new_name("pie1");
 
-  pir2.intcon = &intcon; //get_pir2()->intcon = &intcon;
-  pir2.pie = &pie2; //get_pir2()->pie    = &pie2;
+  pir2.set_intcon(&intcon);
+  pir2.set_pie(&pie2);
   
-  pie2.pir    = &pir2; //get_pir2();
+  pie2.pir    = &pir2;
   pie2.new_name("pie2");
 
   // All of the status bits on the 16bit core are writable
   status->write_mask = 0xff;
 
 
+  adcon0.setAdresLow(&adresl);
+  adcon0.setAdres(&adresh);
+  adcon0.setAdcon1(&adcon1);
+  adcon0.setIntcon(&intcon);
+  adcon0.setPir(&pir1);
+  adcon0.setChannel_Mask(7); // Greater than 4 channels
+  adcon0.setA2DBits(10);
+
+  adcon1.setValidCfgBits(ADCON1::PCFG0 | ADCON1::PCFG1 | 
+			 ADCON1::PCFG2 | ADCON1::PCFG3,0);
+
+  adcon1.setNumberOfChannels(8);	// Allow 8 channel configuration
+  adcon1.setChannelConfiguration(0, 0xff);
+  adcon1.setChannelConfiguration(1, 0xff);
+  adcon1.setChannelConfiguration(2, 0x1f);
+  adcon1.setChannelConfiguration(3, 0x1f);
+  adcon1.setChannelConfiguration(4, 0x0b);
+  adcon1.setChannelConfiguration(5, 0x0b);
+  adcon1.setChannelConfiguration(6, 0x00);
+  adcon1.setChannelConfiguration(7, 0x00);
+  adcon1.setChannelConfiguration(8, 0xff);
+  adcon1.setChannelConfiguration(9, 0x3f);
+  adcon1.setChannelConfiguration(10, 0x3f);
+  adcon1.setChannelConfiguration(11, 0x3f);
+  adcon1.setChannelConfiguration(12, 0x1f);
+  adcon1.setChannelConfiguration(13, 0x0f);
+  adcon1.setChannelConfiguration(14, 0x01);
+  adcon1.setChannelConfiguration(15, 0x0d);
+
+  adcon1.setVrefHiConfiguration(1, 3);
+  adcon1.setVrefHiConfiguration(3, 3);
+  adcon1.setVrefHiConfiguration(5, 3);
+  adcon1.setVrefHiConfiguration(8, 3);
+  adcon1.setVrefHiConfiguration(10, 3);
+  adcon1.setVrefHiConfiguration(11, 3);
+  adcon1.setVrefHiConfiguration(12, 3);
+  adcon1.setVrefHiConfiguration(13, 3);
+  adcon1.setVrefHiConfiguration(15, 3);
+
+  adcon1.setVrefLoConfiguration(8, 2);
+  adcon1.setVrefLoConfiguration(11, 2);
+  adcon1.setVrefLoConfiguration(12, 2);
+  adcon1.setVrefLoConfiguration(13, 2);
+  adcon1.setVrefLoConfiguration(15, 2);
+
+  adcon1.setNumberOfChannels(5);
+  adcon1.setIOPin(0, &(*m_porta)[0]);
+  adcon1.setIOPin(1, &(*m_porta)[1]);
+  adcon1.setIOPin(2, &(*m_porta)[2]);
+  adcon1.setIOPin(3, &(*m_porta)[3]);
+  adcon1.setIOPin(4, &(*m_porta)[5]);
+  // AN5,AN6 and AN7 exist only on devices with a PORTE.
 }
 
 //-------------------------------------------------------------------
@@ -305,10 +424,9 @@ void _16bit_processor :: create ()
   ind0.init(this);
   ind1.init(this);
   ind2.init(this);
-  //usart16 = new USART_MODULE16();
 
   pic_processor::create();
-  _16bit_processor::create_sfr_map();
+  create_sfr_map();
 
   tmr0l.initialize();
 
@@ -324,6 +442,8 @@ void _16bit_processor :: create ()
     pma->SpecialRegisters.push_back(&bsr);
     rma.SpecialRegisters.push_back(&bsr);
   }
+
+
 }
 
 //
@@ -380,73 +500,98 @@ void _16bit_processor::option_new_bits_6_7(unsigned int bits)
 {
 
   //portb.rbpu_intedg_update(bits);
-  cout << "16bit, option bits 6 and/or 7 changed\n";
+  //cout << "16bit, option bits 6 and/or 7 changed\n";
 
 }
 
-#define FOSC0   1<<0
-#define FOSC1   1<<1
-#define FOSC2   1<<2
-#define OSCEN   1<<5
 
-#define WDTEN   ((1<<0) << 8)
-#define WDTPS0  ((1<<1) << 8)
-#define WDTPS1  ((1<<2) << 8)
-#define WDTPS2  ((1<<3) << 8)
-
-
-#define PWRTEN  1<<0
-#define BOREN   1<<1
-#define BORV0   1<<2
-#define BORV1   1<<2
-
-#define CCP2MX  1<<0
-
-#define STVREN  1<<0
 //-------------------------------------------------------------------
-void _16bit_processor::set_out_of_range_pm(unsigned int address, unsigned int value)
+// Fetch the rom contents at a particular address.
+unsigned int _16bit_processor::get_program_memory_at_address(unsigned int address)
 {
-  cout << hex << "16bit proc setting config address 0x" <<(address<<1) << " to 0x"<<value<<'\n';
-  switch (address) {
-  case CONFIG2:
-    if(config_modes) {
-      if(value & WDTEN) {
-	//if(verbose)
-	  cout << "config Enabling WDT\n";
+  unsigned int uIndex = map_pm_address2index(address);
 
-	config_modes->enable_wdt();
-      } else {
-	cout << "config disabling WDT\n";
-	config_modes->disable_wdt();
-      }
+  
+  if (uIndex < program_memory_size())
+    return  program_memory[uIndex] ? program_memory[uIndex]->get_opcode() : 0xffffffff;
+
+  if (address >= CONFIG1L && address <= 0x30000D)
+    return get_config_word(address);
+
+  static const unsigned int DEVID1 = 0x3ffffe;
+  static const unsigned int DEVID2 = 0x3fffff;
+  if (address == DEVID1)
+    return 0;
+  if (address == DEVID2)
+    return 0;
+
+  return 0xffffffff;
+}
+
+unsigned int _16bit_processor::get_config_word(unsigned int address)
+{
+
+  if (!(address >= CONFIG1L && address <= 0x30000D))
+    return 0xffffffff;
+
+  address -= CONFIG1L;
+
+  if (m_configMemory) {
+    address &= 0xfffe; // Clear LSB
+    unsigned int ret = 0xffff;
+
+    if (m_configMemory[address])
+      ret = (ret & 0xff00) |  (((unsigned int )(m_configMemory[address]->getVal())) & 0x00ff);
+
+    address++;
+
+    if (m_configMemory[address])
+      ret = (ret & 0x00ff) |  ((((unsigned int )(m_configMemory[address]->getVal()))<<8) & 0xff00);
+
+    return ret;
+  }
+  return 0xffffffff;
+}
+
+bool  _16bit_processor::set_config_word(unsigned int address, unsigned int cfg_word)
+{
+  if (address >= CONFIG1L && address <= 0x30000D) {
+
+    cout << "Setting config word 0x"<<hex<<address<< " = 0x"<<cfg_word<< endl;
+    address -= CONFIG1L;
+
+    if (m_configMemory) {
+
+      address &= 0xfffe;
+
+      if(m_configMemory[address])
+	m_configMemory[address]->set((int)(cfg_word&0xff));
+
+      address++;
+
+      if(m_configMemory[address])
+	m_configMemory[address]->set((int)((cfg_word>>8)&0xff));
+
+      return true;
     }
-
-    break;
-
-  case CONFIG1:
-    if(((value>>8) & (OSCEN | FOSC0 | FOSC1 | FOSC2)) != (OSCEN | FOSC0 | FOSC1 | FOSC2))
-      cout << "FOSC bits in CONFIG1H are not supported\n";
-
-    cout << "18cxxx config address 0x" << hex << (address<<1) << " Copy protection " <<
-      (value&0xff) << '\n';
-
-    break;
-
-  case CONFIG3:
-  case CONFIG4:
-    cout << "18cxxx config address 0x" << hex << (address<<1) << " is not supported\n";
-    break;
-
-  case DEVID:
-
-    cout << "18cxxx device id address 0x" << hex << (address<<1) << " is not supported\n";
-    break;
-
-  default:
-    cout << "WARNING: 18cxxx is ignoring code at address 0x" << hex <<(address<<1) << '\n';
   }
 
+  return false;
 }
+
+//-------------------------------------------------------------------
+void _16bit_processor::create_config_memory()
+{
+  m_configMemory = new ConfigMemory *[configMemorySize()];
+
+  for (unsigned int i=0; i<configMemorySize(); i++)
+    m_configMemory[i] = 0;
+
+  m_configMemory[CONFIG1H-CONFIG1L] = new Config1H(this, CONFIG1H);
+  m_configMemory[CONFIG2H-CONFIG1L] = new Config2H(this, CONFIG2H);
+
+}
+
 //-------------------------------------------------------------------
 //
 // Package stuff
@@ -456,38 +601,3 @@ void _16bit_processor::create_iopin_map()
   cout << "_16bit_processor::create_iopin_map WARNING --- not creating package \n";
 }
 
-//------------------------------------------------------------------------
-// Latch Register
-// (Probably should put this somewhere else)
-PicLatchRegister::PicLatchRegister(const char *_name, PortRegister *_port)
-  : m_port(_port), m_EnableMask(0xff)
-{
-  new_name(_name);
-}
-
-void PicLatchRegister::put(unsigned int new_value)
-{
-  trace.raw(write_trace.get() | value.data);
-  value.data = new_value & m_EnableMask;
-  m_port->put_value(value.data);
-}
-void PicLatchRegister::put_value(unsigned int new_value)
-{
-  value.data = new_value & m_EnableMask;
-  m_port->put_value(value.data);
-}
-unsigned int PicLatchRegister::get()
-{
-  trace.raw(read_trace.get()  | value.data);
-  trace.raw(read_trace.geti() | value.init);
-
-  return value.data;
-}
-void PicLatchRegister::setbit(unsigned int bit_number, char new_value)
-{
-  printf("PicLatchRegister::setbit() -- shouldn't be called\n");
-}
-void PicLatchRegister::setEnableMask(unsigned int nEnableMask)
-{
-  m_EnableMask = nEnableMask;
-}
