@@ -627,7 +627,8 @@ void Program_Counter16::computed_goto(unsigned int new_address)
 
   value = ( (new_address | cpu_pic->get_pclath_branching_modpcl() )>>1) & memory_size_mask;
 
-  cpu_pic->pcl->value.put((value<<1) & 0xff);    // see Update pcl comment in Program_Counter::increment()
+  // see Update pcl comment in Program_Counter::increment()
+  cpu_pic->pcl->value.put((value<<1) & 0xff);
 
   // The instruction modifying the PCL will also increment the program counter. So, pre-compensate
   // the increment with a decrement:
@@ -674,6 +675,7 @@ void Program_Counter16::put_value(unsigned int new_value)
   value = (new_value) & memory_size_mask;
   cpu_pic->pcl->value.put(value & 0xff);
   cpu_pic->pclath->value.put((value >> 8) & 0xff);
+  cpu16->pclatu.value.put((value >> 16) & 0xff);
 
 
   cpu_pic->pcl->update();
@@ -986,7 +988,7 @@ unsigned int TMR0H::get_value(void)
 // TMR0_16 member functions
 //
 TMR0_16::TMR0_16()
-  : t0con(0), intcon(0), tmr0h(0), value16(0)
+  : TMR0(), t0con(0), intcon(0), tmr0h(0), value16(0)
 {
 }
 //--------------------------------------------------
@@ -1040,11 +1042,12 @@ void TMR0_16::start(int restart_value, int sync)
 void TMR0_16::put_value(unsigned int new_value)
 {
   value.put(new_value & 0xff);
+  value16 = (new_value & 0xff) | (tmr0h ? (tmr0h->get_value()<<8)  : 0);
   if(t0con->value.get() & T0CON::TMR0ON) {
     if(t0con->value.get() & T0CON::T08BIT)
       TMR0::put_value(new_value);
     else
-      start((new_value & 0xff) | (tmr0h ? (tmr0h->get_value()<<8)  : 0));
+      start(value16);
   } else {
     // TMR0 is not enabled
   }
@@ -1102,9 +1105,13 @@ unsigned int TMR0_16::get_value(void)
   if(t0con->value.get() & T0CON::TMR0ON) {
 
     // If TMR0L:H is configured as an 8-bit timer, then treat as an 8-bit timer
-    if(t0con->value.get() & T0CON::T08BIT)
+    if(t0con->value.get() & T0CON::T08BIT) {
+      if (tmr0h)
+	tmr0h->put_value( (value16>>8)&0xff);
+
       return(TMR0::get_value());
 
+    }
     value16 = (int) ((get_cycles().value - last_cycle)/ prescale);
 
     value.put(value16 & 0xff);
@@ -1204,77 +1211,21 @@ void TMR3_MODULE::initialize(T3CON *t3con_, PIR_SET *pir_set_)
 
 }
 
-//--------------------------------------------------
-
-TXREG_16::TXREG_16(void)
-  : pir_set(0)
+//-------------------------------------------------------------------
+ADCON0_16::ADCON0_16()
+  : m_pPir(0)
 {
 }
 
-bool TXREG_16::is_empty(void)
+void ADCON0_16::setPir(PIR *pPir)
 {
-  if(pir_set)
-    return(pir_set->get_txif());
-  return 0;
-
+  m_pPir = pPir;
 }
 
-void TXREG_16::empty(void)
-{
-  if(pir_set)
-    pir_set->set_txif();
-}
-
-void TXREG_16::full(void)
-{
-  if(pir_set)
-    pir_set->clear_txif();
-}
-
-void TXREG_16::assign_pir_set(PIR_SET *new_pir_set)
-{
-  pir_set = new_pir_set;
-
-}
-
-
-RCREG_16::RCREG_16(void)
-  : pir_set(0)
-{
-}
-
-void RCREG_16::assign_pir_set(PIR_SET *new_pir_set)
-{
-  pir_set = new_pir_set;
-
-}
-
-void RCREG_16::push(unsigned int new_value)
-{
-
-  _RCREG::push(new_value);
-
-  if(pir_set)
-    pir_set->set_rcif();
-
-}
-
-void RCREG_16::pop(void)
-{
-
-  _RCREG::pop();
-  if((fifo_sp == 0) && pir_set)
-    pir_set->clear_rcif();
-
-}
-
-
-//--------------------------------------------------
-// member functions for the USART
-//--------------------------------------------------
-
-USART_MODULE16::USART_MODULE16(void)
+void ADCON0_16::set_interrupt()
 {
+  if (m_pPir)
+    m_pPir->set_adif();
 }
 
 //-------------------------------------------------------------------
@@ -1401,92 +1352,6 @@ void TBL_MODULE::write(void)
 }
 
 //-------------------------------------------------------------------
-//-------------------------------------------------------------------
-
-void  SSPCON1::put(unsigned int new_value)
-{
-
-  cout << "SSPCON1 is not implemented\n";
-
-  sfr_register::put(new_value);
-}
-
-SSPCON1::SSPCON1(void)
-{
-
-}
-
-//-------------------------------------------------------------------
-//-------------------------------------------------------------------
-
-void  SSPCON2::put(unsigned int new_value)
-{
-
-  cout << "SSPCON2 is not implemented\n";
-
-  sfr_register::put(new_value);
-}
-
-SSPCON2::SSPCON2(void)
-{
-
-}
-
-//-------------------------------------------------------------------
-//-------------------------------------------------------------------
-
-void  SSPSTAT::put(unsigned int new_value)
-{
-
-  cout << "SSPSTAT is not implemented\n";
-
-  sfr_register::put(new_value);
-}
-
-SSPSTAT::SSPSTAT(void)
-{
-
-}
-
-
-//-------------------------------------------------------------------
-//-------------------------------------------------------------------
-
-void  SSPADD::put(unsigned int new_value)
-{
-
-  cout << "SSPADD is not implemented\n";
-
-  sfr_register::put(new_value);
-}
-
-SSPADD::SSPADD(void)
-{
-
-}
-
-//-------------------------------------------------------------------
-//-------------------------------------------------------------------
-
-void  SSPBUF::put(unsigned int new_value)
-{
-
-  cout << "SSPBUF is not implemented\n";
-
-  sfr_register::put(new_value);
-}
-
-SSPBUF::SSPBUF(void)
-{
-
-}
-
-
-SSPMODULE::SSPMODULE(void)
-{
-  cout << "Warning SSP Module is not implemented\n";
-}
-
 
 #if 0
 //-------------------------------------------------------------------
