@@ -24,12 +24,14 @@ Boston, MA 02111-1307, USA.  */
 
 #include  <iostream>
 #include <iomanip>
-#include <unistd.h>
 #include <glib.h>
 #include <string>
 #include "trigger.h"
 #include "pic-instructions.h"
 #include "registers.h"
+#include "exports.h"
+
+#include "gpsim_object.h" // defines ObjectBreakTypes
 
 using namespace std;
 
@@ -56,18 +58,21 @@ public:
 
   unsigned int address;
 
-  virtual bool set_break(void);
-  virtual Processor* get_cpu(void);
-  virtual void print(void);
-  virtual void clear(void);
+  virtual bool set_break();
+  virtual Processor* get_cpu();
+  virtual void print();
+  virtual int  printTraced(Trace *pTrace, unsigned int tbi,
+			   char *pBuf, int szBuf);
+			   
+  virtual void clear();
   virtual char const * bpName() { return "Execution"; }
 
   Breakpoint_Instruction(Processor *new_cpu, 
 			 unsigned int new_address, 
 			 unsigned int bp);
 
-  virtual INSTRUCTION_TYPES isa(void) {return BREAKPOINT_INSTRUCTION;};
-  virtual void execute(void);
+  virtual enum INSTRUCTION_TYPES isa() {return BREAKPOINT_INSTRUCTION;};
+  virtual void execute();
   virtual bool isBase() { return false;}
   virtual bool eval_Expression();
 };
@@ -81,8 +86,8 @@ class Notify_Instruction : public Breakpoint_Instruction
 		     unsigned int address, 
 		     unsigned int bp, 
 		     TriggerObject *cb);
-  virtual INSTRUCTION_TYPES isa(void) {return NOTIFY_INSTRUCTION;};
-  virtual void execute(void);
+  virtual enum INSTRUCTION_TYPES isa() {return NOTIFY_INSTRUCTION;};
+  virtual void execute();
   virtual char const * bpName() { return "Notify Execution"; }
 
 };
@@ -94,7 +99,7 @@ class Profile_Start_Instruction : public Notify_Instruction
 			    unsigned int address, 
 			    unsigned int bp, 
 			    TriggerObject *cb);
-  virtual INSTRUCTION_TYPES isa(void) {return PROFILE_START_INSTRUCTION;};
+  virtual enum INSTRUCTION_TYPES isa() {return PROFILE_START_INSTRUCTION;};
   virtual char const * bpName() { return "Profile Start"; }
 };
 
@@ -105,7 +110,7 @@ class Profile_Stop_Instruction : public Notify_Instruction
 			   unsigned int address, 
 			   unsigned int bp, 
 			   TriggerObject *cb);
-  virtual INSTRUCTION_TYPES isa(void) {return PROFILE_STOP_INSTRUCTION;};
+  virtual enum INSTRUCTION_TYPES isa() {return PROFILE_STOP_INSTRUCTION;};
   virtual char const * bpName() { return "Profile Stop"; }
 };
 
@@ -168,8 +173,10 @@ class RegisterAssertion : public Breakpoint_Instruction
 		    bool bPostAssertion=false
 		    );
 
-  virtual void execute(void);
-  virtual void print(void);
+  virtual void execute();
+  virtual void print();
+  virtual int  printTraced(Trace *pTrace, unsigned int tbi,
+			   char *pBuf, int szBuf);
   virtual char const * bpName() { return "Register Assertion"; }
 
 
@@ -179,13 +186,13 @@ class Breakpoints;
 
 #if defined(IN_MODULE) && defined(_WIN32)
 // we are in a module: don't access the Breakpoints object directly!
-Breakpoints &get_bp(void);
+LIBGPSIM_EXPORT Breakpoints & get_bp();
 #else
 // we are in gpsim: use of get_bp() is recommended,
 // even if the bp object can be accessed directly.
 extern Breakpoints bp;
 
-inline Breakpoints &get_bp(void)
+inline Breakpoints &get_bp()
 {
   return bp;
 }
@@ -261,115 +268,124 @@ public:
     return iterator(m_iMaxAllocated);
   }
 
-  unsigned int  global_break;
-  bool          m_bExitOnBreak;   // enabled from command line option
+  BreakStatus *get(int index)
+  {
+    return (index>=0 && index<MAX_BREAKPOINTS) ? &break_status[index] : 0;
+  }
+  int  global_break;
+  bool m_bExitOnBreak;   // enabled from command line option
   void EnableExitOnBreak(bool bExitOnBreak) {
     m_bExitOnBreak = bExitOnBreak;
   }
 
-  unsigned int breakpoint_number,last_breakpoint;
+  int breakpoint_number,last_breakpoint;
 
 
-  Breakpoints(void);
-  unsigned int set_breakpoint(BREAKPOINT_TYPES,Processor *,
-			      unsigned int, 
-			      unsigned int,
-			      TriggerObject *f = 0);
-  unsigned int set_breakpoint(TriggerObject *);
+  Breakpoints();
+  int set_breakpoint(BREAKPOINT_TYPES,Processor *,
+		     unsigned int, 
+		     unsigned int,
+		     TriggerObject *f = 0);
+  int set_breakpoint(TriggerObject *,Expression *pExpr=0);
 
-  unsigned int set_execution_break(Processor *cpu, unsigned int address);
-  unsigned int set_notify_break(Processor *cpu, 
+  int set_execution_break(Processor *cpu, unsigned int address, Expression *pExpr=0);
+  int set_notify_break(Processor *cpu, 
 				unsigned int address, 
 				TriggerObject *cb);
-  unsigned int set_profile_start_break(Processor *cpu, 
+  int set_profile_start_break(Processor *cpu, 
 				       unsigned int address, 
 				       TriggerObject *f1 = 0);
-  unsigned int set_profile_stop_break(Processor *cpu, 
+  int set_profile_stop_break(Processor *cpu, 
 				      unsigned int address, 
 				      TriggerObject *f1 = 0);
-  unsigned int set_read_break(Processor *cpu, unsigned int register_number);
-  unsigned int set_write_break(Processor *cpu, unsigned int register_number);
-  unsigned int set_read_value_break(Processor *cpu, 
+  int set_read_break(Processor *cpu, unsigned int register_number);
+  int set_write_break(Processor *cpu, unsigned int register_number);
+  int set_read_value_break(Processor *cpu, 
 				    unsigned int register_number, 
 				    unsigned int value, 
 				    unsigned int mask=0xff);
-  unsigned int set_write_value_break(Processor *cpu,
+  int set_write_value_break(Processor *cpu,
 				    unsigned int register_number,
 				    unsigned int value,
 				    unsigned int mask=0xff);
-  unsigned int set_read_value_break(Processor *cpu, 
+  int set_read_value_break(Processor *cpu, 
             unsigned int register_number, 
             unsigned int op,
             unsigned int value, 
             unsigned int mask=0xff);
-  unsigned int set_write_value_break(Processor *cpu,
+  int set_write_value_break(Processor *cpu,
             unsigned int register_number,
             unsigned int op,
             unsigned int value,
             unsigned int mask=0xff);
-  unsigned int set_cycle_break(Processor *cpu,
+  int set_break(gpsimObject::ObjectBreakTypes bt,
+		gpsimObject::ObjectActionTypes at,
+		Register *,
+		Expression *pExpr=0);
+
+  int set_cycle_break(Processor *cpu,
 			       guint64 cycle,
 			       TriggerObject *f = 0);
-  unsigned int set_wdt_break(Processor *cpu);
-  unsigned int set_stk_overflow_break(Processor *cpu);
-  unsigned int set_stk_underflow_break(Processor *cpu);
-  unsigned int check_cycle_break(unsigned int abp);
+  int set_wdt_break(Processor *cpu);
+  int set_stk_overflow_break(Processor *cpu);
+  int set_stk_underflow_break(Processor *cpu);
+  int check_cycle_break(unsigned int abp);
 
-  unsigned int set_notify_read(Processor *cpu, unsigned int register_number);
-  unsigned int set_notify_write(Processor *cpu, unsigned int register_number);
-  unsigned int set_notify_read_value(Processor *cpu,
-				     unsigned int register_number,
-				     unsigned int value,
-				     unsigned int mask=0xff);
-  unsigned int set_notify_write_value(Processor *cpu, 
+  int set_notify_read(Processor *cpu, unsigned int register_number);
+  int set_notify_write(Processor *cpu, unsigned int register_number);
+  int set_notify_read_value(Processor *cpu,
+			    unsigned int register_number,
+			    unsigned int value,
+			    unsigned int mask=0xff);
+  int set_notify_write_value(Processor *cpu, 
 				      unsigned int register_number,
 				      unsigned int value, 
 				      unsigned int mask=0xff);
   bool set_expression(unsigned bp_number, Expression *pExpr);
 
-  inline void clear_global(void) {global_break = GLOBAL_CLEAR;};
-  void halt(void);
-  inline bool have_halt(void) 
+  inline void clear_global() {global_break = GLOBAL_CLEAR;};
+  void halt();
+  inline bool have_halt() 
     { 
       return( (global_break & GLOBAL_STOP_RUNNING) != 0 );
     }
-  inline void clear_halt(void) 
+  inline void clear_halt() 
     {
       global_break &= ~GLOBAL_STOP_RUNNING;
     }
-  inline bool have_interrupt(void) 
+  inline bool have_interrupt() 
     { 
       return( (global_break & GLOBAL_INTERRUPT) != 0 );
     }
-  inline void clear_interrupt(void) 
+  inline void clear_interrupt() 
     {
       global_break &= ~GLOBAL_INTERRUPT;
     }
-  inline void set_interrupt(void) 
+  inline void set_interrupt() 
     {
       global_break |= GLOBAL_INTERRUPT;
     }
-  inline bool have_sleep(void) 
+  inline bool have_sleep() 
     { 
       return( (global_break & GLOBAL_SLEEP) != 0 );
     }
-  inline void clear_sleep(void) 
+  inline void clear_sleep() 
     {
       global_break &= ~GLOBAL_SLEEP;
     }
-  inline void set_sleep(void) 
+  inline void set_sleep() 
     {
       global_break |= GLOBAL_SLEEP;
     }
-  inline bool have_pm_write(void) 
+  inline bool have_pm_write() 
     { 
       return( (global_break & GLOBAL_PM_WRITE) != 0 );
     }
-  inline void clear_pm_write(void) 
+  inline void clear_pm_write() 
     {
       global_break &= ~GLOBAL_PM_WRITE;
     }
-  inline void set_pm_write(void) 
+  inline void set_pm_write() 
     {
       global_break |= GLOBAL_PM_WRITE;
     }
@@ -387,6 +403,7 @@ public:
     }
 
   bool dump1(unsigned int bp_num, int dump_type = BREAK_DUMP_ALL);
+  bool dump(TriggerObject *);
   void dump(int dump_type = BREAK_DUMP_ALL);
   void dump_traced(unsigned int b);
   void clear(unsigned int b);
@@ -400,7 +417,7 @@ public:
   instruction *find_previous(Processor *cpu, 
 			     unsigned int address, 
 			     instruction *_this);
-  int find_free(void);
+  int find_free();
 };
 
 //
@@ -419,111 +436,39 @@ class BreakpointRegister : public Register, public TriggerObject
 {
 public:
 
-  Register *m_replaced;       // A pointer to the register that this break replaces
 
-  BreakpointRegister(void) : TriggerObject(0)
-  { m_replaced = 0;};
+  BreakpointRegister(Processor *, TriggerAction *, Register *);
+
   BreakpointRegister(Processor *, int, int );
   BreakpointRegister(Processor *, TriggerAction *, int, int );
 
-  virtual REGISTER_TYPES isa(void) {return BP_REGISTER;};
-  virtual string &name(void) const
-    {
-      if(m_replaced)
-        return m_replaced->name();
-      else
-        return gpsimValue::name();
-    };
-  /* direct all accesses to the member functions of the
-   * register that is being replaced. Note that we assume
-   * "m_replaced" is properly initialized which it will be
-   * if this object is accessed. (Why? well, we only access
-   * register notify/breaks via the PIC's file register 
-   * memory and never directly access them. But the only
-   * way this instantiation can be accessed is if it successfully
-   * replaced a file register object */
-
-  virtual void put_value(unsigned int new_value)
-    {
-      m_replaced->put_value(new_value);
-    }
-  virtual void put(unsigned int new_value)
-    {
-      m_replaced->put(new_value);
-    }
-
-  virtual void putRV(RegisterValue rv)
-    {
-      m_replaced->putRV(rv);
-    }
-
-  virtual unsigned int get_value(void)
-    {
-      return(m_replaced->get_value());
-    }
-  virtual RegisterValue getRV(void)
-    {
-      return m_replaced->getRV();
-    }
-  virtual RegisterValue getRVN(void) {
-    return m_replaced->getRVN();
-  }
-  virtual unsigned int get(void)
-    {
-      return(m_replaced->get());
-    }
-
-  virtual Register *getReg(void)
-    {
-      if(m_replaced)
-        return m_replaced;
-      else
-        return this;
-    }
-
-  virtual void setbit(unsigned int bit_number, bool new_value)
-    {
-      m_replaced->setbit(bit_number, new_value);
-    }
-
-  virtual bool get_bit(unsigned int bit_number)
-    {
-      return(m_replaced->get_bit(bit_number));
-    }
-
-  virtual double get_bit_voltage(unsigned int bit_number)
-    {
-      return(m_replaced->get_bit_voltage(bit_number));
-    }
-
-  virtual bool hasBreak(void)
-    { 
-      return true;
-    }
-
-  virtual void update(void)
-    {
-      if(m_replaced)
-        m_replaced->update();
-    }
-
-  virtual void add_xref(void *an_xref)
-    {
-      if(m_replaced)
-        m_replaced->add_xref(an_xref);
-    }
-
-  virtual void remove_xref(void *an_xref)
-    {
-      if(m_replaced)
-        m_replaced->remove_xref(an_xref);
-    }
-
+  virtual REGISTER_TYPES isa() {return BP_REGISTER;};
+  virtual string &name() const;
+  virtual void put_value(unsigned int new_value);
+  virtual void put(unsigned int new_value);
+  virtual void putRV(RegisterValue rv);
+  virtual unsigned int get_value();
+  virtual RegisterValue getRV();
+  virtual RegisterValue getRVN();
+  virtual unsigned int get();
+  virtual Register *getReg();
+  virtual void setbit(unsigned int bit_number, bool new_value);
+  virtual bool get_bit(unsigned int bit_number);
+  virtual double get_bit_voltage(unsigned int bit_number);
+  virtual bool hasBreak();
+  virtual void update();
+  virtual void add_xref(void *an_xref);
+  virtual void remove_xref(void *an_xref);
   void replace(Processor *_cpu, unsigned int reg);
-  virtual bool set_break(void);
+  virtual bool set_break();
   unsigned int clear(unsigned int bp_num);
-  virtual void print(void);
-  virtual void clear(void);
+  virtual void print();
+  virtual int  printTraced(Trace *pTrace, unsigned int tbi,
+			   char *pBuf, int szBuf);
+  virtual void clear();
+
+protected:
+  BreakpointRegister();
 
 };
 
@@ -535,12 +480,11 @@ public:
   unsigned int m_uDefRegMask;
   string m_sOperator;
 
-  BreakpointRegister_Value(void)
-    { 
-      m_replaced = 0;
-      break_value = 0;
-      break_mask = 0;
-    }
+  BreakpointRegister_Value()
+  { 
+    break_value = 0;
+    break_mask = 0;
+  }
 
   BreakpointRegister_Value(Processor *_cpu, 
         int _repl, 
@@ -566,7 +510,8 @@ public:
       unsigned int uRegMask, unsigned int uRegTestValue);
   PFNISBREAKCONDITION m_pfnIsBreak;
 
-  enum {
+  enum BRV_Ops {
+    eBRInvalid,
     eBREquals,
     eBRNotEquals,
     eBRGreaterThen,
@@ -588,8 +533,10 @@ public:
   static bool IsLessThenEqualsBreakCondition(unsigned int uRegValue,
       unsigned int uRegMask, unsigned int uRegTestValue);
 
-  virtual void print(void);
-
+  virtual void invokeAction();
+  virtual void print();
+  virtual int  printTraced(Trace *pTrace, unsigned int tbi,
+			   char *pBuf, int szBuf);
 };
 
 
@@ -602,15 +549,18 @@ public:
     set_action(this);
   }
 
-  virtual unsigned int get(void);
-  virtual RegisterValue getRV(void);
-  virtual RegisterValue getRVN(void);
+  virtual unsigned int get();
+  virtual RegisterValue getRV();
+  virtual RegisterValue getRVN();
   virtual bool get_bit(unsigned int bit_number);
   virtual double get_bit_voltage(unsigned int bit_number);
   virtual char const * bpName() { return "register read"; }
 
+  virtual int  printTraced(Trace *pTrace, unsigned int tbi,
+			   char *pBuf, int szBuf);
+  virtual void invokeAction();
   // TriggerAction overrides
-  virtual void action(void);
+  virtual void action();
 };
 
 class Break_register_write : public BreakpointRegister, TriggerAction
@@ -625,14 +575,17 @@ public:
   virtual void setbit(unsigned int bit_number, bool new_value);
   virtual char const * bpName() { return "register write"; }
 
+  virtual int  printTraced(Trace *pTrace, unsigned int tbi,
+			   char *pBuf, int szBuf);
+  virtual void invokeAction();
   // TriggerAction overrides
-  virtual void action(void);
+  virtual void action();
 };
 
 class Break_register_read_value : public BreakpointRegister_Value, TriggerAction
 {
 public:
-//  Break_register_read_value(void){ };
+//  Break_register_read_value(){ };
   Break_register_read_value(Processor *_cpu, 
 			    int _repl, 
 			    int bp, 
@@ -646,15 +599,17 @@ public:
           unsigned int _operator,
 			    unsigned int bm );
 
-  virtual unsigned int get(void);
-  virtual RegisterValue getRV(void);
-  virtual RegisterValue getRVN(void);
+  virtual unsigned int get();
+  virtual RegisterValue getRV();
+  virtual RegisterValue getRVN();
   virtual bool get_bit(unsigned int bit_number);
   virtual double get_bit_voltage(unsigned int bit_number);
   virtual char const * bpName() { return "register read value"; }
 
+  virtual int  printTraced(Trace *pTrace, unsigned int tbi,
+			   char *pBuf, int szBuf);
   // TriggerAction overrides
-  virtual void action(void);
+  virtual void action();
 };
 
 class Break_register_write_value : public BreakpointRegister_Value, TriggerAction
@@ -670,7 +625,7 @@ public:
 			     int _repl, 
 			     int bp, 
 			     unsigned int bv, 
-           unsigned int _operator,
+			     unsigned int _operator,
 			     unsigned int bm );
 
   virtual void put(unsigned int new_value);
@@ -678,74 +633,96 @@ public:
   virtual void setbit(unsigned int bit_number, bool new_value);
   virtual char const * bpName() { return "register write value"; }
 
+  virtual int  printTraced(Trace *pTrace, unsigned int tbi,
+			   char *pBuf, int szBuf);
   // TriggerAction overrides
-  virtual void action(void);
+  virtual void action();
+};
+
+class CommandAssertion : public Breakpoint_Instruction
+{
+public:
+  bool bPostAssertion; // True if assertion is checked after instruction simulates.
+
+  CommandAssertion(Processor *new_cpu, 
+                   unsigned int instAddress, 
+                   unsigned int bp,
+                   const char *_command,
+                   bool bPostAssertion
+                   );
+
+  virtual void execute();
+  virtual void print();
+  virtual int  printTraced(Trace *pTrace, unsigned int tbi,
+			   char *pBuf, int szBuf);
+  virtual char const * bpName() { return "Register Assertion"; }
+private:
+  char *command;
 };
 
 class Log_Register_Write : public Break_register_write
 {
  public:
 
-//  Log_Register_Write(void){ };
   Log_Register_Write(Processor *_cpu, int _repl, int bp ):
     Break_register_write(_cpu,_repl,bp ) { };
-  virtual void put(unsigned int new_value);
-  virtual void putRV(RegisterValue rv);
-  virtual void setbit(unsigned int bit_number, bool new_value);
+  //virtual void put(unsigned int new_value);
+  //virtual void putRV(RegisterValue rv);
+  //virtual void setbit(unsigned int bit_number, bool new_value);
   virtual char const * bpName() { return "log register write"; }
-
+  virtual void action();
 };
 
 class Log_Register_Read : public Break_register_read
 {
 public:
-
-
-//  Log_Register_Read(void){ };
   Log_Register_Read(Processor *_cpu, int _repl, int bp ):
     Break_register_read(_cpu,_repl,bp ) { };
-  virtual unsigned int get(void);
-  virtual RegisterValue getRV(void);
-  virtual RegisterValue getRVN(void);
-  virtual bool get_bit(unsigned int bit_number);
-  virtual double get_bit_voltage(unsigned int bit_number);
   virtual char const * bpName() { return "log register read"; }
-
+  virtual void action();
 };
 
-class Log_Register_Read_value : public BreakpointRegister_Value
+class Log_Register_Read_value : public  Break_register_read_value
 {
 public:
 
-  Log_Register_Read_value(void){ };
   Log_Register_Read_value(Processor *_cpu, 
 			  int _repl, 
 			  int bp, 
 			  unsigned int bv, 
 			  unsigned int bm ) :
-    BreakpointRegister_Value(_cpu,  _repl, bp, bv, bm ) { };
-  virtual unsigned int get(void);
-  virtual RegisterValue getRV(void);
-  virtual bool get_bit(unsigned int bit_number);
-  virtual double get_bit_voltage(unsigned int bit_number);
+    Break_register_read_value(_cpu,  _repl, bp, bv, bm ) { };
+  Log_Register_Read_value(Processor *_cpu, 
+			  int _repl, 
+			  int bp, 
+			  unsigned int bv,
+			  unsigned int _operator,
+			  unsigned int bm ) :
+    Break_register_read_value(_cpu,  _repl, bp, bv, _operator, bm ) { };
+
   virtual char const * bpName() { return "log register read value"; }
+  virtual void action();
 };
 
-class Log_Register_Write_value : public BreakpointRegister_Value
+class Log_Register_Write_value : public Break_register_write_value
 {
 public:
 
-  Log_Register_Write_value(void){ };
   Log_Register_Write_value(Processor *_cpu, 
 			   int _repl, 
 			   int bp, 
 			   unsigned int bv, 
 			   unsigned int bm ) :
-    BreakpointRegister_Value(_cpu,  _repl, bp, bv, bm ) { };
+    Break_register_write_value(_cpu,  _repl, bp, bv, bm ) { };
 
-  virtual void put(unsigned int new_value);
-  virtual void putRV(RegisterValue rv);
-  virtual char const * bpName() { return "log register write value"; }
+  Log_Register_Write_value(Processor *_cpu, 
+			     int _repl, 
+			     int bp, 
+			     unsigned int bv, 
+			     unsigned int _operator,
+			     unsigned int bm ) :
+    Break_register_write_value(_cpu,  _repl, bp, bv, _operator,bm ) { };
+  virtual void action();
 
 };
 
@@ -753,7 +730,7 @@ public:
 class GuiCallBack: public TriggerObject
 {
 public:
-  virtual void callback(void);
+  virtual void callback();
 
   gpointer gui_callback_data;  // Data to be passed back to the gui
 
@@ -761,7 +738,7 @@ public:
   void  (*gui_callback) (gpointer gui_callback_data);
   void set_break(int, void (*)(gpointer),gpointer );
 
-  GuiCallBack(void);
+  GuiCallBack();
 };
 #endif // HAVE_GUI
 
