@@ -2,7 +2,7 @@
 /*
    Copyright (C) 1998,1999 T. Scott Dattalo
 
-This file is part of gpsim.
+This file is part of gpsim.read_message_area
 
 gpsim is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -390,6 +390,8 @@ int PicCodProgramFileType::read_src_files_from_cod(Processor *cpu)
 
     if(!found_lst_in_cod) {
       cpu->files.Add(lstfilename);
+      cpu->files.list_id(num_files);
+
       if(verbose)
         printf("List file %s wasn't in .cod\n",lstfilename);
     }
@@ -413,7 +415,6 @@ _Cleanup:
 //-----------------------------------------------------------
 void PicCodProgramFileType::read_line_numbers_from_cod(Processor *cpu)
 {
-  int lst_line_number = 6;
   int j,start_block,end_block,offset;
   int file_id, sline,smod;
   unsigned int address;
@@ -435,7 +436,6 @@ void PicCodProgramFileType::read_line_numbers_from_cod(Processor *cpu)
       for(offset=0; offset<(COD_BLOCK_SIZE-COD_LS_SIZE); offset += COD_LS_SIZE) {
 
 	if((temp_block[offset+COD_LS_SMOD] & 4) == 0) {
-	  lst_line_number++;
 	  file_id = temp_block[offset+COD_LS_SFILE];
 	  address = get_short_int(&temp_block[offset+COD_LS_SLOC]);
 	  //address = cpu->map_pm_address2index(address);
@@ -446,7 +446,7 @@ void PicCodProgramFileType::read_line_numbers_from_cod(Processor *cpu)
 	      (address <= cpu->program_memory_size()) &&
 	      (smod == 0x80) )
 
-	    cpu->attach_src_line(address,file_id,sline,lst_line_number);
+	    cpu->attach_src_line(address,file_id,sline,0);
 	  
 	}
       }
@@ -475,8 +475,6 @@ void PicCodProgramFileType::read_message_area(Processor *cpu)
   // this indirect way of accessing the CLI since we don't wish
   // for code in the src/ directory to depend directly on code
   // in the cli/ (or any other) directory.
-  ICommandHandler *pCli = CCommandManager::GetManager().find("gpsimCLI");
-
   start_block = get_short_int(&main_dir.dir.block[COD_DIR_MESSTAB]);
 
   if(start_block) {
@@ -538,32 +536,24 @@ void PicCodProgramFileType::read_message_area(Processor *cpu)
         // between them.
       
         switch(DebugType) {
+        // The 'A' and 'E' options in gpasm specifies a list of gpsim commands
+        // that are to be executed after the .cod file has been loaded.
         case 'a':
         case 'A':
           // assertion
 	  {
-	    string script("assertions");
+	    string script("directive");
 	    char buff[256];
-	    snprintf(buff,sizeof(buff),"break e %d %s\n",laddress,DebugMessage);
+	    snprintf(buff,sizeof(buff),"break e %d, %s\n",laddress,DebugMessage);
 	    string cmd(buff);
 	    cpu->add_command(script,cmd);
 	  }
-	  /*
-	  if(pCli) {
-	    char buff[256];
-	    snprintf(buff,sizeof(buff),"break e %d %s\n",laddress,DebugMessage);
-	    if(CMD_ERR_COMMANDNOTDEFINED == pCli->Execute(buff,0))
-	      printf("cod: Failed to parse assertion:%s\n",buff);
-	  }
-	  */
           break;
         case 'e':
         case 'E':
           // gpsim command
-	  // The 'E' option in gpasm specifies a list of gpsim commands that
-	  // are to be executed after the .cod file has been loaded.
 	  {
-	    string script("startup");
+	    string script("directive");
 	    string cmd(DebugMessage);
 	    cmd = cmd + '\n';
 	    cpu->add_command(script,cmd);
@@ -573,10 +563,15 @@ void PicCodProgramFileType::read_message_area(Processor *cpu)
 	case 'c':
 	case 'C':
 	  // gpsim command
-	  // The 'C' option in gpasm specifies a single gpsim command that is
+	  // The 'c'/'C' option in gpasm specifies a single gpsim command that is
 	  // to be invoked whenever the address associated with this directive
 	  // is being simulated.
-
+	  {
+	    bool bPost = DebugType == 'c';
+	    CommandAssertion *pCA = new CommandAssertion(cpu,laddress,0,
+							DebugMessage,bPost);
+	    get_bp().set_breakpoint(pCA);
+	  }
         case 'f':
         case 'F':
           // printf
@@ -638,10 +633,16 @@ void PicCodProgramFileType::read_symbols( Processor *cpu )
           symbol_table.add(rs);
           }
           break;
+
         case COD_ST_ADDRESS:
           iReturn = get_string(b, s, sizeof b);
           symbol_table.add_address(b, value);
           break;
+
+	case COD_ST_CONSTANT: 	// Ignore as no useful purpose and may
+				// conflict with other symbols - RRR
+	  break;
+
         default:
           iReturn = get_string(b,s,sizeof b);
           symbol_table.add_constant(b,value);
@@ -1019,7 +1020,9 @@ int open_cod_file(Processor **pcpu, const char *filename)
 
 int PicCodProgramFileType::LoadProgramFile(Processor **pcpu,
                                            const char *filename,
-                                           FILE *pFile) {
+                                           FILE *pFile,
+					   const char *pProcessorName)
+{
   int error_code= SUCCESS;
   Processor *ccpu = 0;
 
@@ -1047,45 +1050,51 @@ int PicCodProgramFileType::LoadProgramFile(Processor **pcpu,
   }
 
   // If we get here, then the .cod file is good.
-  if(*pcpu == 0) 
-    {
-      char processor_name[16];
+  if(*pcpu == 0) {
+    
+    char processor_type[16];
+    processor_type[0] = 'p';  // Hack to get around processors whose name begin with a digit.
+
+    if(verbose)
+      cout << "ascertaining cpu from the .cod file\n";
+
+    if(SUCCESS == get_string(&processor_type[1],
+			     &main_dir.dir.block[COD_DIR_PROCESSOR - 1],
+			     sizeof (processor_type)-1)) {
+
+      char *pProcessorTypeOffset = isdigit(processor_type[1]) ? 
+	&processor_type[0] : &processor_type[1];
+
+      if (!pProcessorName)
+	pProcessorName = pProcessorTypeOffset;
+
       if(verbose)
-        cout << "ascertaining cpu from the .cod file\n";
+	cout << "found a " << processor_type << " in the .cod file\n";
 
-      if(SUCCESS == get_string(processor_name,
-                               &main_dir.dir.block[COD_DIR_PROCESSOR - 1],
-                               sizeof processor_name)) {
+      *pcpu = (Processor *)CSimulationContext::GetContext()->add_processor(processor_type,
+									   pProcessorName);
+      if(*pcpu == 0) {
+	if(!ignore_case_in_cod)
+	  return(ERR_UNRECOGNIZED_PROCESSOR);
 
-        if(verbose)
-          cout << "found a " << processor_name << " in the .cod file\n";
+	// Could be that there's a case sensitivity issue:
+	strtolower(processor_type);
+	*pcpu = (Processor *)CSimulationContext::GetContext()->
+	  add_processor(processor_type,pProcessorName);
 
-        *pcpu = (Processor *)CSimulationContext::GetContext()->add_processor(processor_name,processor_name);
-        if(*pcpu == 0) {
-          if(!ignore_case_in_cod)
-            return(ERR_UNRECOGNIZED_PROCESSOR);
-
-          // Could be that there's a case sensitivity issue:
-          strtolower(processor_name);
-          *pcpu = (Processor *)CSimulationContext::GetContext()->
-            add_processor(processor_name,processor_name);
-
-          if(*pcpu == 0)
-            return(ERR_UNRECOGNIZED_PROCESSOR);
-        }
-      }
-      else {
-          return(ERR_UNRECOGNIZED_PROCESSOR);
+	if(*pcpu == 0)
+	  return(ERR_UNRECOGNIZED_PROCESSOR);
       }
     }
+    else {
+      return(ERR_UNRECOGNIZED_PROCESSOR);
+    }
+  }
   else
     cout << "cpu is non NULL\n";
 
   ccpu = *pcpu;
 
-  // assume no configuration word is in the cod file.
-  ccpu->set_config_word(ccpu->config_word_address(),0xffff);
-
   read_hex_from_cod(ccpu);
 
   ccpu->files.SetSourcePath(filename);
@@ -1112,9 +1121,7 @@ _Cleanup:
   if(*pcpu != NULL) {
     (*pcpu)->reset(POR_RESET);
     bp.clear_global();
-    string script("assertions");
-    (*pcpu)->run_script(script);
-    script = "startup";
+    string script("directive");
     (*pcpu)->run_script(script);
   }
   return error_code;
