@@ -166,7 +166,7 @@ public:
   PicCodProgramFileType();
   // ProgramFileType overrides
   virtual int  LoadProgramFile(Processor **pProcessor, const char *pFilename,
-                               FILE *pFile);
+                               FILE *pFile, const char *pProcessorName);
 };
 
 #endif
