@@ -31,10 +31,25 @@ Boston, MA 02111-1307, USA.  */
 
 #include "exports.h"
 #include "pic-processor.h"
+#include "hexutils.h"
 #include "program_files.h"
 
+
+//------------------------------------------------------------------------
+// IntelHexProgramFileType -- constructor
+//
+// When a IntelHexProgramFileType is instantiated, it will get placed
+// on to a 'ProgramFileType' list. This list is then used (ultimately)
+// by the 'load' command to load files of this type.
+
+IntelHexProgramFileType::IntelHexProgramFileType()
+{
+  RegisterProgramFileType(this);
+}
+
+
 int 
-PicHexProgramFileType::getachar (FILE * file)
+IntelHexProgramFileType::getachar (FILE * file)
 {
   int c;
 
@@ -46,7 +61,7 @@ PicHexProgramFileType::getachar (FILE * file)
 }
 
 unsigned char 
-PicHexProgramFileType::getbyte (FILE * file)
+IntelHexProgramFileType::getbyte (FILE * file)
 {
   unsigned char byte;
   unsigned int data;
@@ -60,19 +75,17 @@ PicHexProgramFileType::getbyte (FILE * file)
 }
 
 unsigned int
-PicHexProgramFileType::getword(FILE *file)
+IntelHexProgramFileType::getword(FILE *file)
 {
   unsigned char lo = getbyte(file);
   return ((getbyte(file) << 8) | lo);
 }
+#include "cmd_gpsim.h"
 
-PicHexProgramFileType::PicHexProgramFileType() {
-  RegisterProgramFileType(this);
-}
-
-int PicHexProgramFileType::LoadProgramFile(Processor **pProcessor,
+int IntelHexProgramFileType::LoadProgramFile(Processor **pProcessor,
                                            const char *pFilename,
-                                           FILE *inputfile) {
+                                           FILE *inputfile, const char *pProcessorName) 
+{
   if(verbose)
     cout << "load hex\n";
 
@@ -86,10 +99,6 @@ int PicHexProgramFileType::LoadProgramFile(Processor **pProcessor,
   int iReturn;
   if ((iReturn = readihex16 (pProcessor, inputfile)) != SUCCESS) {
     // No errors were found in the hex file.
-    if(verbose)
-      cout << "Configuration word = 0x"  
-	   << setw(4) << setfill('0') << (*pProcessor)->get_config_word() << '\n';
-
     (*pProcessor)->set_frequency(10e6);
     (*pProcessor)->reset(POR_RESET);
     (*pProcessor)->simulation_mode = eSM_STOPPED;
@@ -100,11 +109,12 @@ int PicHexProgramFileType::LoadProgramFile(Processor **pProcessor,
   return iReturn;
 }
 
-int PicHexProgramFileType::readihex16 (Processor **pProcessor, FILE * file)
+int IntelHexProgramFileType::readihex16 (Processor **pProcessor, FILE * file)
 {
+  int extended_address = 0;
   int address;
   int linetype = 0;
-  int wordsthisline;
+  int wordsthisline, bytesthisline;
   int i;
   int lineCount = 1;
   int csby;
@@ -120,7 +130,8 @@ int PicHexProgramFileType::readihex16 (Processor **pProcessor, FILE * file)
       }
 
       checksum = 0;
-      wordsthisline = getbyte (file) / 2;
+      bytesthisline = getbyte (file);
+      wordsthisline = bytesthisline  / 2;
       hi = getbyte (file);
       lo = getbyte (file);
       address = (hi << 8 | lo) / 2;
@@ -130,28 +141,34 @@ int PicHexProgramFileType::readihex16 (Processor **pProcessor, FILE * file)
 
       linetype = getbyte (file);	/* 0 for data, 1 for end  */
 
-      if (linetype == 1)	/* lets get out of here hit the end */
-        break;
+      switch (linetype ) {
+      case 0:      // Data record
+	{
+	  unsigned char buff[256];
+	  bytesthisline &= 0xff;
+	  for (i = 0; i < bytesthisline; i++) 
+	    buff[i] = getbyte(file);
 
-      if (0 == linetype) {	// data record
-        for (i = 0; i < wordsthisline; i++)
-          cpu->init_program_memory_at_index(address++, getword(file));
-      }
-      else if (4 == linetype) {	// Extended linear address
-        unsigned char b1, b2;
-        b1 = getbyte (file);		// not sure what these mean
-        b2 = getbyte (file);
+	  cpu->init_program_memory_at_index(address|extended_address,
+					    buff, bytesthisline);
+	}
+	break;
+      case 1:      // End of hex file
+	return SUCCESS;
+      case 4:      // Extended address
+	{
+	  unsigned char b1, b2;
+	  b1 = getbyte (file);		// not sure what these mean
+	  b2 = getbyte (file);
 
-        if ((0 != address) || (0 != b1) || (0 != b2)) {
-          printf ("Error! Unhandled Extended linear address! %x %.2x%.2x\n",
-                  address, b1, b2);
-          return ERR_BAD_FILE;
-        }
-	  // Should do something with all this info
-	  // BUG: must fix this for pic18 support
-      }
-      else {
-        printf ("Error! Unknown record type! %d\n", linetype);
+	  extended_address = 	(((unsigned int)b1)<<23) | (((unsigned int)b2)<<15);
+	  printf ("Extended linear address %x %x\n",
+		  address, extended_address);
+
+	}
+	break;
+      default:
+	printf ("Error! Unknown record type! %d\n", linetype);
         return ERR_BAD_FILE;
       }
 
