@@ -18,44 +18,13 @@ along with gpsim; see the file COPYING.  If not, write to
 the Free Software Foundation, 59 Temple Place - Suite 330,
 Boston, MA 02111-1307, USA.  */
 
-
-#include <csignal>
-#include "pic-processor.h"
+#include "../config.h"
 #include "trace.h"
+#include "gpsim_interface.h"
+#include "sim_context.h"
 
-//==============================================================
-
-extern void catch_control_c(int); // In breakpoints.cc
-
-void initialize_signals(void)
-{
-
-  static struct sigaction action;
-
-  action.sa_handler = catch_control_c;
-  sigemptyset(&action.sa_mask);
-  action.sa_flags=0;
-
-  sigaction(SIGINT, &action, 0);
-
-
-}
-
-
-//==============================================================
-// initialize_gpsim 
-//
-// Not much initialization is needed now. However, the CORBA 
-// calls needed later will change this...
-//
-
-void initialize_gpsim(void)
-{
-
-  initialize_signals();
-
-}
-
+// in attribute.cc:
+extern void init_attributes();
 
 //==============================================================
 // simulation_cleanup()
@@ -68,6 +37,18 @@ void simulation_cleanup(void)
 
   // Flush the log file (if there is one).
 
-  trace_log.close_logfile();
+  GetTraceLog().close_logfile();
 
 }
+
+
+//========================================================================
+// initialize_gpsim_core()
+//
+int initialize_gpsim_core()
+{
+
+  init_attributes();
+  CSimulationContext::GetContext()->Initialize();
+  return 0;
+}
