@@ -23,22 +23,19 @@ Boston, MA 02111-1307, USA.  */
 #include <iostream>
 #include <iomanip>
 #include <string>
-
+#include <assert.h>
 
 #include "../config.h"
-#include "pic-processor.h"
-#include "14bit-processors.h"  // %%% FIXME %%% remove the dependencies on this
+#include "cmd_gpsim.h"
 #include "ioports.h"
-#include "interface.h"
-#include "p16x6x.h"
-#include "p16f62x.h"
+#include "trace.h"
 
 #include "stimuli.h"
 
 #include "xref.h"
 //#define DEBUG
 #if defined(DEBUG)
-#define Dprintf(arg) {printf("%s:%d",__FILE__,__LINE__); printf arg; }
+#define Dprintf(arg) {printf("%s:%d ",__FILE__,__LINE__); printf arg; }
 #else
 #define Dprintf(arg) {}
 #endif
@@ -79,7 +76,7 @@ void PeripheralSignalSource::toggle()
     break;
   case '0':
   case 'w':
-    putState('0');
+    putState('1');
     break;
   }
 }
@@ -106,10 +103,10 @@ void PeripheralSignalSource::toggle()
 //
 //-------------------------------------------------------------------
 
-class PicSignalSource : public SignalControl
+class SignalSource : public SignalControl
 {
 public:
-  PicSignalSource(PortRegister *_reg, unsigned int bitPosition)
+  SignalSource(PortRegister *_reg, unsigned int bitPosition)
     : m_register(_reg), m_bitMask(1<<bitPosition)
   {
   }
@@ -119,7 +116,7 @@ public:
     //  (((m_register->getDriving()&m_bitMask)!=0)?'1':'0') : 'Z';
     char r = m_register ? (((m_register->getDriving()&m_bitMask)!=0)?'1':'0') : 'Z';
     /**/
-    Dprintf(("PicSignalSource::getState() %s  bitmask:0x%x state:%c\n",
+    Dprintf(("SignalSource::getState() %s  bitmask:0x%x state:%c\n",
 	     (m_register?m_register->name().c_str():"NULL"),
 	     m_bitMask,r));
     /**/
@@ -131,16 +128,6 @@ private:
 };
 
 
-class PortSink : public SignalSink
-{
-public:
-  PortSink(PortRegister *portReg, unsigned int iobit);
-  virtual void setSinkState(char);
-private:
-  PortRegister *m_PortRegister;
-  unsigned int  m_iobit;
-};
-
 PortSink::PortSink(PortRegister *portReg, unsigned int iobit)
   : m_PortRegister(portReg), m_iobit(iobit)
 {
@@ -163,11 +150,45 @@ PortRegister::PortRegister(unsigned int numIopins, unsigned int _mask)
 {
 
 }
+
+void PortRegister::setEnableMask(unsigned int newEnableMask)
+{
+  //unsigned int maskDiff = getEnableMask() ^ newEnableMask;
+  unsigned int oldEnableMask = getEnableMask();
+
+  for (unsigned int i=0, m=1; i<mNumIopins; i++, m<<= 1)
+    if ((newEnableMask & m) && ! (oldEnableMask & m )) 
+    {
+      PinModule *pmP = PortModule::getIOpins(i); 
+      if (!pmP)
+      {
+          pmP = new PinModule(this,i);
+          PortModule::addPinModule(pmP,i);
+          pmP->setDefaultSource(new SignalSource(this, i));
+          pmP->addSink(new PortSink(this, i));
+      }
+      else
+      {
+        if (pmP->getSourceState() == '?')
+	{
+           pmP->setDefaultSource(new SignalSource(this, i));
+           pmP->addSink(new PortSink(this, i));
+	}
+      }
+    }
+
+  mEnableMask = newEnableMask;
+}
+
 void PortRegister::put(unsigned int new_value)
 {
   trace.raw(write_trace.get() | value.data);
 
-  Dprintf(("PortRegister::put old=0x%x:new=0x%x\n",value.data,new_value));
+  put_value(new_value);
+}
+void PortRegister::put_value(unsigned int new_value)
+{
+  Dprintf(("PortRegister::put_value old=0x%x:new=0x%x\n",value.data,new_value));
 
   unsigned int diff = mEnableMask & (new_value ^ value.data);
   drivingValue = new_value & mEnableMask;
@@ -208,7 +229,7 @@ void PortRegister::setbit(unsigned int bit_number, char new3State)
     trace.raw(write_trace.get()  | value.data);
     trace.raw(write_trace.geti() | value.init);
 
-    Dprintf(("PortRegister::setbit() bit=%d,val=%c\n",bit_number,new3State));
+    Dprintf(("PortRegister::setbit() %s bit=%d,val=%c\n",name().c_str(), bit_number,new3State));
 
     if (new3State=='1' || new3State=='W') {
       rvDrivenValue.data |= (1<<bit_number);
@@ -244,13 +265,6 @@ unsigned int PortRegister::getDriving()
 {
   return drivingValue;
 }
-class PortLatch : public Register
-{
-public:
-
-private:
-  PortRegister *port_register;
-};
 //========================================================================
 //========================================================================
 static PinModule AnInvalidPinModule;
@@ -282,6 +296,16 @@ PinModule &PortModule::operator [] (unsigned int iPinNumber)
   return AnInvalidPinModule;
 }
 
+PinModule * PortModule::getIOpins(unsigned int iPinNumber)
+{
+
+  if (iPinNumber < mNumIopins && iopins[iPinNumber] != &AnInvalidPinModule)
+    return iopins[iPinNumber];
+
+  // error...
+  return (PinModule *)0;
+}
+
 
 void PortModule::updatePort()
 {
@@ -337,12 +361,13 @@ IOPIN *PortModule::getPin(unsigned int iPinNumber)
 // PinModule
 
 PinModule::PinModule()
-  : PinMonitor(), m_pin(0),m_port(0), m_pinNumber(0),
+  : PinMonitor(),
     m_cLastControlState('?'), m_cLastSinkState('?'),
     m_cLastSourceState('?'), m_cLastPullupControlState('?'),
-    m_activeControl(0),m_defaultControl(0),
-    m_activeSource(0),m_defaultSource(0),
-    m_activePullupControl(0),m_defaultPullupControl(0)
+    m_defaultSource(0), m_activeSource(0),
+    m_defaultControl(0), m_activeControl(0),
+    m_defaultPullupControl(0), m_activePullupControl(0),
+    m_pin(0), m_port(0), m_pinNumber(0)
 {
 
 }
@@ -351,11 +376,11 @@ PinModule::PinModule(PortModule *_port, unsigned int _pinNumber, IOPIN *_pin)
   : PinMonitor(),
     m_cLastControlState('?'), m_cLastSinkState('?'),
     m_cLastSourceState('?'), m_cLastPullupControlState('?'),
-    m_activeControl(0),m_defaultControl(0),
-    m_activeSource(0),m_defaultSource(0),
-    m_activePullupControl(0),m_defaultPullupControl(0),
-    m_pin(_pin),m_port(_port), m_pinNumber(_pinNumber),
-  m_bForcedUpdate(false)
+    m_defaultSource(0), m_activeSource(0),
+    m_defaultControl(0), m_activeControl(0),
+    m_defaultPullupControl(0), m_activePullupControl(0),
+    m_pin(_pin), m_port(_port), m_pinNumber(_pinNumber),
+    m_bForcedUpdate(false)
 {
   setPin(m_pin);
 }
@@ -432,6 +457,8 @@ void PinModule::setDefaultControl(SignalControl *newDefaultControl)
     m_defaultControl = newDefaultControl;
     setControl(m_defaultControl);
   }
+  else
+	delete newDefaultControl;
 }
 void PinModule::setControl(SignalControl *newControl)
 {
@@ -462,12 +489,6 @@ void PinModule::setPullupControl(SignalControl *newPullupControl)
   m_activePullupControl = newPullupControl ? newPullupControl : m_defaultPullupControl;
 }
 
-void PinModule::addSink(SignalSink *new_sink)
-{
-  if(new_sink)
-    sinks.push_back(new_sink);
-}
-
 char PinModule::getControlState()
 {
   return m_activeControl ? m_activeControl->getState() : '?';
@@ -516,160 +537,9 @@ void PinModule::updateUI()
 
 
 
+// The IOPORT class is deprecated.
 
-
-//------------------------------------------------------------------------
-
-PicPortRegister::PicPortRegister(const char *port_name,
-				 unsigned int numIopins, 
-				 unsigned int enableMask)
-  : PortRegister(numIopins, false), m_tris(0)
-{
-  new_name(port_name);
-  PicPortRegister::setEnableMask(enableMask);
-}
-
-void PicPortRegister::setEnableMask(unsigned int newEnableMask)
-{
-  unsigned int maskDiff = getEnableMask() ^ newEnableMask;
-
-  for (unsigned int i=0, m=1; i<mNumIopins; i++, m<<= 1)
-    if (maskDiff & m) {
-      PinModule *pmP = new PinModule(this,i);
-      PortModule::addPinModule(pmP,i);
-      pmP->setDefaultSource(new PicSignalSource(this, i));
-      pmP->addSink(new PortSink(this, i));
-    }
-
-  PortRegister::setEnableMask(newEnableMask);
-}
-class PicSignalControl : public SignalControl
-{
-public:
-  PicSignalControl(Register *_reg, unsigned int bitPosition)
-    : m_register(_reg), m_bitMask(1<<bitPosition)
-  {
-  }
-  char getState()
-  {
-    return m_register ? m_register->get3StateBit(m_bitMask) : '?';
-  }
-private:
-  Register *m_register;
-  unsigned int m_bitMask;
-};
-void PicPortRegister::setTris(PicTrisRegister *new_tris)
-{
-  if (!m_tris) {
-    m_tris = new_tris;
-
-    for (unsigned int i=0; i<mNumIopins; i++) {
-      operator[](i).setDefaultControl(new PicSignalControl(m_tris, i));
-    }
-
-  }
-}
-//------------------------------------------------------------------------
-
-PicTrisRegister::PicTrisRegister(const char *tris_name, PicPortRegister *_port)
-  : sfr_register(),m_port(_port)
-{
-  new_name(tris_name);
-  if (m_port)
-    m_port->setTris(this);
-}
-void PicTrisRegister::put(unsigned int new_value)
-{
-  value.data = new_value;
-  if (m_port)
-    m_port->updatePort();
-}
-
-unsigned int PicTrisRegister::get(void)
-{
-  return value.data;
-}
-
-
-
-
-PicPortBRegister::PicPortBRegister(const char *port_name, unsigned int numIopins, unsigned int enableMask)
-  : PicPortRegister(port_name, numIopins, enableMask),
-    m_bRBPU(false),
-    m_bIntEdge(false)
-{
-}
-
-void PicPortBRegister::put(unsigned int new_value)
-{
-  trace.raw(write_trace.get() | value.data);
-
-  unsigned int diff = mEnableMask & (new_value ^ value.data);
-  if(diff) {
-    drivingValue = new_value & mEnableMask;
-    value.data = drivingValue;
-    // If no stimuli are connected to the Port pins, then the driving
-    // value and the driven value are the same. If there are external
-    // stimuli (or perhaps internal peripherals) overdriving or overriding
-    // this port, then the call to updatePort() will update 'drivenValue'
-    // to its proper value.
-    rvDrivenValue.data = drivingValue;
-    rvDrivenValue.init = 0;
-    updatePort();
-  }
-
-  cpu14->intcon->set_rbif(false);
-}
-
-unsigned int PicPortBRegister::get()
-{
-  cpu14->intcon->set_rbif(false);
-
-  return rvDrivenValue.data;
-}
-
-//------------------------------------------------------------------------
-// setbit
-// FIXME - a sink should be created for the intf and rbif functions.
-
-void PicPortBRegister::setbit(unsigned int bit_number, char new3State)
-{
-  Dprintf(("PicPortBRegister::setbit() bit=%d,val=%c\n",bit_number,new3State));
-
-  bool bNewValue = new3State=='1' || new3State=='W';
-  if (bit_number == 0 && (((rvDrivenValue.data&1)==1)!=m_bIntEdge) 
-      && (bNewValue == m_bIntEdge))
-    cpu14->intcon->set_intf(true);
-
-
-  PortRegister::setbit(bit_number, new3State);
-
-  unsigned int bitMask = (1<<bit_number) & 0xF0;
-
-  if ( (drivingValue ^ rvDrivenValue.data) & m_tris->get() & bitMask )
-    cpu14->intcon->set_rbif(true);
-}
-
-void PicPortBRegister::setRBPU(bool bNewRBPU)
-{
-  m_bRBPU = !bNewRBPU;
-
-  Dprintf(("PicPortBRegister::setRBPU() =%d\n",(m_bRBPU?1:0)));
-
-  unsigned int mask = getEnableMask();
-  for (unsigned int i=0, m=1; mask; i++, m<<= 1)
-    if (mask & m) {
-      mask ^= m;
-      operator[](i).getPin().update_pullup(m_bRBPU ? '1' : '0',true);
-    }
-
-}
-
-void PicPortBRegister::setIntEdge(bool bNewIntEdge)
-{
-  m_bIntEdge = bNewIntEdge;
-}
-
+#if defined(OLD_IOPORT_DESIGN)
 
 //-------------------------------------------------------------------
 //
@@ -741,32 +611,28 @@ unsigned int IOPORT::get_value(void)
 
   // Update the stimuli - if there are any
 
-  if(stimulus_mask) {
+  unsigned int current_value = value.get();
+  
+  unsigned int i=0;
+  unsigned int m=1;
 
-    unsigned int current_value = value.get();
+  for(i=0; i<num_iopins; i++, m<<=1) {
+    if(pins[i] && pins[i]->snode) {
 
-    unsigned int i=0;
-    unsigned int m=1;
+      double v = pins[i]->snode->get_nodeVoltage();
 
-    for(i=0; i<num_iopins; i++, m<<=1) {
-      if(pins[i] && pins[i]->snode) {
-
-	double v = pins[i]->snode->get_nodeVoltage();
-
-	if(current_value & m) {
-	  // this io bit is currently a high
-	  if(v <= pins[i]->get_h2l_threshold())
-	    current_value ^= m;
-	} else
-	  if (v > pins[i]->get_l2h_threshold())
-	    current_value ^= m;
-      }
+      if(current_value & m) {
+	// this io bit is currently a high
+	if(v <= pins[i]->get_h2l_threshold())
+	  current_value ^= m;
+      } else
+	if (v > pins[i]->get_l2h_threshold())
+	  current_value ^= m;
     }
-
-    value.put(current_value);
-
   }
 
+  value.put(current_value);
+
   return(value.get());
 }
 //-------------------------------------------------------------------
@@ -819,14 +685,12 @@ void IOPORT::put(unsigned int new_value)
 
     unsigned int diff = current_value ^ new_value;
 
-    guint64 time = get_cycles().value;
-
     // Update all I/O pins that have stimuli attached to
     // them and their state is being changed by this put() operation.
 
     for(unsigned int i = 0; i<num_iopins; i++,diff>>=1)
       if((diff&1) && pins[i] && pins[i]->snode)
-	pins[i]->snode->update(time);
+	pins[i]->snode->update();
   }
 
 }
@@ -922,6 +786,28 @@ IOPIN *IOPORT::getIO(unsigned int pin_number)
 // one of the bits of the I/O port.
 //
 //-------------------------------------------------------------------
+IOPIN *IOPORT::addPin(IOPIN * new_pin, unsigned int bit_position)
+{
+
+  if(bit_position < num_iopins)
+
+    pins[bit_position] = new_pin;
+
+  else
+    cout << "Warning: iopin pin number ("<<bit_position 
+	 <<") is invalid for " << name() << ". Max iopins " << num_iopins << '\n';
+
+  if(verbose)
+    cout << "attaching iopin to ioport " << name() << '\n';
+
+  return new_pin;
+}
+//-------------------------------------------------------------------
+// attach_iopin
+//   This will store a pointer to the iopin that is associated with
+// one of the bits of the I/O port.
+//
+//-------------------------------------------------------------------
 void IOPORT::attach_iopin(IOPIN * new_pin, unsigned int bit_position)
 {
 
@@ -994,6 +880,7 @@ IOPORT::IOPORT(unsigned int _num_iopins)
 {
   stimulus_mask = 0;
   num_iopins = _num_iopins;
+  valid_iopins = (1<<num_iopins) - 1;
   address = 0;
   value.put(0);
   internal_latch = 0;
@@ -1016,313 +903,4 @@ IOPORT::~IOPORT()
     delete pins;
 }
 
-#if 0
-
-//-------------------------------------------------------------------
-//-------------------------------------------------------------------
-void PORTB::rbpu_intedg_update(unsigned int new_configuration)
-{
-
-  // Set the state of the interrupt bit first.
-  // The reason is because it may be that the pullups will cause a
-  // an i/o to toggle and hence cause an interrupt.
-
-  if((new_configuration ^ intedg) & intedg_MASK )
-    intedg = new_configuration & intedg_MASK;
-
-  if((new_configuration ^ rbpu) & rbpu_MASK ){
-
-    // Save the state of the I/O pins before any (potential) changes
-    // are made.
-
-    unsigned int old_value = value.get();
-
-    rbpu = new_configuration & rbpu_MASK;
-    bool pullup = rbpu ? false : true;
-
-    // Update each I/O pin's pullup resistor.
-    for(unsigned int i=0; i<num_iopins; i++) {
-      pins[i]->update_pullup(pullup);
-      //if(pins[i]->snode)
-      //  pins[i]->snode->update();
-    }
-
-    // Get the new state of each I/O pin 
-
-    unsigned int temp_value = get_value();
-
-    // If the pullup resistors are being turned on, and the I/O's are
-    // inputs, and there are no stimuli attached, then drive I/O's high.
-
-    temp_value &= stimulus_mask;
-    if(pullup)
-      temp_value |= (tris->value.get() & ~stimulus_mask);
-
-    // If anything changed, then we record the new state.
-    if(temp_value ^ old_value) {
-      trace.raw(write_trace.get() | old_value);
-      value.put(temp_value);
-    }
-  }
-
-}
-
-
-//-------------------------------------------------------------------
-//-------------------------------------------------------------------
-void PORTB::check_peripherals(RegisterValue oldValue)
-{
-
-  int diff = oldValue.get() ^ value.get(); // The difference between old and new
-
-  // If portb bit 0 changed states, check to see if an interrupt should occur
-  if( diff & 1)
-    {
-      // If the option register is selecting 'interrupt on rising edge' AND
-      // the old_value was low (and hence the new high) then set  INTF
-      //                    OR
-      // If the option register is selecting 'interrupt on falling edge' AND
-      // the old_value was high (and hence the new low) then set  INTF
-
-      bool bInterruptOnRisingEdge = (intedg &  intedg_MASK) ? true : false;
-      bool bRisingEdge = (value.get() & 1) ? true : false;
-
-      if(verbose)
-	cout << "RB0 changed\n";
-      if(bInterruptOnRisingEdge == bRisingEdge) {
-	if(verbose)
-	  cout << "Now setting intf\n";
-
-	cpu14->intcon->set_intf(true);
-      }
-    }
-
-
-  // If any of the upper 4 bits of port b changed states then set RBIF
-  // Note, that the interrupt on change feature only works for I/O's
-  // configured as inputs.
-  if(diff & 0xf0 & tris->value.get())
-    cpu14->intcon->set_rbif(true);
-
-
-}
-
-//-------------------------------------------------------------------
-//-------------------------------------------------------------------
-unsigned int PORTB::get(void)
-{
-  //  cout << "PORTB::get  -- value at entry 0x" << hex << value.get() << endl;
-
-  RegisterValue oldValue = value;
-
-  IOPORT::get();
-
-  check_peripherals(oldValue);
-
-  //  cout << "PORTB::get  -- value at exit 0x" << hex << value.get() << endl;
-
-  return value.get();
-}
-
-//-------------------------------------------------------------------
-//-------------------------------------------------------------------
-void PORTB::setbit(unsigned int bit_number, bool new_value)
-{
-  RegisterValue oldValue = value;
-  IOPORT::setbit( bit_number,  new_value);
-  check_peripherals(oldValue);
-}
-
-//-------------------------------------------------------------------
-//-------------------------------------------------------------------
-void PORTB::put(unsigned int new_value)
-{
-  PIC_IOPORT::put(new_value);
-  
-}
-
-//-------------------------------------------------------------------
-//-------------------------------------------------------------------
-PORTB_62x::PORTB_62x(void)
-{
-  //new_name("portb");
-}
-
-unsigned int PORTB_62x::get(void)
-{
-  unsigned int old_value;
-
-  old_value = value.get();
-
-  IOPORT::get();
-
-  int diff = old_value ^ value.get(); // The difference between old and new
-
-  // If portb bit 0 changed states, check to see if an interrupt should occur
-  if( diff & 1)
-    {
-      // If the option register is selecting 'interrupt on rising edge' AND
-      // the old_value was low (and hence the new high) then set  INTF
-      //                    OR
-      // If the option register is selecting 'interrupt on falling edge' AND
-      // the old_value was high (and hence the new low) then set  INTF
-
-      // These two statements can be combined into an exclusive or:
-
-      if( (intedg &  intedg_MASK) ^ (old_value & 1))
-	cpu14->intcon->set_intf(true);
-    }
-
-
-  // If any of the upper 4 bits of port b changed states then set RBIF
-  // Note, that the interrupt on change feature only works for I/O's
-  // configured as inputs.
-  if(diff & 0xf0 & tris->value.get())
-    cpu14->intcon->set_rbif(true);
-
-  if(ccp1con && ( diff & CCP1) )
-    ccp1con->new_edge(value.get() & CCP1);
-
-  if( usart && (diff & RX))
-    usart->new_rx_edge(value.get() & RX);
-
-  return value.get();
-}
-
-//-------------------------------------------------------------------
-//-------------------------------------------------------------------
-void PORTB_62x::setbit(unsigned int bit_number, bool new_value)
-{
-  unsigned int old_value = value.get();
-
-  IOPORT::setbit( bit_number,  new_value);
-
-  int diff = old_value ^ value.get(); // The difference between old and new
-
-  // If portb bit 0 changed states, check to see if an interrupt should occur
-  if( diff & 1)
-    {
-      // If the option register is selecting 'interrupt on rising edge' AND
-      // the old_value was low (and hence the new high) then set  INTF
-      //                    OR
-      // If the option register is selecting 'interrupt on falling edge' AND
-      // the old_value was high (and hence the new low) then set  INTF
-
-      // These two statements can be combined into an exclusive or:
-      //cout << "PORTB_62x::setbit() - bit changed states\n";
-      if( (intedg &  intedg_MASK) ^ (old_value & 1))
-	cpu14->intcon->set_intf(true);
-    }
-
-
-  // If any of the upper 4 bits of port b changed states then set RBIF
-  // Note, that the interrupt on change feature only works for I/O's
-  // configured as inputs.
-  if(diff & 0xf0 & tris->value.get())
-    cpu14->intcon->set_rbif(true);
-
-  if(ccp1con && ( diff & CCP1) )
-    ccp1con->new_edge(value.get() & CCP1);
-
-  if( usart && (diff & RX))
-    usart->new_rx_edge(value.get() & RX);
-
-}
-
-
-void PORTB_62x::check_peripherals(RegisterValue oldValue)
-{
-}
-
-//-------------------------------------------------------------------
-//  PORTA_62x::put(unsigned int new_value)
-//
-//  inputs:  new_value - here's where the I/O port is written (e.g.
-//                       gpsim is executing a MOVWF IOPORT,F instruction.)
-//  returns: none
-//
-//  The I/O Port is updated with the new value. If there are any stimuli
-// attached to the I/O pins then they will be updated as well.
-// Note that for the 62x device, the comparator module overides the
-// tris output control.
-//
-//-------------------------------------------------------------------
-
-void PORTA_62x::put(unsigned int new_value)
-{
-
-  if(new_value > 255)
-    cout << "PIC_IOPORT::put value >255\n";
-
-  trace.raw(write_trace.get() | value.get());
-  //trace.register_write(address,value.get());
-
-  // The I/O Ports have an internal latch that holds the state of the last
-  // write, even if the I/O pins are configured as inputs. If the tris port
-  // changes an I/O pin from an input to an output, then the contents of this
-  // internal latch will be placed onto the external I/O pin.
-
-  internal_latch = new_value;
-
-  // update only those bits that are really outputs
-  //cout << "PORTA_62X::put trying to put " << new_value << "  Tris is " << tris->value <<'\n';
-
-  // Bit 4 is an open collector output (it can only drive low)
-  // If we're trying to drive bit 4 high and bit 4 is an output
-  // then don't change the value on the I/O pin in the same state.
-  // Also, bit 5 is an input regardless of the TRIS setting
-
-  value.put(((new_value & ~tris->value.get()) | (value.get() & tris->value.get())) & valid_iopins);
-
-  // FIXME - this is obviously wrong
-  //if(comparator && comparator->enabled()) 
-  //  value.put(value.get() & (0xff & ~( AN0 | AN1 | AN2 | AN3)));
-
-
-  // Update the stimuli - if there are any
-  if(stimulus_mask)
-    update_stimuli();
-
-}
-
-
-//-------------------------------------------------------------------
-//  unsigned int PORTA_62x::get(void)
-//
-//  inputs:  none
-//
-//  returns: 
-//
-//-------------------------------------------------------------------
-
-unsigned int PORTA_62x::get(void)
-{
-  unsigned int old_value;
-
-  old_value = value.get();
-
-  if(stimulus_mask) {
-//    value.put(  (value.get() & ~stimulus_mask) | update_stimuli());
-      update_stimuli();
-  }
-
-  // If the comparator is enabled, then all of the "analog" pins are
-  // read as zero.
-
-  if(comparator && comparator->enabled()) {
-
-    value.put(value.get() & (0xff & ~( AN0 | AN1 | AN2 | AN3)));
-
-  }
-
-  int diff = old_value ^ value.get(); // The difference between old and new
-  
-  if( ssp && (diff & SS))
-    ssp->new_ss_edge(value.get() & SS);
-  
-  // cout << " PORTA_62X::get port value is " << value << " \n";
-
-  return value.get();
-}
-
-#endif //0
+#endif
