@@ -40,8 +40,38 @@ Boston, MA 02111-1307, USA.  */
 #include "symbol.h"
 
 
+//========================================================================
+// The P12 devices with an EEPROM contain two die. One is the 12C core and
+// the other is an I2C EEPROM (Actually, it is not know if there are two
+// physical die. However, it is known that there are two functional layouts
+// in the same package.) These two devices are connected internally.
+class P12_I2C_EE : public I2C_EE
+{
+public:
+  P12_I2C_EE(pic_processor *, unsigned int _rom_size);
+  
+protected:
+  RegisterCollection *m_UiAccessOfRom; // User access to the rom.
+};
 
-void P12C508::create_iopin_map(void)
+
+P12_I2C_EE::P12_I2C_EE(pic_processor *pcpu, unsigned int _rom_size)
+  : I2C_EE(_rom_size)
+{
+
+  if(pcpu) {
+    pcpu->ema.set_cpu(pcpu);
+    pcpu->ema.set_Registers(rom, rom_size);
+    m_UiAccessOfRom = new RegisterCollection(pcpu,
+					     "eeData",
+					     rom,
+					     rom_size);
+  }
+
+}
+
+//========================================================================
+void P12C508::create_iopin_map()
 {
 
   package = new Package(8);
@@ -90,13 +120,14 @@ void P12C508::enter_sleep()
 }
 
 
-void  P12C508::option_new_bits_6_7(unsigned int)
+void  P12C508::option_new_bits_6_7(unsigned int bits)
 {
   if(verbose)
     cout << "p12c508 option_new_bits_6_7\n";
+  m_gpio->setPullUp ( (bits & (1<<6)) == (1<<6) );
 }
 
-void P12C508::create_sfr_map(void)
+void P12C508::create_sfr_map()
 {
 
   RegisterValue porVal(0,0);
@@ -116,7 +147,7 @@ void P12C508::create_sfr_map(void)
 
 }
 
-void P12C508::create_symbols(void)
+void P12C508::create_symbols()
 {
   _12bit_processor::create_symbols();
 
@@ -125,7 +156,7 @@ void P12C508::create_symbols(void)
 }
 
 
-void P12C508::dump_registers (void)
+void P12C508::dump_registers ()
 {
 
 
@@ -143,7 +174,7 @@ void P12C508::tris_instruction(unsigned int tris_register)
   trace.write_TRIS(m_tris->value.get());
 }
   
-void P12C508::create(void)
+void P12C508::create()
 {
 
   create_iopin_map();
@@ -154,23 +185,23 @@ void P12C508::create(void)
   P12C508::create_sfr_map();
   create_invalid_registers ();
 
+  tmr0.set_cpu(this,m_gpio,2);
   tmr0.start(0);
 
   pc->reset();
 }
 
 
-Processor * P12C508::construct(void)
+Processor * P12C508::construct(const char *name)
 {
 
-  P12C508 *p = new P12C508;
+  P12C508 *p = new P12C508(name);
 
   p->pc->set_reset_address(0x1ff);
 
   p->create();
   p->create_symbols();
 
-  p->name_str = "p12c508";
   symbol_table.add_module(p,p->name_str.c_str());
 
   return p;
@@ -178,7 +209,8 @@ Processor * P12C508::construct(void)
 }
 
 
-P12C508::P12C508(void)
+P12C508::P12C508(const char *_name, const char *desc)
+  : _12bit_processor(_name,desc)
 {
   if(verbose)
     cout << "12c508 constructor, type = " << isa() << '\n';
@@ -192,28 +224,31 @@ P12C508::P12C508(void)
       config_modes->CM_FOSC1x | config_modes->CM_WDTE | config_modes->CM_MCLRE;
 }
 
+P12C508::~P12C508()
+{
+}
 
 
 //--------------------------------------------------------
 
-void P12C509::create_sfr_map(void)
+void P12C509::create_sfr_map()
 {
 
 }
 
-Processor * P12C509::construct(void)
+Processor * P12C509::construct(const char *name)
 {
 
-  P12C509 *p = new P12C509;
+  P12C509 *p = new P12C509(name);
 
-  cout << " 12c508 construct\n";
+  if (verbose)
+    cout << " 12c508 construct\n";
 
   p->pc->set_reset_address(0x3ff);
 
   p->create();
   p->create_symbols();
 
-  p->name_str = "p12c509";
   symbol_table.add_module(p,p->name_str.c_str());
 
   return p;
@@ -221,7 +256,7 @@ Processor * P12C509::construct(void)
 }
 
 
-void P12C509::create(void)
+void P12C509::create()
 {
 
   if ( verbose )
@@ -237,7 +272,8 @@ void P12C509::create(void)
 
 }
 
-P12C509::P12C509(void)
+P12C509::P12C509(const char *_name, const char *desc)
+  : P12C508(_name,desc)
 {
   if(verbose)
     cout << "12c509 constructor, type = " << isa() << '\n';
@@ -247,10 +283,10 @@ P12C509::P12C509(void)
 //--------------------------------------------------------
 
 // construct function is identical to 12C508 version ??
-Processor * P12CE518::construct(void)
+Processor * P12CE518::construct(const char *name)
 {
 
-  P12CE518 *p = new P12CE518;
+  P12CE518 *p = new P12CE518(name);
 
   if(verbose)
     cout << " 12ce518 construct\n";
@@ -263,7 +299,6 @@ Processor * P12CE518::construct(void)
     cout << " ... create symbols\n";
   p->create_symbols();
 
-  p->name_str = "p12ce518";
   symbol_table.add_module(p,p->name_str.c_str());
 
   return p;
@@ -271,7 +306,7 @@ Processor * P12CE518::construct(void)
 }
 
 
-void P12CE518::create_iopin_map(void)
+void P12CE518::create_iopin_map()
 {
   P12C508::create_iopin_map();
 
@@ -280,10 +315,9 @@ void P12CE518::create_iopin_map(void)
   //gpio.valid_iopins = 0xff;
 }
 
-void P12CE518::create(void)
+void P12CE518::create()
 {
   Stimulus_Node *scl, *sda;
-  I2C_EE * e;
 
   if(verbose)
     cout << " 12ce518 create \n";
@@ -293,10 +327,8 @@ void P12CE518::create(void)
   if(verbose)
     cout << "  adding serial EE\n";
 
-  e = new I2C_EE();
-  e->set_cpu(this);
-  e->initialize ( 0x10 );
-  e->debug();
+  m_eeprom = new P12_I2C_EE(this, 0x10);
+  m_eeprom->debug();
 
   // GPIO bits 6 and 7 are not bonded to physical pins, but are tied
   // to the internal I2C device.
@@ -332,44 +364,17 @@ void P12CE518::create(void)
     sda->update();
   }
 
-#if defined OLD_GPIO
-    // GPIO bits 6 and 7 are not bonded to physical pins, but are tied
-    // to the internal I2C device.
-    gpio.valid_iopins |= 0xc0;
-    gpio.num_iopins = 8;
-
-    RegisterValue por_value(0xc0,0x00);
-    gpio.value       = por_value;
-    gpio.por_value   = por_value;
-    gpio.wdtr_value  = por_value;
-    gpio.put(0xc0);
-
-    if(verbose)
-      cout << " ... create additional (internal) I/O\n";
-    scl = new Stimulus_Node ( "EE_SCL" );
-
-    scl->attach_stimulus ( new IO_bi_directional(&gpio,7) );
-
-    sda = new Stimulus_Node ( "EE_SDA" );
-
-    IO_open_collector *io_sda = new IO_open_collector(&gpio,6);
-
-    // enable the pullup resistor.
-    if(io_sda)
-      io_sda->update_pullup('1',true);
-
-    sda->attach_stimulus (io_sda);
-#endif
-
-
-  e->attach ( scl, sda );
-
-  set_eeprom(e);
 
+  m_eeprom->attach ( scl, sda );
+  /*
+  ema.set_cpu(this);
+  ema.set_Registers(m_eeprom->rom, m_eeprom->rom_size);
+  */
 
 }
 
-P12CE518::P12CE518(void)
+P12CE518::P12CE518(const char *_name, const char *desc)
+  : P12C508(_name,desc)
 {
   if(verbose)
     cout << "12CE518 constructor, type = " << isa() << '\n';
@@ -388,27 +393,18 @@ void P12CE518::tris_instruction(unsigned int tris_register)
   trace.write_TRIS(w_val);
 }
 
-void P12CE518::set_eeprom(I2C_EE *e)
-{
-  eeprom = e; 
-
-  ema.set_cpu(this);
-  ema.set_Registers(e->rom, e->rom_size);
-
-}
-  
 
 //--------------------------------------------------------
 
-void P12CE519::create_sfr_map(void)
+void P12CE519::create_sfr_map()
 {
 
 }
 
-Processor * P12CE519::construct(void)
+Processor * P12CE519::construct(const char *name)
 {
 
-  P12CE519 *p = new P12CE519;
+  P12CE519 *p = new P12CE519(name);
 
   cout << " 12ce519 construct\n";
 
@@ -417,7 +413,6 @@ Processor * P12CE519::construct(void)
   p->create();
   p->create_symbols();
 
-  p->name_str = "p12ce519";
   symbol_table.add_module(p,p->name_str.c_str());
 
   return p;
@@ -425,7 +420,7 @@ Processor * P12CE519::construct(void)
 }
 
 
-void P12CE519::create(void)
+void P12CE519::create()
 {
   if ( verbose )
     cout << " 12ce519 create \n";
@@ -441,7 +436,8 @@ void P12CE519::create(void)
 }
 
 
-P12CE519::P12CE519(void)
+P12CE519::P12CE519(const char *_name, const char *desc)
+  : P12CE518(_name,desc)
 {
   if(verbose)
     cout << "12ce519 constructor, type = " << isa() << '\n';
@@ -484,13 +480,29 @@ so the processor is waking up\n";
   }
 }
 
+void GPIO::setPullUp ( bool bNewPU )
+{
+  m_bPU = !bNewPU;
+
+  if ( verbose & 16 )
+    printf("GPIO::setPullUp() =%d\n",(m_bPU?1:0));
+
+  unsigned int mask = getEnableMask();
+  for (unsigned int i=0, m=1; mask; i++, m<<= 1)
+    if (mask & m)
+    {
+      mask ^= m;
+      getPin(i)->update_pullup ( m_bPU ? '1' : '0', true );
+    }
+}
+
 
 
 //--------------------------------------------------------
 //------------------------------------------------------------------------
 
 
-void P10F200::create_iopin_map(void)
+void P10F200::create_iopin_map()
 {
 
   package = new Package(6);
@@ -508,7 +520,7 @@ void P10F200::create_iopin_map(void)
 }
 
 /*
-void P10F200::create_sfr_map(void)
+void P10F200::create_sfr_map()
 {
 
   RegisterValue porVal(0,0);
@@ -530,7 +542,7 @@ void P10F200::create_sfr_map(void)
 */
 
   
-void P10F200::create(void)
+void P10F200::create()
 {
 
   create_iopin_map();
@@ -541,23 +553,23 @@ void P10F200::create(void)
   P12C508::create_sfr_map();
   create_invalid_registers ();
 
+  tmr0.set_cpu(this,m_gpio,2);
   tmr0.start(0);
 
   pc->reset();
 }
 
 
-Processor * P10F200::construct(void)
+Processor * P10F200::construct(const char *name)
 {
 
-  P10F200 *p = new P10F200;
+  P10F200 *p = new P10F200(name);
 
   p->pc->set_reset_address(0x0ff);
 
   p->create();
   p->create_symbols();
 
-  p->name_str = "p10f200";
   symbol_table.add_module(p,p->name_str.c_str());
 
   return p;
@@ -565,7 +577,8 @@ Processor * P10F200::construct(void)
 }
 
 
-P10F200::P10F200(void)
+P10F200::P10F200(const char *_name, const char *desc)
+  : P12C508(_name,desc)
 {
   if(verbose)
     cout << "10f200 constructor, type = " << isa() << '\n';
@@ -578,4 +591,52 @@ P10F200::P10F200(void)
     config_modes->valid_bits = config_modes->CM_WDTE | config_modes->CM_MCLRE;
 }
 
+P10F200::~P10F200()
+{
+
+}
+//------------------------------------------------------------------------
+
+void P10F202::create()
+{
+
+  create_iopin_map();
+
+  _12bit_processor::create();
+
+  add_file_registers(0x08, 0x1f, 0);    // 10F202 has 24 bytes RAM
+  P12C508::create_sfr_map();
+  create_invalid_registers ();
+
+  tmr0.set_cpu(this,m_gpio,2);
+  tmr0.start(0);
+
+  pc->reset();
+}
+
+
+Processor * P10F202::construct(const char *name)
+{
+
+  P10F202 *p = new P10F202(name);
+
+  p->pc->set_reset_address(0x1ff);
+
+  p->create();
+  p->create_symbols();
+
+  symbol_table.add_module(p,p->name_str.c_str());
+
+  return p;
+
+}
+
+
+P10F202::P10F202(const char *_name, const char *desc)
+  : P10F200(_name,desc)
+{
+  if(verbose)
+    cout << "10f202 constructor, type = " << isa() << '\n';
+}
+
 
