@@ -37,21 +37,16 @@ Boston, MA 02111-1307, USA.  */
 #include "stimuli.h"
 
 #include "p16f62x.h"
+#include "pic-ioports.h"
 #include "packages.h"
 #include "symbol.h"
 
-CMCON::CMCON(void)
+P16F62x::P16F62x(const char *_name, const char *desc)
+  : P16X6X_processor(_name,desc)
 {
-  value.put(0);
 }
 
-VRCON::VRCON(void)
-{
-  value.put(0);
-}
-
-
-void P16F62x::create_iopin_map(void)
+void P16F62x::create_iopin_map()
 {
   package = new Package(18);
   if(!package)
@@ -81,7 +76,7 @@ void P16F62x::create_iopin_map(void)
 
 }
 
-void P16F62x::create_sfr_map(void)
+void P16F62x::create_sfr_map()
 {
  
 
@@ -121,36 +116,55 @@ void P16F62x::create_sfr_map(void)
   add_sfr_register(&intcon_reg, 0x00b, RegisterValue(0,0));
 
   usart.initialize(get_pir_set(),&(*m_portb)[2], &(*m_portb)[1],
-		   new _TXREG(), new _RCREG());
+		   new _TXREG(&usart), new _RCREG(&usart));
 
   add_sfr_register(&usart.rcsta, 0x18, RegisterValue(0,0),"rcsta");
   add_sfr_register(&usart.txsta, 0x98, RegisterValue(2,0),"txsta");
   add_sfr_register(&usart.spbrg, 0x99, RegisterValue(0,0),"spbrg");
   add_sfr_register(usart.txreg,  0x19, RegisterValue(0,0),"txreg");
   add_sfr_register(usart.rcreg,  0x1a, RegisterValue(0,0),"rcreg");
-  //1usart.initialize_14(this,get_pir_set(),portb,1);
-
-  add_sfr_register(&comparator.cmcon, 0x1f, RegisterValue(0,0),"cmcon");
-  add_sfr_register(&comparator.vrcon, 0x9f, RegisterValue(0,0),"vrcon");
 
   intcon = &intcon_reg;
   intcon_reg.set_pir_set(get_pir_set());
 
-  // Link the comparator and porta
-  //1((PORTA_62x*)porta)->comparator = &comparator;
+  // Link the comparator and voltage ref to porta
+  comparator.initialize(get_pir_set(), &(*m_porta)[2], &(*m_porta)[0], 
+	&(*m_porta)[1], &(*m_porta)[2], &(*m_porta)[3], &(*m_porta)[3],
+	&(*m_porta)[4]);
+
+  comparator.cmcon->set_configuration(1, 0, AN0, AN3, AN0, AN3, ZERO);
+  comparator.cmcon->set_configuration(2, 0, AN1, AN2, AN1, AN2, ZERO);
+  comparator.cmcon->set_configuration(1, 1, AN0, AN2, AN3, AN2, NO_OUT);
+  comparator.cmcon->set_configuration(2, 1, AN1, AN2, AN1, AN2, NO_OUT);
+  comparator.cmcon->set_configuration(1, 2, AN0, VREF, AN3, VREF, NO_OUT);
+  comparator.cmcon->set_configuration(2, 2, AN1, VREF, AN2, VREF, NO_OUT);
+  comparator.cmcon->set_configuration(1, 3, AN0, AN2, AN0, AN2, NO_OUT);
+  comparator.cmcon->set_configuration(2, 3, AN1, AN2, AN1, AN3, NO_OUT);
+  comparator.cmcon->set_configuration(1, 4, AN0, AN3, AN0, AN3, NO_OUT);
+  comparator.cmcon->set_configuration(2, 4, AN1, AN2, AN1, AN2, NO_OUT);
+  comparator.cmcon->set_configuration(1, 5, NO_IN, NO_IN, NO_IN, NO_IN, ZERO);
+  comparator.cmcon->set_configuration(2, 5, AN1, AN2, AN1, AN2, NO_OUT);
+  comparator.cmcon->set_configuration(1, 6, AN0, AN2, AN0, AN2, OUT0);
+  comparator.cmcon->set_configuration(2, 6, AN1, AN2, AN1, AN2, OUT1);
+  comparator.cmcon->set_configuration(1, 7, NO_IN, NO_IN, NO_IN, NO_IN, ZERO);
+  comparator.cmcon->set_configuration(2, 7, NO_IN, NO_IN, NO_IN, NO_IN, ZERO);
+  
+
+  add_sfr_register(comparator.cmcon, 0x1f, RegisterValue(0,0),"cmcon");
+  add_sfr_register(&comparator.vrcon, 0x9f, RegisterValue(0,0),"vrcon");
+
+  comparator.cmcon->put(0);
 
   // Link ccp1 and portb
   //1((PORTB_62x*)portb)->ccp1con = &ccp1con;
 }
 
-void P16F62x::create_symbols(void)
+void P16F62x::create_symbols()
 {
   if(verbose)
     cout << "62x create symbols\n";
 
-  symbol_table.add_register(m_portb);
-  symbol_table.add_register(m_porta);
-
+  Pic14Bit::create_symbols();
 }
 
 void P16F62x::set_out_of_range_pm(unsigned int address, unsigned int value)
@@ -176,64 +190,62 @@ bool P16F62x::set_config_word(unsigned int address, unsigned int cfg_word)
 
   if (pic_processor::set_config_word(address, cfg_word)) {
 
-    cout << "p16f628 setting config word 0x" << hex << cfg_word << '\n';
+    if (verbose)
+        cout << "p16f628 setting config word 0x" << hex << cfg_word << '\n';
 
 
     unsigned int valid_pins = m_porta->getEnableMask();
 
+    // Careful these bits not adjacent
     switch(cfg_word & (CFG_FOSC0 | CFG_FOSC1 | CFG_FOSC2)) {
 
     case 0:  // LP oscillator: low power crystal is on RA6 and RA7
-    case 1:  // XT oscillator: crystal/resonator is on RA6 and RA7
-    case 2:  // HS oscillator: crystal/resonator is on RA6 and RA7
-    case 7:  // ER oscillator: RA6 is CLKOUT, resistor (?) on RA7 
+    case 1:     // XT oscillator: crystal/resonator is on RA6 and RA7
+    case 2:     // HS oscillator: crystal/resonator is on RA6 and RA7
+	(m_porta->getPin(6))->newGUIname("OSC2");
+	(m_porta->getPin(7))->newGUIname("OSC1");
+	break;
 
-      //porta->valid_iopins &= 0x3f;
-      m_porta->setEnableMask(valid_pins & 0x3f);
-      break;
+    case 0x13:  // ER oscillator: RA6 is CLKOUT, resistor (?) on RA7 
+	(m_porta->getPin(6))->newGUIname("CLKOUT");
+	(m_porta->getPin(7))->newGUIname("OSC1");
+	break;
 
-    case 3:  // EC:  RA6 is an I/O, RA7 is a CLKIN
-    case 6:  // ER oscillator: RA6 is an I/O, RA7 is a CLKIN
+    case 3:     // EC:  RA6 is an I/O, RA7 is a CLKIN
+    case 0x12:  // ER oscillator: RA6 is an I/O, RA7 is a CLKIN
+        (m_porta->getPin(7))->newGUIname("CLKIN");
+        valid_pins =  (valid_pins & 0x7f)|0x40;
+        break;
 
-      //porta->valid_iopins &= 0x7f;
-      //porta->valid_iopins |= 0x40;
-      m_porta->setEnableMask( (valid_pins & 0x7f)|0x40);
-      break;
+    case 0x10:  // INTRC: Internal Oscillator, RA6 and RA7 are I/O's
+        valid_pins |= 0xc0;
+        break;
 
-    case 4:  // INTRC: Internal Oscillator, RA6 and RA7 are I/O's
-
-      //porta->valid_iopins |= 0xc0;
-      m_porta->setEnableMask( valid_pins | 0xc0);
-      break;
-
-    case 5:  // INTRC: Internal Oscillator, RA7 is an I/O, RA6 is CLKOUT
-      //porta->valid_iopins &= 0xbf;
-      //porta->valid_iopins |= 0x80;
-      m_porta->setEnableMask( (valid_pins & 0xbf)|0x80);
-      break;
+    case 0x11:  // INTRC: Internal Oscillator, RA7 is an I/O, RA6 is CLKOUT
+        valid_pins = (valid_pins & 0xbf)|0x80;
+        break;
 
     }
 
     // If the /MCLRE bit is set then RA5 is the MCLR pin, otherwise it's 
-    // a general purpose input-only pin.
+    // a general purpose I/O pin.
 
-    if (cfg_word & CFG_MCLRE) {
-
-      //unsigned int m = (cfg_word & CFG_MCLRE) ? 0 : (1<<5);
-  
-      valid_pins = m_porta->getEnableMask();
-      m_porta->setEnableMask(valid_pins | (1<<5));
-    
-      //porta->valid_iopins |= m;
-      //porta->valid_iopins &= ~m;
-
-      //trisa.valid_iopins |= m;
-      //trisa.valid_iopins &= ~m;
+    if (! (cfg_word & CFG_MCLRE)) {   
+      valid_pins |= ( 1<< 5); 		// porta5 IO port
+    }
+    else
+    {
+	(m_porta->getPin(5))->newGUIname("MCLR");
     }
 
     //cout << " porta valid_iopins " << porta->valid_iopins << 
     //   "  tris valid io " << trisa.valid_iopins << '\n';
 
+    if (valid_pins != m_porta->getEnableMask()) // enable new pins for IO
+    {
+        m_porta->setEnableMask(valid_pins);
+        m_porta->setTris(m_trisa);
+    }
     return true;
   }
 
@@ -249,10 +261,10 @@ void  P16F62x::create(int ram_top, unsigned int eeprom_size)
 
   _14bit_processor::create();
 
-  e = new EEPROM_PIR;
+  e = new EEPROM_PIR(pir1);
   e->set_cpu(this);
   e->initialize(eeprom_size);
-  e->set_pir_set(get_pir_set());
+  //e->set_pir_set(get_pir_set());
   e->set_intcon(&intcon_reg);
 
   // assign this eeprom to the processor
@@ -275,22 +287,22 @@ void  P16F62x::create(int ram_top, unsigned int eeprom_size)
 // Pic 16F627 
 //
 
-Processor * P16F627::construct(void)
+Processor * P16F627::construct(const char *name)
 {
 
-  P16F627 *p = new P16F627;
+  P16F627 *p = new P16F627(name);
 
   p->P16F62x::create(0x2f, 128);
   p->create_invalid_registers ();
-  p->pic_processor::create_symbols();
-  p->new_name("p16f627");
+  p->create_symbols();
   symbol_table.add_module(p,p->name().c_str());
 
   return p;
 
 }
 
-P16F627::P16F627(void)
+P16F627::P16F627(const char *_name, const char *desc)
+  : P16F62x(_name,desc)
 {
   if(verbose)
     cout << "f627 constructor, type = " << isa() << '\n';
@@ -302,22 +314,22 @@ P16F627::P16F627(void)
 // Pic 16F628 
 //
 
-Processor * P16F628::construct(void)
+Processor * P16F628::construct(const char *name)
 {
 
-  P16F628 *p = new P16F628;
+  P16F628 *p = new P16F628(name);
 
   p->P16F62x::create(0x2f, 128);
   p->create_invalid_registers ();
-  p->pic_processor::create_symbols();
-  p->new_name("p16f628");
+  p->create_symbols();
   symbol_table.add_module(p,p->name().c_str());
 
   return p;
 
 }
 
-P16F628::P16F628(void)
+P16F628::P16F628(const char *_name, const char *desc)
+  : P16F627(_name,desc)
 {
   if(verbose)
     cout << "f628 constructor, type = " << isa() << '\n';
@@ -330,30 +342,30 @@ P16F628::P16F628(void)
 // Pic 16F648 
 //
 
-pic_processor * P16F648::construct(void)
+Processor * P16F648::construct(const char *name)
 {
 
-  P16F648 *p = new P16F648;
+  P16F648 *p = new P16F648(name);
 
   p->P16F62x::create(0x2f, 256);
   p->create_sfr_map();
   p->create_invalid_registers ();
-  p->pic_processor::create_symbols();
-  p->new_name("p16f648");
+  p->create_symbols();
   symbol_table.add_module(p,p->name().c_str());
 
   return p;
 
 }
 
-P16F648::P16F648(void)
+P16F648::P16F648(const char *_name, const char *desc)
+  : P16F628(_name,desc)
 {
   if(verbose)
     cout << "f648 constructor, type = " << isa() << '\n';
 
 }
 
-void P16F648::create_sfr_map(void)
+void P16F648::create_sfr_map()
 {
  
   add_file_registers(0x150,0x16f,0);
