@@ -34,12 +34,16 @@ Boston, MA 02111-1307, USA.  */
 
 #include "packages.h"
 #include "p16x5x.h"
+#include "pic-ioports.h"
 
 #include "symbol.h"
 
 
 void P16C54::create_iopin_map()
 {
+#ifdef USE_PIN_MODULE_FOR_TOCKI
+    IOPIN * tockipin;
+#endif
 
   package = new Package(18);
   if(!package)
@@ -51,17 +55,25 @@ void P16C54::create_iopin_map()
   package->assign_pin(18, m_porta->addPin(new IO_bi_directional("porta1"),1));
   package->assign_pin( 1, m_porta->addPin(new IO_bi_directional("porta2"),2));
   package->assign_pin( 2, m_porta->addPin(new IO_bi_directional("porta3"),3));
-  package->assign_pin( 3, m_porta->addPin(new IO_open_collector("porta4"),4));
+#ifdef USE_PIN_MODULE_FOR_TOCKI
+  // RCP - attempt to add TOCKI without port register
+  tockipin = new IOPIN("tocki");
+  m_tocki->setPin ( tockipin );
+  package->assign_pin( 3, tockipin );
+  // RCP - End new code
+#else
+  package->assign_pin( 3, m_tocki->addPin(new IOPIN("tocki"),0));
+#endif
   package->assign_pin( 4, 0);
   package->assign_pin( 5, 0);
-  package->assign_pin( 6, m_portb->addPin(new IO_bi_directional_pu("portb0"),0));
-  package->assign_pin( 7, m_portb->addPin(new IO_bi_directional_pu("portb1"),1));
-  package->assign_pin( 8, m_portb->addPin(new IO_bi_directional_pu("portb2"),2));
-  package->assign_pin( 9, m_portb->addPin(new IO_bi_directional_pu("portb3"),3));
-  package->assign_pin(10, m_portb->addPin(new IO_bi_directional_pu("portb4"),4));
-  package->assign_pin(11, m_portb->addPin(new IO_bi_directional_pu("portb5"),5));
-  package->assign_pin(12, m_portb->addPin(new IO_bi_directional_pu("portb6"),6));
-  package->assign_pin(13, m_portb->addPin(new IO_bi_directional_pu("portb7"),7));
+  package->assign_pin( 6, m_portb->addPin(new IO_bi_directional("portb0"),0));
+  package->assign_pin( 7, m_portb->addPin(new IO_bi_directional("portb1"),1));
+  package->assign_pin( 8, m_portb->addPin(new IO_bi_directional("portb2"),2));
+  package->assign_pin( 9, m_portb->addPin(new IO_bi_directional("portb3"),3));
+  package->assign_pin(10, m_portb->addPin(new IO_bi_directional("portb4"),4));
+  package->assign_pin(11, m_portb->addPin(new IO_bi_directional("portb5"),5));
+  package->assign_pin(12, m_portb->addPin(new IO_bi_directional("portb6"),6));
+  package->assign_pin(13, m_portb->addPin(new IO_bi_directional("portb7"),7));
   package->assign_pin(14, 0);
   package->assign_pin(15, 0);
   package->assign_pin(16, 0);
@@ -80,20 +92,28 @@ void P16C55::create_iopin_map()
   package->assign_pin( 7, m_porta->addPin(new IO_bi_directional("porta1"),1));
   package->assign_pin( 8, m_porta->addPin(new IO_bi_directional("porta2"),2));
   package->assign_pin( 9, m_porta->addPin(new IO_bi_directional("porta3"),3));
-  package->assign_pin( 1, m_porta->addPin(new IO_open_collector("porta4"),4));
+#ifdef USE_PIN_MODULE_FOR_TOCKI
+  // RCP - attempt to add TOCKI without port register
+  tockipin = new IOPIN("tocki");
+  m_tocki->setPin ( tockipin );
+  package->assign_pin( 1, tockipin );
+  // RCP - End new code
+#else
+  package->assign_pin( 1, m_tocki->addPin(new IOPIN("tocki"),0));
+#endif
   package->assign_pin( 2, 0);
   package->assign_pin( 3, 0);
   package->assign_pin( 4, 0);
   package->assign_pin( 5, 0);
 
-  package->assign_pin(10, m_portb->addPin(new IO_bi_directional_pu("portb0"),0));
-  package->assign_pin(11, m_portb->addPin(new IO_bi_directional_pu("portb1"),1));
-  package->assign_pin(12, m_portb->addPin(new IO_bi_directional_pu("portb2"),2));
-  package->assign_pin(13, m_portb->addPin(new IO_bi_directional_pu("portb3"),3));
-  package->assign_pin(14, m_portb->addPin(new IO_bi_directional_pu("portb4"),4));
-  package->assign_pin(15, m_portb->addPin(new IO_bi_directional_pu("portb5"),5));
-  package->assign_pin(16, m_portb->addPin(new IO_bi_directional_pu("portb6"),6));
-  package->assign_pin(17, m_portb->addPin(new IO_bi_directional_pu("portb7"),7));
+  package->assign_pin(10, m_portb->addPin(new IO_bi_directional("portb0"),0));
+  package->assign_pin(11, m_portb->addPin(new IO_bi_directional("portb1"),1));
+  package->assign_pin(12, m_portb->addPin(new IO_bi_directional("portb2"),2));
+  package->assign_pin(13, m_portb->addPin(new IO_bi_directional("portb3"),3));
+  package->assign_pin(14, m_portb->addPin(new IO_bi_directional("portb4"),4));
+  package->assign_pin(15, m_portb->addPin(new IO_bi_directional("portb5"),5));
+  package->assign_pin(16, m_portb->addPin(new IO_bi_directional("portb6"),6));
+  package->assign_pin(17, m_portb->addPin(new IO_bi_directional("portb7"),7));
 
   package->assign_pin(18, m_portc->addPin(new IO_bi_directional("portc0"),0));
   package->assign_pin(19, m_portc->addPin(new IO_bi_directional("portc1"),1));
@@ -134,6 +154,10 @@ void P16C54::create_sfr_map()
   add_sfr_register(&option_reg,  0xffffffff, RegisterValue(0xff,0));
   add_sfr_register(m_trisa,  0xffffffff, RegisterValue(0x1f,0));
   add_sfr_register(m_trisb,  0xffffffff, RegisterValue(0xff,0));
+#ifndef USE_PIN_MODULE_FOR_TOCKI
+  add_sfr_register(m_tocki,  0xffffffff, RegisterValue(0x01,0));
+  add_sfr_register(m_trist0, 0xffffffff, RegisterValue(0x01,0));
+#endif
 
 }
 
@@ -141,12 +165,8 @@ void P16C54::create_sfr_map()
 void P16C54::create_symbols()
 {
 
-  pic_processor::create_symbols();
-  symbol_table.add_register(m_porta);
-  symbol_table.add_register(m_trisa);
+  _12bit_processor::create_symbols();
 
-  symbol_table.add_register(m_portb);
-  symbol_table.add_register(m_trisb);
 }
 
 void P16C54::create()
@@ -157,16 +177,14 @@ void P16C54::create()
 
 }
 
-Processor * P16C54::construct()
+Processor * P16C54::construct(const char *name)
 {
 
-  P16C54 *p = new P16C54;
+  P16C54 *p = new P16C54(name);
 
   if(verbose)
     cout << " c54 construct\n";
 
-  p->new_name("p16c54");
-
   p->pc->set_reset_address(0x1ff);
 
   p->create();
@@ -180,7 +198,8 @@ Processor * P16C54::construct()
 
 }
 
-P16C54::P16C54()
+P16C54::P16C54(const char *_name, const char *desc)
+  : _12bit_processor(_name,desc)
 {
   if(verbose)
     cout << "c54 constructor, type = " << isa() << '\n';
@@ -190,6 +209,24 @@ P16C54::P16C54()
 
   m_portb = new PicPortRegister("portb",8,0xff);
   m_trisb = new PicTrisRegister("trisb", m_portb);
+
+#ifdef USE_PIN_MODULE_FOR_TOCKI
+//  RCP - Attempt to assign TOCKI without a port register
+  m_tocki = new PinModule();
+  cout << "c54 contructor assigning tmr0\n";
+  tmr0.set_cpu(this, m_tocki);
+#else
+  m_tocki = new PicPortRegister("tockiport",8,0x01);
+  m_trist0 = new PicTrisRegister("trist0", m_tocki);
+//  cout << "c54 contructor assigning tmr0 to tocki register\n";
+  tmr0.set_cpu(this, m_tocki, 0);
+#endif
+  tmr0.start(0);
+
+}
+
+P16C54::~P16C54()
+{
 }
 
 void P16C54::tris_instruction(unsigned int tris_register)
@@ -232,9 +269,6 @@ void P16C55::create_symbols()
 {
   P16C54::create_symbols();
 
-  symbol_table.add_register(m_portc);
-  symbol_table.add_register(m_trisc);
-
 }
 
 void P16C55::create()
@@ -242,12 +276,10 @@ void P16C55::create()
   P16C54::create();
 }
 
-Processor * P16C55::construct()
+Processor * P16C55::construct(const char *name)
 {
 
-  P16C55 *p = new P16C55;
-
-  p->new_name("p16c55");
+  P16C55 *p = new P16C55(name);
 
   if(verbose)
     cout << " c55 construct\n";
@@ -265,7 +297,8 @@ Processor * P16C55::construct()
 
 }
 
-P16C55::P16C55()
+P16C55::P16C55(const char *_name, const char *desc)
+  : P16C54(_name,desc)
 {
   if(verbose)
     cout << "c55 constructor, type = " << isa() << '\n';
@@ -274,6 +307,9 @@ P16C55::P16C55()
   m_trisc = new PicTrisRegister("trisc", m_portc);
 
 }
+P16C55::~P16C55()
+{
+}
 
 void P16C55::tris_instruction(unsigned int tris_register)
 {
@@ -300,12 +336,10 @@ void P16C55::tris_instruction(unsigned int tris_register)
 
 
 
-Processor * P16C56::construct()
+Processor * P16C56::construct(const char *name)
 {
 
-  P16C56 *p = new P16C56;
-
-  p->new_name("p16c56");
+  P16C56 *p = new P16C56(name);
 
   if(verbose)
     cout << " c56 construct\n";
@@ -323,7 +357,8 @@ Processor * P16C56::construct()
 }
 
 
-P16C56::P16C56()
+P16C56::P16C56(const char *_name, const char *desc)
+  : P16C54(_name,desc)
 {
   if(verbose)
     cout << "c56 constructor, type = " << isa() << '\n';
