@@ -38,16 +38,9 @@ Boston, MA 02111-1307, USA.  */
 #include "symbol.h"
 
 #include "p16x6x.h"
+#include "pic-ioports.h"
 #include "intcon.h"
 
-void P16X6X_processor::create_symbols()
-{
-  Pic14Bit::create_symbols();
-
-  symbol_table.add_register(m_portc);
-  symbol_table.add_register(m_trisc);
-}
-
 void P16C61::create(void)
 {
 
@@ -60,26 +53,26 @@ void P16C61::create(void)
 
 }
 
-Processor * P16C61::construct(void)
+Processor * P16C61::construct(const char *name)
 {
 
-  P16C61 *p = new P16C61;
+  P16C61 *p = new P16C61(name);
 
   if(verbose)
     cout << " c61 construct\n";
 
   p->create();
   p->create_invalid_registers ();
-  p->pic_processor::create_symbols();
+  p->create_symbols();
 
-  p->new_name("p16c61");
   symbol_table.add_module(p,p->name().c_str());
 
   return p;
 
 }
 
-P16C61::P16C61(void)
+P16C61::P16C61(const char *_name, const char *desc)
+  
 {
 }
 
@@ -129,11 +122,15 @@ void P16C62::create_iopin_map(void)
   package->assign_pin(28, m_portb->addPin(new IO_bi_directional_pu("portb7"),7));
 
   if (hasSSP()) {
-    ssp.initialize(get_pir_set(),    // PIR
-		   &(*m_portc)[3],   // SCK
-		   &(*m_portc)[4],   // SDO
-		   &(*m_portc)[5],   // SDI
-		   &(*m_porta)[5]);  // SS
+    ssp.initialize(
+		get_pir_set(),    // PIR
+	 	&(*m_portc)[3],   // SCK
+		&(*m_porta)[5],   // SS
+		&(*m_portc)[5],   // SDO
+		&(*m_portc)[4],   // SDI
+		m_trisc,         // I2C port
+		SSP_TYPE_BSSP
+  	);  
   }
 
   tmr1l.setIOpin(&(*m_portc)[0]);
@@ -203,12 +200,23 @@ void P16C64::create_iopin_map(void)
   package->assign_pin(40, m_portb->addPin(new IO_bi_directional_pu("portb7"),7));
 
   if (hasSSP()) {
-    ssp.initialize(get_pir_set(),    // PIR
-		   &(*m_portc)[3],   // SCK
-		   &(*m_portc)[4],   // SDO
-		   &(*m_portc)[5],   // SDI
-		   &(*m_porta)[5]);  // SS
+    ssp.initialize(
+		get_pir_set(),    // PIR
+		&(*m_portc)[3],   // SCK
+		&(*m_porta)[5],   // SS
+		&(*m_portc)[5],   // SDO
+		&(*m_portc)[4],   // SDI
+		m_trisc,         // I2C port
+		SSP_TYPE_BSSP
+		   ); 
   }
+  psp.initialize(get_pir_set(),    // PIR
+		m_portd,	   // Parallel port
+		m_trisd,	   // Parallel tris
+		m_trise,	   // Control tris
+		&(*m_porte)[0],	   // NOT RD
+		&(*m_porte)[1],	   // NOT WR
+		&(*m_porte)[2]);   // NOT CS
 
   tmr1l.setIOpin(&(*m_portc)[0]);
 }
@@ -227,19 +235,21 @@ void P16X6X_processor::create_sfr_map()
 
   Pic14Bit::create_sfr_map();
 
+  // P16x63 and higher have porta5
+  m_porta->setEnableMask(0x3f);
+  m_porta->setTris(m_trisa);
+
   // The 16c62,c64 have general purpose registers
   // at addresses 20-7f and a0-bf
   add_file_registers(0x20, 0x7f, 0);
   add_file_registers(0xa0, 0xbf, 0);
 
-  add_sfr_register(m_portc, 0x07);
-  add_sfr_register(m_trisc, 0x87, RegisterValue(0xff,0));
 
-  add_sfr_register(get_pir1(),   0x0c, RegisterValue(0,0),"pir1");
+  add_sfr_register(pir1,   0x0c, RegisterValue(0,0),"pir1");
   add_sfr_register(&pie1,   0x8c, RegisterValue(0,0));
 
-  add_sfr_register(&tmr1l,  0x0e, RegisterValue(0,0));
-  add_sfr_register(&tmr1h,  0x0f, RegisterValue(0,0));
+  add_sfr_register(&tmr1l,  0x0e, RegisterValue(0,0),"tmr1l");
+  add_sfr_register(&tmr1h,  0x0f, RegisterValue(0,0),"tmr1h");
 
   add_sfr_register(&pcon,   0x8e, RegisterValue(0,0),"pcon");
 
@@ -253,6 +263,7 @@ void P16X6X_processor::create_sfr_map()
     add_sfr_register(&ssp.sspcon,  0x14, RegisterValue(0,0),"sspcon");
     add_sfr_register(&ssp.sspadd,  0x93, RegisterValue(0,0),"sspadd");
     add_sfr_register(&ssp.sspstat, 0x94, RegisterValue(0,0),"sspstat");
+    tmr2.ssp_module = &ssp;
   }
 
   add_sfr_register(&ccpr1l,  0x15, RegisterValue(0,0));
@@ -260,7 +271,7 @@ void P16X6X_processor::create_sfr_map()
   add_sfr_register(&ccp1con, 0x17, RegisterValue(0,0));
 
   // get_pir_set()->set_pir1(get_pir1());
-  pir_set_def.set_pir1(&pir1_reg);
+  pir_set_def.set_pir1(pir1);
 
   intcon = &intcon_reg;
   intcon_reg.set_pir_set(get_pir_set());
@@ -270,7 +281,7 @@ void P16X6X_processor::create_sfr_map()
 
   tmr1l.tmrh = &tmr1h;
   tmr1l.t1con = &t1con;
-  tmr1l.pir_set  = get_pir_set();
+  tmr1l.setInterruptSource(new InterruptSource(pir1, PIR1v1::TMR1IF));
   tmr1l.ccpcon = &ccp1con;
 
   tmr1h.tmrl  = &tmr1l;
@@ -298,17 +309,27 @@ void P16X6X_processor::create_sfr_map()
   ccpr1h.new_name("ccpr1h");
   ccp1con.new_name("ccp1con");
 
-  get_pir1()->intcon = &intcon_reg;
-  get_pir1()->pie    = &pie1;
-  pie1.pir    = get_pir1();
+  if (pir1) {
+    pir1->set_intcon(&intcon_reg);
+    pir1->set_pie(&pie1);
+  }
+  pie1.pir    = pir1;
   pie1.new_name("pie1");
 
 
 }
+//--------------------------------------------------
+
+void P16X6X_processor::create_symbols()
+{
+  Pic14Bit::create_symbols();
+}
+
 
 //--------------------------------------------------
 
-P16X6X_processor::P16X6X_processor(void)
+P16X6X_processor::P16X6X_processor(const char *_name, const char *_desc)
+  : Pic14Bit(_name,_desc)
 {
   
 
@@ -319,8 +340,15 @@ P16X6X_processor::P16X6X_processor(void)
   m_portc = new PicPortRegister("portc",8,0xff);
   m_trisc = new PicTrisRegister("trisc",m_portc);
 
+  pir1 = new PIR1v1(&intcon_reg, &pie1);
+  pir2 = new PIR2v1(&intcon_reg, &pie2);
+
 }
 
+P16X6X_processor::~P16X6X_processor()
+{
+  
+}
 /*******************************************************************
  *
  *        Definitions for the various P16x6x processors
@@ -328,7 +356,8 @@ P16X6X_processor::P16X6X_processor(void)
  */
 
 
-P16C62::P16C62(void)
+P16C62::P16C62(const char *_name, const char *desc)
+  : P16X6X_processor(_name,desc)
 {
   if(verbose)
     cout << "c62 constructor, type = " << isa() << '\n';
@@ -344,6 +373,9 @@ void P16C62::create_sfr_map()
 
   P16X6X_processor::create_sfr_map();
 
+  add_sfr_register(m_portc, 0x07);
+  add_sfr_register(m_trisc, 0x87, RegisterValue(0xff,0));
+
   //1((PORTC*)portc)->ccp1con = &ccp1con;
 
 }
@@ -375,18 +407,17 @@ void  P16C62::create(void)
 
 }
 
-Processor * P16C62::construct(void)
+Processor * P16C62::construct(const char *name)
 {
 
-  P16C62 *p = new P16C62;
+  P16C62 *p = new P16C62(name);
 
   cout << " c62 construct\n";
 
   p->create();
   p->create_invalid_registers ();
-  p->pic_processor::create_symbols();
+  p->create_symbols();
 
-  p->new_name("p16c62");
   symbol_table.add_module(p,p->name().c_str());
 
   return p;
@@ -407,7 +438,7 @@ void P16C63::create_sfr_map(void)
 
   add_file_registers(0xc0, 0xff, 0);
 
-  add_sfr_register(get_pir2(),   0x0d, RegisterValue(0,0),"pir2");
+  add_sfr_register(pir2,    0x0d, RegisterValue(0,0),"pir2");
   add_sfr_register(&pie2,   0x8d, RegisterValue(0,0));
 
   add_sfr_register(&ccpr2l, 0x1b, RegisterValue(0,0));
@@ -415,16 +446,16 @@ void P16C63::create_sfr_map(void)
   add_sfr_register(&ccp2con, 0x1d, RegisterValue(0,0));
 
   // get_pir_set()->set_pir2(get_pir2());
-  pir_set_def.set_pir2(&pir2_reg);
+  pir_set_def.set_pir2(pir2);
 
   ccp2con.setCrosslinks(&ccpr2l, get_pir_set(), &tmr2);
-
+  ccp2con.setIOpin(&((*m_portc)[1]));
   ccpr2l.ccprh  = &ccpr2h;
   ccpr2l.tmrl   = &tmr1l;
   ccpr2h.ccprl  = &ccpr2l;
 
   usart.initialize(get_pir_set(),&(*m_portc)[6], &(*m_portc)[7],
-		   new _TXREG(), new _RCREG());
+		   new _TXREG(&usart), new _RCREG(&usart));
 
   add_sfr_register(&usart.rcsta, 0x18, RegisterValue(0,0),"rcsta");
   add_sfr_register(&usart.txsta, 0x98, RegisterValue(2,0),"txsta");
@@ -436,13 +467,15 @@ void P16C63::create_sfr_map(void)
   ccpr2h.new_name("ccpr2h");
   ccp2con.new_name("ccp2con");
 
-  get_pir2()->intcon = &intcon_reg;
-  get_pir2()->pie    = &pie2;
+  if (pir2) {
+    pir2->set_intcon(&intcon_reg);
+    pir2->set_pie(&pie2);
+  }
+
   pie2.pir    = get_pir2();
   pie2.new_name("pie2");
 
 
-  //1((PORTC*)portc)->usart = &usart;
 }
 
 void P16C63::create_symbols(void)
@@ -464,7 +497,8 @@ void P16C63::create_symbols(void)
 // is called, the C62 constructor will be called. Most of the initialization
 // is done within the 'C62 constructor.
 
-P16C63::P16C63(void)
+P16C63::P16C63(const char *_name, const char *desc)
+  : P16C62(_name,desc)
 {
 
   if(verbose)
@@ -486,10 +520,10 @@ void P16C63::create(void)
 
 }
 
-Processor * P16C63::construct(void)
+Processor * P16C63::construct(const char *name)
 {
 
-  P16C63 *p = new P16C63;
+  P16C63 *p = new P16C63(name);
 
   if(verbose)
     cout << " c63 construct\n";
@@ -497,9 +531,8 @@ Processor * P16C63::construct(void)
   p->create();
   p->create_invalid_registers ();
 
-  p->pic_processor::create_symbols();
+  p->create_symbols();
 
-  p->new_name("p16c63");
   symbol_table.add_module(p,p->name().c_str());
 
   return p;
@@ -516,6 +549,8 @@ void P16C64::create_sfr_map(void)
    if(verbose)
     cout << "creating c64 registers\n";
 
+  pir_set_2_def.set_pir1(&pir1_2_reg);
+
 
   P16X6X_processor::create_sfr_map();
 
@@ -567,35 +602,41 @@ void  P16C64::create(void)
 
 }
 
-Processor * P16C64::construct(void)
+Processor * P16C64::construct(const char *name)
 {
 
-  P16C64 *p = new P16C64;
+  P16C64 *p = new P16C64(name);
 
   p->create();
   p->create_invalid_registers ();
   p->create_symbols();
 
-  p->new_name("p16c64");
   symbol_table.add_module(p,p->name().c_str());
 
   return p;
 
 }
 
-P16C64::P16C64(void)
+P16C64::P16C64(const char *_name, const char *desc)
+  : P16X6X_processor(_name,desc), 
+  pir1_2_reg(&intcon_reg,&pie1)
 {
   if(verbose)
     cout << "c64 constructor, type = " << isa() << '\n';
 
-  m_portd = new PicPortRegister("portd",8,0xff);
-  m_trisd = new PicTrisRegister("trisd",m_portd);
+  pir1 = &pir1_2_reg;
+
+
+  m_portd = new PicPSP_PortRegister("portd",8,0xff);
+  m_trisd = new PicTrisRegister("trisd",(PicPortRegister *)m_portd);
 
   m_porte = new PicPortRegister("porte",8,0x07);
-  m_trise = new PicTrisRegister("trise",m_porte);
+  m_trise =  new PicPSP_TrisRegister("trise",m_porte);
 
 }
-
+P16C64::~P16C64()
+{
+}
 //------------------------------------------------------------------------
 //
 //
@@ -606,11 +647,11 @@ void P16C65::create_sfr_map(void)
   if(verbose)
     cout << "creating c65 registers\n";
 
-  P16C64::create_sfr_map();
+  //P16C64::create_sfr_map();
 
   add_file_registers(0xc0, 0xff, 0);
 
-  add_sfr_register(get_pir2(),   0x0d, RegisterValue(0,0),"pir2");
+  add_sfr_register(pir2,    0x0d, RegisterValue(0,0),"pir2");
   add_sfr_register(&pie2,   0x8d, RegisterValue(0,0));
 
   add_sfr_register(&ccpr2l, 0x1b, RegisterValue(0,0));
@@ -618,16 +659,18 @@ void P16C65::create_sfr_map(void)
   add_sfr_register(&ccp2con, 0x1d, RegisterValue(0,0));
 
   // get_pir_set()->set_pir2(&get_pir2());
-  pir_set_def.set_pir2(&pir2_reg);
+  pir_set_def.set_pir2(pir2);
+
 
   ccp2con.setCrosslinks(&ccpr2l, get_pir_set(), &tmr2);
+  ccp2con.setIOpin(&((*m_portc)[1]));
 
   ccpr2l.ccprh  = &ccpr2h;
   ccpr2l.tmrl   = &tmr1l;
   ccpr2h.ccprl  = &ccpr2l;
 
   usart.initialize(get_pir_set(),&(*m_portc)[6], &(*m_portc)[7],
-		   new _TXREG(), new _RCREG());
+		   new _TXREG(&usart), new _RCREG(&usart));
 
   add_sfr_register(&usart.rcsta, 0x18, RegisterValue(0,0),"rcsta");
   add_sfr_register(&usart.txsta, 0x98, RegisterValue(2,0),"txsta");
@@ -639,13 +682,14 @@ void P16C65::create_sfr_map(void)
   ccpr2h.new_name("ccpr2h");
   ccp2con.new_name("ccp2con");
 
-  get_pir2()->intcon = &intcon_reg;
-  get_pir2()->pie    = &pie2;
+  if (pir2) {
+    pir2->set_intcon(&intcon_reg);
+    pir2->set_pie(&pie2);
+  }
+
   pie2.pir    = get_pir2();
   pie2.new_name("pie2");
 
-
-  //1((PORTC*)portc)->usart = &usart;
 }
 
 void P16C65::create_symbols(void)
@@ -667,7 +711,8 @@ void P16C65::create_symbols(void)
 // is called, the C64 constructor will be called. Most of the initialization
 // is done within the 'C64 constructor.
 
-P16C65::P16C65(void)
+P16C65::P16C65(const char *_name, const char *desc)
+  : P16C64(_name,desc)
 {
 
   if(verbose)
@@ -689,10 +734,10 @@ void P16C65::create(void)
 
 }
 
-Processor * P16C65::construct(void)
+Processor * P16C65::construct(const char *name)
 {
 
-  P16C65 *p = new P16C65;
+  P16C65 *p = new P16C65(name);
 
   if(verbose)
     cout << " c65 construct\n";
@@ -702,7 +747,6 @@ Processor * P16C65::construct(void)
 
   p->create_symbols();
 
-  p->new_name("p16c65");
   symbol_table.add_module(p,p->name().c_str());
 
   return p;
