@@ -20,7 +20,7 @@ Boston, MA 02111-1307, USA.  */
 
 
 #include <stdio.h>
-#include <iostream.h>
+#include <iostream>
 #include <string>
 
 #include "symbol.h"
@@ -28,6 +28,7 @@ Boston, MA 02111-1307, USA.  */
 #include "../config.h"
 #include "p17c75x.h"
 
+#if 0
 void _68pins::create_iopin_map(void)
 {
   // ---- This is probably going to be moved:
@@ -85,37 +86,37 @@ void _68pins::create_iopin_map(void)
   create_pkg(68);
 
   // Vdd and Vss pins
-  assign_pin(2, NULL);
-  assign_pin(20, NULL);
-  assign_pin(37, NULL);
-  assign_pin(49, NULL);
-  assign_pin(19, NULL);
-  assign_pin(36, NULL);
-  assign_pin(53, NULL);
-  assign_pin(68, NULL);
+  assign_pin(2, 0);
+  assign_pin(20, 0);
+  assign_pin(37, 0);
+  assign_pin(49, 0);
+  assign_pin(19, 0);
+  assign_pin(36, 0);
+  assign_pin(53, 0);
+  assign_pin(68, 0);
 
   // AVdd and AVss pins
-  assign_pin(29, NULL);
-  assign_pin(30, NULL);
+  assign_pin(29, 0);
+  assign_pin(30, 0);
 
   // NC pins
-  assign_pin(1, NULL);
-  assign_pin(18, NULL);
-  assign_pin(35, NULL);
-  assign_pin(52, NULL);
+  assign_pin(1, 0);
+  assign_pin(18, 0);
+  assign_pin(35, 0);
+  assign_pin(52, 0);
 
   // Test pin
-  assign_pin(17, NULL);
+  assign_pin(17, 0);
 
   // Reset pin
-  assign_pin(16, NULL);
+  assign_pin(16, 0);
   
   // Oscillator pins
-  assign_pin(50, NULL);
-  assign_pin(51, NULL);
+  assign_pin(50, 0);
+  assign_pin(51, 0);
 
-  assign_pin(60, new IO_input(porta, 0));
-  assign_pin(44, new IO_input(porta, 1));
+  assign_pin(60, new IOPIN(porta, 0));
+  assign_pin(44, new IOPIN(porta, 1));
   assign_pin(45, new IO_bi_directional_pu(porta, 2));
   assign_pin(46, new IO_bi_directional_pu(porta, 3));
   assign_pin(43, new IO_bi_directional(porta, 4));
@@ -172,6 +173,7 @@ void _68pins::create_iopin_map(void)
   assign_pin(40, new IO_bi_directional(portg, 7));
 
 }
+#endif
 
 
 //========================================================================
@@ -180,7 +182,7 @@ void _68pins::create_iopin_map(void)
 //
 
 
-pic_processor * P17C7xx::construct(void)
+Processor * P17C7xx::construct(const char *name)
 {
 
   P17C7xx *p = new P17C7xx;
@@ -190,13 +192,15 @@ pic_processor * P17C7xx::construct(void)
   p->create(0x1fff);
   p->create_invalid_registers ();
   p->pic_processor::create_symbols();
-  p->name_str = "17c7xx";
+
+  p->new_name("p17c7xx");
+  symbol_table.add_module(p,p->name().c_str());
 
   return p;
 
 }
 
-P17C7xx::P17C7xx(void)
+P17C7xx::P17C7xx()
 {
   if(verbose)
     cout << "17c7xx constructor, type = " << isa() << '\n';
@@ -205,7 +209,7 @@ P17C7xx::P17C7xx(void)
   
   //  create_iopins(iopin_map, num_of_iopins);
   
-  name_str = "17c7xx";
+  name_str = "p17c7xx";
   
 }
 
@@ -228,7 +232,6 @@ void  P17C7xx::create(int ram_top)
   ind0.init(this);
   ind1.init(this);
   ind2.init(this);
-  trace.program_counter (pc.value);
   tmr0l.initialize();
   intcon.initialize();
 
@@ -248,11 +251,11 @@ void  P17C7xx::create(int ram_top)
 
 
 
-void P17C7xx::create_symbols(void)
+void P17C7xx::create_symbols()
 {
 }
 
-void P17C7xx::create_sfr_map(void)
+void P17C7xx::create_sfr_map()
 {  
   if(verbose)
     cout << "creating 17c7xx common registers\n";
@@ -266,20 +269,19 @@ void P17C7xx::create_sfr_map(void)
 // Pic 17C75x
 //
 
-pic_processor * P17C75x::construct(void)
+Processor * P17C75x::construct(const char *name)
 {
 
   cout << " 17c75x construct\n";
 
   P17C75x *p = new P17C75x;
-  cout << "1\n";
+
   p->create(0x1fff);
-  cout << "2\n";
   p->create_invalid_registers ();
-  cout << "3\n";
   p->pic_processor::create_symbols();
-  cout << "4\n";
-  p->name_str = "17c75x";
+
+  p->new_name("p17c75x");
+  symbol_table.add_module(p,p->name().c_str());
 
   return p;
 
@@ -304,13 +306,13 @@ void P17C75x::create(int ram_top)
 
 }
 
-P17C75x::P17C75x(void)
+P17C75x::P17C75x()
 {
   //if(verbose)
     cout << "17c75x constructor, type = " << isa() << '\n';
 }
 
-void P17C75x::create_symbols(void)
+void P17C75x::create_symbols()
 {
   if(verbose)
     cout << "p17c75x create symbols\n";
@@ -328,8 +330,9 @@ void P17C75x::create_symbols(void)
 */
 }
 
-void P17C75x::create_sfr_map(void)
+void P17C75x::create_sfr_map()
 {
+#if 0 
   if (verbose) 
     cout << "creating p17c75x common registers\n";
   
@@ -468,7 +471,7 @@ void P17C75x::create_sfr_map(void)
   // All of the status bits on the 16bit core are writable
   status.write_mask = 0xff;
 
-
+#endif
 }
 
 
@@ -477,7 +480,7 @@ void P17C75x::create_sfr_map(void)
 // Pic 17C756
 //
 
-pic_processor * P17C756::construct(void)
+Processor * P17C756::construct(const char *name)
 {
 
   P17C756 *p = new P17C756;
@@ -487,13 +490,15 @@ pic_processor * P17C756::construct(void)
   p->P17C7xx::create(0x1fff);
   p->create_invalid_registers ();
   p->pic_processor::create_symbols();
-  p->name_str = "17c756";
+
+  p->new_name("p17c756");
+  symbol_table.add_module(p,p->name().c_str());
 
   return p;
 
 }
 
-void P17C756::create(void)
+void P17C756::create()
 {
 
   create_iopin_map();
@@ -508,19 +513,19 @@ void P17C756::create(void)
 
 }
 
-void P17C756::create_sfr_map(void)
+void P17C756::create_sfr_map()
 {
 
   cout << "create_sfr_map P17C756\n";
 }
 
-void P17C756::create_symbols(void)
+void P17C756::create_symbols()
 {
   cout << "P17C756 create symbols\n";
 
 }
 
-P17C756::P17C756(void)
+P17C756::P17C756()
 {
 
   if(verbose)
@@ -534,7 +539,7 @@ P17C756::P17C756(void)
 // P17C756A
 // 
 
-pic_processor * P17C756A::construct(void)
+Processor * P17C756A::construct(const char *name)
 {
 
   P17C756A *p = new P17C756A;
@@ -544,13 +549,15 @@ pic_processor * P17C756A::construct(void)
   p->P17C7xx::create(0x1fff);
   //p->create_invalid_registers ();
   p->pic_processor::create_symbols();
-  p->name_str = "17c756a";
+
+  p->new_name("p17c756a");
+  symbol_table.add_module(p,p->name().c_str());
 
   return p;
 
 }
 
-P17C756A::P17C756A(void)
+P17C756A::P17C756A()
 {
 
   if(verbose)
@@ -558,7 +565,7 @@ P17C756A::P17C756A(void)
 
 }
 
-void P17C756A::create(void)
+void P17C756A::create()
 {
 
   create_iopin_map();
@@ -576,13 +583,13 @@ void P17C756A::create(void)
 }
 
 
-void P17C756A::create_sfr_map(void)
+void P17C756A::create_sfr_map()
 {
 
   cout << "create_sfr_map P17C756A\n";
 }
 
-void P17C756A::create_symbols(void)
+void P17C756A::create_symbols()
 {
   cout << "P17C756A create symbols\n";
 
@@ -594,7 +601,7 @@ void P17C756A::create_symbols(void)
 // Pic 17C752
 //
 
-pic_processor * P17C752::construct(void)
+Processor * P17C752::construct(const char *name)
 {
 
   P17C752 *p = new P17C752;
@@ -604,13 +611,15 @@ pic_processor * P17C752::construct(void)
   p->P17C7xx::create(0x1fff);
   p->create_invalid_registers ();
   p->pic_processor::create_symbols();
-  p->name_str = "17c752";
+
+  p->new_name("p17c752");
+  symbol_table.add_module(p,p->name().c_str());
 
   return p;
 
 }
 
-void P17C752::create(void)
+void P17C752::create()
 {
 
   create_iopin_map();
@@ -625,19 +634,19 @@ void P17C752::create(void)
 
 }
 
-void P17C752::create_sfr_map(void)
+void P17C752::create_sfr_map()
 {
 
   cout << "create_sfr_map P17C752\n";
 }
 
-void P17C752::create_symbols(void)
+void P17C752::create_symbols()
 {
   cout << "P17C752 create symbols\n";
 
 }
 
-P17C752::P17C752(void)
+P17C752::P17C752()
 {
 
   if(verbose)
@@ -650,7 +659,7 @@ P17C752::P17C752(void)
 // Pic 17C762
 //
 
-pic_processor * P17C762::construct(void)
+Processor * P17C762::construct(const char *name)
 {
 
   P17C762 *p = new P17C762;
@@ -660,13 +669,15 @@ pic_processor * P17C762::construct(void)
   p->P17C7xx::create(0x1fff);
   p->create_invalid_registers ();
   p->pic_processor::create_symbols();
-  p->name_str = "17c762";
+
+  p->new_name("p17c762");
+  symbol_table.add_module(p,p->name().c_str());
 
   return p;
 
 }
 
-void P17C762::create(void)
+void P17C762::create()
 {
 
   create_iopin_map();
@@ -681,19 +692,19 @@ void P17C762::create(void)
 
 }
 
-void P17C762::create_sfr_map(void)
+void P17C762::create_sfr_map()
 {
 
   cout << "create_sfr_map P17C762\n";
 }
 
-void P17C762::create_symbols(void)
+void P17C762::create_symbols()
 {
   cout << "P17C762 create symbols\n";
 
 }
 
-P17C762::P17C762(void)
+P17C762::P17C762()
 {
 
   if(verbose)
@@ -706,7 +717,7 @@ P17C762::P17C762(void)
 // Pic 17C766
 //
 
-pic_processor * P17C766::construct(void)
+Processor * P17C766::construct(const char *name)
 {
 
   P17C766 *p = new P17C766;
@@ -716,13 +727,15 @@ pic_processor * P17C766::construct(void)
   p->P17C7xx::create(0x1fff);
   p->create_invalid_registers ();
   p->pic_processor::create_symbols();
-  p->name_str = "17c766";
+
+  p->new_name("p17c766");
+  symbol_table.add_module(p,p->name().c_str());
 
   return p;
 
 }
 
-void P17C766::create(void)
+void P17C766::create()
 {
 
   create_iopin_map();
@@ -737,19 +750,19 @@ void P17C766::create(void)
 
 }
 
-void P17C766::create_sfr_map(void)
+void P17C766::create_sfr_map()
 {
 
   cout << "create_sfr_map P17C766\n";
 }
 
-void P17C766::create_symbols(void)
+void P17C766::create_symbols()
 {
   cout << "P17C766 create symbols\n";
 
 }
 
-P17C766::P17C766(void)
+P17C766::P17C766()
 {
 
   if(verbose)
