@@ -25,6 +25,7 @@ Boston, MA 02111-1307, USA.  */
 
 #include "../config.h"
 #include "p18x.h"
+#include "pic-ioports.h"
 #include "packages.h"
 #include "stimuli.h"
 #include "symbol.h"
@@ -92,7 +93,17 @@ void P18C2x2::create_iopin_map()
   package->assign_pin(27, m_portb->addPin(new IO_bi_directional_pu("portb6"),6));
   package->assign_pin(28, m_portb->addPin(new IO_bi_directional_pu("portb7"),7));
 
-  //1portc.ccp1con = &ccp1con;
+  tmr1l.setIOpin(&(*m_portc)[0]);
+  ssp.initialize(&pir_set_def,    // PIR
+                &(*m_portc)[3],   // SCK
+                &(*m_porta)[5],   // SS
+                &(*m_portc)[5],   // SDO
+                &(*m_portc)[4],   // SDI
+                m_trisc,         // i2c tris port
+		SSP_TYPE_MSSP
+       );
+
+
   //1portc.usart = &usart16;
 
 }
@@ -104,7 +115,8 @@ void P18C2x2::create_symbols()
   _16bit_processor::create_symbols();
 }
 
-P18C2x2::P18C2x2()
+P18C2x2::P18C2x2(const char *_name, const char *desc)
+  : _16bit_processor(_name,desc)
 {
 
   if(verbose)
@@ -119,7 +131,8 @@ P18C2x2::P18C2x2()
 // P18C242
 // 
 
-P18C242::P18C242()
+P18C242::P18C242(const char *_name, const char *desc)
+  : P18C2x2(_name,desc)
 {
 
   if(verbose)
@@ -137,15 +150,14 @@ void P18C242::create()
 
 }
 
-Processor * P18C242::construct()
+Processor * P18C242::construct(const char *name)
 {
 
-  P18C242 *p = new P18C242;
+  P18C242 *p = new P18C242(name);
 
   if(verbose)
     cout << " 18c242 construct\n";
 
-  p->new_name("p18c242");
   p->create();
   p->create_invalid_registers();
   p->create_symbols();
@@ -162,7 +174,8 @@ Processor * P18C242::construct()
 // P18C252
 // 
 
-P18C252::P18C252()
+P18C252::P18C252(const char *_name, const char *desc)
+  : P18C242(_name,desc)
 {
 
   if(verbose)
@@ -182,10 +195,10 @@ void P18C252::create()
 }
 
 
-Processor * P18C252::construct()
+Processor * P18C252::construct(const char *name)
 {
 
-  P18C252 *p = new P18C252;
+  P18C252 *p = new P18C252(name);;
 
   if(verbose)
     cout << " 18c252 construct\n";
@@ -194,7 +207,6 @@ Processor * P18C252::construct()
   p->create_invalid_registers();
   p->create_symbols();
 
-  p->new_name("p18c252");
   symbol_table.add_module(p,p->name().c_str());
 
   return p;
@@ -280,7 +292,24 @@ void P18C4x2::create_iopin_map()
   package->assign_pin(39, m_portb->addPin(new IO_bi_directional_pu("portb6"),6));
   package->assign_pin(40, m_portb->addPin(new IO_bi_directional_pu("portb7"),7));
 
+  psp.initialize(&pir_set_def,    // PIR
+                m_portd,           // Parallel port
+                m_trisd,           // Parallel tris
+                m_trise,           // Control tris
+                &(*m_porte)[0],    // NOT RD
+                &(*m_porte)[1],    // NOT WR
+                &(*m_porte)[2]);   // NOT CS
 
+  tmr1l.setIOpin(&(*m_portc)[0]);
+
+  ssp.initialize(&pir_set_def,    // PIR
+                &(*m_portc)[3],   // SCK
+                &(*m_porta)[5],   // SS
+                &(*m_portc)[5],   // SDO
+                &(*m_portc)[4],   // SDI
+                m_trisc,         // i2c tris port
+		SSP_TYPE_MSSP
+       );
   //1portc.ccp1con = &ccp1con;
   //1portc.usart = &usart16;
 
@@ -295,18 +324,19 @@ void P18C4x2::create_symbols()
 
 }
 
-P18C4x2::P18C4x2()
+P18C4x2::P18C4x2(const char *_name, const char *desc)
+  : _16bit_processor(_name,desc)
 {
 
   if(verbose)
     cout << "18c4x2 constructor, type = " << isa() << '\n';
 
-  m_portd = new PicPortRegister("portd",8,0xff);
-  m_trisd = new PicTrisRegister("trisd", m_portd);
+  m_portd = new PicPSP_PortRegister("portd",8,0xff);
+  m_trisd = new PicTrisRegister("trisd", (PicPortRegister *)m_portd);
   m_latd  = new PicLatchRegister("latd", m_portd);
 
   m_porte = new PicPortRegister("porte",8,0x07);
-  m_trise = new PicTrisRegister("trise", m_porte);
+  m_trise = new PicPSP_TrisRegister("trise", m_porte);
   m_late  = new PicLatchRegister("late", m_porte);
 
 }
@@ -331,6 +361,11 @@ void P18C4x2::create_sfr_map()
   add_sfr_register(m_trisd,       0xf95,RegisterValue(0xff,0));
   add_sfr_register(m_trise,       0xf96,RegisterValue(0x07,0));
 
+  // rest of configureation in parent class
+  adcon1.setNumberOfChannels(8);
+  adcon1.setIOPin(5, &(*m_porte)[0]);
+  adcon1.setIOPin(6, &(*m_porte)[1]);
+  adcon1.setIOPin(7, &(*m_porte)[2]);
 
   //1 usart16.initialize_16(this,&pir_set_def,&portc);
 
@@ -343,7 +378,8 @@ void P18C4x2::create_sfr_map()
 // P18C442
 // 
 
-P18C442::P18C442()
+P18C442::P18C442(const char *_name, const char *desc)
+  : P18C4x2(_name,desc)
 {
 
   if(verbose)
@@ -361,10 +397,10 @@ void P18C442::create()
 
 }
 
-Processor * P18C442::construct()
+Processor * P18C442::construct(const char *name)
 {
 
-  P18C442 *p = new P18C442;
+  P18C442 *p = new P18C442(name);
 
   if(verbose)
     cout << " 18c442 construct\n";
@@ -373,7 +409,6 @@ Processor * P18C442::construct()
   p->create_invalid_registers();
   p->create_symbols();
 
-  p->new_name("p18c442");
   symbol_table.add_module(p,p->name().c_str());
 
   return p;
@@ -388,7 +423,8 @@ Processor * P18C442::construct()
 // P18C452
 // 
 
-P18C452::P18C452()
+P18C452::P18C452(const char *_name, const char *desc)
+  : P18C442(_name,desc)
 {
 
   if(verbose)
@@ -405,11 +441,10 @@ void P18C452::create()
   P18C442::create();
 }
 
-Processor * P18C452::construct()
+Processor * P18C452::construct(const char *name)
 {
 
-  P18C452 *p = new P18C452;
-  p->new_name("p18c452");
+  P18C452 *p = new P18C452(name);
 
   if(verbose)
     cout << " 18c452 construct\n";
@@ -431,7 +466,8 @@ Processor * P18C452::construct()
 // P18F242
 // 
 
-P18F242::P18F242()
+P18F242::P18F242(const char *_name, const char *desc)
+  : P18C242(_name,desc)
 {
 
   if(verbose)
@@ -446,19 +482,17 @@ void P18F242::create()
   if(verbose)
     cout << " 18f242 create \n";
 
-  P18C242::create();
-
-  e = new EEPROM_PIR;
+  e = new EEPROM_PIR(&pir2);
   e->set_cpu(this);
   // We might want to pass this value in for larger eeproms
   e->initialize(256);
-  //e->set_pir_set(get_pir_set());
-  e->set_pir_set(&pir_set_def);
+  //e->set_pir_set(&pir_set_def);
   e->set_intcon(&intcon);
 
   // assign this eeprom to the processor
   set_eeprom_pir(e);
 
+  P18C242::create();
 }
 
 /*
@@ -484,11 +518,10 @@ void P18F242::set_out_of_range_pm(unsigned int address, unsigned int value)
     }
 }
 
-Processor * P18F242::construct()
+Processor * P18F242::construct(const char *name)
 {
 
-  P18F242 *p = new P18F242;
-  p->new_name("p18f242");
+  P18F242 *p = new P18F242(name);
 
   if(verbose)
     cout << " 18F242 construct\n";
@@ -511,7 +544,9 @@ Processor * P18F242::construct()
 // P18F252
 // 
 
-P18F252::P18F252()
+P18F252::P18F252(const char *_name, const char *desc)
+  : P18F242(_name,desc)
+
 {
 
   if(verbose)
@@ -527,11 +562,10 @@ void P18F252::create()
   P18F242::create();
 
 }
-Processor * P18F252::construct()
+Processor * P18F252::construct(const char *name)
 {
 
-  P18F252 *p = new P18F252;
-  p->new_name("p18f252");
+  P18F252 *p = new P18F252(name);
 
   if(verbose)
     cout << " 18F252 construct\n";
@@ -552,7 +586,9 @@ Processor * P18F252::construct()
 // P18F442
 // 
 
-P18F442::P18F442()
+P18F442::P18F442(const char *_name, const char *desc)
+  : P18C442(_name,desc)
+
 {
 
   if(verbose)
@@ -567,19 +603,18 @@ void P18F442::create()
   if(verbose)
     cout << " 18f442 create \n";
 
-  P18C442::create();
-
-  e = new EEPROM_PIR;
+  e = new EEPROM_PIR(&pir2);
   e->set_cpu(this);
   // We might want to pass this value in for larger eeproms
   e->initialize(256);
   //e->set_pir_set(get_pir_set());
-  e->set_pir_set(&pir_set_def);
+  //e->set_pir_set(&pir_set_def);
   e->set_intcon(&intcon);
 
   // assign this eeprom to the processor
   set_eeprom_pir(e);
 
+  P18C442::create();
 }
 /*
 void P18F442::create_sfr_map()
@@ -603,11 +638,10 @@ void P18F442::set_out_of_range_pm(unsigned int address, unsigned int value)
     }
 }
 
-Processor * P18F442::construct()
+Processor * P18F442::construct(const char *name)
 {
 
-  P18F442 *p = new P18F442;
-  p->new_name("p18f442");
+  P18F442 *p = new P18F442(name);
 
   if(verbose)
     cout << " 18F442 construct\n";
@@ -629,7 +663,8 @@ Processor * P18F442::construct()
 // P18F258
 // 
 
-P18F248::P18F248()
+P18F248::P18F248(const char *_name, const char *desc)
+  : P18F242(_name,desc)
 {
 
   if(verbose)
@@ -643,14 +678,13 @@ void P18F248::create()
   if(verbose)
     cout << " 18f248 create \n";
 
-  P18F442::create();
+  P18F242::create();
 }
 
-Processor * P18F248::construct()
+Processor * P18F248::construct(const char *name)
 {
 
-  P18F248 *p = new P18F248;
-  p->new_name("p18f248");
+  P18F248 *p = new P18F248(name);
 
   if(verbose)
     cout << " 18F248 construct\n";
@@ -667,12 +701,56 @@ Processor * P18F248::construct()
 }
 
 
+//------------------------------------------------------------------------
+//
+// P18F458
+// 
+
+P18F448::P18F448(const char *_name, const char *desc)
+  : P18F442(_name,desc)
+{
+
+  if(verbose)
+    cout << "18f448 constructor, type = " << isa() << '\n';
+
+}
+
+void P18F448::create()
+{
+
+  if(verbose)
+    cout << " 18f448 create \n";
+
+  P18F442::create();
+}
+
+Processor * P18F448::construct(const char *name)
+{
+
+  P18F448 *p = new P18F448(name);
+
+  if(verbose)
+    cout << " 18F448 construct\n";
+
+  p->create();
+  p->create_invalid_registers();
+  p->create_symbols();
+
+  symbol_table.add_module(p,p->name().c_str());
+
+  return p;
+
+
+}
+
+
 //------------------------------------------------------------------------
 //
 // P18F452
 // 
 
-P18F452::P18F452()
+P18F452::P18F452(const char *_name, const char *desc)
+  : P18F442(_name,desc)
 {
 
   if(verbose)
@@ -689,11 +767,10 @@ void P18F452::create()
   P18F442::create();
 }
 
-Processor * P18F452::construct()
+Processor * P18F452::construct(const char *name)
 {
 
-  P18F452 *p = new P18F452;
-  p->new_name("p18f452");
+  P18F452 *p = new P18F452(name);
 
   if(verbose)
     cout << " 18F452 construct\n";
@@ -709,17 +786,22 @@ Processor * P18F452::construct()
 
 }
 
+//------------------------------------------------------------------------
+
+P18Fxx20::P18Fxx20(const char *_name, const char *desc)
+  : _16bit_processor(_name,desc)
+{
+}
 
 //------------------------------------------------------------------------
 //
 // P18F1220
 // 
 
-Processor * P18F1220::construct()
+Processor * P18F1220::construct(const char *name)
 {
 
-  P18F1220 *p = new P18F1220;
-  p->new_name("p18f1220");
+  P18F1220 *p = new P18F1220(name);
 
   if(verbose)
     cout << " 18F1220 construct\n";
@@ -775,7 +857,8 @@ void P18F1220::create_iopin_map()
 
 }
 
-P18F1220::P18F1220()
+P18F1220::P18F1220(const char *_name, const char *desc)
+  : P18Fxx20(_name,desc)
 {
 
   if(verbose)
@@ -790,7 +873,8 @@ P18F1220::P18F1220()
 // P18Fx320
 // 
 
-P18F1320::P18F1320()
+P18F1320::P18F1320(const char *_name, const char *desc)
+  : P18F1220(_name,desc)
 {
 
   if(verbose)
@@ -809,15 +893,14 @@ void P18F1320::create()
 
 }
 
-Processor * P18F1320::construct()
+Processor * P18F1320::construct(const char *name)
 {
 
-  P18F1320 *p = new P18F1320;
+  P18F1320 *p = new P18F1320(name);
 
   if(verbose)
     cout << " 18F1320 construct\n";
 
-  p->new_name("p18f1320");
   p->create();
   p->create_invalid_registers();
   p->create_symbols();
