@@ -31,35 +31,22 @@ Boston, MA 02111-1307, USA.  */
 #include "xref.h"
 
 
-instruction::instruction()
+instruction::instruction(Processor *pProcessor,
+			 unsigned int uOpCode, 
+			 unsigned int uAddrOfInstr)
+  : gpsimValue(pProcessor),
+    m_bIsModified(false),
+    cycle_count(0),
+    opcode(uOpCode), 
+    m_uAddrOfInstr(uAddrOfInstr),
+    file_id(-1),
+    hll_file_id(-1),
+    src_line(-1),
+    lst_line(-1),
+    hll_src_line(-1)
 {
-  Initialize(0, 0, 0);
 }
 
-instruction::instruction(Processor *pProcessor, unsigned int uOpCode, unsigned int uAddrOfInstr)
-{
-  Initialize(pProcessor, uOpCode, uAddrOfInstr);
-}
-
-void instruction::Initialize(Processor *pProcessor, unsigned int uOpCode, unsigned int uAddrOfInstr)
-{
-  cpu = pProcessor;
-  opcode = uOpCode;
-
-  m_uAddrOfInstr = uAddrOfInstr;
-
-  m_bIsModified = false;
-  cycle_count = 0;
-
-  file_id = -1;
-  hll_file_id = -1;
-  src_line = -1;
-  lst_line = -1;
-  hll_src_line = -1;
-
-}
-
-
 void instruction::decode(Processor *new_cpu, unsigned int new_opcode)
 {
   cpu = new_cpu;
@@ -80,9 +67,16 @@ void invalid_instruction::execute(void)
 };
 
 invalid_instruction::invalid_instruction(Processor *new_cpu,unsigned int new_opcode)
+  : instruction(new_cpu,new_opcode,0)
+{
+  new_name("INVALID");
+}
+
+invalid_instruction::invalid_instruction(Processor *new_cpu,
+                                         unsigned int address, 
+                                         unsigned int new_opcode)
+  : instruction(new_cpu,new_opcode,address)
 {
-  cpu=new_cpu;
-  opcode=new_opcode;
   new_name("INVALID");
 }
 
@@ -97,24 +91,54 @@ void instruction::add_line_number_symbol(int address)
   symbol_table.add_line_number(address);
 
 }
+//------------------------------------------------------------------------
+// update_line_number(int file, int sline, int lline, int hllfile, int hllsline)
+// 
+// map the instruction to the source file that generated it.
+// If a line number is less than 0, then this means that the
+// existing mapping should remain unchanged. This allows this 
+// method to be called multiple times; once for each type of
+// mapping.
 
 void instruction::update_line_number(int file, int sline, int lline, int hllfile, int hllsline)
 {
-  file_id = file;
-  src_line = sline;
-  lst_line = lline;
-  hll_src_line = hllsline;
-  hll_file_id = hllfile;
+  file_id      = file<0     ? file_id      : file;
+  src_line     = sline<0    ? src_line     : sline;
+  lst_line     = lline<0    ? lst_line     : lline;
+  hll_src_line = hllsline<0 ? hll_src_line : hllsline;
+  hll_file_id  = hllfile<0  ? hll_file_id  : hllfile;
+}
+char *instruction::ReadSrcLine(char *buf, int nBytes)
+{
+  if (buf && nBytes && cpu)
+    return ((Processor *)cpu)->files.ReadLine(file_id,src_line,buf,nBytes);
+
+  return buf;
+}
+char *instruction::ReadHLLLine(char *buf, int nBytes)
+{
+  if (buf && nBytes && cpu)
+    return ((Processor *)cpu)->files.ReadLine(hll_file_id,hll_src_line,buf,nBytes);
+
+  return buf;
+}
+
+char *instruction::ReadLstLine(char *buf, int nBytes)
+{
+  if (buf && nBytes && cpu)
+    return ((Processor *)cpu)->files.ReadLine(((Processor *)cpu)->files.list_id(),lst_line,buf,nBytes);
+
+  return buf;
 }
 
 //------------------------------------------------------------------------
 AliasedInstruction::AliasedInstruction(instruction *_replaced)
-  : instruction(), m_replaced(_replaced)
+  : instruction(0,0,0), m_replaced(_replaced)
 {
 }
 
 AliasedInstruction::AliasedInstruction()
-  : instruction(), m_replaced(0)
+  : instruction(0,0,0), m_replaced(0)
 {
 }
 
@@ -181,7 +205,7 @@ int AliasedInstruction::get_hll_file_id()
 {
   return getReplaced()->get_hll_file_id();
 }
-instruction::INSTRUCTION_TYPES AliasedInstruction::isa()
+enum instruction::INSTRUCTION_TYPES AliasedInstruction::isa()
 {
   return getReplaced()->isa();
 }
@@ -212,6 +236,13 @@ bool AliasedInstruction::isBase()
 
 
 //------------------------------------------------------------------------
+Literal_op::Literal_op(Processor *pProcessor, 
+		       unsigned int uOpCode, 
+		       unsigned int uAddrOfInstr)
+  : instruction(pProcessor, uOpCode, uAddrOfInstr),
+    L(uOpCode&0xff)
+{
+}
 char *Literal_op::name(char *return_str,int len)
 {
 
@@ -228,6 +259,16 @@ void Literal_op::decode(Processor *new_cpu, unsigned int new_opcode)
   L = opcode & 0xff;
 }
 
+Bit_op::Bit_op(Processor *pProcessor, 
+		       unsigned int uOpCode, 
+		       unsigned int uAddrOfInstr)
+  : instruction(pProcessor, uOpCode, uAddrOfInstr),
+    mask(0),register_address(0),
+    access(false),
+    reg(0)
+{
+}
+
 void Bit_op::decode(Processor *new_cpu, unsigned int new_opcode)
 {
   opcode = new_opcode;
@@ -247,6 +288,7 @@ void Bit_op::decode(Processor *new_cpu, unsigned int new_opcode)
 	case  _P18F252_:
 	case  _P18F442_:
 	case  _P18F248_:
+	case  _P18F448_:
 	case  _P18F452_:
 	case  _P18F1220_:
 	case  _P18F1320_:
@@ -335,6 +377,16 @@ char * Bit_op::name(char *return_str,int len)
 }
 
 
+//----------------------------------------------------------------
+Register_op::Register_op(Processor *pProcessor, 
+		       unsigned int uOpCode, 
+		       unsigned int uAddrOfInstr)
+  : instruction(pProcessor, uOpCode, uAddrOfInstr),
+    register_address(0),
+    destination(false), access(false)
+{
+}
+
 //----------------------------------------------------------------
 //
 // Register_op::name
@@ -412,6 +464,7 @@ Register * Register_op::source = 0;
 //--------------------------------------------------
 
 ADDWF::ADDWF (Processor *new_cpu, unsigned int new_opcode)
+  : Register_op(new_cpu, new_opcode, 0)
 {
 
   decode(new_cpu, new_opcode);
@@ -423,8 +476,6 @@ void ADDWF::execute(void)
 {
   unsigned int new_value,src_value,w_value;
 
-  // trace.instruction(opcode);
-
   if(!access)
     source = cpu_pic->registers[register_address];
   else
@@ -448,19 +499,16 @@ void ADDWF::execute(void)
 //--------------------------------------------------
 
 ANDLW::ANDLW (Processor *new_cpu, unsigned int new_opcode)
+  : Literal_op(new_cpu, new_opcode, 0)
 {
-
   decode(new_cpu, new_opcode);
   new_name("andlw");
-
 }
 
 void ANDLW::execute(void)
 {
   unsigned int new_value;
 
-  // trace.instruction(opcode);
-
   new_value = cpu_pic->W->value.get() & L;
 
   cpu_pic->W->put(new_value);
@@ -473,6 +521,7 @@ void ANDLW::execute(void)
 //--------------------------------------------------
 
 ANDWF::ANDWF (Processor *new_cpu, unsigned int new_opcode)
+  : Register_op(new_cpu, new_opcode, 0)
 {
 
   decode(new_cpu, new_opcode);
@@ -484,8 +533,6 @@ void ANDWF::execute(void)
 {
   unsigned int new_value;
 
-  // trace.instruction(opcode);
-
   if(!access)
     source = cpu_pic->registers[register_address];
   else
@@ -508,21 +555,16 @@ void ANDWF::execute(void)
 //--------------------------------------------------
 
 BCF::BCF (Processor *new_cpu, unsigned int new_opcode)
+  : Bit_op(new_cpu, new_opcode, 0)
 {
-
   decode(new_cpu, new_opcode);
-
   mask ^= 0xff;        // decode initializes the mask to 1<<bit
-
   new_name("bcf");
-
 }
 
 void BCF::execute(void)
 {
 
-  // trace.instruction(opcode);
-
   if(!access)
     reg = cpu_pic->registers[register_address];
   else
@@ -537,19 +579,15 @@ void BCF::execute(void)
 //--------------------------------------------------
 
 BSF::BSF (Processor *new_cpu, unsigned int new_opcode)
+  : Bit_op(new_cpu, new_opcode, 0)
 {
-
   decode(new_cpu, new_opcode);
-
   new_name("bsf");
-
 }
 
 void BSF::execute(void)
 {
 
-  // trace.instruction(opcode);
-
   if(!access)
     reg = cpu_pic->registers[register_address];
   else
@@ -565,19 +603,15 @@ void BSF::execute(void)
 //--------------------------------------------------
 
 BTFSC::BTFSC (Processor *new_cpu, unsigned int new_opcode)
+  : Bit_op(new_cpu, new_opcode, 0)
 {
-
   decode(new_cpu, new_opcode);
-
   new_name("btfsc");
-
 }
 
 void BTFSC::execute(void)
 {
 
-  // trace.instruction(opcode);
-
   if(!access)
     reg = cpu_pic->registers[register_address];
   else
@@ -595,19 +629,15 @@ void BTFSC::execute(void)
 //--------------------------------------------------
 
 BTFSS::BTFSS (Processor *new_cpu, unsigned int new_opcode)
+  : Bit_op(new_cpu, new_opcode, 0)
 {
-
   decode(new_cpu, new_opcode);
-
   new_name("btfss");
-
 }
 
 void BTFSS::execute(void)
 {
 
-  // trace.instruction(opcode);
-
   if(!access)
     reg = cpu_pic->registers[register_address];
   else
@@ -624,9 +654,8 @@ void BTFSS::execute(void)
 
 //--------------------------------------------------
 CALL::CALL (Processor *new_cpu, unsigned int new_opcode)
+  : instruction(new_cpu, new_opcode, 0)
 {
-  opcode = new_opcode;
-  cpu = new_cpu;
 
   switch(cpu_pic->base_isa())
     {
@@ -646,7 +675,6 @@ CALL::CALL (Processor *new_cpu, unsigned int new_opcode)
 
 void CALL::execute(void)
 {
-  // trace.instruction(opcode);
 
   cpu_pic->stack->push(cpu_pic->pc->get_next());
 
@@ -667,6 +695,7 @@ char * CALL::name(char *return_str,int len)
 
 //--------------------------------------------------
 CLRF::CLRF (Processor *new_cpu, unsigned int new_opcode)
+  : Register_op(new_cpu, new_opcode, 0)
 {
   decode(new_cpu, new_opcode);
 
@@ -675,7 +704,6 @@ CLRF::CLRF (Processor *new_cpu, unsigned int new_opcode)
 
 void CLRF::execute(void)
 {
-  // trace.instruction(opcode);
 
   if(!access)
     cpu_pic->registers[register_address]->put(0);
@@ -701,18 +729,15 @@ char * CLRF::name(char *return_str,int len)
 //--------------------------------------------------
 
 CLRW::CLRW (Processor *new_cpu, unsigned int new_opcode)
+  : instruction(new_cpu, new_opcode, 0)
 {
-
   decode(new_cpu, new_opcode);
-
   new_name("clrw");
-
 }
 
 void CLRW::execute(void)
 {
 
-  // trace.instruction(opcode);
   cpu_pic->W->put(0);
 
   cpu_pic->status->put_Z(1);
@@ -723,33 +748,20 @@ void CLRW::execute(void)
 //--------------------------------------------------
 
 CLRWDT::CLRWDT (Processor *new_cpu, unsigned int new_opcode)
+  : instruction(new_cpu, new_opcode, 0)
 {
-
   decode(new_cpu, new_opcode);
-
   new_name("clrwdt");
-
 }
 
 void CLRWDT::execute(void)
 {
 
-  // trace.instruction(opcode);
+
   cpu_pic->wdt.clear();
 
-  if(cpu_pic->base_isa() != _16BIT_PROCESSOR_)
-    {
-      cpu_pic->status->put_TO(1);
-      cpu_pic->status->put_PD(1);
-    }
-  else {
-    static int warned = 0;
-    if(!warned) {
-      cout << "FIXME: CLRWDT for 16 bit processors\n";
-      warned=1;
-    }
-  }
-
+  cpu_pic->status->put_TO(1);
+  cpu_pic->status->put_PD(1);
   cpu_pic->pc->increment();
 
 }
@@ -757,21 +769,16 @@ void CLRWDT::execute(void)
 //--------------------------------------------------
 
 COMF::COMF (Processor *new_cpu, unsigned int new_opcode)
+  : Register_op(new_cpu, new_opcode, 0)
 {
-
   decode(new_cpu, new_opcode);
-
   new_name("comf");
-
 }
 
 void COMF::execute(void)
 {
   unsigned int new_value;
 
-  // trace.instruction(opcode);
-
-
   if(!access)
     source = cpu_pic->registers[register_address];
   else
@@ -793,21 +800,16 @@ void COMF::execute(void)
 //--------------------------------------------------
 
 DECF::DECF (Processor *new_cpu, unsigned int new_opcode)
+  : Register_op(new_cpu, new_opcode, 0)
 {
-
   decode(new_cpu, new_opcode);
-
   new_name("decf");
-
 }
 
 void DECF::execute(void)
 {
   unsigned int new_value;
 
-  // trace.instruction(opcode);
-
-
   if(!access)
     source = cpu_pic->registers[register_address];
   else
@@ -829,21 +831,16 @@ void DECF::execute(void)
 //--------------------------------------------------
 
 DECFSZ::DECFSZ (Processor *new_cpu, unsigned int new_opcode)
+  : Register_op(new_cpu, new_opcode, 0)
 {
-
   decode(new_cpu, new_opcode);
-
   new_name("decfsz");
-
 }
 
 void DECFSZ::execute(void)
 {
   unsigned int new_value;
 
-  // trace.instruction(opcode);
-
-
   if(!access)
     source = cpu_pic->registers[register_address];
   else
@@ -865,9 +862,8 @@ void DECFSZ::execute(void)
 
 //--------------------------------------------------
 GOTO::GOTO (Processor *new_cpu, unsigned int new_opcode)
+  : instruction(new_cpu, new_opcode, 0)
 {
-  opcode = new_opcode;
-  cpu = new_cpu;
 
   switch(cpu_pic->base_isa())
     {
@@ -887,7 +883,6 @@ GOTO::GOTO (Processor *new_cpu, unsigned int new_opcode)
 
 void GOTO::execute(void)
 {
-  // trace.instruction(opcode);
 
   cpu_pic->pc->jump(cpu_pic->get_pclath_branching_jump() | destination);
 
@@ -906,20 +901,16 @@ char * GOTO::name(char *return_str,int len)
 //--------------------------------------------------
 
 INCF::INCF (Processor *new_cpu, unsigned int new_opcode)
+  : Register_op(new_cpu, new_opcode, 0)
 {
-
   decode(new_cpu, new_opcode);
-
   new_name("incf");
-
 }
 
 void INCF::execute(void)
 {
   unsigned int new_value;
 
-  // trace.instruction(opcode);
-
   if(!access)
     source = cpu_pic->registers[register_address];
   else
@@ -944,21 +935,16 @@ void INCF::execute(void)
 //--------------------------------------------------
 
 INCFSZ::INCFSZ (Processor *new_cpu, unsigned int new_opcode)
+  : Register_op(new_cpu, new_opcode, 0)
 {
-
   decode(new_cpu, new_opcode);
-
   new_name("incfsz");
-
 }
 
 void INCFSZ::execute(void)
 {
   unsigned int new_value;
 
-  // trace.instruction(opcode);
-
-
   if(!access)
     source = cpu_pic->registers[register_address];
   else
@@ -981,20 +967,16 @@ void INCFSZ::execute(void)
 //--------------------------------------------------
 
 IORLW::IORLW (Processor *new_cpu, unsigned int new_opcode)
+  : Literal_op(new_cpu, new_opcode, 0)
 {
-
   decode(new_cpu, new_opcode);
-
   new_name("iorlw");
-
 }
 
 void IORLW::execute(void)
 {
   unsigned int new_value;
 
-  // trace.instruction(opcode);
-
   new_value = cpu_pic->W->value.get() | L;
 
   cpu_pic->W->put(new_value);
@@ -1007,21 +989,16 @@ void IORLW::execute(void)
 //--------------------------------------------------
 
 IORWF::IORWF (Processor *new_cpu, unsigned int new_opcode)
+  : Register_op(new_cpu, new_opcode, 0)
 {
-
   decode(new_cpu, new_opcode);
-
   new_name("iorwf");
-
 }
 
 void IORWF::execute(void)
 {
   unsigned int new_value;
 
-  // trace.instruction(opcode);
-
-
   if(!access)
     source = cpu_pic->registers[register_address];
   else
@@ -1043,17 +1020,14 @@ void IORWF::execute(void)
 //--------------------------------------------------
 
 MOVLW::MOVLW (Processor *new_cpu, unsigned int new_opcode)
+  : Literal_op(new_cpu, new_opcode, 0)
 {
-
   decode(new_cpu, new_opcode);
-
   new_name("movlw");
-
 }
 
 void MOVLW::execute(void)
 {
-  // trace.instruction(opcode);
 
   cpu_pic->W->put(L);
 
@@ -1064,20 +1038,16 @@ void MOVLW::execute(void)
 //--------------------------------------------------
 
 MOVF::MOVF (Processor *new_cpu, unsigned int new_opcode)
+  : Register_op(new_cpu, new_opcode, 0)
 {
-
   decode(new_cpu, new_opcode);
-
   new_name("movf");
-
 }
 
 void MOVF::execute(void)
 {
   unsigned int source_value;
 
-  // trace.instruction(opcode);
-
   if(!access)
     source = cpu_pic->registers[register_address];
   else
@@ -1109,16 +1079,14 @@ void MOVF::debug(void)
 
 //--------------------------------------------------
 MOVWF::MOVWF (Processor *new_cpu, unsigned int new_opcode)
+  : Register_op(new_cpu, new_opcode, 0)
 {
-
   decode(new_cpu, new_opcode);
-
   new_name("movwf");
 }
 
 void MOVWF::execute(void)
 {
-  // trace.instruction(opcode);
 
   if(!access)
     cpu_pic->registers[register_address]->put(cpu_pic->W->get());
@@ -1142,6 +1110,7 @@ char * MOVWF::name(char *return_str, int len)
 //--------------------------------------------------
 
 NOP::NOP (Processor *new_cpu, unsigned int new_opcode)
+  : instruction(new_cpu,new_opcode,0)
 {
 
   decode(new_cpu,new_opcode);
@@ -1153,38 +1122,30 @@ NOP::NOP (Processor *new_cpu, unsigned int new_opcode)
   // (Subsequent initialization code will overwrite this,
   // but there is a chance that this info will be accessed
   // before that occurs).
-
+  /*
   file_id = 0;
   src_line = 0;
   lst_line = 0;
-
+  */
 }
 
 void NOP::execute(void)
 {
-
-  // trace.instruction(opcode);
-
   cpu_pic->pc->increment();
-
 }
 
 //--------------------------------------------------
 
 OPTION::OPTION (Processor *new_cpu, unsigned int new_opcode)
+  : instruction(new_cpu,new_opcode,0)
 {
-
   decode(new_cpu, new_opcode);
-
   new_name("option");
-
 }
 
 void OPTION::execute(void)
 {
 
-  // trace.instruction(opcode);
-
   cpu_pic->option_reg.put(cpu_pic->W->get());
 
   cpu_pic->pc->increment();
@@ -1194,19 +1155,15 @@ void OPTION::execute(void)
 //--------------------------------------------------
 
 RETLW::RETLW (Processor *new_cpu, unsigned int new_opcode)
+  : Literal_op(new_cpu, new_opcode, 0)
 {
-
   decode(new_cpu, new_opcode);
-
   new_name("retlw");
-
 }
 
 void RETLW::execute(void)
 {
 
-  // trace.instruction(opcode);
-
   cpu_pic->W->put(L);
 
   cpu_pic->pc->new_address(cpu_pic->stack->pop());
@@ -1216,21 +1173,16 @@ void RETLW::execute(void)
 //--------------------------------------------------
 
 RLF::RLF (Processor *new_cpu, unsigned int new_opcode)
+  : Register_op(new_cpu, new_opcode, 0)
 {
-
   decode(new_cpu, new_opcode);
-
   new_name("rlf");
-
 }
 
 void RLF::execute(void)
 {
   unsigned int new_value;
 
-  // trace.instruction(opcode);
-
-
   if(!access)
     source = cpu_pic->registers[register_address];
   else
@@ -1252,12 +1204,10 @@ void RLF::execute(void)
 //--------------------------------------------------
 
 RRF::RRF (Processor *new_cpu, unsigned int new_opcode)
+  : Register_op(new_cpu, new_opcode, 0)
 {
-
   decode(new_cpu, new_opcode);
-
   new_name("rrf");
-
 }
 
 void RRF::execute(void)
@@ -1286,12 +1236,10 @@ void RRF::execute(void)
 //--------------------------------------------------
 
 SLEEP::SLEEP (Processor *new_cpu, unsigned int new_opcode)
+  : instruction(new_cpu,new_opcode,0)
 {
-
   decode(new_cpu, new_opcode);
-
   new_name("sleep");
-
 }
 
 void SLEEP::execute(void)
@@ -1304,21 +1252,16 @@ void SLEEP::execute(void)
 //--------------------------------------------------
 
 SUBWF::SUBWF (Processor *new_cpu, unsigned int new_opcode)
+  : Register_op(new_cpu, new_opcode, 0)
 {
-
   decode(new_cpu, new_opcode);
-
   new_name("subwf");
-
 }
 
 void SUBWF::execute(void)
 {
   unsigned int new_value,src_value,w_value;
 
-  // trace.instruction(opcode);
-
-
   if(!access)
     source = cpu_pic->registers[register_address];
   else
@@ -1343,21 +1286,16 @@ void SUBWF::execute(void)
 //--------------------------------------------------
 
 SWAPF::SWAPF (Processor *new_cpu, unsigned int new_opcode)
+  : Register_op(new_cpu, new_opcode, 0)
 {
-
   decode(new_cpu, new_opcode);
-
   new_name("swapf");
-
 }
 
 void SWAPF::execute(void)
 {
   unsigned int src_value;
 
-  // trace.instruction(opcode);
-
-
   if(!access)
     source = cpu_pic->registers[register_address];
   else
@@ -1377,8 +1315,8 @@ void SWAPF::execute(void)
 
 //--------------------------------------------------
 TRIS::TRIS (Processor *new_cpu, unsigned int new_opcode)
+  : Register_op(new_cpu, new_opcode, 0)
 {
-
   decode(new_cpu, new_opcode);
 
   // The TRIS instruction only uses the lower three bits to determine
@@ -1406,8 +1344,6 @@ TRIS::TRIS (Processor *new_cpu, unsigned int new_opcode)
 
 void TRIS::execute(void)
 {
-  // trace.instruction(opcode);
-
   if(register_address)
     {
       // Execute the instruction only if the register is valid.
@@ -1434,20 +1370,16 @@ char * TRIS::name(char *return_str,int len)
 //--------------------------------------------------
 
 XORLW::XORLW (Processor *new_cpu, unsigned int new_opcode)
+  : Literal_op(new_cpu, new_opcode, 0)
 {
-
   decode(new_cpu, new_opcode);
-
   new_name("xorlw");
-
 }
 
 void XORLW::execute(void)
 {
   unsigned int new_value;
 
-  // trace.instruction(opcode);
-
   new_value = cpu_pic->W->value.get() ^ L;
 
   cpu_pic->W->put(new_value);
@@ -1460,20 +1392,16 @@ void XORLW::execute(void)
 //--------------------------------------------------
 
 XORWF::XORWF (Processor *new_cpu, unsigned int new_opcode)
+  : Register_op(new_cpu, new_opcode, 0)
 {
-
   decode(new_cpu, new_opcode);
-
   new_name("xorwf");
-
 }
 
 void XORWF::execute(void)
 {
   unsigned int new_value;
 
-  // trace.instruction(opcode);
-
   if(!access)
     source = cpu_pic->registers[register_address];
   else
