@@ -24,7 +24,6 @@ Boston, MA 02111-1307, USA.  */
 
 
 #ifdef HAVE_GUI
-#include <unistd.h>
 #include <glib.h>
 #endif
 
@@ -69,9 +68,7 @@ public:
   };
 
 
-  instruction();
   instruction(Processor *pProcessor, unsigned int uOpCode, unsigned int uAddrOfInstr);
-  void Initialize(Processor *pProcessor, unsigned int uOpCode, unsigned int uAddrOfInstr);
 
   virtual void execute() = 0;
   virtual void debug(){ }
@@ -86,12 +83,16 @@ public:
   virtual int get_lst_line() { return(lst_line); }
   virtual int get_file_id() {return(file_id); }
   virtual int get_hll_file_id() {return(hll_file_id); }
-  virtual INSTRUCTION_TYPES isa() {return NORMAL_INSTRUCTION;}
+  virtual enum INSTRUCTION_TYPES isa() {return NORMAL_INSTRUCTION;}
   virtual guint64 getCyclesUsed() { return cycle_count;}
   virtual bool isBase() = 0;
   void decode(Processor *new_cpu, unsigned int new_opcode);
   void add_line_number_symbol(int address);
-  void update_line_number(int file, int sline, int lline, int hllfile, int hllsline);
+  virtual void update_line_number(int file, int sline, int lline, int hllfile, int hllsline);
+
+  virtual char *ReadSrcLine(char *buf, int nBytes);
+  virtual char *ReadLstLine(char *buf, int nBytes);
+  virtual char *ReadHLLLine(char *buf, int nBytes);
 
   // Some instructions require special initialization after they've
   // been instantiated. For those that do, the instruction base class
@@ -146,7 +147,7 @@ public:
   virtual int get_lst_line();
   virtual int get_file_id();
   virtual int get_hll_file_id();
-  virtual INSTRUCTION_TYPES isa();
+  virtual enum INSTRUCTION_TYPES isa();
   virtual void initialize(bool init_state);
   virtual char *name(char *,int len);
   virtual bool isBase();
@@ -173,7 +174,9 @@ public:
   };
 
   invalid_instruction(Processor *new_cpu=0,unsigned int new_opcode=0);
-  virtual INSTRUCTION_TYPES isa() {return INVALID_INSTRUCTION;};
+  invalid_instruction(Processor *new_cpu,unsigned int address, 
+    unsigned int new_opcode);
+  virtual enum INSTRUCTION_TYPES isa() {return INVALID_INSTRUCTION;};
   //virtual char *name(char *str){return("INVALID");};
   static instruction *construct(Processor *new_cpu, unsigned int new_opcode)
     {return new invalid_instruction(new_cpu,new_opcode);}
@@ -186,6 +189,8 @@ public:
 class Literal_op : public instruction
 {
 public:
+  Literal_op(Processor *pProcessor, unsigned int uOpCode, unsigned int uAddrOfInstr);
+
   unsigned int L;
 
   virtual void debug(){ };
@@ -200,6 +205,8 @@ public:
 class Bit_op : public instruction
 {
 public:
+  Bit_op(Processor *pProcessor, unsigned int uOpCode, unsigned int uAddrOfInstr);
+
   unsigned int mask,register_address;
   bool access;
   Register *reg;
@@ -218,6 +225,8 @@ class Register_op : public instruction
 {
 public:
 
+  Register_op(Processor *pProcessor, unsigned int uOpCode, unsigned int uAddrOfInstr);
+
   static Register *source;
   unsigned int register_address;
   bool destination, access;
