@@ -21,6 +21,9 @@ Boston, MA 02111-1307, USA.  */
 #include <stdio.h>
 #ifdef _WIN32
 #include "uxtime.h"
+#include "unistd.h"
+#else
+#include <unistd.h>
 #endif
 #ifndef _MSC_VER
 #include <sys/time.h>
@@ -40,12 +43,15 @@ Boston, MA 02111-1307, USA.  */
 #include "stimuli.h"
 #include "p16x5x.h"
 #include "p16f62x.h"
+#include "p16f8x.h"
 #include "p16x8x.h"
 #include "p16f87x.h"
 #include "p16x6x.h"
 #include "p16x7x.h"
 #include "p12x.h"
+#ifdef P17C7XX	// code no longer works
 #include "p17c75x.h"
+#endif
 #include "p18x.h"
 #include "icd.h"
 
@@ -61,7 +67,6 @@ guint64 simulation_start_cycle;
 //  FIXME -  move these global references somewhere else
 
 // don't print out a bunch of garbage while initializing
-int     verbose=0;
 
 
 //================================================================================
@@ -75,6 +80,8 @@ int     verbose=0;
 
 ProcessorConstructor pP10F200(P10F200::construct ,
 			      "__10F200", "pic10f200",  "p10f200", "10f200");
+ProcessorConstructor pP10F202(P10F202::construct ,
+			      "__10F202", "pic10f202",  "p10f202", "10f202");
 ProcessorConstructor pP12C508(P12C508::construct ,
 			      "__12C508", "pic12c508",  "p12c508", "12c508");
 ProcessorConstructor pP12C509(P12C509::construct ,
@@ -127,10 +134,26 @@ ProcessorConstructor pP16C73(P16C73::construct ,
 			     "__16C73",   "pic16c73",   "p16c73", "16c73");
 ProcessorConstructor pP16C74(P16C74::construct ,
 			     "__16C74",   "pic16c74",   "p16c74", "16c74");
+ProcessorConstructor pP16F73(P16F73::construct ,
+			     "__16F73",   "pic16f73",   "p16f73", "16f73");
+ProcessorConstructor pP16F74(P16F74::construct ,
+			     "__16F74",   "pic16f74",   "p16f74", "16f74");
 ProcessorConstructor pP16F627(P16F627::construct ,
 			      "__16F627", "pic16f627",  "p16f627", "16f627");
+ProcessorConstructor pP16F627A(P16F627::construct ,
+			      "__16F627A", "pic16f627a",  "p16f627a", "16f627a");
 ProcessorConstructor pP16F628(P16F628::construct ,
 			      "__16F628", "pic16f628",  "p16f628", "16f628");
+ProcessorConstructor pP16F628A(P16F628::construct ,
+			      "__16F628A", "pic16f628a",  "p16f628a", "16f628a");
+ProcessorConstructor pP16F648(P16F648::construct ,
+			      "__16F648", "pic16f648",  "p16f648", "16f648");
+ProcessorConstructor pP16F648A(P16F648::construct ,
+			      "__16F648A", "pic16f648a",  "p16f648a", "16f648a");
+ProcessorConstructor pP16F87(P16F87::construct ,
+			      "__16F87", "pic16f87",  "p16f87", "16f87");
+ProcessorConstructor pP16F88(P16F88::construct ,
+			      "__16F88", "pic16f88",  "p16f88", "16f88");
 ProcessorConstructor pP16F871(P16F871::construct ,
 			      "__16F871", "pic16f871",  "p16f871", "16f871");
 ProcessorConstructor pP16F873(P16F873::construct ,
@@ -141,6 +164,15 @@ ProcessorConstructor pP16F876(P16F876::construct ,
 			      "__16F876", "pic16f876",  "p16f876", "16f876");
 ProcessorConstructor pP16F877(P16F877::construct ,
 			      "__16F877", "pic16f877",  "p16f877", "16f877");
+ProcessorConstructor pP16F873A(P16F873A::construct ,
+			      "__16F873a", "pic16f873a", "p16f873a", "16f873a");
+ProcessorConstructor pP16F874A(P16F874A::construct ,
+			      "__16F874a", "pic16f874a", "p16f874a", "16f874a");
+ProcessorConstructor pP16F876A(P16F876A::construct ,
+			      "__16F876a", "pic16f876a", "p16f876a", "16f876a");
+ProcessorConstructor pP16F877A(P16F877A::construct ,
+			      "__16F877a", "pic16f877a", "p16f877a", "16f877a");
+#ifdef P17C7XX	// code no longer works
 ProcessorConstructor pP17C7xx(P17C7xx::construct ,
 			      "__17C7xx", "pic17c7xx",  "p17c7xx", "17c7xx");
 ProcessorConstructor pP17C75x(P17C75x::construct ,
@@ -155,6 +187,7 @@ ProcessorConstructor pP17C762(P17C762::construct ,
 			      "__17C762", "pic17c762",  "p17c762", "17c762");
 ProcessorConstructor pP17C766(P17C766::construct ,
 			      "__17C766", "pic17c766",  "p17c766", "17c766");
+#endif // P17C7XX
 ProcessorConstructor pP18C242(P18C242::construct ,
 			      "__18C242", "pic18c242",  "p18c242", "18c242");
 ProcessorConstructor pP18C252(P18C252::construct ,
@@ -171,6 +204,8 @@ ProcessorConstructor pP18F442(P18F442::construct ,
 			      "__18F442", "pic18f442",  "p18f442", "18f442");
 ProcessorConstructor pP18F248(P18F248::construct ,
 			      "__18F248", "pic18f248",  "p18f248", "18f248");
+ProcessorConstructor pP18F448(P18F448::construct ,
+			      "__18F448", "pic18f448",  "p18f448", "18f448");
 ProcessorConstructor pP18F452(P18F452::construct,
 			      "__18F452", "pic18f452",  "p18f452", "18f452");
 ProcessorConstructor pP18F1220(P18F1220::construct,
@@ -179,17 +214,6 @@ ProcessorConstructor pP18F1320(P18F1320::construct,
 			      "__18F1320", "pic18f1320",  "p18f1320", "18f1320");
 
 
-//-------------------------------------------------------------------
-//
-Processor * pic_processor::construct(void)
-{
-
-  cout << " Can't create a generic pic processor\n";
-
-  return 0;
-
-}
-
 void pic_processor::set_eeprom(EEPROM *e)
 { 
   eeprom = e; 
@@ -203,7 +227,7 @@ void pic_processor::set_eeprom(EEPROM *e)
 // sleep - Begin sleeping and stay asleep until something causes a wake
 //
 
-void pic_processor::sleep (void)
+void pic_processor::sleep ()
 {
   simulation_mode = eSM_SLEEPING;
 
@@ -237,7 +261,7 @@ void pic_processor::enter_sleep()
 // pm_write - program memory write
 //
 
-void pic_processor::pm_write (void)
+void pic_processor::pm_write ()
 {
   //  simulation_mode = PM_WRITE;
 
@@ -250,9 +274,18 @@ void pic_processor::pm_write (void)
 
 }
 
-int realtime_mode=0;
-int realtime_mode_with_gui=0;
-extern void update_gui(void);
+static bool realtime_mode = false;
+static bool realtime_mode_with_gui = false;
+
+void EnableRealTimeMode(bool bEnable) {
+  realtime_mode = bEnable;
+}
+
+void EnableRealTimeModeWithGui(bool bEnable) {
+  realtime_mode_with_gui = bEnable;
+}
+
+extern void update_gui();
 
 class RealTimeBreakPoint : public TriggerObject
 {
@@ -264,7 +297,7 @@ public:
   int warntimer;
   guint64 period;
 
-  RealTimeBreakPoint(void)
+  RealTimeBreakPoint()
   {
     cpu = 0;
     warntimer = 1;
@@ -300,7 +333,7 @@ public:
 
   }
 
-  void stop(void)
+  void stop()
   {
 
     // Clear any pending break point.
@@ -314,7 +347,7 @@ public:
       
   }
 
-  void callback(void)
+  void callback()
   {
     gint64 system_time;
     double diff;
@@ -420,7 +453,7 @@ void pic_processor::save_state()
 //
 void pic_processor::run (bool refresh)
 {
-  if(use_icd)
+  if(get_use_icd())
   {
     cout  << "WARNING: gui_refresh is not being called " 
 	  <<  __FILE__<<':'<<__LINE__<<endl;
@@ -513,7 +546,7 @@ void pic_processor::run (bool refresh)
 void pic_processor::step (unsigned int steps, bool refresh)
 {
 
-  if(use_icd)
+  if(get_use_icd())
   {
       if(steps!=1)
       {
@@ -593,7 +626,7 @@ void pic_processor::step_over (bool refresh)
 //
 // this method really only applies to processors with stacks.
 
-void pic_processor::finish(void)
+void pic_processor::finish()
 {
   if(!stack)
     return;
@@ -612,7 +645,7 @@ void pic_processor::reset (RESET_TYPE r)
 {
   bool bHaltSimulation = true;
 
-  if(use_icd)
+  if(get_use_icd())
   {
       puts("RESET");
       icd_reset();
@@ -649,8 +682,7 @@ void pic_processor::reset (RESET_TYPE r)
       cout << "POR\n";
       if(config_modes) config_modes->print();
     }
-    if(config_modes)
-      wdt.initialize( config_modes->get_wdt() , nominal_wdt_timeout);
+    wdt.reset();
 
     bHaltSimulation = false;
     break;
@@ -672,10 +704,9 @@ void pic_processor::reset (RESET_TYPE r)
 // por - power on reset %%% FIX ME %%% needs lots of work...
 //
 
-void pic_processor::por(void)
+void pic_processor::por()
 {
-  if(config_modes)
-    wdt.initialize( config_modes->get_wdt(), nominal_wdt_timeout);
+  wdt.reset();
 
 }
 
@@ -684,26 +715,30 @@ void pic_processor::por(void)
 // pic_processor -- constructor
 //
 
-pic_processor::pic_processor(void)
+pic_processor::pic_processor(const char *_name, const char *_desc)
+  : Processor(_name,_desc),
+    wdt(this, 18.0e-3),indf(0),fsr(0), stack(0), status(0), W(0), pcl(0), pclath(0), m_configMemory(0)
 {
   m_Capabilities = eSTACK | eWATCHDOGTIMER;
 
   if(verbose)
     cout << "pic_processor constructor\n";
 
-  pc = 0;
-
   eeprom = 0;
   config_modes = create_ConfigMode();
 
-  set_frequency(DEFAULT_PIC_CLOCK);
-  set_ClockCycles_per_Instruction(4);
   pll_factor = 0;
 
-  // Test code for logging to disk:
-  trace_log.switch_cpus(this);
-}
+  Integer::setDefaultBitmask(0xff);
 
+  // Test code for logging to disk:
+  GetTraceLog().switch_cpus(this);
+}
+//-------------------------------------------------------------------
+pic_processor::~pic_processor()
+{
+
+}
 //-------------------------------------------------------------------
 //
 // 
@@ -713,24 +748,19 @@ pic_processor::pic_processor(void)
 // Since this is a base class member function, only those things that
 // are common to all pics are created.
 
-void pic_processor::create (void)
+void pic_processor::create ()
 {
 
   init_program_memory (program_memory_size());
 
   init_register_memory (register_memory_size());
 
-  //  create_invalid_registers ();
-  //  create_symbols();
   create_stack();
 
   // Now, initialize the core stuff:
   pc->set_cpu(this);
-  //  cycles.cpu = this;
-  wdt.cpu = this;
 
   W = new WREG(this);
-  //  W->set_cpu(this);
 
   pcl = new PCL;
   pclath = new PCLATH;
@@ -744,8 +774,6 @@ void pic_processor::create (void)
   register_bank = &registers[0];  // Define the active register bank 
   W->value.put(0);
 
-  nominal_wdt_timeout = 18e-3;    // 18ms according to the data sheet (no prescale)
-
   Vdd = 5.0;                      // Assume 5.0 volt power supply
 
   if(pma) {
@@ -760,6 +788,8 @@ void pic_processor::create (void)
 
   }
 
+  create_config_memory();
+
 }
 
 //-------------------------------------------------------------------
@@ -788,8 +818,10 @@ void pic_processor::add_sfr_register(Register *reg, unsigned int addr,
       if(new_name)
         registers[addr]->new_name(new_name);
 
-      registers[addr]->set_write_trace(getWriteTT(addr));
-      registers[addr]->set_read_trace(getReadTT(addr));
+      RegisterValue rv = getWriteTT(addr);
+      registers[addr]->set_write_trace(rv);
+      rv = getReadTT(addr);
+      registers[addr]->set_read_trace(rv);
     }
 
   reg->value       = por_value;
@@ -829,7 +861,7 @@ void pic_processor::init_program_memory (unsigned int memory_size)
 // Add a symbol table entry for each one of the sfr's
 //
 
-void pic_processor::create_symbols (void)
+void pic_processor::create_symbols ()
 {
 
   if(verbose)
@@ -863,7 +895,6 @@ bool pic_processor::set_config_word(unsigned int address,unsigned int cfg_word)
   // reset each of them based on the config bits in cfg_word:
   //config_modes &= ~(CM_WDTE);
   //config_modes |= ( (cfg_word & WDTE) ? CM_WDTE : 0);
-  //cout << " setting cfg_word and cfg_modes " << hex << config_word << "  " << config_modes << '\n';
 
   if((address == config_word_address()) && config_modes) {
 
@@ -871,7 +902,7 @@ bool pic_processor::set_config_word(unsigned int address,unsigned int cfg_word)
 
     config_modes->config_mode = (config_modes->config_mode & ~7) | (cfg_word & 7);
 
-    if(verbose && config_modes)
+    if((bool)verbose && config_modes)
       config_modes->print();
 
     return true;
@@ -882,15 +913,21 @@ bool pic_processor::set_config_word(unsigned int address,unsigned int cfg_word)
 }
 
 
+unsigned int pic_processor::get_config_word(unsigned int address)
+{
+  return address == config_word_address() ? config_word : 0xffffffff;
+}
+
 
 
 //-------------------------------------------------------------------
 //
 // load_hex
 //
-extern int readihex16 (pic_processor *cpu, FILE * file);
 
-bool pic_processor::LoadProgramFile(const char *pFilename, FILE *pFile) {
+bool pic_processor::LoadProgramFile(const char *pFilename, FILE *pFile,
+				    const char *pProcessorName)
+{
   Processor * pProcessor = this;
   // Tries the file type based on the file extension first.
   // If it fails tries the other type. This code will need
@@ -898,17 +935,17 @@ bool pic_processor::LoadProgramFile(const char *pFilename, FILE *pFile) {
   // because then we cannot garrentee that these file types
   // will be the first two in the list.
   ProgramFileType * aFileTypes[] = {
-    ProgramFileTypeList::GetList()[0],  // PicHexProgramFileType
+    ProgramFileTypeList::GetList()[0],  // IntelHexProgramFileType
     ProgramFileTypeList::GetList()[1]   // PicCodProgramFileType
   };
   if(IsFileExtension(pFilename,"cod")) {
     // If 'cod' file extension, try PicCodProgramFileType first
     swap(aFileTypes[0], aFileTypes[1]);
   }
-  int iReturn  = aFileTypes[0]->LoadProgramFile(&pProcessor, pFilename, pFile);
+  int iReturn  = aFileTypes[0]->LoadProgramFile(&pProcessor, pFilename, pFile, pProcessorName);
   if (iReturn != ProgramFileType::SUCCESS) {
     fseek(pFile, 0, SEEK_SET);
-    iReturn = aFileTypes[1]->LoadProgramFile(&pProcessor, pFilename, pFile);
+    iReturn = aFileTypes[1]->LoadProgramFile(&pProcessor, pFilename, pFile, pProcessorName);
   }
   return iReturn == ProgramFileType::SUCCESS;
 }
@@ -917,7 +954,7 @@ bool pic_processor::LoadProgramFile(const char *pFilename, FILE *pFile) {
 //-------------------------------------------------------------------
 //  ConfigMode
 //
-void ConfigMode::print(void)
+void ConfigMode::print()
 {
 
 
@@ -965,7 +1002,7 @@ void ConfigMode::print(void)
 
 //-------------------------------------------------------------------
 //-------------------------------------------------------------------
-void ProgramMemoryAccess::callback(void)
+void ProgramMemoryAccess::callback()
 {
 
   if(_state)
@@ -979,3 +1016,174 @@ void ProgramMemoryAccess::callback(void)
     }
 
 }
+
+//--------------------------------------------------
+WDT::WDT(pic_processor *p_cpu, double _timeout)
+  : cpu(p_cpu), breakpoint(0), future_cycle(0), timeout(_timeout), wdte(false)
+{
+}
+
+//--------------------------------------------------
+void WDT::update()
+{
+  if(wdte){
+
+    value = (unsigned int )(cpu->get_frequency()*timeout);
+    prescale = cpu->option_reg.get_psa() ? (cpu->option_reg.get_prescale()) : 0;
+
+    if(future_cycle) {
+
+      guint64 fc = get_cycles().value + value * (1<<prescale);
+
+      //cout << "WDT::update:  moving break from " << future_cycle << " to " << fc << '\n';
+
+      get_cycles().reassign_break(future_cycle, fc, this);
+      future_cycle = fc;
+
+    } else {
+    
+      future_cycle = get_cycles().value + value * (1<<prescale);
+
+      get_cycles().set_break(future_cycle, this);
+    }
+  }
+
+}
+
+//--------------------------------------------------
+// WDT::put - shouldn't be called?
+//
+
+void WDT::put(unsigned int new_value)
+{
+  value = new_value;
+
+  update();
+
+}
+void WDT::set_timeout( double _timeout)
+{
+  timeout = _timeout;
+  update();
+}
+void WDT::initialize(bool enable)
+{
+  wdte = enable;
+  warned = 0;
+
+  if(verbose)
+    cout << " WDT init called "<< ( (enable) ? "enabling\n" :", but disabling WDT\n");
+
+  if(wdte) {
+    cout << "Enabling WDT " << " timeout = " << timeout << " seconds\n";
+    value = (unsigned int) (cpu->get_frequency()*timeout);
+    prescale = cpu->option_reg.get_psa() ? (cpu->option_reg.get_prescale()) : 0;
+
+    future_cycle = get_cycles().value + value * (1<<prescale);
+
+    get_cycles().set_break(future_cycle, this);
+
+  } else {
+
+    if (future_cycle) {
+      cout << "Disabling WDT\n";
+      get_cycles().clear_break(this);
+      future_cycle = 0;
+    }
+  }
+
+}
+
+void WDT::reset()
+{
+  update();
+}
+void WDT::set_breakpoint(unsigned int bpn)
+{
+  breakpoint = bpn;
+}
+
+void WDT::callback()
+{
+
+
+  if(wdte) {
+    cout<<"WDT timeout: " << hex << get_cycles().value << '\n';
+
+    //future_cycle = 0;
+    update();
+
+    // The TO bit gets cleared when the WDT times out.
+    cpu->status->put_TO(0);
+
+    if(breakpoint)
+      bp.halt();
+    else 
+    {
+      bp.clear_sleep();
+      cpu->reset(WDT_RESET);
+    }
+    /*    else if(bp.have_sleep()) {
+      bp.clear_sleep();
+    }else
+      cpu->reset(WDT_RESET);
+    */
+  }
+
+}
+
+void WDT::clear()
+{
+  if(wdte)
+    update();
+  else
+    {
+      if(!warned)
+	{
+	  warned = 1;
+	  cout << "The WDT is not enabled - clrwdt has no effect!\n";
+	}
+    }
+
+}
+
+void WDT::start_sleep()
+{
+
+  if(wdte) {
+    prescale = 0;
+
+    guint64 fc = get_cycles().value + value * (1<<prescale);
+
+    //cout << "WDT::start_sleep:  moving break from " << future_cycle << " to " << fc << '\n';
+
+    get_cycles().reassign_break(future_cycle, fc, this);
+
+    future_cycle = fc;
+  }
+}
+
+void WDT::new_prescale()
+{
+
+  update();
+
+}
+
+void WDT::callback_print()
+{
+
+  cout << "WDT\n";
+}
+
+
+//------------------------------------------------------------------------
+// ConfigMemory - Base class
+ConfigMemory::ConfigMemory(const char *_name, unsigned int default_val, const char *desc,
+			   pic_processor *pCpu, unsigned int addr)
+  : Integer(_name, default_val, desc), m_pCpu(pCpu), m_addr(addr)
+{
+  if (m_pCpu)
+    m_pCpu->add_attribute(this);
+}
+
