@@ -39,7 +39,6 @@ Boston, MA 02111-1307, USA.  */
 
 #include "../config.h"
 #include "gpsim_def.h"
-#include <unistd.h>
 #include <glib.h>
 
 #include "gpsim_classes.h"
@@ -71,19 +70,21 @@ static char pkg_version[] = PACKAGE_VERSION;
 // Processor - Constructor
 //
 
-Processor::Processor(void)
+Processor::Processor(const char *_name, const char *_desc)
+  : Module(_name, _desc)
 {
   registers = 0;
   pma = 0;
   m_pConstructorObject = 0;
   m_Capabilities = 0;
   if(verbose)
-    cout << "pic_processor constructor\n";
+    cout << "processor constructor\n";
 
   pc = 0;
 
   mFrequency = new Float("frequency",20e6, " oscillator frequency.");
-  set_ClockCycles_per_Instruction(1);
+  set_ClockCycles_per_Instruction(4);
+  get_cycles().set_instruction_cps((guint64)(get_frequency()/clocks_per_inst));
   set_Vdd(5.0);
   setWarnMode(true);
   setSafeMode(true);
@@ -168,6 +169,7 @@ void Processor::set_frequency(double f)
 {
   if(mFrequency)
     mFrequency->set(f);
+  get_cycles().set_instruction_cps((guint64)(f/clocks_per_inst));
 }
 double Processor::get_frequency()
 {
@@ -213,6 +215,10 @@ void Processor::init_register_memory (unsigned int memory_size)
     cout << __FUNCTION__ << " memory size: " << memory_size << '\n';
 
   registers = new Register *[memory_size];
+  m_UiAccessOfRegisters = new RegisterCollection(this,
+						 "ramData",
+						 registers,
+						 memory_size);
 
   if (registers  == 0)
     {
@@ -251,9 +257,9 @@ void Processor::init_register_memory (unsigned int memory_size)
 // fills the voids.
 //
 
-void Processor::create_invalid_registers (void)
+void Processor::create_invalid_registers ()
 {
-  unsigned int i;
+  unsigned int addr;
 
   if(verbose)
     cout << "Creating invalid registers " << register_memory_size()<<"\n";
@@ -265,19 +271,18 @@ void Processor::create_invalid_registers (void)
   // individual invalid file registers. By default, gpsim halts whenever
   // there is an invalid file register access.
 
-  for (i = 0; i < register_memory_size(); i++)
-    {
-      if (0 == registers[i])
-      {
-	  registers[i] = new InvalidRegister(i);
-	  //registers[i]->address = 0;    // BAD_REGISTER;
-	  registers[i]->alias_mask = 0;
-	  registers[i]->value.put(0);	// unimplemented registers are read as 0
+  for (addr = 0; addr < register_memory_size(); addr+=map_rm_index2address(1)) {
 
-	  registers[i]->set_cpu(this);
+    unsigned int index = map_rm_address2index(addr);
+
+    if (0 == registers[index]) {
+
+      registers[index] = new InvalidRegister(addr);
+      registers[index]->alias_mask = 0;
+      registers[index]->set_cpu(this);
 
-	}
     }
+  }
 
 }
 
@@ -307,9 +312,10 @@ void Processor::add_file_registers(unsigned int start_address, unsigned int end_
       registers[j]->alias_mask = 0;
 
     registers[j]->address = j;
-
-    registers[j]->set_write_trace(getWriteTT(j));
-    registers[j]->set_read_trace(getReadTT(j));
+    RegisterValue rv = getWriteTT(j);
+    registers[j]->set_write_trace(rv);
+    rv = getReadTT(j);
+    registers[j]->set_read_trace(rv);
 
     //The default register name is simply its address
     sprintf (str, "0x%02x", j);
@@ -450,12 +456,13 @@ void Processor::destroyProgramMemoryAccess(ProgramMemoryAccess *pma) {
 
 void Processor::init_program_memory(unsigned int address, unsigned int value)
 {
-  init_program_memory_at_index(map_pm_address2index(address), value);
-}
+  unsigned int uIndex = map_pm_address2index(address);
+
+  if (!program_memory) {
+    printf("ERROR: internal bug %s:%d",__FILE__,__LINE__);
+    exit(1);
+  }
 
-void Processor::init_program_memory_at_index(unsigned int uIndex, unsigned int value)
-{
-  unsigned int address = map_pm_index2address(uIndex);
   if(uIndex < program_memory_size()) {
 
     if(program_memory[uIndex] != 0 && program_memory[uIndex]->isa() != instruction::INVALID_INSTRUCTION) {
@@ -472,9 +479,35 @@ void Processor::init_program_memory_at_index(unsigned int uIndex, unsigned int v
   else
     set_out_of_range_pm(address,value);  // could be e2prom
 
+
 }
 
+void Processor::init_program_memory_at_index(unsigned int uIndex, unsigned int value)
+{
+  init_program_memory(map_pm_index2address(uIndex), value);
+}
 
+void Processor::init_program_memory_at_index(unsigned int uIndex, 
+					     const unsigned char *bytes, int nBytes)
+{
+
+  for (int i =0; i<nBytes/2; i++)
+    init_program_memory_at_index(uIndex+i, (((unsigned int)bytes[2*i+1])<<8)  | bytes[2*i]);
+
+}
+
+//------------------------------------------------------------------
+// Fetch the rom contents at a particular address.
+unsigned int Processor::get_program_memory_at_address(unsigned int address)
+{
+  unsigned int uIndex = map_pm_address2index(address);
+
+  
+  return (uIndex < program_memory_size() && program_memory[uIndex])
+    ? program_memory[uIndex]->get_opcode() 
+    : 0xffffffff;
+
+}
 
 //-------------------------------------------------------------------
 // build_program_memory - given an array of opcodes this function
@@ -498,7 +531,7 @@ void Processor::set_out_of_range_pm(unsigned int address, unsigned int value)
 {
 
   cout << "Warning::Out of range address " << address << " value " << value << endl;
-  cout << "Max allowed address is " << (program_memory_size()-1) << '\n';
+  cout << "Max allowed address is 0x" << hex << (program_memory_size()-1) << '\n';
 
 }
 
@@ -514,28 +547,8 @@ void Processor::attach_src_line(unsigned int address,
 				unsigned int lst_line)
 {
   unsigned int uIndex = map_pm_address2index(address);
-  if(uIndex < program_memory_size()) {
-
-    program_memory[uIndex]->update_line_number(file_id,sline,lst_line,0,0);
-
-    //printf("%s address=%x, File ID= %d, sline=%d, lst_line=%d\n", __FUNCTION__,address,file_id,sline,lst_line);
-
-    FileContext *fc = files[file_id];
-
-    if(fc && sline > fc->max_line())
-      fc->max_line(sline);
-
-    // If there's a listing file then handle it as well
-    if(files.list_id() >=0 && lst_line != 0) {
-      fc = files[files.list_id()];
-    
-      // FIX ME - what is this '+5' junk?
-      if(fc && lst_line+5 > fc->max_line())
-        fc->max_line(lst_line+5);
-    }
-
-  }
-
+  if(uIndex < program_memory_size())
+    program_memory[uIndex]->update_line_number(file_id,sline,lst_line,-1,-1);
 }
 
 //-------------------------------------------------------------------
@@ -546,48 +559,6 @@ void Processor::attach_src_line(unsigned int address,
 
 void Processor::read_src_files(void)
 {
-#if USE_OLD_FILE_CONTEXT == 1
-
-  // Are there any src files ?
-  for(int i=0; i<number_of_source_files; i++) {
-
-
-    // did this src file generate any code?
-    if(files[i].max_line > 0) {
-      
-      // Create an array whose index corresponds to the
-      // line number of a source file line and whose data
-      // is the offset (in bytes) from the beginning of the 
-      // file. (e.g. files[3].line_seek[20] references the
-      // 20th line of the third source file.)
-      files[i].line_seek = new int[files[i].max_line+1];
-
-      // Create an array whose index is a source file line 
-      // number and whose data is the program memory address
-      // associated with this lines. Note, this array is initalized
-      // to all -1's here, but later will get filled with the
-      // proper addresses.
-
-      files[i].pm_address = new int[files[i].max_line+1];
-
-      if( 0 == (files[i].file_ptr = fopen_path(files[i].name,"r")))
-        continue;
-
-      rewind(files[i].file_ptr);
-
-      char buf[256],*s;
-      files[i].line_seek[0] = 0;
-      for(int j=1; j<=files[i].max_line; j++) {
-        files[i].pm_address[j] = -1;
-        files[i].line_seek[j] = ftell(files[i].file_ptr);
-        s = fgets(buf,256,files[i].file_ptr);
-        if(s != buf)
-	        break;
-      }
-    }
-  }
-#else
-
   int i;
   // Are there any src files ?
   for(i=0; i<files.nsrc_files(); i++) {
@@ -608,6 +579,7 @@ void Processor::read_src_files(void)
     }
   }
 
+  // Associate source files with the instructions they generated.
   unsigned int addr;
   for(addr = 0; addr<program_memory_size(); addr++) {
 
@@ -622,7 +594,39 @@ void Processor::read_src_files(void)
     }
   }
 
-#endif
+  // Associate the list file with 
+  if (files.list_id() >= 0) {
+
+    // Parse the list file.
+    //printf("read_src_files List file:%d %d\n",files.list_id(),files.nsrc_files());
+
+    FileContext *fc = files[files.list_id()];
+    if (!fc)
+      return;
+
+    fc->ReadSource();
+    fc->rewind();
+
+    char buf[256];
+    int line = 1;
+
+    while(fc->gets(buf,sizeof(buf))) {
+
+      int address;
+      int opcode;
+
+      if (sscanf(buf,"%x   %x",&address, &opcode) == 2) {
+	unsigned int uIndex = map_pm_address2index(address);
+	if (uIndex < program_memory_size()) {
+	  program_memory[uIndex]->update_line_number(-1,-1,line,-1,-1);
+	  fc->put_address(line,address);
+	}
+      }
+
+      line++;
+    }
+
+  }
 }
 
 
@@ -634,8 +638,8 @@ void Processor::read_src_files(void)
 //
 void Processor::list(unsigned int file_id, 
 		     unsigned int pc_val, 
-		     unsigned int start_line, 
-		     unsigned int end_line)
+		     int start_line, 
+		     int end_line)
 {
 
 
@@ -672,23 +676,22 @@ void Processor::list(unsigned int file_id,
   if(fc == NULL)
     return;
 
-  if(start_line < 0) start_line = 0;
-
-  if(end_line > fc->max_line())
-    end_line = fc->max_line();
-
+  start_line = (start_line < 0) ? 0 : start_line;
+  end_line   = (end_line <= start_line) ? (start_line + 5) : end_line;
+  end_line   = (end_line > (int)fc->max_line()) ? fc->max_line() : end_line;
   cout << " listing " << fc->name() << " Starting line " << start_line
        << " Ending line " << end_line << '\n';
 
+  if (end_line == start_line)
+    return;
 
-  for(unsigned int i=start_line; i<=end_line; i++)
+
+  for(unsigned int i=start_line; i<=(unsigned int)end_line; i++)
   {
 
     char buf[256];
 
-    fc->ReadLine(i,
-		    buf,
-		    sizeof(buf));
+    fc->ReadLine(i, buf, sizeof(buf));
 
     if (pc_line == i)
       cout << "==>";
@@ -759,6 +762,10 @@ void Processor::disassemble (signed int s, signed int e)
   const int iConsoleWidth = 80;
   char str[iConsoleWidth];
   char str2[iConsoleWidth];
+  if (!pc) {
+    printf("ERROR: Internal bug %s:%d\n",__FILE__,__LINE__);
+    exit(1);
+  }
   unsigned uPCAddress = pc->get_value();
   const char *pszPC;
   char cBreak;
@@ -824,8 +831,8 @@ void Processor::disassemble (signed int s, signed int e)
       int iSrc = iOperandsWidth - (strlen(str) - iNumonicWidth - 1);
       //        Console.Printf("0.........1.........2.........3.........4.........5.........6.........7.........");
       //        Console.Printf("%d, strlen(str)=%d\n", iNumonicWidth, strlen(str));
-      Console.Printf(
-		     "% 3s%c%04x  %04x  %s %*s%s\n",
+      const char *pFormat = (opcode_size() <=2) ?  "% 3s%c%04x  %04x  %s %*s%s\n" :  "% 3s%c%04x  %06x  %s %*s%s\n";
+      Console.Printf(pFormat,		     
 		     pszPC, cBreak, uAddress, inst->get_opcode(), 
 		     str, iSrc, "", str2);
     }
@@ -902,8 +909,8 @@ int ProgramMemoryAccess::find_closest_address_to_line(int file_id, int src_line)
   
   if(fc)
   {
-  	int offset=0;
-  	while((unsigned int)(src_line+offset)<fc->max_line())
+    int offset=0;
+    while((unsigned int)(src_line+offset)<fc->max_line())
     {
       closest_address = fc->get_address(src_line+offset);
       if(closest_address>=0)
@@ -948,12 +955,7 @@ int ProgramMemoryAccess::find_closest_address_to_line(int file_id, int src_line)
 //-------------------------------------------------------------------
 int ProgramMemoryAccess::find_address_from_line(FileContext *fc, int src_line)
 {
-  if(!cpu)
-    return -1;
-
-  if(fc)
-    return fc->get_address(src_line);
-  return -1;
+  return (cpu && fc) ? fc->get_address(src_line) : -1;
 }
 //--------------------------------------------------------------------------
 //
@@ -1221,33 +1223,34 @@ int Processor::trace_dump1(int type, char *buffer, int bufsize)
 // at the moment), we can enhance this function to return different Trace
 // type objects base on the upper address bits.
 
-unsigned int Processor::getWriteTT(unsigned int j)
+RegisterValue Processor::getWriteTT(unsigned int j)
 {
   if(!writeTT) {
-    writeTT = new RegisterWriteTraceType(this,0,1);
+    writeTT = new RegisterWriteTraceType(this,2);
     trace.allocateTraceType(writeTT);
   }
 
   // The upper 8-bits define the dynamically allocated trace type
   // The lower 8-bits will record the register value that is written.
   // The middle 16-bits are the register address
-  
-  return writeTT->type = (writeTT->type & 0xff000000) | ((j & 0xffff) << 8);
+  unsigned int tt = (writeTT->type() & 0xff000000) | ((j & 0xffff) << 8);
 
+  return RegisterValue(tt, tt + (1<<24) );
 }
 
-unsigned int Processor::getReadTT(unsigned int j)
+RegisterValue Processor::getReadTT(unsigned int j)
 {
   if(!readTT) {
-    readTT = new RegisterReadTraceType(this,0,1);
+    readTT = new RegisterReadTraceType(this,2);
     trace.allocateTraceType(readTT);
   }
 
   // The upper 8-bits define the dynamically allocated trace type
   // The lower 8-bits will record the register value that is written.
   // The middle 16-bits are the register address
+  unsigned int tt = (readTT->type() & 0xff000000) | ((j & 0xffff) << 8);
 
-  return (readTT->type & 0xff000000) | ((j & 0xffff) << 8);
+  return RegisterValue(tt, tt + (1<<24) );
 }
 
 //-------------------------------------------------------------------
@@ -1457,13 +1460,139 @@ void MemoryAccess::set_cpu(Processor *p)
 
 //-------------------------------------------------------------------
 //-------------------------------------------------------------------
-//
+// Program memory interface used by the command line
+class ProgramMemoryCollection : public IIndexedCollection 
+{
+public:
+  ProgramMemoryCollection(Processor *pProcessor, 
+			  const char *collection_name,
+			  ProgramMemoryAccess *pPma);
 
+  virtual unsigned int GetSize();
+  virtual Value &GetAt(unsigned int uAddress, Value *pValue=0);
+  virtual void SetAt(unsigned int uAddress, Value *pValue);
+  virtual void ConsolidateValues(int &iColumnWidth,
+                                 vector<string> &aList,
+                                 vector<string> &aValue);
+  virtual unsigned int GetLowerBound();
+  virtual unsigned int GetUpperBound();
+  virtual bool bIsIndexInRange(unsigned int uIndex);
+private:
+  Processor *   m_pProcessor;
+  ProgramMemoryAccess   *m_pPma;
+  Integer       m_ReturnValue;
+};
+
+
+ProgramMemoryCollection::ProgramMemoryCollection (Processor   *pProcessor, 
+						  const char  *pC_collection_name,
+						  ProgramMemoryAccess *pPma) :
+  IIndexedCollection(16), m_ReturnValue(0) 
+{
+  m_pProcessor = pProcessor;
+  Value::new_name(pC_collection_name);
+  m_pPma = pPma;
+  get_symbol_table().add(this);
+}
+
+unsigned int ProgramMemoryCollection::GetSize()
+{
+  return m_pProcessor->map_pm_address2index(m_pProcessor->program_memory_size());
+}
+
+Value &ProgramMemoryCollection::GetAt(unsigned int uAddress, Value *) 
+{
+  //m_pProcessor->map_pm_address2index
+  m_ReturnValue.set((int)m_pPma->get_rom(uAddress));
+  m_ReturnValue.setBitmask( (1<<(m_pProcessor->opcode_size()*8)) - 1);
+  ostringstream sIndex;
+  sIndex << Value::name() << "[" << hex << m_szPrefix << uAddress << "]" << '\000';
+  m_ReturnValue.new_name(sIndex.str().c_str());
+  return m_ReturnValue;
+}
+
+void ProgramMemoryCollection::SetAt(unsigned int uAddress, Value *pValue)
+{
+  Integer *pInt = dynamic_cast<Integer*>(pValue);
+  if(pInt == NULL) {
+    throw Error("rValue is not an Integer");
+  }
+  else {
+    m_pPma->put_rom(uAddress, (unsigned int)(int)*pInt);
+  }
+}
+
+void ProgramMemoryCollection::ConsolidateValues(int &iColumnWidth,
+                                           vector<string> &aList,
+                                           vector<string> &aValue) 
+{
+  unsigned int  uFirstAddress = 0;
+  unsigned int  uAddress;
+  //Register *    pReg = m_ppRegisters[0];
+  //Integer       uLastValue(pReg->getRV_notrace().data);
+  Integer uLastValue(m_pPma->get_opcode(0));
+  uLastValue.setBitmask((1<<(m_pProcessor->opcode_size()*8)) - 1);
+
+  unsigned int uSize = GetSize();
+
+  for(uAddress = 0; uAddress < uSize; uAddress++) {
+    ostringstream sIndex;
+
+    unsigned int ui_opcode = m_pPma->get_opcode(uAddress);
+
+    if((unsigned int)uLastValue != ui_opcode) {
+      PushValue(uFirstAddress, uAddress,
+                &uLastValue, aList, aValue);
+      iColumnWidth = max(iColumnWidth, (int)aList.back().size());
+      uFirstAddress = uAddress;
+      uLastValue = ui_opcode;
+    }
+  }
+  uAddress--;
+  // Record the last set of elements
+  if(uFirstAddress <= uAddress) {
+    PushValue(uFirstAddress, uAddress,
+              &uLastValue, aList, aValue);
+    iColumnWidth = max(iColumnWidth, (int)aList.back().size());
+  }
+}
+
+//void RegisterCollection::SetAt(ExprList_t* pIndexers, Expression *pExpr) {
+//  throw Error("RegisterCollection::SetAt() not implemented");
+//}
+
+unsigned int ProgramMemoryCollection::GetLowerBound()
+{
+  return 0;
+}
+
+unsigned int ProgramMemoryCollection::GetUpperBound() 
+{
+  return GetSize() - 1;
+}
+bool ProgramMemoryCollection::bIsIndexInRange(unsigned int uAddress) 
+{
+  return m_pPma->get_rom(uAddress) != 0xffffffff;
+  // (uIndex >= GetLowerBound() &&  uIndex <= GetUpperBound()) || 
+
+}
+//-------------------------------------------------------------------
+//-------------------------------------------------------------------
+//
+// ProgramMemoryAccess
+//
+// The ProgramMemoryAccess class provides an interface to the processor's
+// program memory. On Pic processors, this is the memory where instructions
+// are stored. 
+//
 
 ProgramMemoryAccess::ProgramMemoryAccess(Processor *new_cpu) :
   MemoryAccess(new_cpu)
 {
   init(new_cpu);
+  m_pRomCollection = new ProgramMemoryCollection(new_cpu,
+						 "romData",
+						 this);
 }
 
 void ProgramMemoryAccess::init(Processor *new_cpu)
@@ -1502,13 +1631,13 @@ void ProgramMemoryAccess::putToIndex(unsigned int uIndex, instruction *new_instr
   if(!new_instruction)
     return;
 
- 
-  if(hasValid_opcode_at_index(uIndex)) {
+  //??? TSD 15MAY06 why would you care if the instruction you're replacing is valid or not?
+  //???  if(hasValid_opcode_at_index(uIndex)) {
 
     cpu->program_memory[uIndex] = new_instruction;
 
     new_instruction->update();
-  }
+  //???}
 
 }
 
@@ -1593,6 +1722,30 @@ instruction *ProgramMemoryAccess::get_base_instruction(unsigned int uIndex)
     return 0;
 }
 
+//----------------------------------------
+// get_rom - return the rom contents from program memory
+//           If the address is normal program memory, then the opcode
+//           of the instruction at that address is returned.
+//           If the address is some other special memory (like configuration
+//           memory in a PIC) then that data is returned instead.
+
+unsigned int ProgramMemoryAccess::get_rom(unsigned int addr)
+{
+  return cpu->get_program_memory_at_address(addr);
+}
+
+//----------------------------------------
+// put_rom - write new data to the program memory.
+//           If the address is in normal program memory, then a new instruction
+//           will be generated (if possible). If the address is some other 
+//           special memory (like configuration memory), then that area will
+//           be updated.
+//
+void ProgramMemoryAccess::put_rom(unsigned int addr,unsigned int value)
+{
+  return cpu->init_program_memory(addr,value);
+}
+
 //----------------------------------------
 // get_opcode - return an opcode from program memory.
 //              If the address is out of range return 0.
@@ -1905,12 +2058,7 @@ Register *RegisterMemoryAccess::get_register(unsigned int address)
   // If there are breakpoints set on the register, then drill down
   // through them until we get to the real register.
 
-  while(reg->isa() == Register::BP_REGISTER)
-    reg = ((BreakpointRegister *)reg)->m_replaced;
-
-
-  return reg;
-
+  return reg ? reg->getReg() : 0;
 }
 
 //--------------------------------------------------------------------------
@@ -1919,6 +2067,56 @@ void RegisterMemoryAccess::set_Registers(Register **_registers, int _nRegisters)
   nRegisters = _nRegisters; 
   registers = _registers;
 }
+//------------------------------------------------------------------------
+// insertRegister - Each register address may contain a linked list of registers.
+// The top most register is the one that is referenced whenever a processor
+// accesses the register memory. The primary purpose of the linked list is to
+// support register breakpoints. For example, a write breakpoint is implemented
+// with a breakpoint class derived from the register class. Setting a write
+// breakpoint involves creating the write breakpoint object and placing it
+// at the top of the register linked list. Then, when a processor attempts
+// to write to this register, the breakpoint object will capture this and
+// halt the simulation.
+
+bool RegisterMemoryAccess::insertRegister(int address, Register *pReg)
+{
+
+  if(!cpu || !registers || nRegisters<=address ||!pReg)
+    return false;
+
+  Register *ptop = registers[address];
+  pReg->setReplaced(ptop);
+  registers[address] = pReg;
+
+  return true;
+}
+
+//------------------------------------------------------------------------
+// removeRegister - see comment on insertRegister. This method removes
+// a register object from the breakpoint linked list.
+
+bool RegisterMemoryAccess::removeRegister(int address, Register *pReg)
+{
+  if(!cpu || !registers || nRegisters<=address ||!pReg)
+    return false;
+
+  Register *ptop = registers[address];
+
+  if (ptop == pReg  &&  pReg->getReplaced())
+    registers[address] = pReg->getReplaced();
+  else 
+    while (ptop) {
+      Register *pNext = ptop->getReplaced();
+      if (pNext == pReg) {
+	ptop->setReplaced(pNext->getReplaced());
+	return true;
+      }
+      ptop = pNext;
+    }
+
+  return false;
+}
+
 //-------------------------------------------------------------------
 bool RegisterMemoryAccess::hasBreak(unsigned int address)
 {
@@ -1966,13 +2164,24 @@ ProcessorConstructor::ProcessorConstructor(tCpuContructor _cpu_constructor,
   GetList()->push_back(this);
 }
 
-Processor * ProcessorConstructor::ConstructProcessor() {
-  return cpu_constructor();
+//------------------------------------------------------------
+Processor * ProcessorConstructor::ConstructProcessor(const char *opt_name)
+{
+  // Instantiate a specific processor. If a name is provided, then that
+  // will be used. Otherwise, the third name in the list of aliases for
+  // this processor will be used instead. (Why 3rd?... Before optional
+  // processor names were allowed, the default name matched what is now
+  // the third alias; this maintains a backward compatibility).
+
+  if (opt_name && strlen(opt_name))
+    return cpu_constructor(opt_name);
+  return cpu_constructor(names[2]);
 }
 
-ProcessorConstructorList * ProcessorConstructor::processor_list = new ProcessorConstructorList();
+ProcessorConstructorList * ProcessorConstructor::processor_list;
 
-ProcessorConstructorList * ProcessorConstructor::GetList() {
+ProcessorConstructorList * ProcessorConstructor::GetList() 
+{
   if(processor_list == NULL) {
     processor_list = new ProcessorConstructorList();
   }
@@ -2005,9 +2214,9 @@ ProcessorConstructor *ProcessorConstructorList::findByType(const char *name)
 // dump() --  Print out a list of all of the processors
 //
 
-void ProcessorConstructorList::dump(void)
+string ProcessorConstructorList::DisplayString(void)
 {
-
+  ostringstream stream;
   list <ProcessorConstructor *> :: iterator processor_iterator;
 
   const int nPerRow = 4;   // Number of names to print per row.
@@ -2043,7 +2252,7 @@ void ProcessorConstructorList::dump(void)
     for(i=0; i<nPerRow && processor_iterator != processor_list->end(); i++) {
 
       p = *processor_iterator++;
-      cout << p->names[1];
+      stream << p->names[1];
 
       if(i<nPerRow-1) {
 
@@ -2052,12 +2261,13 @@ void ProcessorConstructorList::dump(void)
 
         k = longest + 2 - strlen(p->names[1]);
         for(j=0; j<k; j++)
-          cout << ' ';
+          stream << ' ';
       }
     }
-    cout << '\n';
+    stream << endl;
   } 
-
+  stream << ends;
+  return string(stream.str());
 }
 
 
@@ -2068,17 +2278,19 @@ FileContext::FileContext(string &new_name)
   name_str = new_name;
   fptr = NULL;
   line_seek = 0;
-  _max_line =0;
+  m_uiMaxLine = 0;
   pm_address = 0;
+  m_bIsList = false;
 }
 
-FileContext::FileContext(char *new_name)
+FileContext::FileContext(const char *new_name)
 {
   name_str = string(new_name);
   fptr = NULL;
   line_seek = 0;
-  _max_line =0;
+  m_uiMaxLine = 0;
   pm_address = 0;
+  m_bIsList = false;
 }
 
 FileContext::~FileContext(void)
@@ -2101,7 +2313,7 @@ FileContext::~FileContext(void)
 void FileContext::ReadSource(void)
 {
 
-  if( (max_line() < 0) || name_str.length() == 0)
+  if( (max_line() <= 0) || name_str.length() == 0)
     return;
 
   const char *str = name_str.c_str();
@@ -2167,7 +2379,20 @@ char *FileContext::gets(char *buf, unsigned int nBytes)
 }
 
 //----------------------------------------
-void FileContext::rewind(void)
+unsigned int FileContext::max_line()
+{
+  if(fptr && !m_uiMaxLine) {
+    char buff[256];
+    rewind();
+    m_uiMaxLine=0;
+    while(fgets(buff,sizeof(buff),fptr))
+      m_uiMaxLine++;
+  }
+  return m_uiMaxLine;
+}
+
+//----------------------------------------
+void FileContext::rewind()
 {
   if(fptr)
     fseek(fptr,0,SEEK_SET);
@@ -2176,12 +2401,14 @@ void FileContext::rewind(void)
 //----------------------------------------
 void FileContext::open(const char *mode)
 {
-  if(!fptr)
+  if(!fptr) {
     fptr = fopen_path(name_str.c_str(), mode);
-
+    max_line();
+  }
 }
 //----------------------------------------
-void FileContext::close() {
+void FileContext::close()
+{
   if(fptr != NULL) {
     fclose(fptr);
     fptr = NULL;
@@ -2191,14 +2418,14 @@ void FileContext::close() {
 //----------------------------------------
 int FileContext::get_address(unsigned int line_number)
 {
-  if(line_number < max_line()  && pm_address)
+  if(line_number <= max_line()  && pm_address)
     return (*pm_address)[line_number];
   return -1;
 }
 //----------------------------------------
 void FileContext::put_address(unsigned int line_number, unsigned int address)
 {
-  if(line_number < max_line()  && pm_address)
+  if(line_number <= max_line()  && pm_address && (*pm_address)[line_number]<0)
     (*pm_address)[line_number] = address;
 }
 
@@ -2215,12 +2442,12 @@ FileContextList::~FileContextList(void)
 {
   FileContextList::iterator it;
   FileContextList::iterator itEnd = end();
-  for (it = begin(); it != itEnd; it++) {
+  for (it = begin(); it != itEnd; it++)
     it->close();
-  }
 }
 
-bool EndsWith(string &sSubject, string &sSubstring) {
+static bool EndsWith(string &sSubject, string &sSubstring)
+{
   if(sSubject.size() < sSubstring.size()) {
     return false;
   }
@@ -2235,7 +2462,6 @@ int FileContextList::Find(string &fname)
 {
   if(lastFile) {
     for (int i = 0; i < lastFile; i++) {
-//      if ((*this)[i]->name() == fname)
       if(EndsWith((*this)[i]->name(), fname)) {
         return i;
       }
@@ -2244,9 +2470,12 @@ int FileContextList::Find(string &fname)
   return -1;
 }
 
+extern bool bHasAbsolutePath(string &fname);
+
 int FileContextList::Add(string &new_name)
 {
-  string sFull = sSourcePath + new_name;
+  
+  string sFull = bHasAbsolutePath(new_name) ? new_name : (sSourcePath + new_name);
   push_back(FileContext(sFull));
   lastFile++;
   if(CSimulationContext::GetContext()->IsSourceEnabled()) {
@@ -2259,17 +2488,10 @@ int FileContextList::Add(string &new_name)
   return lastFile-1;
 }
 
-int FileContextList::Add(char *new_name)
+int FileContextList::Add(const char *new_name)
 {
-  string sFull = sSourcePath + new_name;
-  push_back(FileContext(sFull));
-  lastFile++;
-  back().open("r");
-  if(verbose)
-    cout << "Added new file named: " << new_name 
-         << "  id = " << lastFile << endl;
-
-  return lastFile-1;
+  string sNewName(new_name);
+  return Add (sNewName);
 }
 
 FileContext *FileContextList::operator [] (int file_id)
@@ -2318,3 +2540,18 @@ void FileContextList::SetSourcePath(const char *pPath) {
   SplitPathAndFile(sPath, sSourcePath, sFile);
   EnsureTrailingFolderDelimiter(sSourcePath);
 }
+
+//----------------------------------------
+// 
+void FileContextList::list_id(int new_list_id)
+{
+  FileContext *fc = operator[](list_file_id);
+  if (fc)
+    fc->setListId(false);
+
+  list_file_id = new_list_id;
+  fc = operator[](list_file_id);
+  if (fc)
+    fc->setListId(true);
+
+}
