@@ -1,5 +1,6 @@
 /*
    Copyright (C) 1998 T. Scott Dattalo
+   Copyright (C) 2006 Roy R Rankin
 
 This file is part of gpsim.
 
@@ -28,6 +29,8 @@ Boston, MA 02111-1307, USA.  */
 #include <vector>
 #include <sstream>
 
+#include <math.h>
+
 #include "../config.h"
 #include "pic-processor.h"
 #include "stimuli.h"
@@ -95,6 +98,12 @@ void Stimulus_Node::new_name(string &rName)
   new_name(rName.c_str());
 }
 
+double Stimulus_Node::get_nodeVoltage()
+{
+  if (future_cycle) // RC calculation in progress, get current value
+    callback();
+  return(voltage);
+}
 
 void dump_node_list(void)
 {
@@ -145,16 +154,15 @@ void dump_stimulus_list(void)
   Symbol_Table::stimulus_symbol_iterator it;
   Symbol_Table::stimulus_symbol_iterator itEnd = ST.endStimulusSymbol();
   for(it = ST.beginStimulusSymbol(); it != itEnd; it++) {
-      stimulus *t = (*it)->getStimulus();
-      if(t) {
-        cout << "stimulus ";
-        cout << t->name();
-        if(t->snode)
-          cout << " attached to " << t->snode->name();
-        cout << '\n';
-      }
+    stimulus *t = (*it)->getStimulus();
+    if(t) {
+      cout << t->name();
+      //if(t->snode)
+      // cout << " attached to " << t->snode->name();
+      t->show();
+      cout << '\n';
     }
-  cout << "returning from dump\n";
+  }
 
 }
 
@@ -171,7 +179,10 @@ Stimulus_Node::Stimulus_Node(const char *n)
   warned  = 0;
   current_time_constant = 0.0;
   delta_voltage = 0.0;
-  min_time_constant = 1e6; // by making this large, most stimuli will update instantly.
+  cap_start_cycle = 0;
+  future_cycle = 0;
+  minThreshold = 0.1; // volts
+  min_time_constant = 1000; // in cycles
   bSettling = false;  
   if(n)
     {
@@ -347,7 +358,7 @@ void Stimulus_Node::refresh()
 
     case 1:
       // Only one stimulus is attached.
-      voltage = sptr->get_Vth();
+      finalVoltage = sptr->get_Vth();   // RP - was just voltage
       Zth =  sptr->get_Zth();
       break;
 
@@ -358,13 +369,15 @@ void Stimulus_Node::refresh()
       stimulus *sptr2 = sptr ? sptr->next : 0;
       if(!sptr2)
         break;     // error, nStimuli is two, but there aren't two stimuli
-
-      double Z1 = sptr->get_Zth();
-      double Z2 = sptr2->get_Zth();
-      double resistance = Z1 + Z2;
-      finalVoltage = (sptr->get_Vth()*Z2  + sptr2->get_Vth()*Z1) / resistance;
-      Zth = Z1*Z2/resistance;
-      Cth = sptr->get_Cth() + sptr2->get_Cth();
+      
+      double V1,Z1,C1;
+      double V2,Z2,C2;
+      sptr->getThevenin(V1,Z1,C1);
+      sptr2->getThevenin(V2,Z2,C2);
+      finalVoltage = (V1*Z2  + V2*Z1) / (Z1+Z2);
+      Zth = Z1*Z2/(Z1+Z2);
+      Cth = C1+C2;
+      
       }
       break;
 
@@ -391,22 +404,89 @@ void Stimulus_Node::refresh()
 
       //cout << "multi-node summing:\n";
       while(sptr) {
+
+	double V1,Z1,C1;
+	sptr->getThevenin(V1,Z1,C1);
         /*
         cout << " N: " <<sptr->name() 
-        << " V=" << sptr->get_Vth() 
-        << " Z=" << sptr->get_Zth()
-        << " C=" << sptr->get_Cth() << endl; 
+        << " V=" << V1
+        << " Z=" << Z1
+        << " C=" << C1 << endl;
         */
-        double Cs = 1 / sptr->get_Zth();
-        finalVoltage += sptr->get_Vth() * Cs;
+
+        double Cs = 1 / Z1;
+        finalVoltage += V1 * Cs;
         conductance += Cs;
-        Cth += sptr->get_Cth();
+        Cth += C1;
         sptr = sptr->next;
       }
       Zth = 1.0/conductance;
       finalVoltage *= Zth;
       }
     }
+
+    current_time_constant = Cth * Zth;
+    if(((guint64)(current_time_constant*get_cycles().instruction_cps()) 
+      < min_time_constant) 
+      ||
+      (fabs(finalVoltage - voltage) < minThreshold))
+    {
+      if (verbose)
+        cout << "Stimulus_Node::refresh " << name() << " use DC " <<
+		finalVoltage << " as current_time_constant=" <<
+          current_time_constant << endl;
+
+      if (future_cycle)	// callback is active
+      {
+	get_cycles().clear_break(this);
+      }
+
+      voltage = finalVoltage;
+      future_cycle = 0;
+
+    } else {
+	settlingTimeStep = (guint64) (0.11 * 
+	  get_cycles().instruction_cps() * current_time_constant);
+	voltage = initial_voltage;
+
+        if (verbose)
+	  cout << "Stimulus_Node::refresh " << name() << " settlingTimeStep=" 
+	    << settlingTimeStep << " voltage=" << voltage << " Finalvoltage=" 
+	    << finalVoltage << endl;
+  /*
+	If future_cycle is not 0 we are in the middle of an RC
+	calculation, but an input condition has changed.
+  */
+
+      if (future_cycle)
+      {
+	callback();
+      }
+      else
+      {
+	cap_start_cycle = get_cycles().value; 
+        future_cycle = cap_start_cycle + settlingTimeStep;
+        get_cycles().set_break(future_cycle,this);
+      }
+    }
+
+  }
+
+
+}
+
+//------------------------------------------------------------------------
+// updateStimuli
+// 
+// drive all the stimuli connected to this node.
+
+void Stimulus_Node::updateStimuli()
+{
+  stimulus *sptr = stimuli;
+
+  while(sptr) {
+    sptr->set_nodeVoltage(voltage);
+    sptr = sptr->next;
   }
 }
 
@@ -416,51 +496,76 @@ void Stimulus_Node::update()
   if(stimuli) {
 
     refresh();
-
-    delta_voltage = 0.0;
-    current_time_constant = Cth * Zth;
-
-    if(current_time_constant < min_time_constant) {
-      voltage = finalVoltage;
-
-      stimulus *sptr = stimuli;
-
-      while(sptr) {
-        sptr->set_nodeVoltage(voltage);
-        sptr = sptr->next;
-      }
-    } else {
-
-      // Capacitive loading must be relatively large.
-      delta_voltage = finalVoltage - initial_voltage;
-
-      if(bSettling) 
-        get_cycles().reassign_break(future_cycle,get_cycles().value + 1,this);
-      else
-        get_cycles().set_break(get_cycles().value +1,this);
-
-      bSettling = true;
-    }
+    updateStimuli();
   }
 }
 
+void Stimulus_Node::set_nodeVoltage(double v)
+{
+  voltage = v;
+  updateStimuli();
+}
+
 //------------------------------------------------------------------------
 void Stimulus_Node::callback()
 {
-  voltage = initial_voltage + delta_voltage;
 
-  callback_print();
-  cout << " - updating voltage from " 
-       << initial_voltage << "V to "
-       << voltage << "V\n";
+    if (verbose)
+	callback_print();
 
-  stimulus *sptr = stimuli;
+    initial_voltage = voltage;
+    double Time_Step;
+    double expz;
+      //
+      // increase time step as capacitor charges more slowly as final
+      // voltage is approached.
+      //
+                                                                            
+      //
+      // The following is an exact calculation, assuming no circuit
+      // changes,  regardless of time step.
+      //
+      Time_Step = (get_cycles().value - cap_start_cycle)/
+        (get_cycles().instruction_cps()*current_time_constant);
+      expz = exp(-Time_Step);
+      voltage = finalVoltage* (1.-expz) + voltage * expz;
 
-  while(sptr) {
-    sptr->set_nodeVoltage(voltage);
-    sptr = sptr->next;
-  }
+      if (verbose)
+  	cout << "\tVoltage was " << initial_voltage << "V now "
+          << voltage << "V\n";
 
+      if (fabs(finalVoltage - voltage) < minThreshold) {
+    	voltage = finalVoltage;
+        future_cycle = 0;
+	if (verbose)
+          cout << "\t" << name() << 
+	    " Final voltage " << finalVoltage << " reached at " 
+	    << get_cycles().value << " cycles\n";
+      } 
+      else if(get_cycles().value >= future_cycle) // got here via break
+      {
+        settlingTimeStep  = (guint64) (1.5 * settlingTimeStep);
+        cap_start_cycle = get_cycles().value;
+	future_cycle = cap_start_cycle + settlingTimeStep;
+    	get_cycles().set_break(future_cycle, this);
+        if (verbose)
+	  cout << "\tBreak reached at " << cap_start_cycle << 
+	    " cycles, next break set for " 
+	    << future_cycle << " delta=" << settlingTimeStep << endl;
+      }
+      else	// updating value before break don't increase step size
+      {
+        cap_start_cycle = get_cycles().value;
+        get_cycles().reassign_break(future_cycle, 
+		cap_start_cycle + settlingTimeStep, this);
+	future_cycle = get_cycles().value + settlingTimeStep;
+        if (verbose)
+	  cout << "\tcallback called at " << cap_start_cycle << 
+	    " cycles, next break set for " << future_cycle << " delta=" 
+	    << settlingTimeStep << endl;
+      }
+
+  updateStimuli();
 }
 
 //------------------------------------------------------------------------
@@ -472,7 +577,7 @@ void Stimulus_Node::callback_print()
 //------------------------------------------------------------------------
 void Stimulus_Node::time_constant(double new_tc)
 {
-  min_time_constant = new_tc;
+  min_time_constant = (unsigned int)(new_tc*get_cycles().instruction_cps());
 }
 
 //------------------------------------------------------------------------
@@ -540,20 +645,21 @@ void stimulus::show()
 string stimulus::toString() 
 {
   ostringstream s;
-  s << name();
+
+  s << " stimulus ";
   if(snode)
     s << " attached to " << snode->name();
   s << endl
-       << "  Vth=" << get_Vth() << "V"
-       << "  Zth=" << get_Zth() << " ohms"
-       << "  Cth=" << get_Cth() << " F"
-       << "  nodeVoltage= " << get_nodeVoltage() << "V"
-       << endl 
-       << " Driving=" << getDriving()
-       << " drivingState=" << getDrivingState()
-       << " drivenState=" << getDrivenState()
-       << " bitState=" << getBitChar()
-       << endl;
+    << " Vth=" << get_Vth() << "V"
+    << " Zth=" << get_Zth() << " ohms"
+    << " Cth=" << get_Cth() << "F"
+    << " nodeVoltage= " << get_nodeVoltage() << "V"
+    << endl 
+    << " Driving=" << getDriving()
+    << " drivingState=" << getDrivingState()
+    << " drivenState=" << getDrivenState()
+    << " bitState=" << getBitChar();
+
   return s.str();
 }
 void stimulus::attach(Stimulus_Node *s)
@@ -566,6 +672,36 @@ void stimulus::detach(Stimulus_Node *s)
   if(snode == s)
     snode = 0; 
 }
+
+void   stimulus::getThevenin(double &v, double &z, double &c)
+{
+  v = get_Vth();
+  z = get_Zth();
+  c = get_Cth();
+}
+
+//========================================================================
+// 
+PinMonitor::PinMonitor()
+{
+}
+
+PinMonitor::~PinMonitor()
+{
+}
+
+void PinMonitor::addSink(SignalSink *new_sink)
+{
+  if(new_sink)
+    sinks.push_back(new_sink);
+}
+
+void PinMonitor::removeSink(SignalSink *pSink)
+{
+  if(pSink)
+    sinks.remove(pSink);
+}
+
 //========================================================================
 
 
@@ -699,7 +835,7 @@ void Event::callback(void)
 
   // If there's a node attached to this stimulus, then update it.
   if(snode)
-    snode->update(get_cycles().value);
+    snode->update();
 
   // If the event is inactive.
 
@@ -728,70 +864,34 @@ void source_stimulus::show()
 {
   stimulus::show();
 }
-//========================================================================
-//
-
-IOPIN::IOPIN(IOPORT *i, unsigned int b,const char *opt_name, Register **_iopp)
-  : stimulus()
+void source_stimulus::put_period(Value *pValue)
 {
-  iop = i;
-  iopp = _iopp;
-  iobit=b;
-  l2h_threshold = 2.0;
-  h2l_threshold = 1.0;
-  bDrivenState = false;
-  cForcedDrivenState = 'Z';
-
-  Zth = 1e8;
-  Vth = 5.0;
-  ZthWeak = 1e3;
-  ZthFloating = 1e6;
-
-  snode = 0;
-  m_monitor=0;
-
-  if(iop) {
-    iop->attach_iopin(this,b);
-
-    // assign the name to the I/O pin.
-    // If one was passed to us (opt_name), then use it
-    // otherwise, derive the name from the I/O port to 
-    // which this pin is attached.
-
-    char name_str[100];
-    if(opt_name) {
-      snprintf(name_str,sizeof(name_str),"%s.%s",
-	       iop->name().c_str(),
-	       opt_name);
-
-    } else {
-
-      char bs[3];
-
-      strncpy(name_str, iop->name().c_str(),sizeof(name_str) - sizeof(bs));
-      if(iobit < 10) {
-        bs[0] = iobit+'0';
-        bs[1] = 0;
-      } else {
-        bs[0] = (iobit / 10) + '0';
-        bs[1] = (iobit % 10) + '0';
-        bs[2] = 0;
-      }
-
-      strcat(name_str,bs);
-    }
-
-    new_name(name_str);
-  } else {
-    // there's no IO port associated with this pin.
-
-    // If a name was provided, use it:
-    if(opt_name)
-      new_name(opt_name);
-  }
-
+  if (pValue)
+    pValue->get(period);
+}
+void source_stimulus::put_duty(Value *pValue)
+{
+  if (pValue)
+    pValue->get(duty);
+}
+void source_stimulus::put_phase(Value *pValue)
+{
+  if (pValue)
+    pValue->get(phase);
+}
+void source_stimulus::put_initial_state(Value *pValue)
+{
+  if (pValue)
+    pValue->get(initial_state);
+}
+void source_stimulus::put_start_cycle(Value *pValue)
+{
+  if (pValue)
+    pValue->get(start_cycle);
 }
 
+//========================================================================
+//
 IOPIN::IOPIN(const char *_name,
 	     double _Vth, 
 	     double _Zth,
@@ -805,10 +905,7 @@ IOPIN::IOPIN(const char *_name,
   if(verbose)
     cout << "IOPIN default constructor\n";
 
-  iop = 0;
-  iopp = 0;
-  iobit=0;
-  l2h_threshold = 2.0;
+  l2h_threshold = 2.0;      // PICs are CMOS and use CMOS-like thresholds
   h2l_threshold = 1.0;
   bDrivenState = false;
   cForcedDrivenState = 'Z';
@@ -823,22 +920,6 @@ void IOPIN::setMonitor(PinMonitor *new_pinMonitor)
     m_monitor = new_pinMonitor;
 }
 
-void IOPIN::attach_to_port(IOPORT *i, unsigned int b)
-{
-  iop = i; 
-  iobit=b;
-  if(iop)
-    iop->attach_iopin(this,b);
-}
-
-void IOPIN::disconnect_from_port()
-{
-  if(iop) {
-    //iop->detach_iopin(this);
-    iop = 0;
-  }
-  iobit = 0;
-}
 
 IOPIN::~IOPIN()
 {
@@ -847,9 +928,6 @@ IOPIN::~IOPIN()
 
 void IOPIN::attach(Stimulus_Node *s)
 {
-  if(iop)
-    iop->attach_node(s,iobit);
-
   snode = s;
 }
 
@@ -858,21 +936,6 @@ void IOPIN::show()
   stimulus::show();
 }
 
-//
-// Accomodate breakpoints by providing an indirect way
-// through which the ioport is selected. The breakpoint
-// engine is cabable of intercepting this indirect access.
-//
-Register *IOPIN::get_iop(void)
-{
-  if(iopp)
-    return *iopp;
-  else if(iop)
-    return iop;
-  else
-    return 0;
-}
-
 //--------------------
 // set_nodeVoltage()
 //
@@ -925,12 +988,7 @@ void IOPIN::putState(bool new_state)
     // the I/O port directly.
 
     if(snode)
-      snode->update(0);
-    else {
-      Register *port = get_iop();
-      if(port)
-	port->setbit(iobit, new_state);
-    }
+      snode->update();
   }
   if(m_monitor)
     m_monitor->putState(new_state?'1':'0');
@@ -966,11 +1024,6 @@ void IOPIN::setDrivingState(char new3State)
 
 bool IOPIN::getDrivingState(void)
 {
-  Register *port = get_iop();
-
-  if(port)
-    bDrivingState = port->get_bit(iobit);
-
   return bDrivingState;
 }
 
@@ -995,14 +1048,14 @@ void IOPIN::setDrivenState(bool new_state)
     cout << name()<< " setDrivenState= " 
 	 << (new_state ? "high" : "low") << endl;
 
-  Register *port = get_iop();
-  if(port)
-    port->setbit(iobit, new_state);
-
   // Propagate the new state to those things monitoring this pin.
   // (note that the 3-state value is what's propagated).
   if(m_monitor)
+  {
     m_monitor->setDrivenState(getBitChar());
+    if(verbose & 16)
+      cout << name() << " setting state of monitor to " << getBitChar() << endl;
+  }
 }
 
 //------------------------------------------------------------------------
@@ -1049,9 +1102,6 @@ void IOPIN::toggle()
  */
 double IOPIN::get_Vth()
 {
-  if(!snode && iop)
-    return ( (iop->value.get() & (1<<iobit)) ? Vth : 0.0);
-
   return Vth;
 
 }
@@ -1059,7 +1109,8 @@ double IOPIN::get_Vth()
 char IOPIN::getBitChar()
 {
   if(!snode)
-    return 'Z';  // High impedance - unknown state.
+    return getForcedDrivenState();      // RCP - Changed to match IO_bi_directional
+//  was  return 'Z';  // High impedance - unknown state.
 
   if(snode->get_nodeZth() > ZthFloating)
     return 'Z';
@@ -1067,25 +1118,23 @@ char IOPIN::getBitChar()
   if(snode->get_nodeZth() > ZthWeak)
     return getDrivenState() ? 'W' : 'w';
 
-  return getDrivingState() ? '1' : '0';
+  return getDrivenState() ? '1' : '0';
+}
+void IOPIN::newGUIname(const char *s)
+{
+                                                                                
+  if(s)
+  {
+    gui_name_updated = true;
+    gui_name = string(s);
+  }
+}
+string &IOPIN::GUIname(void) const
+{
+  return (string &)gui_name;
 }
-
 //========================================================================
 //
-IO_bi_directional::IO_bi_directional(IOPORT *i, unsigned int b,const char *opt_name, Register **_iopp)
-  : IOPIN(i,b,opt_name,_iopp)
-{
-
-  // Thevenin equivalent while configured as an output 
-  Vth = 5.0;
-  Zth = 250;
-
-  // Thevenin equivalent while configured as an input 
-  VthIn = 0.3;
-  ZthIn = 1e8;
-
-}
-
 IO_bi_directional::IO_bi_directional(const char *_name,
 				     double _Vth, 
 				     double _Zth,
@@ -1122,48 +1171,41 @@ double IO_bi_directional::get_Zth()
 
 }
 
+/*
+   getBitChar() returns bit status as follows
+     Input pin
+	1> Pin considered floating, 
+	   return 'Z'
+	2> Weak Impedance on pin, 
+	   return 'W" if high or 'w' if low
+	3> Pin being driven externally
+	   return '1' node voltage high '0' if low
+     Output pin
+	1> Node voltage opposite driven value
+	   return 'X' if node voltage high or 'x' if inode voltage low
+	2> Node voltage same as driven value
+	   return '1' node voltage high '0' if low
+*/
+
 char IO_bi_directional::getBitChar()
 {
   if(!snode && !getDriving() )
     return getForcedDrivenState();
 
   if(snode) {
-
-    if(snode->get_nodeZth() > ZthFloating)
-      return 'Z';
-
-    if(snode->get_nodeZth() > ZthWeak)
-      return getDrivenState() ? 'W' : 'w';
-
-    // There's at least one strong driver tied to the node
-    if(!getDriving()) {
-      if(getDrivenState()) {
-	if(nodeVoltage < 4.5)
-	  return 'X';
-	else
-	  return '1';
-      } else {
-	if(nodeVoltage > 0.5)
-	  return 'X';
-	else
-	  return '0';
-      }
-    }
+                                                                                
+    if (!getDriving())		// input pin
+    {
+      if(snode->get_nodeZth() > ZthFloating)
+        return 'Z';
+                                                                                
+      if(snode->get_nodeZth() > ZthWeak)
+        return getDrivenState() ? 'W' : 'w';
+     }
+    else if(getDrivenState() != getDrivingState())
+        return getDrivenState() ? 'X' : 'x';
   }
 
-  if(getDriving()) {
-    if(getDrivingState()) {
-      if(nodeVoltage < 4.5)
-	return 'X';
-      else
-	return '1';
-    } else {
-      if(nodeVoltage > 0.5)
-	return 'X';
-      else
-	return '0';
-    }
-  }
 
   return getDrivenState() ? '1' : '0';
 }
@@ -1182,18 +1224,8 @@ void IO_bi_directional::update_direction(unsigned int new_direction, bool refres
 
   // If this pin is not associated with an IO Port, but it's tied
   // to a stimulus, then we need to update the stimulus.
-  if(refresh && !iop && snode)
-    snode->update(0);
-}
-
-
-IO_bi_directional_pu::IO_bi_directional_pu(IOPORT *i, unsigned int b,
-					   const char *opt_name, Register **_iopp)
-  : IO_bi_directional(i, b,opt_name,_iopp)
-{
-  Vpullup = Vth;
-  Zpullup = 10e3;
-  bPullUp = false;
+  if(refresh && snode)
+    snode->update();
 }
 
 IO_bi_directional_pu::IO_bi_directional_pu(const char *_name,
@@ -1241,9 +1273,9 @@ double IO_bi_directional_pu::get_Vth()
   
   /**/
   if(verbose & 1)
-    cout << name() << "get_Vth "
+    cout << name() << " get_Vth PU "
 	 << " driving=" << getDriving()
-	 << " bDrivingState=" << bDrivingState
+	 << " DrivingState=" << getDrivingState()
 	 << " bDrivenState=" << bDrivenState
 	 << " Vth=" << Vth
 	 << " VthIn=" << VthIn
@@ -1263,6 +1295,22 @@ double IO_bi_directional_pu::get_Vth()
 
 }
 
+/*
+   getBitChar() returns bit status as follows
+     Input pin
+	1> Pin considered floating, 
+	   return 'Z'
+	2> Weak Impedance on pin, 
+	   return 'W" if high or 'w' if low
+	3> Pin being driven externally
+	   return '1' node voltage high '0' if low
+     Output pin
+	1> Node voltage opposite driven value
+	   return 'X' if node voltage high or 'x' if inode voltage low
+	2> Node voltage same as driven value
+	   return '1' node voltage high '0' if low
+*/
+
 char IO_bi_directional_pu::getBitChar()
 {
   if(!snode && !getDriving() ) {
@@ -1271,50 +1319,23 @@ char IO_bi_directional_pu::getBitChar()
   }
 
   if(snode) {
-
-    if(snode->get_nodeZth() > ZthFloating)
-      return 'Z';
-
-    if(snode->get_nodeZth() > ZthWeak)
-      return getDrivenState() ? 'W' : 'w';
-
-    // There's at least one strong driver tied to the node
-    if(!getDriving()) {
-      if(getDrivenState()) {
-        if(nodeVoltage < 4.5)
-          return 'X';
-        else
-          return '1';
-      } else {
-        if(nodeVoltage > 0.9)
-          return 'X';
-        else
-          return '0';
-      }
-    }
+                                                                                
+    if (!getDriving())		// input pin
+    {
+      if(snode->get_nodeZth() > ZthFloating)
+        return 'Z';
+                                                                                
+      if(snode->get_nodeZth() > ZthWeak)
+        return getDrivenState() ? 'W' : 'w';
+     }
+    else if(getDrivenState() != getDrivingState())
+        return getDrivenState() ? 'X' : 'x';
   }
 
-  if(getDriving()) {
-    if(getDrivingState()) {
-      if(nodeVoltage < 4.5)
-        return 'X';
-      else
-        return '1';
-    } else {
-      if(nodeVoltage > 0.5)
-        return 'X';
-      else
-        return '0';
-    }
-  }
+
   return getDrivenState() ? '1' : '0';
 }
 
-IO_open_collector::IO_open_collector(IOPORT *i, unsigned int b,
-				     const char *opt_name, Register **_iopp)
-  : IO_bi_directional_pu(i,b,opt_name,_iopp)
-{
-}
 IO_open_collector::IO_open_collector(const char *_name)
   : IO_bi_directional_pu(_name)
 {
@@ -1323,6 +1344,17 @@ IO_open_collector::IO_open_collector(const char *_name)
 
 double IO_open_collector::get_Vth()
 {
+  /**/
+  if(verbose & 1)
+    cout << name() << " get_Vth OC"
+	 << " driving=" << getDriving()
+	 << " DrivingState=" << getDrivingState()
+	 << " bDrivenState=" << bDrivenState
+	 << " Vth=" << Vth
+	 << " VthIn=" << VthIn
+	 << " bPullUp=" << bPullUp << endl;
+  /**/  
+
   if(getDriving() && !getDrivingState())
     return 0.0;
 
@@ -1384,7 +1416,8 @@ char IO_open_collector::getBitChar()
 ValueStimulus::ValueStimulus(const char *n)
   : source_stimulus() 
 {
-  initial = 0;
+  initial.time = 0;
+  initial.v = 0;
   current = 0;
 
   if(n)
@@ -1401,7 +1434,7 @@ ValueStimulus::ValueStimulus(const char *n)
 
 ValueStimulus::~ValueStimulus()
 {
-  delete initial;
+  delete initial.v;
   delete current;
 
   for(sample_iterator = samples.begin();
@@ -1426,16 +1459,18 @@ void ValueStimulus::show()
       si != samples.end();
       ++si) {
 
-    double d;
-    (*si).v->get(d);
-    cout << "    " << dec << (*si).time
-	 <<  '\t'  << d
+    //double d;
+    //(*si).v->get(d);
+    cout << "    t=" << dec << (*si).time
+	 <<  ",v=" << (*si).v->toString()
 	 << '\n';
 
   }
 
+  if (initial.v)
+    cout << "  initial=" << initial.v->toString() << '\n';
+
   cout
-    << "  initial=" << initial << '\n'
     << "  period=" << period << '\n'
     << "  start_cycle=" << start_cycle << '\n'
     << "  Next break cycle=" << future_cycle << '\n';
@@ -1487,19 +1522,23 @@ void ValueStimulus::callback()
     cout <<"  next transition = " << future_cycle << '\n';
 }
 
-/*
-void ValueStimulus::put_initial(ValueStimulusData &data_point)
+void ValueStimulus::put_initial_state(Value *pValue)
 {
-  samples.push_front(data_point);
-  initial = data_point.v;
+  if (pValue && !initial.v) {
+    initial.time = 0;
+    initial.v = pValue->copy();
+  }
 }
-*/
 
 void ValueStimulus::put_data(ValueStimulusData &data_point)
 {
-  samples.push_back(data_point);
+  ValueStimulusData *sample = new ValueStimulusData;
+  sample->time = data_point.time;
+  sample->v = data_point.v;
+  samples.push_back(*sample);
 
 }
+
 double ValueStimulus::get_Vth()
 {
   double v=initial_state;
@@ -1529,11 +1568,16 @@ void ValueStimulus::start()
     cout << "Starting asynchronous stimulus\n";
 
   if(period) {
+
     // Create a data point for the rollover condition.
-    ValueStimulusData *vRestart = new ValueStimulusData();
-    vRestart->v = new Float(initial_state);
-    vRestart->time = period;
-    put_data(*vRestart);
+    // If an initial value was supplied when the stimulus was created, then
+    // that's what we'll use for the rollover. Otherwise, we'll create 
+    // a rollover based on 'initial_state' (which should be a default value).
+
+    ValueStimulusData vSample;
+    vSample.time = period;
+    vSample.v = initial.v ? initial.v : new Float(initial_state);
+    put_data(vSample);
   }
 
   sample_iterator = samples.begin();
@@ -1544,7 +1588,7 @@ void ValueStimulus::start()
     if(digital)
       initial_state = (initial_state > 0.0) ? Vth : 0.0;
 
-    current       = initial;
+    current       = initial.v;
     next_sample   = *sample_iterator;
     future_cycle  = next_sample.time;// + start_cycle;
 
@@ -1601,7 +1645,9 @@ AttributeStimulus::~AttributeStimulus()
 */
 void AttributeStimulus::show()
 {
-
+  if (attr)
+    cout << "\nDriving Attribute:" << attr->name() << endl;
+  ValueStimulus::show();
 }
 
 void AttributeStimulus::callback()
@@ -1656,7 +1702,7 @@ void AttributeStimulus::setClientAttribute(Value *v)
 
   attr = v;
 
-  if(verbose && v)
+  if((bool)verbose && v)
     cout << " attached " << name() << " to attribute: " << v->name() << endl;
 }
 
@@ -1700,7 +1746,7 @@ void stimorb_attach(char *node, char_list *stimuli)
 
       stimuli = stimuli->next;
     }
-    sn->update(0);
+    sn->update();
   }
   else {
     cout << "Warning: Node \"" << node << "\" was not found in the node list\n";
@@ -1737,7 +1783,7 @@ void stimuli_attach(StringList_t *sl)
       {
         AttachStimulusToNode(sn, *si);
       }
-      sn->update(0);
+      sn->update();
   }
   else {
     cout << "Warning: Node \"" << (*si) << "\" was not found in the node list\n";
@@ -1763,7 +1809,7 @@ void stimuli_attach(SymbolList_t *sl)
     {
       AttachStimulusToNode(sn, (*si)->name());
     }
-    sn->update(0);
+    sn->update();
   }
   else {
     // The first symbol is not a node - so let's assume that 
@@ -1827,11 +1873,11 @@ void stimuli_attach(Value *pNode, PinList_t *pPinList)
         AttachStimulusToNode(sn, pPin->name(), pPin);
       }
       else {
-        Module *pMod;
+        Module *pMod = NULL;
         if(pPinArgument->m_iFlags & Pin_t::eActiveProc) {
           pMod = get_active_cpu();
         }
-        else {
+        else if ( pPinArgument->m_sModuleName ) {
           // this dynamic_cast always fails here
           pMod = dynamic_cast<Module*>(pPinArgument->m_sModuleName);
           if(pMod == NULL) {
@@ -1847,9 +1893,16 @@ void stimuli_attach(Value *pNode, PinList_t *pPinList)
           }
         }
         if(pMod == NULL) {
-          GetUserInterface().DisplayMessage(
-            "attach error: did not find module '%s'\n",
-            pPinArgument->m_sModuleName->name().c_str());
+          if (NULL == pPinArgument->m_sModuleName) {
+            GetUserInterface().DisplayMessage(
+              "attach error: did not find pin '%s'\n",
+              pPinArgument->m_sPin->name().c_str());
+          }
+          else {
+            GetUserInterface().DisplayMessage(
+              "attach error: did not find module '%s'\n",
+              pPinArgument->m_sModuleName->name().c_str());
+          }
           bSuccess = false;
         }
         else {
@@ -1887,7 +1940,8 @@ void stimuli_attach(Value *pNode, PinList_t *pPinList)
           }
           else {
             bSuccess = false;
-            if(pPin == NULL) {
+            if(pPin == NULL && 
+		strcmp(pPinArgument->m_sPin->showType().c_str(), "module_symbol")) {
               int iValue = -1;
               if(pPinArgument->m_sPin) {
                 pPinArgument->m_sPin->get(iValue);
@@ -1902,7 +1956,7 @@ void stimuli_attach(Value *pNode, PinList_t *pPinList)
             }
             else {
               GetUserInterface().DisplayMessage(
-                  "attach error: pin argument '%s' type(%s) is not of type Integer\n",
+                  "attach error: pin argument '%s' type(%s) is not of type Integer or stimulus\n",
                   pPinArgument->m_sPin->name().c_str(),
                   pPinArgument->m_sPin->showType().c_str());
               }
@@ -1911,7 +1965,7 @@ void stimuli_attach(Value *pNode, PinList_t *pPinList)
       }
 #endif
     }
-    sn->update(0);
+    sn->update();
   }
   else {
     // The first symbol is not a node - so let's assume that 
@@ -1979,7 +2033,7 @@ void AttachStimulusToNode(Stimulus_Node *sn, string &sStimulusName, stimulus *st
 }
 
 stimulus *Pin_t::GetStimulus() {
-  stimulus_symbol * pPinSymbol;
+  stimulus_symbol * pPinSymbol = NULL;
   if(m_sPin) {
     pPinSymbol = dynamic_cast<stimulus_symbol*>(m_sPin);
   }
@@ -2024,9 +2078,16 @@ IOPIN *Pin_t::GetIOPin() {
     }
   }
   if(pMod == NULL) {
-    GetUserInterface().DisplayMessage(
-      "attach error: did not find module '%s'\n",
-      m_sModuleName->name().c_str());
+    if (NULL == m_sModuleName) {
+      GetUserInterface().DisplayMessage(
+        "attach error: did not find pin '%s'\n",
+        m_sPin->name().c_str());
+    }
+    else {
+      GetUserInterface().DisplayMessage(
+        "attach error: did not find module '%s'\n",
+        m_sModuleName->name().c_str());
+    }
     bSuccess = false;
   }
   else {
