@@ -40,10 +40,14 @@ Boston, MA 02111-1307, USA.  */
 #include "stimuli.h"
 #include "symbol_orb.h"
 #include "expr.h"
+#include "ValueCollections.h"
 #include "operator.h"
 #include "errors.h"
 #include "protocol.h"
 #include "cmd_gpsim.h"
+#include "sim_context.h"
+
+class IIndexedCollection;
 
 //
 // ***NOTE*** Ideally, I would like to use a the std container 'map' 
@@ -132,10 +136,33 @@ void Symbol_Table::add_stimulus(stimulus *s)
   }
 }
 
-bool Symbol_Table::add(Value *s) {
+// This is an experiment to have the symbol table ordered
+// case insensitive unless their is a match for a case insensitive
+// compare, then a case sensitive compare is made.
+int SymbolCompare(const char *pLeft, const char *pRight) {
+#if 0
+  int iResult = stricmp(pLeft, pRight);
+  if(iResult < 0) {
+    return -1;
+  }
+  else if(iResult > 0) {
+    return 1;
+  }
+  else {
+    return strcmp(pLeft, pRight);
+  }
+#else
+    return strcmp(pLeft, pRight);
+#endif
+}
+
+//------------------------------------------------------------------------
+// add(Value *) - add a symbol to the symbol table
+bool Symbol_Table::add(Value *s)
+{
   if(s) {
     if(s->name().empty()) {
-      printf("Symbol_Table::add() attempt to add a symbol with no name: %s",
+      printf("Symbol_Table::add() attempt to add a symbol with no name: %s\n",
         s->toString().c_str());
     }
     else {
@@ -143,91 +170,34 @@ bool Symbol_Table::add(Value *s) {
         s, NameLessThan());
       if (it != end() &&
         (*it)->name() == s->name()) {
-//          printf("Symbol_Table::add(): Warning: previous symbol %s overwritten\n", s->name().c_str());
-//          erase(it);
         GetUserInterface().DisplayMessage(
             "Symbol_Table::add(): Warning: failed to add symbol "
             "because a symbol by the name '%s' already exists, new object is type %s\n",
             s->name().c_str(), s->showType().c_str());
         return false;
       }
+      if(verbose)
+	cout << "Adding '" << s->name() << "' to the symbol table\n";
       insert(it, s);
-// for testing
-//      GetUserInterface().DisplayMessage("Symbol added: %s\n",
-//        s->name().c_str());
+
       return true;
     }
   }
   return false;
 }
 
-void Symbol_Table::AddFromCommandLine(char * pSymbol) {
-  char *pSymName;
-  char *pSymValue = strchr(pSymbol, '=');
-  // The name and value are stored as single string with
-  // a null character delimiting between the name and value.
-  if(pSymValue == NULL) {
-    // if no equals sign, define as a null string value
-    pSymName = (char*)malloc(strlen(pSymbol) + 2);   // space for two null chars
-    strcpy(pSymName, pSymbol);
-    pSymValue = strchr(pSymName, 0);
-    pSymValue++;
-    *pSymValue = 0;
-  }
-  else {
-    pSymName = strdup(pSymbol);
-    pSymValue = strchr(pSymName, '=');
-    *pSymValue = 0;
-    pSymValue++;
-  }
-  // Stored each name value pair from the command line it
-  // its own list container.
-  s_CmdLineSymbolList.push_back(pSymName);
-}
-
-void Symbol_Table::PopulateWithCommandLineSymbols() {
-  Value *pValue;
-  SymbolList::iterator it;
-  char *pSymName;
-  char *pSymValue;
-  for(it = s_CmdLineSymbolList.begin(); it != s_CmdLineSymbolList.end(); it++) {
-    pSymName = *it;
-    pSymValue = strchr(pSymName, 0) + 1;
-    if(*pSymValue == '\'' || *pSymValue == '\"') {
-      char * pUnquoted = UnquoteString(strdup(pSymValue));
-      pValue = new String(pSymName, pUnquoted);
-      free(pUnquoted);
-    }
-    else {
-      pValue = Integer::New(pSymName, pSymValue, "derived from gpsim command line");
-      if(pValue == NULL) {
-        pValue = Float::New(pSymName, pSymValue, "derived from gpsim command line");
-        if(pValue == NULL) {
-          pValue = Boolean::New(pSymName, pSymValue, "derived from gpsim command line");
-          if(pValue == NULL) {
-            // assume string
-            pValue = new String(pSymName, pSymValue);
-          }
-        }
-      }
-    }
-    pValue->setClearableSymbol(false);
-    add(pValue);
-  }
-}
-
-register_symbol *
-Symbol_Table::add_register(Register *new_reg, const char *symbol_name)
+//------------------------------------------------------------------------
+// add_register
+register_symbol *Symbol_Table::add_register(Register *new_reg, const char *symbol_name)
 {
   // mask of zero lets the register_symbol calculate a default mask
   return add_register(new_reg, symbol_name, 0);
 }
 
-register_symbol *
-Symbol_Table::add_register(Register *new_reg, const char *symbol_name,
-                           unsigned int uMask)
+register_symbol * Symbol_Table::add_register(Register *new_reg, 
+					     const char *symbol_name,
+					     unsigned int uMask)
 {
-
   if(!new_reg)
     return 0;
 
@@ -264,7 +234,6 @@ void Symbol_Table::add_w(WREG *new_w)
 
 void Symbol_Table::add_constant(const char *_name, int value, bool bClearable)
 {
-
   Integer *i = new Integer(value);
   i->new_name(_name);
   i->setClearableSymbol(bClearable);
@@ -297,7 +266,8 @@ void Symbol_Table::add_module(Module * m, const char *cPname)
   }
 }
 
-void Symbol_Table::remove_module(Module * m) {
+void Symbol_Table::remove_module(Module * m)
+{
   iterator sti = FindIt(m->name());
   Value *sym;
 
@@ -331,7 +301,6 @@ void Symbol_Table::rename(const char *pOldName, const char *pNewName)
     if(it != end() && (*it)->name() == pOldName) {
       Value *pValue = *it;
       erase(it);
-      // pValue->gpsimObject::new_name(pNewName);
       pValue->new_name(pNewName);
       add(pValue);
     }
@@ -405,10 +374,14 @@ register_symbol * Symbol_Table::findRegisterSymbol(unsigned int uAddress)
     register_symbol *pRegSymbol = dynamic_cast<register_symbol*>(*sti);
     if(pRegSymbol != 0) {
       Register * pReg = pRegSymbol->getReg();
-      if (pReg && pReg->get_cpu() == NULL)
-        cout << " Null cpu for reg named:"<<pReg->name()<<endl;
-      assert(pReg->get_cpu() != NULL);
-      if(pRegSymbol->getAddress() == uAddress &&
+      if (0 && pReg && pReg->get_cpu() == NULL) {
+	// hmmm  we need to fix this... It's possible for 
+	// modules (which are not processors) to have registers.
+        //cout << " Null cpu for reg named:"<<pReg->name()<<endl;
+
+      }
+      //assert(pReg->get_cpu() != NULL);
+      if(pReg && pReg->get_cpu() && pRegSymbol->getAddress() == uAddress &&
         pRegSymbol->getBitmask() == pReg->get_cpu()->register_mask() &&
         // This function will find the first symbol that
         // uses the entire bit field of the register.
@@ -534,6 +507,11 @@ Stimulus_Node * Symbol_Table::findNode(const char *s)
   return ((Stimulus_Node *)0);
 }
 
+String * Symbol_Table::findString(const char *s)
+{
+  return findSymbol(s, (String *)NULL);
+}
+
 template<class _symbol_iterator_t, class _symbol_t>
 _symbol_iterator_t Symbol_Table::beginSymbol(_symbol_iterator_t *pit, _symbol_t*psym) {
   iterator it;
@@ -551,12 +529,14 @@ _symbol_iterator_t Symbol_Table::endSymbol(_symbol_iterator_t *pit, _symbol_t*ps
   return _symbol_iterator_t(this,_Myt::end());
 }
 
-Symbol_Table::node_symbol_iterator Symbol_Table::beginNodeSymbol() {
+Symbol_Table::node_symbol_iterator Symbol_Table::beginNodeSymbol()
+{
   return (node_symbol_iterator)beginSymbol((node_symbol_iterator*)0,
-    (node_symbol*)0);
+					   (node_symbol*)0);
 }
 
-Symbol_Table::node_symbol_iterator Symbol_Table::endNodeSymbol() {
+Symbol_Table::node_symbol_iterator Symbol_Table::endNodeSymbol()
+{
     return endSymbol((node_symbol_iterator*) 0, (node_symbol*)0);
 }
 
@@ -597,6 +577,41 @@ Symbol_Table::stimulus_symbol_iterator Symbol_Table::endStimulusSymbol() {
     (stimulus_symbol*)NULL);
 }
 
+
+Symbol_Table::module_symbol_iterator Symbol_Table::beginModuleSymbol() {
+  iterator it;
+  iterator itEnd = _Myt::end();
+  for(it = _Myt::begin(); itEnd != it; it++) {
+    if(dynamic_cast<module_symbol*>(*it) != NULL &&
+      dynamic_cast<attribute_symbol*>(*it) == NULL) {
+      return module_symbol_iterator(this, it);
+    }
+  }
+  return module_symbol_iterator(this, itEnd);
+//  return (module_symbol_iterator)beginSymbol(
+//    (module_symbol_iterator*)NULL,
+//    (module_symbol*)NULL);
+}
+
+Symbol_Table::module_symbol_iterator Symbol_Table::endModuleSymbol() {
+  return endSymbol((module_symbol_iterator*) NULL,
+    (module_symbol*)NULL);
+}
+
+Symbol_Table::module_symbol_iterator
+Symbol_Table::module_symbol_iterator::operator++(int) {
+  // postincrement
+  Symbol_Table::iterator & it = (Symbol_Table::iterator&)*this;
+  for(it++; it != m_pSymbolTable->end(); it++) {
+    if(dynamic_cast<module_symbol*>(*it) != NULL &&
+      dynamic_cast<attribute_symbol*>(*it) == NULL) {
+      return (*this);
+    }
+  }
+  return (*this);
+}
+
+
 bool Symbol_Table::Exist(const char *s) {
   return this->FindIt(s) != end();
 }
@@ -618,19 +633,29 @@ void Symbol_Table::dump_one(const char *str)
 void Symbol_Table::dump_all(void)
 {
   cout << "  Symbol Table\n";
+  bool bUserCanceled = false;
+  CSimulationContext::GetContext()->SetUserCanceledFlag(&bUserCanceled);
   iterator sti = begin();
   iterator last;
   while( sti != end()) {
     Value *val = *sti;
-    if(val &&(typeid(*val) != typeid(line_number_symbol)))
-      cout << val->name() << " = " << val->toString() << endl;
-      //cout << val->name() << ": " << val->showType() << endl;
+    if(val && (typeid(*val) != typeid(line_number_symbol))) {
+      if(dynamic_cast<IIndexedCollection*>(val) == NULL) {
+        cout << val->name() << " = " ;
+      }
+      cout << val->toString() << endl;
+    }
     last = sti;
     sti++;
     if(sti != end() && (*last)->name() == (*sti)->name()) {
       cout << "***************** Duplicate Found ***********" << endl;
     }
+    if(bUserCanceled) {
+      cout << endl << "Symbol dump canceled." << endl;
+      break;
+    }
   }
+  CSimulationContext::GetContext()->SetUserCanceledFlag(NULL);
 }
 
 bool beginsWith(string &sTarget, string &sBeginsWith) {
@@ -647,6 +672,8 @@ void Symbol_Table::dump_filtered(const string & sSymbol)
     dump_all();
     return;
   }
+  bool bUserCanceled = false;
+  CSimulationContext::GetContext()->SetUserCanceledFlag(&bUserCanceled);
   if(sSymbol[nLastCharPos] == '.') {
     sBeginsWith = sSymbol.substr(0, nLastCharPos);
   }
@@ -661,16 +688,29 @@ void Symbol_Table::dump_filtered(const string & sSymbol)
     Value *val = *sti;
     if(val && (typeid(*val) != typeid(line_number_symbol)) &&
       beginsWith(val->name(), sBeginsWith)) {
-      cout << val->name() << " = " << val->toString() << endl;
-      //cout << val->name() << ": " << val->showType() << endl;
+      if(dynamic_cast<IIndexedCollection*>(val) == NULL) {
+        cout << val->name() << " = " ;
+      }
+      cout << val->toString() << endl;
       }
     last = sti;
     sti++;
+    if(bUserCanceled) {
+      cout << endl << "Symbol dump canceled." << endl;
+      break;
+    }
   }
+  CSimulationContext::GetContext()->SetUserCanceledFlag(NULL);
 }
 
 void Symbol_Table::dump_type(type_info const &symt)
 {
+  cout << DisplayType(symt);
+}
+
+string Symbol_Table::DisplayType(type_info const &symt)
+{
+  ostringstream stream;
   // Now loop through the whole table and display all instances of the type of interest
   int first=1;     // On the first encounter of one, display the title
   iterator sti = begin();
@@ -679,15 +719,16 @@ void Symbol_Table::dump_type(type_info const &symt)
     if(sym && (typeid(*sym) == symt)) {
       if(first) {
         first = 0;
-        cout << "Symbol Table for \"" << sym->showType() << "\"\n";
+        stream << "Symbol Table for \"" << sym->showType() << "\"" << endl;
       }
 
-      cout << sym->toString() << endl;
+      stream << sym->toString() << endl;
     }
     sti++;
   }
   if(first)
-    cout << "No symbols found\n";
+    stream << "No symbols found" << endl << ends;
+  return string(stream.str());
 }
 
 bool IsClearable(Value* value)
@@ -695,38 +736,44 @@ bool IsClearable(Value* value)
   return value->isClearable();
 }
 
-void Symbol_Table::clear() {
+void Symbol_Table::clear()
+{
   iterator it;
-  iterator itEnd = end();
-  int i = 0;
-  for(it = begin(); it != itEnd;) {
+
+  for(it = begin(); it != end();) {
     Value *value = *it;
-    if(value->isClearable()) {
+
+    if(value && value->isClearable()) {
+
       delete value;
       erase(it);
     }
     else {
-      it++;
+      ++it;
     }
-    i++;
   }
 //  remove_if(begin(), end(), IsClearable);
 }
 
-void Symbol_Table::Initialize() {
+void Symbol_Table::Initialize() 
+{
+#if 0
   PopulateWithCommandLineSymbols();
+#endif
 }
 
-void Symbol_Table::Reinitialize() {
+void Symbol_Table::Reinitialize()
+{
   clear();
 }
 
-void Symbol_Table::clear_all() {
+void Symbol_Table::clear_all() 
+{
   iterator it;
-  iterator itEnd = end();
-  for(it = begin(); it != itEnd; it++) {
+
+  for(it = begin(); it != end(); ++it)
     delete *it;
-  }
+
   _Myt::clear();
 }
 
@@ -773,7 +820,7 @@ Symbol_Table::FindIt(Value *key) {
   iterator it = lower_bound(begin( ), end( ),
     key, NameLessThan());
   if (it != end() &&
-    (*it)->name() == key->name()) {
+      (*it)->name() == key->name()) {
     return it;
   }
   return end();
@@ -798,6 +845,16 @@ symbol::~symbol(void)
 {
 }
 
+Value* symbol::evaluate()
+{
+  string msg("symbol '");
+  msg.append(this->name());
+  msg.append("' of type '");
+  msg.append(showType());
+  msg.append("' cannot not be evaluated in an expression");
+  throw Error( msg );
+}
+
 //------------------------------------------------------------------------
 string symbol::toString()
 {
@@ -885,15 +942,17 @@ void register_symbol::setMask(Register *pReg) {
   m_uMaskShift = BitShiftCount(m_uMask);
 }
 
-unsigned int register_symbol::getAddress(void) {
-  return reg->address;
+unsigned int register_symbol::getAddress()
+{
+  return reg ? reg->address : 0xffffffff;
 }
 
 unsigned int register_symbol::getBitmask(void) {
   return m_uMask;
 }
 
-string &register_symbol::name(void) const {
+string &register_symbol::name() const 
+{
   return (string &)name_str;
 }
 
@@ -914,15 +973,15 @@ string register_symbol::toString()
     unsigned int uValue = reg->get_value() & m_uMask;
     uValue  = uValue >> m_uMaskShift;
     if ( (unsigned int)((1<<(4*iDigits))-1) != m_uMask)
-      snprintf(buff,sizeof(buff)," [0x%x] BITS 0x%0*x = 0x%0*x = 0b",
+      snprintf(buff,sizeof(buff),"[0x%x] BITS 0x%0*x = 0x%0*x = 0b",
 	       reg->address, iDigits, m_uMask,
 	       iDigits, uValue);
     else
-      snprintf(buff,sizeof(buff)," [0x%x] = 0x%0*x = 0b",
+      snprintf(buff,sizeof(buff),"[0x%x] = 0x%0*x = 0b",
 	       reg->address, 
 	       iDigits, uValue);
 
-    return name() + string(buff) + string(bits);
+    return string(buff) + string(bits);
   }
   return string("");
 }
@@ -1071,6 +1130,11 @@ bool register_symbol::compare(ComparisonOperator *compOp, Value *rvalue)
   return compOp->equal();
 }
 
+void register_symbol::update(void) {
+  reg->update();
+}
+
+
 symbol *register_symbol::copy()
 {
   return new register_symbol(*this);
@@ -1080,11 +1144,43 @@ Register *register_symbol::getReg()
   return reg;
 }
 
+Value* register_symbol::evaluate() {
+  gint64 v;
+  get(v);
+  return new Integer(v);
+}
+
+int register_symbol::set_break(ObjectBreakTypes bt, ObjectActionTypes at,
+			       Expression *pExpr)
+{
+  return get_bp().set_break(bt,at, reg,pExpr);
+}
+
+int register_symbol::clear_break()
+{
+  cout << showType() << " objects breakpoints can only be cleared by 'clear #'\n   where # is the breakpoint number\n";
+  return -1;
+}
+
 //------------------------------------------------------------------------
 w_symbol::w_symbol(const char *_name, Register *_reg)
   : register_symbol(_name, _reg)
 {
 }
+string w_symbol::toString()
+{
+  if(reg) {
+    char buff[256];
+    char bits[256];
+
+    reg->toBitStr(bits,sizeof(bits));
+
+    snprintf(buff,sizeof(buff)," = 0x%02x = 0b", reg->get_value() & 0xff);
+
+    return string(buff) + string(bits);
+  }
+  return string("");
+}
 
 //------------------------------------------------------------------------
 ioport_symbol::ioport_symbol(PortRegister *_ioport)
@@ -1106,7 +1202,20 @@ string address_symbol::toString()
   int i = (int)getVal();
   snprintf(buf,sizeof(buf), " at address %d = 0x%X",i,i);
   
-  return name() + string(buf);
+  return string(buf);
+}
+
+Value* address_symbol::evaluate()
+{
+  return copy();
+}
+int address_symbol::set_break(ObjectBreakTypes bt, ObjectActionTypes at,
+			      Expression *pExpr)
+{
+  if (bt == gpsimObject::eBreakExecute)
+    return get_bp().set_execution_break(get_active_cpu(),getVal(),pExpr);
+
+  return -1;
 }
 
 line_number_symbol::line_number_symbol(const char *_name, unsigned int _val)
@@ -1133,16 +1242,27 @@ Value *module_symbol::copy()
 }
 void module_symbol::set(const char *cP,int len)
 {
-  if(module)
-    module->set(cP,len);
+  throw new Error("object cannot be assigned a value\n");
 }
 
 void module_symbol::get(char *cP, int len)
 {
-  if(module)
-    module->get(cP,len);
+  if(cP) {
+    *cP = 0;
+  }
 }
 
+string module_symbol::description()
+{
+  return module ? module->description() : string("no description");
+}
+
+string module_symbol::toString()
+{
+  return module ? module->toString() : name();
+}
+
+
 //------------------------------------------------------------------------
 
 attribute_symbol::attribute_symbol(Module *_module, Value *_attribute)
@@ -1163,7 +1283,7 @@ string attribute_symbol::toString()
 {
 
   if(attribute)
-    return attribute->showType()+": " + attribute->name() + " = " + attribute->toString();
+    return attribute->toString();
   else
     return string("(null)");
 }
@@ -1203,6 +1323,11 @@ Value *attribute_symbol::copy()
   return copy();
 }
 
+Value* attribute_symbol::evaluate()
+{
+  return copy();
+}
+
 void attribute_symbol::set(double d)
 {
   if(attribute)
@@ -1273,11 +1398,6 @@ void attribute_symbol::get(Value **v)
 }
 
 
-string module_symbol::toString()
-{
-  return name();
-}
-
 void attribute_symbol::set_xref(Value *v)
 {
   if(attribute) {
@@ -1336,7 +1456,7 @@ void stimulus_symbol::new_name(string &sNewName) {
 string stimulus_symbol::toString()
 {
   if(s)
-    return name() + ": " + s->toString();
+    return s->toString();
   return name();
 }
 
