@@ -26,6 +26,7 @@ Boston, MA 02111-1307, USA.  */
 #include <vector>
 #include "value.h"
 #include "registers.h"
+#include "exports.h"
 
 using namespace std;
 
@@ -69,7 +70,7 @@ enum SYMBOL_TYPE
 };
 
 
-class Symbol_Table : private vector<Value*> {
+class Symbol_Table : protected vector<Value*> {
 private:
   struct NameLessThan : binary_function<Value*, Value*, bool> {
     bool operator()(const Value* left, const Value* right) const {
@@ -109,6 +110,7 @@ public:
   void dump_one(string *s);
   void dump_type(type_info const&t);
   void dump_filtered(const string & sSymbol);
+  string DisplayType(type_info const &symt);
 
   Value *find(const char *s);
   Value *find(const string &s);
@@ -120,19 +122,26 @@ public:
   register_symbol * findRegisterSymbol(unsigned int uAddress,
                                        unsigned int uBitmask);
   const char *  findProgramAddressLabel(unsigned int address);
-  // This was intended to be used by disassembly code. I added
+
+  // This was intended to be used by disassembly code. I (JR) added
   // this then I figured out that it is not useful unless
   // you have a cross reference to every address that references
   // the symbol. So I'm leaving the code for the future when
   // the constant object includes a list of address references.
   const char *  findConstant(unsigned int uValue,
     unsigned int uReferencedFromAddress);
+  // Convenience functions for accessing symbols of a specific type.
+  // In general, provide a string containing the symbol name to access
+  // the symbol.
   Integer *     findInteger(const char *s);
   Boolean *     findBoolean(const char *s);
   module_symbol *findModuleSymbol(const char *s);
   Module *      findModule(const char *s);
+  String *      findString(const char *s);
 
-private:
+protected:
+  // 'findSymbol' is a template that will create type-specific code
+  // for searching the symbol table.
   template<class _symbol_t>
   _symbol_t * findSymbol(const char *s, _symbol_t*)
   {
@@ -190,9 +199,10 @@ public:
     return _Myt::end();
   }
 
-private:
+protected:
   template<class _symbol_t>
   class symbol_iterator_t : iterator {
+  friend class module_symbol_iterator;
   public:
     symbol_iterator_t() {
       m_pSymbolTable = NULL;
@@ -233,14 +243,14 @@ private:
 			++*this;
 			return (_Tmp);
 			}
-  private:
+  protected:
     Symbol_Table *m_pSymbolTable;
   };
 
   ///
   /// Note:: Apparently every argument in the template also needs
   ///        to be in the function argument list. The function
-  ///        arguments are only used to as place holders to allow
+  ///        arguments are only used as place holders to allow
   //         using this function template for a variety of functions
   ///        that only need to vary by return type.
   template<class _symbol_iterator_t, class _symbol_t>
@@ -266,15 +276,36 @@ public:
   stimulus_symbol_iterator beginStimulusSymbol();
   stimulus_symbol_iterator endStimulusSymbol();
 
+  ///
+  /// module_symbol iterator declarations
+  ///
+  class module_symbol_iterator : public symbol_iterator_t<module_symbol> {
+  public:
+    module_symbol_iterator() {
+    }
+
+    module_symbol_iterator(const module_symbol_iterator & it) :
+      symbol_iterator_t<module_symbol>(it) {
+    }
+
+    module_symbol_iterator(Symbol_Table *pSymbolTable, iterator it) :
+      symbol_iterator_t<module_symbol>(pSymbolTable, it) {
+    }
+
+    module_symbol_iterator operator++(int);
+  };
+  module_symbol_iterator beginModuleSymbol();
+  module_symbol_iterator endModuleSymbol();
 
   ///
   ///   Symbols defined from gpsim command line
   ///
+  /*
   typedef list<char*> SymbolList;
   SymbolList s_CmdLineSymbolList;
   void AddFromCommandLine(char * pSymbol);
   void PopulateWithCommandLineSymbols();
-
+  */
 };
 
 
@@ -284,7 +315,7 @@ public:
   extern Symbol_Table symbol_table;
   #endif
 // we are in Windows: don't access symbol_table object directly!
-extern "C" Symbol_Table &get_symbol_table(void);
+LIBGPSIM_EXPORT Symbol_Table & get_symbol_table(void);
 #else
 // we are in gpsim: use of get_symbol_table() is recommended,
 // even if trace object can be accessed directly.
@@ -307,6 +338,7 @@ class symbol : public Value
 {
 public:
 
+  virtual Value* evaluate();
   virtual string toString();
 
   symbol(const char *);
@@ -314,6 +346,48 @@ public:
   virtual ~symbol();
 };
 
+class AliasedSymbol : public symbol {
+public:
+  AliasedSymbol(const char * pName, Value *pSymbol) :
+    symbol(pName) {
+    m_pData = pSymbol;
+  }
+  AliasedSymbol(const char * pName, Value *pSymbol, const char *pDoc) :
+    symbol(pName) {
+    m_pData = pSymbol;
+    set_description(pDoc);
+  }
+  // The following functions are implemented to call the
+  // contained object. The name() and new_name() function
+  // will return and set the name of the alias.
+  virtual void set(const char *v,int len=0) {m_pData->set(v, len);}
+  virtual void set(double v) {m_pData->set(v);}
+  virtual void set(gint64 v) {m_pData->set(v);}
+  virtual void set(int v) {m_pData->set(v);}
+  virtual void set(bool v) {m_pData->set(v);}
+  virtual void set(Value *v) {m_pData->set(v);}
+  virtual void set(Expression *v) {m_pData->set(v);}
+  virtual void set(Packet &v) {m_pData->set(v);}
+
+  /// Value 'get' methods provide a mechanism of casting Value objects
+  /// to other value types. If the type cast is not supported in a
+  /// derived class, an Error will be thrown.
+
+  virtual void get(bool &v) {m_pData->get(v);}
+  virtual void get(int &v) {m_pData->get(v);}
+  virtual void get(guint64 &v) {m_pData->get(v);}
+  virtual void get(gint64 &v) {m_pData->get(v);}
+  virtual void get(double &v) {m_pData->get(v);}
+  virtual void get(char *v, int len) {m_pData->get(v,len);}
+  virtual void get(Packet &v) {m_pData->get(v);}
+  virtual Value *getValue() { return m_pData; }
+
+  virtual char *toString(char *v, int len) { return m_pData->toString(v,len);}
+  virtual string toString() { return m_pData->toString(); }
+  virtual Value* evaluate() { return m_pData->evaluate(); }
+protected:
+  Value *m_pData;
+};
 
 class node_symbol : public symbol
 {
@@ -332,7 +406,8 @@ class register_symbol : public symbol
 {
 protected:
   Register *reg;
-  /** m_uMask is used to mask out bits not associated
+  /** Scott to JR -- I don't understand this comment about m_uMask
+    * m_uMask is used to mask out bits not associated
     * with the symbol when retrieving the value from the
     * register. Masked bit values are right shifted such
     * that if the 16th bit is high in the register and
@@ -352,6 +427,9 @@ protected:
   unsigned int m_uMaskShift;
 
   unsigned int SetMaskedValue(unsigned int uValue);
+  /// breakpoints
+  virtual int set_break(ObjectBreakTypes bt=eBreakAny, ObjectActionTypes at=eActionHalt, Expression *expr=0);
+  virtual int clear_break();
 
 public:
   register_symbol(const register_symbol &);
@@ -374,6 +452,8 @@ public:
   virtual void set(const char *cP,int len=0);
   virtual void set(Packet &);
 
+  virtual void update(void);
+
   inline operator gint64() {
     gint64 i;
     get(i);
@@ -416,6 +496,7 @@ public:
   unsigned int getBitmask(void);
 
   virtual bool compare(ComparisonOperator *compOp, Value *rvalue);
+  Value*       evaluate();
 
   Register *getReg();
   /// copy the object value to a user char array
@@ -457,6 +538,9 @@ public:
 
   address_symbol(const char *, unsigned int);
   virtual string toString();
+  virtual Value* evaluate();
+  /// breakpoints
+  virtual int set_break(ObjectBreakTypes bt=eBreakAny, ObjectActionTypes at=eActionHalt, Expression *expr=0);
 };
 
 class line_number_symbol : public address_symbol
@@ -482,6 +566,7 @@ public:
   virtual string toString();
   Module *get_module() { return module;}
   virtual Value *copy();
+  virtual string description();
 
   /// The set and get methods for the module_symbol take char strings
   /// as their inputs. These strings are dynamically decoded such
@@ -532,6 +617,7 @@ public:
   virtual void set_xref(Value *);
   virtual Value *get_xref();
   virtual Value *copy();
+  virtual Value* evaluate();
 
 };
 
@@ -540,6 +626,7 @@ class w_symbol : public register_symbol
 {
  public:
   w_symbol(const char*, Register *);
+  virtual string toString();
 };
 
 
