@@ -45,15 +45,15 @@ Trace trace;               /* Instantiate the trace buffer class.
 // create an instance of inline get_trace() method by taking its address
 Trace &(*dummy_trace)(void) = get_trace;
 
-/*Trace trace_log_buffer;   * The trace_log_buffer is a special trace
-			    * buffer intended for logging events that will
-			    * ultimately be written to a file. Each logged
-			    * event is individually time tagged making it
-			    * easy to post process.
-			    */
 TraceLog trace_log;
 ProfileKeeper profile_keeper;
 
+#if defined(_WIN32)
+TraceLog &GetTraceLog(void) {
+  return trace_log;
+}
+#endif
+
 //========================================================================
 traceValue::traceValue(void)
 {
@@ -119,10 +119,60 @@ unsigned int traceValue::get_value(void)
    centuries!) The trace member function is_cycle_trace() describes
    the cycle counter encoding in detail.
 
-   Trace frames.
+   Trace Types.
 
-   A trace frame is defined to be the contents of the trace buffer
-   corresponding to a single time quantum. 
+   gpsim differentiates individually traced items by the TT field
+   (upper 8 bits of the 32-bit trace word). Except for the cycle
+   counter trace, these trace types are dynamically allocated whenever
+   a TraceType class is instantiated. As described above, the other
+   gpsim classes use this dynamically allocated 32-bit trace word as a
+   handle for efficiently storing information into the trace
+   buffer. In addition to allocating the 32-bit word for tracing, the
+   TraceType class will use it as a hash index into the
+   'trace_map'. The trace_map is a locally (to trace.cc) instantiated
+   STL map that cross references the 32-bit trace types to the
+   instantiated TraceType class that create them.
+
+   When the trace buffer is decoded, the trace type (upper 8-bits) of
+   the 32-bit trace word is extracted and used to look up the
+   TraceType object in the trace_map map. The lower 24-bits of the
+   trace word are then passed to the TraceType object's decode()
+   method. Continuing with the example from above, 
+
+       TTAAAAVV
+
+      TT - Used to look up the TraceType object in the trace_map
+      AAAAVV - Passed to the decode() method of the object.
+
+   The TraceType decode() method will usually create a TraceObject and
+   place it on a TraceFrame.
+
+
+   Trace Frames and Trace Objects
+
+   A trace frame is defined to be the decoded contents of the trace
+   buffer corresponding to a single time quantum (i.e. single
+   simulation cycle). Each frame has an STL list to hold the
+   TraceObjects. The TraceObjects are created by the TraceType
+   classes. This happens when the 32-bit trace word is decoded by the
+   TraceType class. The TraceObject has several purposes. First,
+   unique TraceObjects are created for the variety of things gpsim
+   traces. For example, when the simulated processor writes to a
+   register, a corresponding RegisterWriteTraceObject will get created
+   when the trace buffer is decoded. Another purpose of the
+   TraceObject is to record the system state. Take for example the
+   register write trace. When a register write occurs, the register
+   has a value before the write and a value after the write. When the
+   register write is traced, only the value *before* the write is
+   recorded. When the trace buffer is decoded, the simulation is
+   effectively run backwards. The current state is known before trace
+   decoding commences. Then as the decoding steps backwards through
+   the trace history, the state change at each trace event is recorded
+   in the trace object. So the register write trace event gets decoded
+   into a TraceObject. This trace object knows the current state of
+   the register; that's simply the register's current contents. The
+   trace object knows the contents of the register prior to the
+   register write operation; that's stored in the trace buffer. 
 
 
 ****************************************************************************/
@@ -205,11 +255,14 @@ void TraceRawLog::disable(void)
 }
 
 
-//========================================================================
-// Experimental trace code
-
 //========================================================================
 // TraceFrame
+//
+// A TraceFrame is a collection of traced items that belong to a single
+// simulation cycle. The TraceFrame is only built up whenever the user
+// requests trace history. Each frame contains a list of traceObjects
+// that describe the specific information that the simulation has traced.
+
 TraceFrame::TraceFrame( ) 
 {
   cycle_time = 0;
@@ -235,7 +288,6 @@ void TraceFrame::print(FILE *fp)
 {
   list <TraceObject *> :: iterator toIter;
 
-  //  fprintf(fp,"=== Trace Frame 0x%016LX\n",cycle_time);
   for(toIter = traceObjects.begin();
       toIter != traceObjects.end();
       ++toIter) 
@@ -252,6 +304,12 @@ void TraceFrame::update_state()
     (*toIter)->getState(this);
 }
 
+//============================================================
+// Trace::addFrame
+//
+// The Trace class maintains a list of traceFrames. Here is where
+// a new one gets added. Note that traceFrames are created only
+// when a user requests to see the trace history.
 
 void Trace::addFrame(TraceFrame *newFrame)
 {
@@ -261,13 +319,15 @@ void Trace::addFrame(TraceFrame *newFrame)
 
 void Trace::addToCurrentFrame(TraceObject *to)
 {
-
+  
   if(current_frame)
     current_frame->add(to);
 
 }
 void Trace::deleteTraceFrame(void)
 {
+  if (!current_frame)
+    return;
   list <TraceFrame *> :: iterator tfIter;
 
   for(tfIter = traceFrames.begin();
@@ -298,7 +358,8 @@ void Trace::printTraceFrame(FILE *fp)
 //========================================================================
 // TraceObject
 //
-// Base class for decode traces.
+// A TraceObject is a base class for decoded traces. TraceObjects are only
+// created when the user requests to see the TraceHistory.
 //
 TraceObject::TraceObject()
 {
@@ -316,6 +377,31 @@ void TraceObject::getState(TraceFrame *tf)
   // information to the TraceFrame.
 }
 
+//========================================================================
+// InvalidTraceObject
+//
+InvalidTraceObject::InvalidTraceObject(int type)
+  : mType(type)
+{
+}
+void InvalidTraceObject::print(FILE *fp)
+{
+  fprintf(fp, "  Invalid Trace entry: 0x%x\n",mType);
+}
+
+//========================================================================
+// ModuleTraceObject
+
+void ModuleTraceObject::print(FILE *fp)
+{
+  fprintf(fp, " Module Trace: ");
+  if (pModule)
+    fprintf(fp, "%s ", pModule->name().c_str());
+  if (pModuleTraceType && pModuleTraceType->pDescription)
+    fprintf(fp, "%s ", pModuleTraceType->pDescription);
+  fprintf (fp, "0x%x\n", mTracedData & 0xffffff);
+}
+
 //========================================================================
 // RegisterTraceObject
 //
@@ -393,7 +479,8 @@ void PCTraceObject::print_frame(TraceFrame *tf,FILE *fp)
 
   list <TraceObject *> :: reverse_iterator toIter;
 
-  fprintf(fp,"0x%016LX %s ",tf->cycle_time,cpu->name().c_str());
+  fprintf(fp,"0x%016" PRINTF_INT64_MODIFIER "X %s ",
+    tf->cycle_time,cpu->name().c_str());
   print(fp);
 
   for(toIter = tf->traceObjects.rbegin();
@@ -408,10 +495,9 @@ void PCTraceObject::print_frame(TraceFrame *tf,FILE *fp)
 
 //========================================================================
 
-TraceType::TraceType(unsigned int t, unsigned int s)
-  : type(t), size(s)
+TraceType::TraceType(unsigned int nTraceEntries)
+  : mType(0), mSize(nTraceEntries)
 {
-
 }
 
 //----------------------------------------
@@ -421,16 +507,20 @@ TraceType::TraceType(unsigned int t, unsigned int s)
 // If the trace record starting at the trace buffer index 'tbi' is of the
 // same type as this trace object, then return true.
 //
-bool TraceType::isValid(unsigned int tbi)
+bool TraceType::isValid(Trace *pTrace, unsigned int tbi)
 {
+  if (!pTrace)
+    return false;
+
   unsigned int i;
 
   // The upper 8-bits of the 'type' specify the trace type for this object.
   // This is assigned whenever Trace::allocateTraceType() is called. Multi-
   // sized trace records occupy consecutive types.
-  for(i=0; i<size; i++) {
+  for(i=0; i<size(); i++) {
 
-    if(trace.type(tbi + i) != (type + (i<<24))) 
+    //if(pTrace->type(tbi + i) != (type() + (i<<24))) 
+    if(!isValid(pTrace->get(tbi+i)))
       return false;
   }
 
@@ -438,34 +528,109 @@ bool TraceType::isValid(unsigned int tbi)
 
 }
 
-int TraceType::dump_raw(unsigned int tbi, char *buf, int bufsize)
+int TraceType::dump_raw(Trace *pTrace,unsigned int tbi, char *buf, int bufsize)
 {
+  if (!pTrace || !buf)
+    return 0;
+
   int total_chars=0;
-  if(isValid(tbi)) {
-    unsigned int i=0;
+  int iUsed = entriesUsed(pTrace,tbi);
 
-    for(i=0; i< size; i++) {
+  for(unsigned int i=0; i<iUsed; i++) {
 
-      int n = snprintf(buf,bufsize,"%08X:", trace.get(tbi+i));
-      if(n < 0)
-	break;
+    int n = snprintf(buf,bufsize,"%08X:", pTrace->get(tbi+i));
+    if(n < 0)
+      break;
 
-      total_chars += n;
-      buf += n;
-      bufsize -= n;
-    }      
-  }
+    total_chars += n;
+    buf += n;
+    bufsize -= n;
+  }      
 
   return total_chars;
-    
+}
+
+//============================================================
+//
+// entriesUsed
+//
+// given a trace buffer and an index into it, return the number
+// of trace buffer entries at that point that match the type of
+// this trace.
+
+int TraceType::entriesUsed(Trace *pTrace,unsigned int tbi)
+{
+  int iUsed=0;
+  if (pTrace)
+    while (pTrace->type(tbi+iUsed) == (mType + (iUsed<<24)))
+      iUsed++;
+  return iUsed;
+}
+//========================================================================
+TraceObject *ModuleTraceType::decode(unsigned int tbi)
+{
+  ModuleTraceObject *mto = new ModuleTraceObject(pModule, this, trace.get(tbi)&0xffffff);
+
+  return mto;
+}
+
+int ModuleTraceType::dump_raw(Trace *pTrace,unsigned int tbi, char *buf, int bufsize)
+{
+  if (!pTrace)
+    return 0;
+
+  int n = TraceType::dump_raw(pTrace, tbi,buf,bufsize);
+
+  buf += n;
+  bufsize -= n;
+
+  unsigned int tv = pTrace->get(tbi);
+
+  int m = snprintf(buf, bufsize,
+		   " Module: %s 0x%x",
+		   (pModule ? pModule->name().c_str() : "no name"),
+		   (tv & 0xffffff));
+
+  return m > 0 ? (m+n) : n;
 }
 
+//========================================================================
+CycleTraceType::CycleTraceType(unsigned int s)
+  : TraceType(s)
+{
+}
+TraceObject *CycleTraceType::decode(unsigned int tbi)
+{
+  return 0;
+}
+bool CycleTraceType::isFrameBoundary()
+{
+  return false;
+}
+int CycleTraceType::dump_raw(Trace *pTrace,unsigned tbi, char *buf, int bufsize)
+{
+  int n = TraceType::dump_raw(pTrace, tbi,buf,bufsize);
+  buf += n;
+  bufsize -= n;
+
+  int m=0;
+  if (pTrace) {
+    guint64 cycle;
+    if (pTrace->is_cycle_trace(tbi,&cycle) == 2)
+      m = snprintf(buf,bufsize,"  Cycle 0x%016" PRINTF_INT64_MODIFIER "X",cycle);
+  }
+
+  return m > 0 ? (m+n) : n;
+}
+int CycleTraceType::entriesUsed(Trace *pTrace,unsigned int tbi)
+{
+  return pTrace ? pTrace->is_cycle_trace(tbi,0) : 0;
+}
 //========================================================================
 
 RegisterWriteTraceType::RegisterWriteTraceType(Processor *_cpu, 
-				     unsigned int t,
-				     unsigned int s)
-  : ProcessorTraceType(_cpu,t,s)
+					       unsigned int s)
+  : ProcessorTraceType(_cpu,s)
 {
     
 }
@@ -478,21 +643,28 @@ TraceObject *RegisterWriteTraceType::decode(unsigned int tbi)
   unsigned int address = (tv >> 8) & 0xfff;
 
   RegisterWriteTraceObject *rto = new RegisterWriteTraceObject(cpu, cpu->rma.get_register(address), rv);
-  trace.addToCurrentFrame(rto);
 
   return rto;
 }
 
-int RegisterWriteTraceType::dump_raw(unsigned int tbi, char *buf, int bufsize)
+int RegisterWriteTraceType::dump_raw(Trace *pTrace,unsigned int tbi, char *buf, int bufsize)
 {
-  int n = TraceType::dump_raw(tbi,buf,bufsize);
+  if (!pTrace)
+    return 0;
+
+  int n = TraceType::dump_raw(pTrace, tbi,buf,bufsize);
 
   buf += n;
   bufsize -= n;
 
-  unsigned int tv = trace.get(tbi);
+  unsigned int tv = pTrace->get(tbi);
+  unsigned int address = (tv >> 8) & 0xfff;
   
-  int m = snprintf(buf, bufsize," Register 0x%02x was 0x%02x",(tv>>8)&0xfff, tv&0xff);
+  Register *reg = cpu->rma.get_register(address);
+  int m = snprintf(buf, bufsize,
+		   "  Reg Write: %s(0x%04X) was 0x%x ",
+		   (reg ? reg->name().c_str() : ""), address,
+		   tv & 0xff);
   if(m>0)
     n += m;
 
@@ -503,9 +675,8 @@ int RegisterWriteTraceType::dump_raw(unsigned int tbi, char *buf, int bufsize)
 //========================================================================
 
 RegisterReadTraceType::RegisterReadTraceType(Processor *_cpu, 
-				     unsigned int t,
-				     unsigned int s)
-  : ProcessorTraceType(_cpu,t,s)
+					     unsigned int s)
+  : ProcessorTraceType(_cpu,s)
 {
     
 }
@@ -518,21 +689,29 @@ TraceObject *RegisterReadTraceType::decode(unsigned int tbi)
   unsigned int address = (tv >> 8) & 0xfff;
 
   RegisterReadTraceObject *rto = new RegisterReadTraceObject(cpu, cpu->rma.get_register(address), rv);
-  trace.addToCurrentFrame(rto);
 
   return rto;
 }
 
-int RegisterReadTraceType::dump_raw(unsigned int tbi, char *buf, int bufsize)
+int RegisterReadTraceType::dump_raw(Trace *pTrace, unsigned int tbi, char *buf, int bufsize)
 {
-  int n = TraceType::dump_raw(tbi,buf,bufsize);
+  if (!pTrace)
+    return 0;
+
+  int n = TraceType::dump_raw(pTrace, tbi,buf,bufsize);
 
   buf += n;
   bufsize -= n;
 
-  unsigned int tv = trace.get(tbi);
-  
-  int m = snprintf(buf, bufsize," Read 0x%02x from Register 0x%02x", tv&0xff,(tv>>8)&0xfff);
+  unsigned int tv = pTrace->get(tbi);
+  unsigned int address = (tv >> 8) & 0xfff;
+
+  Register *reg = cpu->rma.get_register(address);
+  int m = snprintf(buf, bufsize,
+		   "  Reg Read:  %s(0x%04X) was 0x%0X",
+		   (reg ? reg->name().c_str() : ""), address,
+		   tv & 0xff);
+
   if(m>0)
     n += m;
 
@@ -542,9 +721,8 @@ int RegisterReadTraceType::dump_raw(unsigned int tbi, char *buf, int bufsize)
 
 //========================================================================
 PCTraceType::PCTraceType(Processor *_cpu, 
-			 unsigned int t,
 			 unsigned int s)
-  : ProcessorTraceType(_cpu,t,s)
+  : ProcessorTraceType(_cpu,s)
 {
 }
 
@@ -556,7 +734,6 @@ TraceObject *PCTraceType::decode(unsigned int tbi)
   trace.addFrame(new TraceFrame( ));
 
   PCTraceObject *pcto = new PCTraceObject(cpu, tv);
-  trace.addToCurrentFrame(pcto);
 
   if((tv & (3<<22)) == (1<<22))
     trace.current_cycle_time -= 2;
@@ -568,14 +745,18 @@ TraceObject *PCTraceType::decode(unsigned int tbi)
   return pcto;
 }
 
-int PCTraceType::dump_raw(unsigned int tbi, char *buf, int bufsize)
+int PCTraceType::dump_raw(Trace *pTrace, unsigned int tbi, char *buf, int bufsize)
 {
-  int n = TraceType::dump_raw(tbi,buf,bufsize);
+  if (!pTrace)
+    return 0;
+
+  int n = TraceType::dump_raw(pTrace,tbi,buf,bufsize);
 
   buf += n;
   bufsize -= n;
 
-  int m = snprintf(buf, bufsize," PC: %04X",trace.get(tbi) & 0xffff);
+  int m = snprintf(buf, bufsize,"FRAME ==============  PC: %04X",
+		   cpu->map_pm_index2address(pTrace->get(tbi) & 0xffff));
   if(m>0)
     n += m;
 
@@ -604,8 +785,14 @@ int PCTraceType::dump_raw(unsigned int tbi, char *buf, int bufsize)
 // type for tracing 8-bit register writes. 
 //
 map <unsigned int, TraceType *> trace_map;
+CycleTraceType *pCycleTrace=0;
 
-Trace::Trace(void)
+
+Trace::Trace()
+  : cpu(0),
+    current_frame(0),
+    lastTraceType(LAST_TRACE_TYPE),
+    lastSubTraceType(1<<16)
 {
 
   for(trace_index = 0; trace_index < TRACE_BUFFER_SIZE; trace_index++)
@@ -614,8 +801,7 @@ Trace::Trace(void)
   trace_index = 0;
   string_cycle = 0;
 
-  lastTraceType = LAST_TRACE_TYPE;
-  lastSubTraceType = 1<<16;
+  traceFrames.clear();
 
   xref = new XrefObject(&trace_value);
 
@@ -628,6 +814,16 @@ Trace::~Trace(void)
 
 }
 
+//--------------------------------------------------------------
+//
+unsigned int Trace::type(unsigned int index)
+{
+  unsigned int traceType = operator[](index) & TYPE_MASK;
+  unsigned int cycleType = traceType & (CYCLE_COUNTER_LO | CYCLE_COUNTER_HI);
+
+  return cycleType ? cycleType : traceType;
+
+}
 //--------------------------------------------------------------
 // is_cycle_trace(unsigned int index)
 //
@@ -757,16 +953,12 @@ int Trace::dump1(unsigned index, char *buffer, int bufsize)
   switch (type(index))
     {
     case NOTHING:
-      snprintf(buffer, bufsize,"  empty trace cycle\n");
+      snprintf(buffer, bufsize,"  empty trace cycle");
       break;
     case WRITE_TRIS:
       snprintf(buffer, bufsize,"  wrote: 0x%02x to TRIS",
 	       get(index)&0xff);
       break;
-    case WRITE_OPTION:
-      snprintf(buffer, bufsize,"  wrote: 0x%02x to OPTION",
-	       get(index)&0xff);
-      break;
     case BREAKPOINT:
       snprintf(buffer, bufsize,"BREAK: ");
       bp.dump_traced(get(index) & 0xffffff);
@@ -804,15 +996,15 @@ int Trace::dump1(unsigned index, char *buffer, int bufsize)
 	  TraceType *tt = (*tti).second;
 
 	  if(tt) {
-	    tt->dump_raw(index,buffer,bufsize);
-	    return_value = tt->size;
+	    tt->dump_raw(this,index,buffer,bufsize);
+	    return_value = tt->size();
 	  }
 	  break;
 	} 
 
 
 	if(cpu)
-	  return_value = cpu->trace_dump1(index,buffer,bufsize);
+	  return_value = cpu->trace_dump1(get(index),buffer,bufsize);
       }
     }
 
@@ -833,7 +1025,7 @@ void Trace::disableLogging()
 }
 
 //------------------------------------------------------------------
-// int Trace::dump(unsigned int n=0)
+// int Trace::dump(int n, FILE *out_stream)
 //
 
 int Trace::dump(int n, FILE *out_stream)
@@ -844,7 +1036,7 @@ int Trace::dump(int n, FILE *out_stream)
     return 0;
 
   if(n<0)
-    n = TRACE_BUFFER_SIZE;
+    n = TRACE_BUFFER_SIZE-1;
 
   if(!n)
     n = 5;
@@ -853,6 +1045,14 @@ int Trace::dump(int n, FILE *out_stream)
   if(!out_stream)
     return 0;
 
+  if (!pCycleTrace) {
+    // ugh
+    // the trace_map needs to be a member of Trace, other wise
+    // there's a global constructor initialization race condition.
+    pCycleTrace = new CycleTraceType(2);
+    trace_map[CYCLE_COUNTER_LO] = pCycleTrace;
+    trace_map[CYCLE_COUNTER_HI] = pCycleTrace;
+  }
 
   unsigned int frames = n;
 
@@ -871,7 +1071,6 @@ int Trace::dump(int n, FILE *out_stream)
   // Save the state of the CPU here. 
   cpu->save_state();
 
-
   //
   // Decode the trace buffer
   //
@@ -893,21 +1092,19 @@ int Trace::dump(int n, FILE *out_stream)
 
       TraceType *tt = (*tti).second;
 
-      if(tt)
-	tt->decode(k);
+      if(tt) {
+	TraceObject *pTO = tt->decode(k);
+	if (pTO)
+	  addToCurrentFrame(pTO);
+      }
 
-    } else if(is_cycle_trace(k,&cycle) == 2) {
+      if(is_cycle_trace(k,&cycle) == 2)
+	current_cycle_time = cycle;
+	
+    } else if (get(k) != NOTHING) {
 
-      current_cycle_time = cycle;
-    } else {
-
-      if(type(k) == 0)
-	break;
-
-      /*
-      if(is_cycle_trace(tbi(k-1),0) != 2)
-	fprintf(out_stream," could not decode 0x%x\n",get(k));
-      */
+      cout << " could not decode trace type: 0x" << hex << get(k) << endl;
+      addToCurrentFrame(new InvalidTraceObject(get(k)));
     }
 
     k = tbi(k-1);
@@ -924,7 +1121,7 @@ int Trace::dump(int n, FILE *out_stream)
 // allocateTraceType - allocate one or more trace commands
 //
 //
-unsigned int Trace::allocateTraceType(TraceType *tt, int nSlots)
+unsigned int Trace::allocateTraceType(TraceType *tt)
 {
   
   if(tt) {
@@ -943,14 +1140,14 @@ unsigned int Trace::allocateTraceType(TraceType *tt, int nSlots)
       n = 1<<16;
     }
 
-    tt->type = *ltt;;
+    tt->setType(*ltt);;
 
-    for(i=0; i<nSlots; i++) {
+    for(i=0; i<tt->size(); i++) {
       trace_map[*ltt] = tt;
       *ltt += n;
     }
 
-    return tt->type;
+    return tt->type();
   }
   return 0;
 }
@@ -964,7 +1161,7 @@ void Trace::dump_raw(int n)
 
   FILE *out_stream = stdout;
 
-  const int BUFFER_SIZE = 50;
+  const int BUFFER_SIZE = 256;
 
   char buffer[BUFFER_SIZE];
 
@@ -973,16 +1170,32 @@ void Trace::dump_raw(int n)
   trace_flag = TRACE_ALL;
 
   do {
-    printf("%04X: ",i);
-    if(is_cycle_trace(i,0))
-      fprintf(out_stream, "%08X:%08X",trace_buffer[i], trace_buffer[(i+1) & TRACE_BUFFER_MASK]);
-    else
-      printf("%08X         ",trace_buffer[i]);
+    fprintf(out_stream,"%04X:",i);
 
-    i = (i + dump1(i,buffer,BUFFER_SIZE)) & TRACE_BUFFER_MASK;
+    map<unsigned int, TraceType *>::iterator tti = trace_map.find(type(i));
+    unsigned int tSize = 1;
+    TraceType *tt = tti != trace_map.end() ? (*tti).second : 0;
 
+    buffer[0]=0;
+    tSize = 0;
+    if(tt) {
+      tSize = tt->entriesUsed(this,i);
+      /*
+	fprintf(out_stream, "%02X:",tSize);
+	for (unsigned int ii=0; ii<tSize; ii++)
+	  fprintf(out_stream, "%08X:",get(i+ii));
+      */
+
+      tt->dump_raw(this,i,buffer,sizeof(buffer));
+    }
+    
+    if(!tSize)
+      fprintf(out_stream, "%08X:  ??",get(i));
     if(buffer[0]) 
       fprintf(out_stream,"%s",buffer);
+
+    tSize = tSize ? tSize : 1;
+    i = (i + tSize) & TRACE_BUFFER_MASK;
     putc('\n',out_stream);
 
   } while((i!=trace_index) && (i!=((trace_index+1)&TRACE_BUFFER_MASK)));
@@ -1132,9 +1345,11 @@ void TraceLog::write_logfile(void)
 
   unsigned int i,j;
   char buf[256];
+  guint64 cycle=0;
 
   if(log_file) {
 
+
     buffer.trace_flag = TRACE_ALL;
 
     // Loop through the trace buffer and decode each entry.
@@ -1142,14 +1357,15 @@ void TraceLog::write_logfile(void)
 
     for(i=0,j=0; i<buffer.trace_index && j<buffer.trace_index; j++) {
       buf[0] = 0;
+
+      if(buffer.is_cycle_trace(i,&cycle))
+	fprintf(log_file,"Cycle 0x%016" PRINTF_INT64_MODIFIER "X\n",cycle);
+
       i = (i + buffer.dump1(i,buf, sizeof(buf))) & TRACE_BUFFER_MASK;
 
       if(buf[0]) {
 	items_logged++;
 	fprintf(log_file,"%s\n", buf);
-      } else {
-	cout << " write_logfile: ERROR, couldn't decode trace buffer\n";
-	return;
       }
     }
 
@@ -1200,11 +1416,11 @@ void TraceLog::status(void)
       switch(file_format)
       {
       case TRACE_FILE_FORMAT_LXT:
-	  cout << "in LXT mode" << endl;
+	  cout << " in LXT mode" << endl;
 	  break;
       case TRACE_FILE_FORMAT_ASCII:
       default:
-	  cout << "in ASCII mode" << endl;
+	  cout << " in ASCII mode" << endl;
 	  break;
       }
 
@@ -1271,61 +1487,79 @@ void TraceLog::lxt_trace(unsigned int address, unsigned int value, guint64 cc)
     lt_emit_value_int(lxtp, symp, 0, value);
 }
 
-void TraceLog::register_read(unsigned int address, unsigned int value, guint64 cc)
+void TraceLog::register_read(Register *pReg, guint64 cc)
 {
-    switch(file_format)
+  if (!pReg)
+    return;
+
+  switch(file_format)
     {
     case TRACE_FILE_FORMAT_ASCII:
-	buffer.cycle_counter(cc);
-	//buffer.register_read(address, value);
-	break;
+      buffer.cycle_counter(cc);
+      buffer.raw(pReg->read_trace.get() | pReg->get_value());
+      if(buffer.near_full())
+	write_logfile();
+      break;
     case TRACE_FILE_FORMAT_LXT:
-	lxt_trace(address, value, cc);
-	break;
+      lxt_trace(pReg->getAddress(), pReg->get_value(), cc);
+      break;
     }
 }
 
-void TraceLog::register_write(unsigned int address, unsigned int value, guint64 cc)
+void TraceLog::register_write(Register *pReg, guint64 cc)
 {
-    switch(file_format)
+  if (!pReg)
+    return;
+
+  switch(file_format)
     {
     case TRACE_FILE_FORMAT_ASCII:
-	buffer.cycle_counter(cc);
-	//buffer.register_write(address, value);
-	if(buffer.near_full())
-	    write_logfile();
-	break;
+      buffer.cycle_counter(cc);
+      buffer.raw(pReg->write_trace.get() | pReg->get_value());
+      if(buffer.near_full())
+	write_logfile();
+      break;
     case TRACE_FILE_FORMAT_LXT:
-	lxt_trace(address, value, cc);
-	break;
+      lxt_trace(pReg->getAddress(), pReg->get_value(), cc);
+      break;
     }
 }
 
-void TraceLog::register_read_value(unsigned int address, unsigned int value, guint64 cc)
+void TraceLog::register_read_value(Register *pReg, guint64 cc)
 {
-    switch(file_format)
+  if (!pReg)
+    return;
+
+  switch(file_format)
     {
     case TRACE_FILE_FORMAT_ASCII:
-	buffer.cycle_counter(cc);
-	//buffer.register_read_value(address, value);
-        break;
+      buffer.cycle_counter(cc);
+      buffer.raw(pReg->read_trace.get() | pReg->get_value());
+      if(buffer.near_full())
+	write_logfile();
+      break;
     case TRACE_FILE_FORMAT_LXT:
-	lxt_trace(address, value, cc);
-	break;
+      lxt_trace(pReg->getAddress(), pReg->get_value(), cc);
+      break;
     }
 }
 
-void TraceLog::register_write_value(unsigned int address, unsigned int value, guint64 cc)
+void TraceLog::register_write_value(Register *pReg, guint64 cc)
 {
-    switch(file_format)
+  if (!pReg)
+    return;
+
+  switch(file_format)
     {
     case TRACE_FILE_FORMAT_ASCII:
-	buffer.cycle_counter(cc);
-	//buffer.register_write_value(address, value);
-        break;
+      buffer.cycle_counter(cc);
+      buffer.raw(pReg->write_trace.get() | pReg->get_value());
+      if(buffer.near_full())
+	write_logfile();
+      break;
     case TRACE_FILE_FORMAT_LXT:
-	lxt_trace(address, value, cc);
-	break;
+      lxt_trace(pReg->getAddress(), pReg->get_value(), cc);
+      break;
     }
 }
 
