@@ -87,16 +87,14 @@ string &toupper(string & sStr) {
 
 //------------------------------------------------------------------------
 Value::Value()
-  : cpDescription(0), xref(0)
+  : xref(0), m_bClearableSymbol(true)
 {
-  m_bClearableSymbol = true;
 }
 
 Value::Value(const char *_name, const char *desc)
-  : cpDescription(desc), xref(0)
+  : gpsimObject(_name,desc), xref(0), m_bClearableSymbol(true)
 {
-  m_bClearableSymbol = true;
-  new_name(_name);
+  
 }
 
 Value::~Value()
@@ -132,11 +130,13 @@ void Value::set(Value *v)
   throw new Error(" cannot assign a Value to a " + showType());
 }
 
+// JRH - Note: If this function Value::set(Expression) is ever
+//       used anywhere besides yyparse() you may want to move
+//       the delete expr; statement into the calling code.
 void Value::set(Expression *expr)
 {
-  Value *v=0;
-
   try {
+    Value *v=0;
 
     if(!expr)
       throw new Error(" null expression ");
@@ -146,7 +146,7 @@ void Value::set(Expression *expr)
       throw new Error(" cannot evaluate expression ");
 
     set(v);
-
+    delete v;
   }
 
 
@@ -157,7 +157,6 @@ void Value::set(Expression *expr)
   }
 
 
-  delete v;
   delete expr;
 
 }
@@ -191,7 +190,7 @@ void Value::get(guint64 &i)
 void Value::get(bool &b)
 {
   throw new Error(showType() +
-		  " cannot be converted to an integer ");
+		  " cannot be converted to a boolean");
 }
 
 void Value::get(double &d)
@@ -227,19 +226,6 @@ Value *Value::copy()
 
 }
 
-string Value::description()
-{
-  if(cpDescription)
-    return string(cpDescription);
-  else
-    return string("no description");
-}
-
-void  Value::set_description(const char *new_description)
-{
-  cpDescription = new_description;
-}
-
 void Value::set_xref(Value *v)
 {
   delete xref;
@@ -259,14 +245,6 @@ Value *Value::get_xref()
   return xref;
 }
 
-void Value::set_break()
-{
-  cout << showType() << " objects do not support break points\n";
-}
-void Value::clear_break()
-{
-  cout << showType() << " objects do not support break points\n";
-}
 //------------------------------------------------------------------------
 // gpsimValue
 
@@ -384,6 +362,12 @@ bool AbstractRange::compare(ComparisonOperator *compOp, Value *rvalue)
   throw new Error(compOp->showOp() + 
 		  " comparison is not defined for " + showType());
 }
+
+Value *AbstractRange::copy()
+{
+  return new AbstractRange(get_leftVal(),get_rightVal());
+}
+
 void AbstractRange::set(Value *v)
 {
   AbstractRange *ar=typeCheck(v, string(""));
@@ -426,7 +410,7 @@ bool Boolean::Parse(const char *pValue, bool &bValue) {
   return false;
 }
 
-Boolean * Boolean::New(const char *_name, const char *pValue, const char *desc) {
+Boolean * Boolean::NewObject(const char *_name, const char *pValue, const char *desc) {
   bool bValue;
   if(Parse(pValue, bValue)) {
     return new Boolean(_name, bValue);
@@ -510,6 +494,13 @@ bool Boolean::compare(ComparisonOperator *compOp, Value *rvalue)
 }
 
 
+Value *Boolean::copy()
+{
+  bool b;
+  get(b);
+  return new Boolean(b);
+}
+
 // get(bool&) - primary method for accessing the value.
 void Boolean::get(bool &b)
 {
@@ -614,20 +605,42 @@ bool Boolean::operator!=(Value *rv)
 /*****************************************************************
  * The Integer class.
  */
+Integer::Integer(const Integer &new_value) {
+  Integer & nv = (Integer&)new_value;
+  nv.get(value);
+  bitmask = new_value.bitmask;
+}
+
 Integer::Integer(gint64 newValue)
 {
   value = newValue;
+  bitmask = def_bitmask;
 }
 
 Integer::Integer(const char *_name, gint64 newValue,const char *_desc)
   : Value(_name,_desc)
 {
-   value = newValue;
+  value = newValue;
+  bitmask = def_bitmask;
 }
 
+gint64 Integer::def_bitmask = 0xffffffff;
+
 Integer::~Integer()
 {
 }
+
+void Integer::setDefaultBitmask(gint64 bitmask) {
+  def_bitmask = bitmask;
+}
+
+Value *Integer::copy()
+{ 
+  gint64 i;
+  get(i);
+  return new Integer(i); 
+}
+
 void Integer::set(double d)
 {
   gint64 i = (gint64)d;
@@ -647,8 +660,11 @@ void Integer::set(int i)
 }
 void Integer::set(Value *v)
 {
-  Integer *iv = typeCheck(v,string("set "));
-  set(iv->getVal());
+  gint64 iv = 0;
+  if (v)
+    v->get(iv);
+
+  set(iv);
 }
 
 void Integer::set(Packet &p)
@@ -681,7 +697,7 @@ void Integer::set(const char *buffer, int buf_size)
 bool Integer::Parse(const char *pValue, gint64 &iValue) {
     if(::isdigit(*pValue)) {
       if(strchr(pValue, '.')) {
-        false;
+        return false;
       }
       else {
         // decimal or 0x integer
@@ -697,7 +713,7 @@ bool Integer::Parse(const char *pValue, gint64 &iValue) {
     return false;
 }
 
-Integer * Integer::New(const char *_name, const char *pValue, const char *desc) {
+Integer * Integer::NewObject(const char *_name, const char *pValue, const char *desc) {
   gint64 iValue;
   if(Parse(pValue, iValue)) {
     return new Integer(_name, iValue, desc);
@@ -710,11 +726,12 @@ void Integer::get(gint64 &i)
 { 
   i = value;
 }
+
 void Integer::get(double &d)
 { 
   gint64 i;
   get(i);
-  d = (double)i;;
+  d = (double)i;
 }
 
 void Integer::get(char *buffer, int buf_size)
@@ -737,12 +754,35 @@ void Integer::get(Packet &pb)
   pb.EncodeUInt32(j);
 }
 
+int Integer::set_break(ObjectBreakTypes bt, ObjectActionTypes at, Expression *expr)
+{
+  Processor *pCpu = get_active_cpu();
+  if (pCpu) {
+
+    // Legacy code compatibility!
+
+    if ( bt == eBreakWrite || bt == eBreakRead ) {
+
+      // Cast the integer into a register and set a register break point
+      unsigned int iRegAddress = (unsigned int) value;
+      Register *pReg = &pCpu->rma[iRegAddress];
+      return get_bp().set_break(bt, at, pReg, expr);
+    } else if ( bt == eBreakExecute) {
+
+      unsigned int iProgAddress = (unsigned int) value;
+      return get_bp().set_execution_break(pCpu, iProgAddress, expr);
+    }
+  }
+  
+  return -1;
+}
+
 string Integer::toString()
 {
   gint64 i;
   get(i);
   IUserInterface & TheUI = GetUserInterface();
-  return string(TheUI.FormatValue(i));
+  return string(TheUI.FormatValue(i, (unsigned int)bitmask));
 }
 
 
@@ -912,10 +952,10 @@ Float::Float(const char *_name, double newValue,const char *_desc)
 
 bool Float::Parse(const char *pValue, double &fValue) 
 {
-  return pValue ? sscanf(pValue,"%lg",&fValue) : false;
+  return pValue ? sscanf(pValue,"%lg",&fValue) == 1 : false;
 }
 
-Float * Float::New(const char *_name, const char *pValue, const char *desc) {
+Float * Float::NewObject(const char *_name, const char *pValue, const char *desc) {
   double fValue;
   if(Parse(pValue, fValue)) {
     return new Float(_name, fValue);
@@ -998,6 +1038,12 @@ void Float::get(Packet &pb)
   pb.EncodeFloat(d);
 }
 
+Value *Float::copy() {
+  double d;
+  get(d);
+  return new Float(d);
+}
+
 string Float::toString()
 {
   return toString("%#-16.16g");
